package provide buildsystem 1.0

namespace eval buildsystem {
  variable package_info
  variable package_default 1;       # default is release as binary
  namespace export read_componentdeps get_componentdeps_string get_componentdeps_string_release
}

#expected options_string: d(.*)
proc buildsystem::parse_debug_options {options_string} {
  set result_string ""

  # we are only interested in the part between the brackets
  if [regexp -all {\((.*)\)} $options_string match debuginfo] {
    set result_string "SAH_DEBUG=1"
    foreach command [split $debuginfo ","] {
      set command [string trim $command]
      if { [split $command "=" ] == "$command" } {
        set command "$command=1"
      }
      set result_string "$result_string SAH_DEBUG_$command"
    }
  }
  return $result_string
}

# parse component.dep files
proc buildsystem::read_componentdeps {filename componentdepsarray use_package_and_debug_info} {
  variable package_info
  variable package_default
  upvar $componentdepsarray componentdeps

  set cnames " "

  log::log debug "Reading component dependencies"
  if [file exists $filename] {
	  log::log debug "Opening $filename"
	  set infile [open $filename r]
	  set names "test"

	  while { [gets $infile line] >= 0} {
	    # is line not empty
	    if [string length $line] {
        # line starting with #! is treated as additional information
        if [regexp {^#! package_default=source} $line] {
          log::log debug "Default package behavior: release as source"
          set package_default 0
          continue
        } elseif [regexp {^#} $line] {
		    # line starting with # are treated as comment
		        continue
		    }

        set debug_options "";                          # store all debug options here
        set release_as_binary $package_default;        # do we need to package the component as binary or as source
        set extra_info ""

		    if { [string first "#" $line] > 0 } {
		        set parts [split $line "#"]
		        lassign $parts line comment
		    }
                    # lets see whether there is a : character
		    if { [string first ":" $line] != -1 } {
		        set parts [split $line ":"]
		        lassign $parts extra_info line

		        # let's see whether we can find a debug section-> d(.....)
			if [regexp -all {(d\(.*\))} $extra_info match debuginfo] {
			  # found the section we were looking for, get the options
			  set debug_options [parse_debug_options $debuginfo]
			  # remove the section from the option_string
			  regsub {d\(.*\)} $extra_info "" extra_info
		        }

			# let's investigate the rest of the option_string
			for { set i 0 } { $i < [string length $extra_info] } { incr i } {
			  set char [string index $extra_info $i ]
			  if { $char == "d"} {
			    set debug_options "$debug_options SAH_DEBUG=1"
            } elseif { $char == "b"} {
              set release_as_binary 1
			  } elseif { $char == "s"} {
			    set release_as_binary 0
			  } else {
			    puts "\nWarning ($filename): skipping invalid option \"$char\" in line \"$line\"\n"
			    #break
			  }
		       }
		    }

		    if { [string first "=" $line] > 0 } {
		        #split name from the version
		        set parts [split $line "="]
		        set assgm "EQUALS"
	            } elseif { [string first "<" $line] > 0 } {
		        #split name from the version
		        set parts [split $line "<"]
		        set assgm "LESSER"
		    } elseif { [string first ">" $line] > 0} {
		        #split name from the version
		        set parts [split $line ">"]
		        set assgm "GREATER"
		    } else {
		        continue
		    }

		    lassign $parts componentname componentversion
		    set componentstruct("depfile") $filename
		    set componentstruct("assignment") $assgm
		    set componentstruct("version") [string trim $componentversion]

                # this info can only be obtained from the project.list, not from the Component.dep files
		if { $use_package_and_debug_info == 1 } {
		  set component_package_info(release_as_binary) $release_as_binary
		  if {[string length $debug_options] > 0 } {
		    set component_package_info(enable_debug) 1
                    set component_package_info(debug_options) "$debug_options"
                  } else {
		    set component_package_info(enable_debug) 0
		  }
		  set package_info($componentname) [list [array get component_package_info]]

          log::log debug "Componentname: $componentname - release_as_binary=($release_as_binary)"
		}

		log::log debug "Componentname: $componentname Componentversion: $componentversion"

		if { [info exists componentdeps($componentname)] } {
		  lappend componentdeps($componentname) [array get componentstruct]
		  lappend cnames $componentname
		} else {
		  set componentdeps($componentname) [list [array get componentstruct]]
		  lappend cnames $componentname
		}
	    }
	  }
	  close $infile
  }
  log::log debug "Reading component dependencies done"
  return $cnames
}

# get all the dependencies of the component as string (used in Makefile generation)
#proc buildsystem::get_componentdeps_string {filename} {
#  set names [buildsystem::get_componentdeps_string_with_suffix $filename ""]
#  return $names
#}

proc buildsystem::get_componentdeps_string_with_suffix {filename suffix} {
  log::log debug "Reading component dependencies"

  set names " "
  set nametest(init) "0"
  variable checkpath

  set components " "

  foreach name [array names checkpath] {
      set components "$components $name"
  }  

  if [file exists $filename] {
    log::log debug "Opening $filename"
    set infile [open $filename r]

    while { [gets $infile line] >= 0} {
      # is line not empty
      if [string length $line] {
	      # line starting with # are treated as comment
	      if [regexp {^#} $line] {
	          continue
	      }

               # lets see whether there is a : character
	      if { [string first ":" $line] != -1 } {
		set parts [split $line ":"]
		lassign $parts extra_info line
	      }

	      if { [string first "=" $line] > 0 } {
	          #split name from the version
	          set parts [split $line "="]
	      } elseif { [string first "<" $line] > 0 } {
	          #split name from the version
	          set parts [split $line "<"]
	      } elseif { [string first ">" $line] > 0} {
	          #split name from the version
	          set parts [split $line ">"]
	      } else {
	          continue
	      }

	      lassign $parts componentname componentversion

	      # check if the component has allready been handled
	      if { [info exists nametest($componentname)] == 0 } {
            # Wildcard and don't care if the component is not present
            if { [string compare $componentversion "?"] == 0 } {
                if { [lsearch $components $componentname] != -1 } {
		set nametest($componentname) "1"
		set names "$names $componentname$suffix"
	      }
            } elseif { [string compare $componentversion "*"] == 0 } {
                if { [lsearch $components $componentname] != -1 } {
                    set nametest($componentname) "1"
                    set names "$names $componentname$suffix"
                } else {
                    puts stderr "Depending on non-existing component ($componentname)"
                    exit 2
                }
            } else { 
                set nametest($componentname) "1"
                set names "$names $componentname$suffix"
            }
          }
	}
    }
    close $infile
  }
  return $names
}
