#!/usr/bin/perl
use v5.14.1;
use strict;

our $configs     = {};
our $components  = {};

our $diff_method = $ARGV[0];
my  $file1       = $ARGV[1];
my  $file2       = $ARGV[2];

#my $file1="MAIN/ft/lbv2pl/zte_h202n/ft_multi_sip_rescue/project.list";
#my $file2="GEN/V4.6/PROJ/FT_STEP4/MAIN/ft/lbv2pl/zte_h202n/ft_multi_sip_rescue/project.list";

parseConfig("LEFT",  $file1);
parseConfig("RIGHT", $file2);

#printConfigs();
printComponents();

exit 0;

sub parseConfig {
    my $fh;
    my $name = shift;
    my $file = shift;
    open($fh, "cat $file |");
    while (!eof($fh)) {
        my $line = <$fh>;
        parseLine($name, $line);
    }
    close $fh;
}

sub printConfigs {
    our $configs;
    my $config;
    my $cp;
    my $version;

    print "Configs:\n";
    foreach $config (sort keys $configs) {
        foreach $cp (sort keys $configs->{$config}) {
            $version = $configs->{$config}->{$cp};
            print "$config : $cp = $version\n";
        }
    } 
}

sub printComponents {
    our $diff_method;
    our $configs;
    my $config;
    my $component;
    my $version;

    my $prev_cp = "";
    my $prev_ver = "";
    my $prev_config = "";
    print "Components:\n";
    foreach $component (sort keys $components) {
        foreach $config (sort keys $components->{$component}) {
            $version = $components->{$component}->{$config};
            for($diff_method) {
                when (/^eq/) {
                    if (($prev_ver eq $version) && ($prev_ver ne "")) {
                        print "$component=$prev_ver : $prev_config\n";
                        print "$component=$version : $config\n";
                    }
                }
                when (/^ne/) {
                    if (($prev_ver ne $version) && ($prev_ver ne "")) {
                        print "$component=$prev_ver : $prev_config\n";
                        print "$component=$version : $config\n";
                    }
                }
                when (/^lt/) {
                    if (($prev_ver lt $version) && ($prev_ver ne "")) {
                        print "$component=$prev_ver : $prev_config\n";
                        print "$component=$version : $config\n";
                    }
                }
                when (/^gt/) {
                    if (($prev_ver gt $version) && ($prev_ver ne "")) {
                        print "$component=$prev_ver : $prev_config\n";
                        print "$component=$version : $config\n";
                    }
                }
                default { }
            }
            $prev_ver       = $version;
            $prev_config    = $config;
        }
        $prev_cp    = $component;
        $prev_ver   = "";
    } 
    print "Extra Components:\n";
    foreach $component (sort keys $components) {
        if (keys($components->{$component}) != 2) {
            foreach $config (sort keys $components->{$component}) {
                $version = $components->{$component}->{$config};
                print "Component $component=$version : $config\n";
            }
        }
    } 
}

sub parseLine {
    our $configs;
    my $name = shift;
    my $line = shift;
    my $cp;
    my $version;
    if ($line =~ /^(.*)=(.*)$/) {
        $cp         = $1;
        $version    = $2;
        $configs->{$name}->{$cp} = $version;
        $components->{$cp}->{$name} = $version;
    }
}
