#!/usr/bin/perl
##
# p4 to Rel
# Quick integration script for integrating a branch to a new/latest rel
#
# Install
# In p4v select Tools -> Manage Custom Tools
# Select New -> Tool
#   Under 
#       Name:                               p4ToRel
#       Add to applicable context menus:    YES
#       Application:                        /home/sah5056/ws_w1/build/MAIN/tools/p4ToRel.pl
#       Arguments:                          --base %D (--update-latest || --update-major || --update-minor || --update-micro)
#       Start In:                           /home/sah5056/ws_w1/build/MAIN
#       Run tool in terminal:               this is up to you!
#       Refresh P4V upon completion:        YES
#       Press OK 
#       And Press OK again!
#
#       Right click on a BRANCH p4ToRel should now be visable!
#
#       Email: <david.cluytens_ext@softathome.com>
##
use strict;
use v5.10.1;
use Time::Piece;
use Getopt::Long;

my $p4From;
my $p4Dir;
my $p4To;
my $p4Rel;
my $rDate;
my $rMajor;
my $rMinor;
my $rMicro;
my $src_is_rel      = 0;
my $help            = 0;
my $update_mode     = 0;
my $cDate           = Time::Piece->new->strftime('%Y-%m-%d');

GetOptions('help'           => sub { $help = 1 },
        'base=s'            => \$p4From,
        'update-latest'     => sub { $update_mode = 1 },
        'update-micro'      => sub { $update_mode = 2 },
        'update-minor'      => sub { $update_mode = 3 },
        'update-major'      => sub { $update_mode = 4 },
        ) or printSyntax();


if ($p4From =~ /^(.*)\/DEV\/.*\/\.\.\.$/) {
    $p4Dir = $1;
} elsif ($p4From =~ /^(.*)\/MAIN\/\.\.\.$/) {
    $p4Dir = $1;
} elsif ($p4From =~ /^(.*)\/REL\/(.*)_V([\d]+)\.([\d]+)\.([\d]+)\/\.\.\.$/) {
    $p4Dir  = $1;
    $rDate  = $2;
    $rMajor = $3;
    $rMinor = $4;
    $rMicro = $5;
    $src_is_rel = 1;
} else {
    printSyntax();
}

if (!$src_is_rel) {
    ($rDate, $rMajor, $rMinor, $rMicro) = p4GetLatestComponent($p4Dir);
}

for($update_mode) {
    when (/^1$/) {
    }
    when (/^2$/) {
        $rDate = $cDate;    
        $rMicro++;
    }
    when (/^3$/) {
        $rDate = $cDate;
        $rMinor++;
        $rMicro=0;
    }
    when (/^4$/) {
        $rDate = $cDate;
        $rMajor++;
        $rMinor = 0;
        $rMicro = 0;
    }
    default {
        printSyntax();
    }
}

$p4Rel = $rDate . "_V" . $rMajor . "." . $rMinor . "." . $rMicro;
$p4To = $p4Dir . "/REL/" . $p4Rel . "/...";

print "p4 integrate -d -o $p4From $p4To\n";
system("p4 integrate -d -o $p4From $p4To");
exit $?;

sub printSyntax {
    print "p4ToRel --base <//depot/URL/...> --update-latest | --update-major | --update-minor | --update-micro\n\n";
    exit 1;
}

sub trim {
    my $string = shift;
    $string =~ s/^\s+//;
    $string =~ s/\s+$//;
    return $string;
}

sub p4GetLatestComponent {
    my $location    = shift;
    my $fh;
    my $p4location;
    my $lver        = ""; 
    my $ldate       = "";
    my $cdate       = "";
    my $cmajor      = 0;
    my $cminor      = 0;
    my $cmicro      = 0;
    my $lmajor      = 0;
    my $lminor      = 0;
    my $lmicro      = 0;

    $p4location = "${location}/REL/";
    open($fh, "p4 -p sahwp4s01.be.softathome.com:1666 dirs ${p4location}* |");

    while (!eof($fh)) {
        my $line = <$fh>;
        my $len  = length $p4location;
        my $cver = substr $line, $len;
        my $update_ver = 0;
        $cver = trim($cver);
        if ($cver =~ /^([\d]+-[\d]+-[\d]+)_V([\d]+)\.([\d]+)\.([\d]+)$/) {
            $cdate   = $1;
            $cmajor  = $2;
            $cminor  = $3;
            $cmicro  = $4;
            if ($cmajor > $lmajor) {
                $update_ver = 1;
            } elsif (($cmajor == $lmajor) && ($cminor > $lminor)) {
                $update_ver = 1;
            } elsif (($cmajor == $lmajor) && ($cminor == $lminor) && ($cmicro > $lmicro)) {
                $update_ver = 1;
            }
            if ($update_ver) {
                $ldate  = $cdate; 
                $lmajor = $cmajor;
                $lminor = $cminor;
                $lmicro = $cmicro;
            }
        } 
    }   
    close $fh;
    return($ldate, $lmajor, $lminor, $lmicro);
}

