/* ============================================================================
 *
 * Copyright : (c) 2005 SAGEM SA - Site Velizy - URD2
 *
 * The information and source code contained herein is the exclusive
 * property of SAGEM and may not be disclosed, examined, or
 * reproduced in whole or in part without explicit written authorization
 * of the company.
 *
 * ============================================================================
 */

/**
 * \file bsp_but.c Bsp Button functions
 *
 * This file contain all Bsp Button functions.
 */

/*
 * Includes
 * ~~~~~~~~
 */

#include "bsp_pri.h"

/*
 * Functions
 * ~~~~~~~~~
 */

/** Get BUTTON state. */
BOARD_BUTTON_STATE kSysGetButtonState(BOARD_BUTTON_NAME _nameUser)
{
	BOARD_BUTTON_STATE	_stateUser;

	/* Check */
	if ( (_nameUser < 0) || (_nameUser >= kButtonEnd) ){
		printk(	"%s: WARNING -> Button >= Max(%u)\n",
				__FUNCTION__,
				kButtonEnd);

		return(-1);
	}

	/* Retrieve state */
	{
		bsp_gpio_state_e	_stateGpio;
		bsp_gpio_name_e	_nameGpio;

		/* Retrieve nameGpio */
		_nameGpio = bsp_base.Index.But.Name.user2gpio[_nameUser];

		/* Retrieve stateGpio */
		if ((_stateGpio = bsp_gpio_get_state(_nameGpio)) == BSP_Gpio_state_failure) {
			/* Error */
			return(-1);
		}

		/* Retrieve userState */
		_stateUser = bsp_base.Index.But.State.gpio2user[_stateGpio];

#ifdef BSP_DEBUG
		/* If bsp */
		if (bsp_base.debug & BSP_FLAGS_DEBUG_BUT) {
			printk(	"%s: But -> nameUser(%u) nameGpio(%u) stateUser(%u) stateGpio(%u)\n",
					__FUNCTION__,
					_nameUser,
					_nameGpio,
					_stateUser,
					_stateGpio);
		}
#endif
	}

	return(_stateUser);
}
EXPORT_SYMBOL(kSysGetButtonState);

/** sysctl bsp_but_maps_proc.
 *
 *   \par Description
 *	<b>This function allow to set connexion tampon.</b>\n
 *
 *	@param _pTable:	Sysctl table reference.
 *	@param _write:		If write.
 *	@param _pFile:		File reference.
 *	@param _pBuffer:	Buffer reference.
 *	@param _pLen:		Length reference.
 *	@param _pPpos:		Position reference.
 *
 *	@return proc_dostring return.
 */
int bsp_but_maps_proc(	ctl_table		*_pTable,
					int			_write,
					struct file	*_pFile,
					void __user	*_pBuffer,
					size_t		*_pLen,
					loff_t		*_pPpos)
{
	char				*_pTampon		= (char *)_pTable->data;
	BOARD_RELAY_STATE	_userState;

	sprintf(	_pTampon,
			"\n");

	/* Next */
	_pTampon += strlen(_pTampon);

	/* Retrieve tampon */
	if ((_pTampon = (char *)_pTable->data) != (char *)0) {
		sprintf(	_pTampon,
				"| Reboot | Reset | Pairing |\n");

		/* Next */
		_pTampon += strlen(_pTampon);

		/* Adsl part */
		{
			/* Retrieve but state */
			_userState = kSysGetButtonState(kButtonReboot);
	
			switch (_userState) {
				case kButtonPushed:
				{
					/* Print */
					sprintf(	_pTampon,
							"|   1    |");
				}
				break;
	
				case kButtonReleased:
				{
					/* Print */
					sprintf(	_pTampon,
							"|   0    |");
				}
				break;
	
				default:
				{
					/* Print */
					sprintf(	_pTampon,
							"|   -    |");
				}
				break;
			}
		}

		/* Next */
		_pTampon += strlen(_pTampon);

		/* Internet green part */
		{
			/* Retrieve but state */
			_userState = kSysGetButtonState(kButtonReset);
	
			switch (_userState) {
				case kButtonPushed:
				{
					/* Print */
					sprintf(	_pTampon,
							"   1   |");
				}
				break;
	
				case kButtonReleased:
				{
					/* Print */
					sprintf(	_pTampon,
							"   0   |");
				}
				break;
	
				default:
				{
					/* Print */
					sprintf(	_pTampon,
							"   -   |");
				}
				break;
			}
		}

		/* Next */
		_pTampon += strlen(_pTampon);

		/* Internet red part */
		{
			/* Retrieve but state */
			_userState = kSysGetButtonState(kButtonPairing);
	
			switch (_userState) {
				case kButtonPushed:
				{
					/* Print */
					sprintf(	_pTampon,
							"    1    |");
				}
				break;
	
				case kButtonReleased:
				{
					/* Print */
					sprintf(	_pTampon,
							"    0    |");
				}
				break;
	
				default:
				{
					/* Print */
					sprintf(	_pTampon,
							"    -    |");
				}
				break;
			}
		}

		/* Next */
		_pTampon += strlen(_pTampon);
	}
	else {
		/* Print */
		sprintf(	_pTampon,
				"Process Error\n");

		/* Next */
		_pTampon += strlen(_pTampon);
	}

	sprintf(	_pTampon,
			"\n");

	/* Next */
	_pTampon += strlen(_pTampon);

	/* Call proc_dostring */
	return(proc_dostring(	_pTable,
						_write,
						_pFile,
						_pBuffer,
						_pLen,
						_pPpos));
}

// vim:ts=5:
