/*
 * ============================================================================
 *
 * Copyright : (c) 2005 SAGEM SA - Site Velizy - URD2
 *
 * The information and source code contained herein is the exclusive
 * property of SAGEM and may not be disclosed, examined, or
 * reproduced in whole or in part without explicit written authorization
 * of the company.
 *
 * ============================================================================
 */

/**
 * \file bsp.c Bsp Global Parameters
 *
 * This file contain all Bsp Global parameters.
 */

/*
 * Includes
 * ~~~~~~~~
 */

#include "bsp_pri.h"

/*
 * Globals
 * ~~~~~~~
 */

/* permParam will be copied from NOR FLASH into RAM at init time in order to avoid conflict with R/W filesystem, e.g. JFFS2 */
static bsp_permParam_s bsp_flashAddrInRAM;

/* Base struct. */
bsp_base_s		bsp_base = {
	.debug												= 0,
	.flags												= 0,
	/* Sysctl. */
	.SysCtl.head											= (struct ctl_table_header *)0,
	/** Sagem. */
	.SysCtl.sagem[0]									= {
		.ctl_name											= BSP_SysCtl_sagem,
		.procname											= "sagem",
		.mode											= 0555,
		.child											= bsp_base.SysCtl.sagem_list,
	},
	.SysCtl.sagem[1]									= {
		.ctl_name											= 0,
	},
	/** Sagem List. */
	.SysCtl.sagem_list[0]								= {
		.ctl_name											= BSP_SysCtl_sagem_list_type_firmware,
		.procname											= "firmwareType",
		.data											= &(bsp_base.Env.Krnopt.typeFirmware),
		.maxlen											= sizeof(bsp_base.Env.Krnopt.typeFirmware),
		.mode											= 0644,
		.proc_handler										= &proc_dointvec,
	},
	.SysCtl.sagem_list[1]								= {
		.ctl_name											= BSP_SysCtl_sagem_list_type_board,
		.procname											= "boardType",
		.data											= &(bsp_base.Board.Cur.boardType),
		.maxlen											= sizeof(bsp_base.Board.Cur.boardType),
		.mode											= 0444,
		.proc_handler										= &proc_dointvec,
	},
	.SysCtl.sagem_list[2]								= {
		.ctl_name											= BSP_SysCtl_sagem_list_boot,
		.procname											= "boot",
		.mode											= 0555,
		.child											= bsp_base.SysCtl.sagem_list_boot,
	},
	.SysCtl.sagem_list[3]								= {
		.ctl_name											= BSP_SysCtl_sagem_list_relay,
		.procname											= "relay",
		.mode											= 0555,
		.child											= bsp_base.SysCtl.sagem_list_relay,
	},
	.SysCtl.sagem_list[4]								= {
		.ctl_name											= BSP_SysCtl_sagem_list_led,
		.procname											= "led",
		.mode											= 0555,
		.child											= bsp_base.SysCtl.sagem_list_led,
	},
	.SysCtl.sagem_list[5]								= {
		.ctl_name											= BSP_SysCtl_sagem_list_but,
		.procname											= "but",
		.mode											= 0555,
		.child											= bsp_base.SysCtl.sagem_list_but,
	},
	.SysCtl.sagem_list[6]								= {
		.ctl_name											= BSP_SysCtl_sagem_list_gpio,
		.procname											= "gpio",
		.mode											= 0555,
		.child											= bsp_base.SysCtl.sagem_list_gpio,
	},
	.SysCtl.sagem_list[7]								= {
		.ctl_name											= 0,
	},
	/** Sagem List Boot. */
	.SysCtl.sagem_list_boot[0]							= {
		.ctl_name											= BSP_SysCtl_sagem_list_boot_recovery_force,
		.procname											= "recoveryForce",
		.data											= &(bsp_base.Env.Boot.Recovery.force),
		.maxlen											= sizeof(bsp_base.Env.Boot.Recovery.force),
		.mode											= 0644,
		.proc_handler										= &bsp_boot_recovery_force_proc_dointvec_minmax,
		.strategy											= &sysctl_intvec,
		.extra1											= &bsp_base.Env.Boot.Recovery.min,
		.extra2											= &bsp_base.Env.Boot.Recovery.max,
	},
	.SysCtl.sagem_list_boot[1]							= {
		.ctl_name											= 0,
	},
	/** Sagem List relay. */
	.SysCtl.sagem_list_relay[0]							= {
		.ctl_name											= BSP_SysCtl_sagem_list_relay_mode,
		.procname											= "mode",
		.data											= bsp_base.Env.Relay.Mode.Cur.name,
		.maxlen											= BSP_RELAY_MODE_NAME_MAX_SIZE,
		.mode											= 0644,
		.proc_handler										= &bsp_relay_mode_proc_dostring,
	},
	/** Sagem List relay. */
	.SysCtl.sagem_list_relay[1]							= {
		.ctl_name											= BSP_SysCtl_sagem_list_relay_maps,
		.procname											= "maps",
		.data											= bsp_base.SysCtl.tampon,
		.maxlen											= BSP_SYSCTL_TAMPON_MAX_SIZE,
		.mode											= 0444,
		.proc_handler										= &bsp_relay_maps_proc,
	},
	.SysCtl.sagem_list_relay[2]							= {
		.ctl_name											= 0,
	},
	/** Sagem List led. */
	.SysCtl.sagem_list_led[0]							= {
		.ctl_name											= BSP_SysCtl_sagem_list_led_maps,
		.procname											= "maps",
		.data											= bsp_base.SysCtl.tampon,
		.maxlen											= BSP_SYSCTL_TAMPON_MAX_SIZE,
		.mode											= 0444,
		.proc_handler										= &bsp_led_maps_proc,
	},
	.SysCtl.sagem_list_led[1]							= {
		.ctl_name											= 0,
	},
	/** Sagem List but. */
	.SysCtl.sagem_list_but[0]							= {
		.ctl_name											= BSP_SysCtl_sagem_list_but_maps,
		.procname											= "maps",
		.data											= bsp_base.SysCtl.tampon,
		.maxlen											= BSP_SYSCTL_TAMPON_MAX_SIZE,
		.mode											= 0444,
		.proc_handler										= &bsp_but_maps_proc,
	},
	.SysCtl.sagem_list_but[1]							= {
		.ctl_name											= 0,
	},
	/** Sagem List gpio maps. */
	.SysCtl.sagem_list_gpio[0]							= {
		.ctl_name											= BSP_SysCtl_sagem_list_gpio_maps,
		.procname											= "maps",
		.data											= bsp_base.SysCtl.tampon,
		.maxlen											= BSP_SYSCTL_TAMPON_MAX_SIZE,
		.mode											= 0444,
		.proc_handler										= &bsp_gpio_maps_proc,
	},
	.SysCtl.sagem_list_gpio[1]							= {
		.ctl_name											= 0,
	},
	/** Dev Perm Param. */
	.Dev[BSP_Dev_minor_pp]								= {
		.name											= "bsp",
		.fops.open										= bsp_devOpen,
		.fops.release										= bsp_devRelease,
		.fops.ioctl										= bsp_devIoctl,
	},
	/** Perm Parm Address. */
	.Pp.pFlashAddr											= &bsp_flashAddrInRAM,
	/** Perm Parm callback. */
	.Pp.getPpCb[BSP_Get_permParam_all] 						= (bsp_getPermParamCb_f)bsp_get_permParam_all,
	.Pp.getPpCb[BSP_Get_permParam_version] 						= (bsp_getPermParamCb_f)bsp_get_permParam_version,
	.Pp.getPpCb[BSP_Get_permParam_serial_number]					= (bsp_getPermParamCb_f)bsp_get_permParam_serialNumber,
	.Pp.getPpCb[BSP_Get_permParam_sagem_product_id]				= (bsp_getPermParamCb_f)bsp_get_permParam_sagemProductId,
	.Pp.getPpCb[BSP_Get_permParam_sagem_hardware_version] 			= (bsp_getPermParamCb_f)bsp_get_permParam_sagemHardwareVersion,
	.Pp.getPpCb[BSP_Get_permParam_sagem_hardware_revision]			= (bsp_getPermParamCb_f)bsp_get_permParam_sagemHardwareRevision,
	.Pp.getPpCb[BSP_Get_permParam_customer_product_id] 			= (bsp_getPermParamCb_f)bsp_get_permParam_customerProductId,
	.Pp.getPpCb[BSP_Get_permParam_customer_hardware_version] 		= (bsp_getPermParamCb_f)bsp_get_permParam_customerHardwareVersion,
	.Pp.getPpCb[BSP_Get_permParam_customer_hardware_revision]		= (bsp_getPermParamCb_f)bsp_get_permParam_customerHardwareRevision,
	.Pp.getPpCb[BSP_Get_permParam_ram_size] 					= (bsp_getPermParamCb_f)bsp_get_permParam_ramSize,
	.Pp.getPpCb[BSP_Get_permParam_flash_size] 					= (bsp_getPermParamCb_f)bsp_get_permParam_flashSize,
	.Pp.getPpCb[BSP_Get_permParam_firmware_version] 				= (bsp_getPermParamCb_f)bsp_get_permParam_firmVersion,
	.Pp.getPpCb[BSP_Get_permParam_lan_base_mac_address]			= (bsp_getPermParamCb_f)bsp_get_permParam_lanBaseMacAddr,
	.Pp.getPpCb[BSP_Get_permParam_lan_mac_address_range]			= (bsp_getPermParamCb_f)bsp_get_permParam_lanMacAddrRange,
	.Pp.getPpCb[BSP_Get_permParam_wan_base_mac_address] 			= (bsp_getPermParamCb_f)bsp_get_permParam_wanBaseMacAddr,
	.Pp.getPpCb[BSP_Get_permParam_wan_mac_address_range]			= (bsp_getPermParamCb_f)bsp_get_permParam_wanMacAddrRange,
	.Pp.getPpCb[BSP_Get_permParam_usb_mac] 						= (bsp_getPermParamCb_f)bsp_get_permParam_usbMacAddr,
	.Pp.getPpCb[BSP_Get_permParam_wifi_mac_address_1]				= (bsp_getPermParamCb_f)bsp_get_permParam_wifiMac1Addr,
	.Pp.getPpCb[BSP_Get_permParam_wifi_ssid_1] 					= (bsp_getPermParamCb_f)bsp_get_permParam_wifiSsid1,
	.Pp.getPpCb[BSP_Get_permParam_wifi_mac_address_2] 			= (bsp_getPermParamCb_f)bsp_get_permParam_wifiMac2Addr,
	.Pp.getPpCb[BSP_Get_permParam_wifi_ssid_2] 					= (bsp_getPermParamCb_f)bsp_get_permParam_wifiSsid2,
	.Pp.getPpCb[BSP_Get_permParam_manufacturer] 					= (bsp_getPermParamCb_f)bsp_get_permParam_manufacturer,
	.Pp.getPpCb[BSP_Get_permParam_manufacturer_oui]				= (bsp_getPermParamCb_f)bsp_get_permParam_manufacturerBaseMacAddr,
	.Pp.getPpCb[BSP_Get_permParam_manufacturer_url] 				= (bsp_getPermParamCb_f)bsp_get_permParam_manufacturerUrl,
	.Pp.getPpCb[BSP_Get_permParam_country_code] 					= (bsp_getPermParamCb_f)bsp_get_permParam_comCountry,
	.Pp.getPpCb[BSP_Get_permParam_wifi_key_1] 					= (bsp_getPermParamCb_f)bsp_get_permParam_wifiKey1,
	.Pp.getPpCb[BSP_Get_permParam_wifi_key_2] 					= (bsp_getPermParamCb_f)bsp_get_permParam_wifiKey2,
	.Pp.getPpCb[BSP_Get_permParam_h235_key] 					= (bsp_getPermParamCb_f)bsp_get_permParam_h235Key,
	.Pp.getPpCb[BSP_Get_permParam_public_dsa_syst] 				= (bsp_getPermParamCb_f)bsp_get_permParam_publicDsaSyst,
	.Pp.getPpCb[BSP_Get_permParam_public_dsa_resc] 				= (bsp_getPermParamCb_f)bsp_get_permParam_publicDsaResc,
	.Pp.getPpCb[BSP_Get_permParam_private_key] 					= (bsp_getPermParamCb_f)bsp_get_permParam_privateDsaKey,
	.Pp.getPpCb[BSP_Get_permParam_certificate] 					= (bsp_getPermParamCb_f)bsp_get_permParam_certificate,
	.Pp.getPpCb[BSP_Get_permParam_dsa] 						= (bsp_getPermParamCb_f)bsp_get_permParam_dsa,
	.Pp.getPpCb[BSP_Get_permParam_vendor_info_1] 				= (bsp_getPermParamCb_f)bsp_get_permParam_vendorInfo1,
	.Pp.getPpCb[BSP_Get_permParam_vendor_info_2] 				= (bsp_getPermParamCb_f)bsp_get_permParam_vendorInfo2,
	.Pp.getPpCb[BSP_Get_permParam_vendor_info_3] 				= (bsp_getPermParamCb_f)bsp_get_permParam_vendorInfo3,
	.Pp.getPpCb[BSP_Get_permParam_vendor_info_4] 				= (bsp_getPermParamCb_f)bsp_get_permParam_vendorInfo4,
	.Pp.getPpCb[BSP_Get_permParam_vendor_info_5] 				= (bsp_getPermParamCb_f)bsp_get_permParam_vendorInfo5,
	.Pp.getPpCb[BSP_Get_permParam_vendor_info_6] 				= (bsp_getPermParamCb_f)bsp_get_permParam_vendorInfo6,
	.Pp.getPpCb[BSP_Get_permParam_vendor_info_7] 				= (bsp_getPermParamCb_f)bsp_get_permParam_vendorInfo7,
	.Pp.getPpCb[BSP_Get_permParam_vendor_info_8] 				= (bsp_getPermParamCb_f)bsp_get_permParam_vendorInfo8,
	.Pp.getPpCb[BSP_Get_permParam_vendor_info_9] 				= (bsp_getPermParamCb_f)bsp_get_permParam_vendorInfo9,
	.Pp.getPpCb[BSP_Get_permParam_vendor_info_10] 				= (bsp_getPermParamCb_f)bsp_get_permParam_vendorInfo10,
	/** Permparam To Board Unknown. */
	.Pp.pp2board[BSP_Pp2board_unknown]						= {
		.pInfo[BSP_Pp_info_sagem_product_id]					= (char *)0,
		.pInfo[BSP_Pp_info_sagem_hardware_version]				= (char *)0,
		.pInfo[BSP_Pp_info_sagem_hardware_revision]				= (char *)0,
		.pInfo[BSP_Pp_info_customer_product_id]					= (char *)0,
		.pInfo[BSP_Pp_info_customer_hardware_version]			= (char *)0,
		.pInfo[BSP_Pp_info_customer_hardware_revision]			= (char *)0,
		.boardType										= BSP_Board_type_unknown,
		.boardInfo										= BSP_Board_info_unknown,
	},
	/** Permparam To Board Ftth_v2_1. */
	.Pp.pp2board[BSP_Pp2board_lb_ftth_v2_1]					= {
		.pInfo[BSP_Pp_info_sagem_product_id]					= (char *)0,
		.pInfo[BSP_Pp_info_sagem_hardware_version]				= "1.0",
		.pInfo[BSP_Pp_info_sagem_hardware_revision]				= (char *)0,
		.pInfo[BSP_Pp_info_customer_product_id]					= (char *)0,
		.pInfo[BSP_Pp_info_customer_hardware_version]			= "Livebox FTTH v2",
		.pInfo[BSP_Pp_info_customer_hardware_revision]			= (char *)0,
		.boardType										= BSP_Board_type_lb_v2,
		.boardInfo										= BSP_Board_info_lb_v2_1,
	},
	/** Permparam To Board Ftth_v2_2. */
	.Pp.pp2board[BSP_Pp2board_lb_ftth_v2_2]					= {
		.pInfo[BSP_Pp_info_sagem_product_id]					= (char *)0,
		.pInfo[BSP_Pp_info_sagem_hardware_version]				= "2.0",
		.pInfo[BSP_Pp_info_sagem_hardware_revision]				= (char *)0,
		.pInfo[BSP_Pp_info_customer_product_id]					= (char *)0,
		.pInfo[BSP_Pp_info_customer_hardware_version]			= "Livebox FTTH v2",
		.pInfo[BSP_Pp_info_customer_hardware_revision]			= (char *)0,
		.boardType										= BSP_Board_type_lb_v2,
		.boardInfo										= BSP_Board_info_lb_v2_2,
	},
	/** Permparam To Board lb_2. */
	.Pp.pp2board[BSP_Pp2board_lb_2]						= {
		.pInfo[BSP_Pp_info_sagem_product_id]					= (char *)0,
		.pInfo[BSP_Pp_info_sagem_hardware_version]				= (char *)0,
		.pInfo[BSP_Pp_info_sagem_hardware_revision]				= (char *)0,
		.pInfo[BSP_Pp_info_customer_product_id]					= (char *)0,
		.pInfo[BSP_Pp_info_customer_hardware_version]			= "Livebox 2",
		.pInfo[BSP_Pp_info_customer_hardware_revision]			= (char *)0,
		.boardType										= BSP_Board_type_lb_v2,
		.boardInfo										= BSP_Board_info_lb_v2_1,
	},
	/** Permparam To Board lb_2_mini. */
	.Pp.pp2board[BSP_Pp2board_lb_2_mini]					= {
		.pInfo[BSP_Pp_info_sagem_product_id]					= (char *)0,
		.pInfo[BSP_Pp_info_sagem_hardware_version]				= (char *)0,
		.pInfo[BSP_Pp_info_sagem_hardware_revision]				= (char *)0,
		.pInfo[BSP_Pp_info_customer_product_id]					= (char *)0,
		.pInfo[BSP_Pp_info_customer_hardware_version]			= "Livebox 2 Mini",
		.pInfo[BSP_Pp_info_customer_hardware_revision]			= (char *)0,
		.boardType										= BSP_Board_type_lb_mini,
		.boardInfo										= BSP_Board_info_lb_mini,
	},
	/** Permparam To Board Ftthv2. */
	.Pp.pp2board[BSP_Pp2board_lb_ftthv2]					= {
		.pInfo[BSP_Pp_info_sagem_product_id]					= (char *)0,
		.pInfo[BSP_Pp_info_sagem_hardware_version]				= (char *)0,
		.pInfo[BSP_Pp_info_sagem_hardware_revision]				= (char *)0,
		.pInfo[BSP_Pp_info_customer_product_id]					= (char *)0,
		.pInfo[BSP_Pp_info_customer_hardware_version]			= "Livebox FTTHv2",
		.pInfo[BSP_Pp_info_customer_hardware_revision]			= (char *)0,
		.boardType										= BSP_Board_type_lb_ftth_v2,
		.boardInfo										= BSP_Board_info_lb_ftth_v2,
	},
	/** Permparam To Board Fast_3504. */
	.Pp.pp2board[BSP_Pp2board_fast_3504]					= {
		.pInfo[BSP_Pp_info_sagem_product_id]					= (char *)0,
		.pInfo[BSP_Pp_info_sagem_hardware_version]				= (char *)0,
		.pInfo[BSP_Pp_info_sagem_hardware_revision]				= (char *)0,
		.pInfo[BSP_Pp_info_customer_product_id]					= (char *)0,
		.pInfo[BSP_Pp_info_customer_hardware_version]			= "Fast3504",
		.pInfo[BSP_Pp_info_customer_hardware_revision]			= (char *)0,
		.boardType										= BSP_Board_type_fast_3504,
		.boardInfo										= BSP_Board_info_fast_3504,
	},
	/** Permparam To Board lb2. */
	.Pp.pp2board[BSP_Pp2board_lb2]						= {
		.pInfo[BSP_Pp_info_sagem_product_id]					= (char *)0,
		.pInfo[BSP_Pp_info_sagem_hardware_version]				= (char *)0,
		.pInfo[BSP_Pp_info_sagem_hardware_revision]				= (char *)0,
		.pInfo[BSP_Pp_info_customer_product_id]					= (char *)0,
		.pInfo[BSP_Pp_info_customer_hardware_version]			= "lb2",
		.pInfo[BSP_Pp_info_customer_hardware_revision]			= (char *)0,
		.boardType										= BSP_Board_type_lb_v2,
		.boardInfo										= BSP_Board_info_lb_v2_1,
	},
	/** Index. */
	.Index											= {
		/** Relay. */
		.Relay.Name.user2gpio[kRelay1]						= BSP_Gpio_name_relay_fxs_all,
		.Relay.Name.user2gpio[kRelay2]						= BSP_Gpio_name_relay_fxo,
		.Relay.State.user2gpio[kRelayOff]						= BSP_Gpio_state_unset,
		.Relay.State.user2gpio[kRelayOn]						= BSP_Gpio_state_set,
		.Relay.State.gpio2user[BSP_Gpio_state_unset]				= kRelayOff,
		.Relay.State.gpio2user[BSP_Gpio_state_set]				= kRelayOn,
		/** Button. */
		.But.Name.user2gpio[kButtonReset]						= BSP_Gpio_name_button_reset,
		.But.Name.user2gpio[kButtonReboot]						= BSP_Gpio_name_button_reboot,
		.But.Name.user2gpio[kButtonPairing]					= BSP_Gpio_name_button_pairing,
		.But.State.user2gpio[kButtonReleased]					= BSP_Gpio_state_unset,
		.But.State.user2gpio[kButtonPushed]					= BSP_Gpio_state_set,
		.But.State.gpio2user[BSP_Gpio_state_unset]				= kButtonReleased,
		.But.State.gpio2user[BSP_Gpio_state_set]				= kButtonPushed,
		/** Led. */
		.Led.Name.user2gpio[kLedPower]						= BSP_Gpio_name_led_power,
		.Led.Name.user2gpio[kLedInternetRed]					= BSP_Gpio_name_led_internet_red,
		.Led.Name.user2gpio[kLedInternetGreen]					= BSP_Gpio_name_led_internet_green,
		.Led.Name.user2gpio[kLedVoip]							= BSP_Gpio_name_led_voip,
		.Led.Name.user2gpio[kLedLan]							= BSP_Gpio_name_led_lan,
		.Led.Name.user2gpio[kLedWireless]						= BSP_Gpio_name_led_wireless,
		.Led.Name.user2gpio[kLedUpgrade]						= BSP_Gpio_name_led_upgrade,
		.Led.State.user2gpio[kLedStateOff]						= BSP_Gpio_state_unset,
		.Led.State.user2gpio[kLedStateOn]						= BSP_Gpio_state_set,
		.Led.State.gpio2user[BSP_Gpio_state_unset]				= kLedStateOff,
		.Led.State.gpio2user[BSP_Gpio_state_set]				= kLedStateOn,
	},
	/** Env. */
	.Env												= {
		/** Boot. */
		.Boot.Recovery.force								= BSP_Boot_recovery_force_unknown,
		.Boot.Recovery.force								= BSP_Boot_recovery_force_unknown,
		.Boot.Recovery.min									= 0,
		.Boot.Recovery.max									= BSP_Boot_recovery_force_max - 1,
		/** Firmware. */
		.Firmware.Type[BSP_Firmware_type_operational].name		= "operational",
		.Firmware.Type[BSP_Firmware_type_lanrescue].name			= "lanrescue",
		.Firmware.Type[BSP_Firmware_type_rescue].name			= "rescue",
		/** Relay. */
		.Relay.Mode.All[BSP_Relay_mode_none].name				= "none",
		.Relay.Mode.All[BSP_Relay_mode_none].notify				= BSP_Notify_evt_relay_mode_none,
		.Relay.Mode.All[BSP_Relay_mode_adsl].name				= "adsl",
		.Relay.Mode.All[BSP_Relay_mode_adsl].notify				= BSP_Notify_evt_relay_mode_adsl,
		.Relay.Mode.All[BSP_Relay_mode_ftth].name				= "ftth",
		.Relay.Mode.All[BSP_Relay_mode_ftth].notify				= BSP_Notify_evt_relay_mode_ftth,
		/** Kernel Options. */
		.Krnopt.typeFirmware								= BSP_Firmware_type_unknown,
		.Krnopt.myfs_start									= 0,
		.Krnopt.myfs_size									= 0,
		.Krnopt.operational_start							= 0,
		.Krnopt.rescue_start								= 0,
	},
	/** Board. */
	.Board											= {
		/** Gpio Intr. */
		.GpioIntr.low										= BSP_Gpio_intr_0_to_15,
		.GpioIntr.high										= BSP_Gpio_intr_16_to_31,
		/** Current Type. */
		.Cur.pp2board										= BSP_Pp2board_unknown,
		.Cur.boardType										= BSP_Board_type_unknown,
		.Cur.boardInfo										= BSP_Board_info_unknown,
		/** Gpio Name. */
		.pGpioNameTxt[BSP_Gpio_name_reserved]					= "        Reserved        ",
		.pGpioNameTxt[BSP_Gpio_name_led_adsl]					= "        Led Adsl        ",
		.pGpioNameTxt[BSP_Gpio_name_led_internet_green]			= "       Led @ green      ",
		.pGpioNameTxt[BSP_Gpio_name_led_internet_red]			= "        Led @ red       ",
		.pGpioNameTxt[BSP_Gpio_name_led_voip]					= "        Led voip        ",
		.pGpioNameTxt[BSP_Gpio_name_led_wireless]				= "      Led wireless      ",
		.pGpioNameTxt[BSP_Gpio_name_led_power]					= "        Led power       ",
		.pGpioNameTxt[BSP_Gpio_name_led_lan]					= "        Led lan         ",
		.pGpioNameTxt[BSP_Gpio_name_led_upgrade]				= "      Led upgrade       ",
		.pGpioNameTxt[BSP_Gpio_name_led_phone_1]				= "      Led phone 1       ",
		.pGpioNameTxt[BSP_Gpio_name_led_phone_2]				= "      Led phone 2       ",
		.pGpioNameTxt[BSP_Gpio_name_button_reset]				= "       But reset        ",
		.pGpioNameTxt[BSP_Gpio_name_button_reboot]				= "       But reboot       ",
		.pGpioNameTxt[BSP_Gpio_name_button_pairing]				= "       But pairing      ",
		.pGpioNameTxt[BSP_Gpio_name_relay_fxs_all]				= "      Relay Fxs all     ",
		.pGpioNameTxt[BSP_Gpio_name_relay_fxs_1]				= "       Relay Fxs 1      ",
		.pGpioNameTxt[BSP_Gpio_name_relay_fxs_2]				= "       Relay Fxs 2      ",
		.pGpioNameTxt[BSP_Gpio_name_relay_fxo]					= "        Relay Fxo       ",
		.pGpioNameTxt[BSP_Gpio_name_spi_cs_fxs_all_and_fxo]		= "  Spi Cs Fxs all & Fxo  ",
		.pGpioNameTxt[BSP_Gpio_name_spi_cs_fxs_all]				= "     Spi Cs Fxs all     ",
		.pGpioNameTxt[BSP_Gpio_name_spi_cs_fxo]					= "       Spi Cs Fxo       ",
		.pGpioNameTxt[BSP_Gpio_name_voice_rst_fxs_all_and_fxo]		= "   Reset Fxs all & Fxo  ",
		.pGpioNameTxt[BSP_Gpio_name_voice_rst_fxs_1]				= "       Reset Fxs 1      ",
		.pGpioNameTxt[BSP_Gpio_name_voice_rst_fxs_2]				= "       Reset Fxs 2      ",
		.pGpioNameTxt[BSP_Gpio_name_voice_rst_fxs_2_and_fxo]		= "    Reset Fxs 2 & Fxo   ",
		.pGpioNameTxt[BSP_Gpio_name_voice_int_fxs_1]				= "        Int Fxs 1       ",
		.pGpioNameTxt[BSP_Gpio_name_voice_int_fxs_2]				= "        Int Fxs 2       ",
		.pGpioNameTxt[BSP_Gpio_name_voice_int_fxs_2_and_fxo]		= "     Int Fxs 2 & Fxo    ",
		.pGpioNameTxt[BSP_Gpio_name_config_1]					= "        Config 1        ",
		.pGpioNameTxt[BSP_Gpio_name_config_2]					= "        Config 2        ",
		.pGpioNameTxt[BSP_Gpio_name_config_3]					= "        Config 3        ",
		.pGpioNameTxt[BSP_Gpio_name_usb_pwr_en_master_all]		= " Usb pwr en master all  ",
		.pGpioNameTxt[BSP_Gpio_name_usb_pwr_en_master_1]			= "   Usb pwr en master 1  ",
		.pGpioNameTxt[BSP_Gpio_name_usb_pwr_en_master_2]			= "   Usb pwr en master 2  ",
		.pGpioNameTxt[BSP_Gpio_name_usb_overcurrent_master_all]	= " Usb overcur master all ",
		.pGpioNameTxt[BSP_Gpio_name_usb_overcurrent_master_1]		= "  Usb overcur master 1  ",
		.pGpioNameTxt[BSP_Gpio_name_usb_overcurrent_master_2]		= "  Usb overcur master 2  ",
		.pGpioNameTxt[BSP_Gpio_name_usb_pwr_en_slave]			= "     Usb pwr en slave   ",
		.pGpioNameTxt[BSP_Gpio_name_usb_pwr_detect_slave]			= "  Usb pwr detect slave  ",
		.pGpioNameTxt[BSP_Gpio_name_usb_rst_hub]				= "        Usb rst hub     ",
		.pGpioNameTxt[BSP_Gpio_name_mini_led_act_eth_all]			= "  Mini led act eth all  ",
		.pGpioNameTxt[BSP_Gpio_name_mini_led_act_eth_0]			= "   Mini led act eth 0   ",
		.pGpioNameTxt[BSP_Gpio_name_mini_led_act_eth_1]			= "   Mini led act eth 1   ",
		.pGpioNameTxt[BSP_Gpio_name_mini_pin_crs_eth_0]			= "   Mini pin crs eth 0   ",
		.pGpioNameTxt[BSP_Gpio_name_mini_pin_crs_eth_1]			= "   Mini pin crs eth 1   ",
		.pGpioNameTxt[BSP_Gpio_name_adsl_rst_afe]				= "       Adsl rst afe     ",
		.pGpioNameTxt[BSP_Gpio_name_dying_gasp]					= "        Dying gasp      ",
		.pGpioNameTxt[BSP_Gpio_name_switch_rst]					= "        Switch rst      ",
		.pGpioNameTxt[BSP_Gpio_name_switch_int]					= "        Switch int      ",
		.pGpioNameTxt[BSP_Gpio_name_phy_rst_all]				= "       Phy rst all      ",
		.pGpioNameTxt[BSP_Gpio_name_phy_rst_1]					= "        Phy rst 1       ",
		.pGpioNameTxt[BSP_Gpio_name_phy_rst_2]					= "        Phy rst 2       ",
		.pGpioNameTxt[BSP_Gpio_name_wifi_rst]					= "        Wifi rst        ",
		.pGpioNameTxt[BSP_Gpio_name_cmd_nl]					= "         Cmd Nl         ",
	},
	/** Board All Unknwon. */
	.Board.All[BSP_Board_info_unknown]						= {
		.Gpio[0].name				= BSP_Gpio_name_reserved,
		.Gpio[0].flags				= (BSP_Gpio_flags_dir_out),
		.Gpio[1].name				= BSP_Gpio_name_reserved,
		.Gpio[1].flags				= (BSP_Gpio_flags_dir_out),
		.Gpio[2].name				= BSP_Gpio_name_reserved,
		.Gpio[2].flags				= (BSP_Gpio_flags_dir_out),
		.Gpio[3].name				= BSP_Gpio_name_reserved,
		.Gpio[3].flags				= (BSP_Gpio_flags_dir_out),
		.Gpio[4].name				= BSP_Gpio_name_reserved,
		.Gpio[4].flags				= (BSP_Gpio_flags_dir_out),
		.Gpio[5].name				= BSP_Gpio_name_reserved,
		.Gpio[5].flags				= (BSP_Gpio_flags_dir_out),
		.Gpio[6].name				= BSP_Gpio_name_reserved,
		.Gpio[6].flags				= (BSP_Gpio_flags_dir_out),
		.Gpio[7].name				= BSP_Gpio_name_reserved,
		.Gpio[7].flags				= (BSP_Gpio_flags_dir_out),
		.Gpio[8].name				= BSP_Gpio_name_reserved,
		.Gpio[8].flags				= (BSP_Gpio_flags_dir_out),
		.Gpio[9].name				= BSP_Gpio_name_reserved,
		.Gpio[9].flags				= (BSP_Gpio_flags_dir_out),
		.Gpio[10].name				= BSP_Gpio_name_reserved,
		.Gpio[10].flags			= (BSP_Gpio_flags_dir_out),
		.Gpio[11].name				= BSP_Gpio_name_reserved,
		.Gpio[11].flags			= (BSP_Gpio_flags_dir_out),
		.Gpio[12].name				= BSP_Gpio_name_reserved,
		.Gpio[12].flags			= (BSP_Gpio_flags_dir_out),
		.Gpio[13].name				= BSP_Gpio_name_reserved,
		.Gpio[13].flags			= (BSP_Gpio_flags_dir_out),
		.Gpio[14].name				= BSP_Gpio_name_reserved,
		.Gpio[14].flags			= (BSP_Gpio_flags_dir_out),
		.Gpio[15].name				= BSP_Gpio_name_reserved,
		.Gpio[15].flags			= (BSP_Gpio_flags_dir_out),
		.Gpio[16].name				= BSP_Gpio_name_reserved,
		.Gpio[16].flags			= (BSP_Gpio_flags_dir_out),
		.Gpio[17].name				= BSP_Gpio_name_reserved,
		.Gpio[17].flags			= (BSP_Gpio_flags_dir_out),
		.Gpio[18].name				= BSP_Gpio_name_reserved,
		.Gpio[18].flags			= (BSP_Gpio_flags_dir_out),
		.Gpio[19].name				= BSP_Gpio_name_reserved,
		.Gpio[19].flags			= (BSP_Gpio_flags_dir_out),
		.Gpio[20].name				= BSP_Gpio_name_reserved,
		.Gpio[20].flags			= (BSP_Gpio_flags_dir_out),
		.Gpio[21].name				= BSP_Gpio_name_reserved,
		.Gpio[21].flags			= (BSP_Gpio_flags_dir_out),
		.Gpio[22].name				= BSP_Gpio_name_reserved,
		.Gpio[22].flags			= (BSP_Gpio_flags_dir_out),
		.Gpio[23].name				= BSP_Gpio_name_reserved,
		.Gpio[23].flags			= (BSP_Gpio_flags_dir_out),
		.Gpio[24].name				= BSP_Gpio_name_reserved,
		.Gpio[24].flags			= (BSP_Gpio_flags_dir_out),
		.Gpio[25].name				= BSP_Gpio_name_reserved,
		.Gpio[25].flags			= (BSP_Gpio_flags_dir_out),
		.Gpio[26].name				= BSP_Gpio_name_reserved,
		.Gpio[26].flags			= (BSP_Gpio_flags_dir_out),
		.Gpio[27].name				= BSP_Gpio_name_reserved,
		.Gpio[27].flags			= (BSP_Gpio_flags_dir_out),
		.Gpio[28].name				= BSP_Gpio_name_reserved,
		.Gpio[28].flags			= (BSP_Gpio_flags_dir_out),
		.Gpio[29].name				= BSP_Gpio_name_reserved,
		.Gpio[29].flags			= (BSP_Gpio_flags_dir_out),
		.Gpio[30].name				= BSP_Gpio_name_reserved,
		.Gpio[30].flags			= (BSP_Gpio_flags_dir_out),
		.Gpio[31].name				= BSP_Gpio_name_reserved,
		.Gpio[31].flags			= (BSP_Gpio_flags_dir_out),
		.Lan.Mii[0].dev			= BSP_Lan_mii_dev_switch,
		.Lan.Mii[0].smiAddr			= 0xff,
		.Lan.Mii[1].dev			= BSP_Lan_mii_dev_switch,
		.Lan.Mii[1].smiAddr			= 0xff,
		.Lan.MiiDev.Switch.nb		= 0,
		.Lan.MiiDev.Phy.nb			= 0,
		.Lan.all[0]				= BSP_Gpio_name_reserved,
		.Usb.Hub.nb				= 0,
		.Usb.Hub.all[0]			= BSP_Gpio_name_reserved,
		.Usb.Master.nb				= 0,
		.Usb.Master.all[0]			= BSP_Gpio_name_reserved,
		.Usb.Slave.nb				= 0,
		.Usb.Slave.all[0]			= BSP_Gpio_name_reserved,
		.Voice.Fxs.nb				= 0,
		.Voice.Fxs.all[0]			= BSP_Gpio_name_reserved,
		.Voice.Fxo.nb				= 0,
		.Voice.Fxo.all[0]			= BSP_Gpio_name_reserved,
		.Relay.Fxs.nb				= 0,
		.Relay.Fxs.map				= 0,
		.Relay.Fxs.all[0]			= BSP_Gpio_name_reserved,
		.Relay.Fxo.nb				= 0,
		.Relay.Fxo.map				= 0,
		.Relay.Fxo.all[0]			= BSP_Gpio_name_reserved,
	},
	/** Board All 3504. */
	.Board.All[BSP_Board_info_fast_3504]					= {
		.Gpio[0].name				= BSP_Gpio_name_led_phone_2,
		.Gpio[0].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio),
		.Gpio[1].name				= BSP_Gpio_name_spi_cs_fxs_all,
		.Gpio[1].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_spi_cs),
		.Gpio[2].name				= BSP_Gpio_name_spi_cs_fxo,
		.Gpio[2].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_spi_cs),
		.Gpio[3].name				= BSP_Gpio_name_reserved,
		.Gpio[3].flags				= (BSP_Gpio_flags_dir_out),
		.Gpio[4].name				= BSP_Gpio_name_reserved,
		.Gpio[4].flags				= (BSP_Gpio_flags_dir_out),
		.Gpio[5].name				= BSP_Gpio_name_usb_rst_hub,
		.Gpio[5].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio),
		.Gpio[6].name				= BSP_Gpio_name_reserved,
		.Gpio[6].flags				= (BSP_Gpio_flags_dir_out),
		.Gpio[7].name				= BSP_Gpio_name_reserved,
		.Gpio[7].flags				= (BSP_Gpio_flags_dir_out),
		.Gpio[8].name				= BSP_Gpio_name_reserved,
		.Gpio[8].flags				= (BSP_Gpio_flags_dir_out),
		.Gpio[9].name				= BSP_Gpio_name_led_upgrade,
		.Gpio[9].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio),
		.Gpio[10].name				= BSP_Gpio_name_voice_rst_fxs_all_and_fxo,
		.Gpio[10].flags			= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio),
		.Gpio[11].name				= BSP_Gpio_name_reserved,
		.Gpio[11].flags			= (BSP_Gpio_flags_dir_out),
		.Gpio[12].name				= BSP_Gpio_name_usb_pwr_en_slave,
		.Gpio[12].flags			= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_used_as_gpio),
		.Gpio[13].name				= BSP_Gpio_name_usb_pwr_detect_slave,
		.Gpio[13].flags			= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_used_as_gpio),
		.Gpio[14].name				= BSP_Gpio_name_relay_fxo,
		.Gpio[14].flags			= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio),
		.Gpio[15].name				= BSP_Gpio_name_usb_pwr_en_master_2,
		.Gpio[15].flags			= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio),
		.Gpio[16].name				= BSP_Gpio_name_usb_pwr_en_master_1,
		.Gpio[16].flags			= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio),
		.Gpio[17].name				= BSP_Gpio_name_reserved,
		.Gpio[17].flags			= (BSP_Gpio_flags_dir_out),
		.Gpio[18].name				= BSP_Gpio_name_reserved,
		.Gpio[18].flags			= (BSP_Gpio_flags_dir_out),
		.Gpio[19].name				= BSP_Gpio_name_relay_fxs_all,
		.Gpio[19].flags			= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio),
		.Gpio[20].name				= BSP_Gpio_name_reserved,
		.Gpio[20].flags			= (BSP_Gpio_flags_dir_out),
		.Gpio[21].name				= BSP_Gpio_name_reserved,
		.Gpio[21].flags			= (BSP_Gpio_flags_dir_out),
		.Gpio[22].name				= BSP_Gpio_name_reserved,
		.Gpio[22].flags			= (BSP_Gpio_flags_dir_out),
		.Gpio[23].name				= BSP_Gpio_name_reserved,
		.Gpio[23].flags			= (BSP_Gpio_flags_dir_out),
		.Gpio[24].name				= BSP_Gpio_name_button_reset,
		.Gpio[24].flags			= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio),
		.Gpio[25].name				= BSP_Gpio_name_reserved,
		.Gpio[25].flags			= (BSP_Gpio_flags_dir_out),
		.Gpio[26].name				= BSP_Gpio_name_button_pairing,
		.Gpio[26].flags			= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio),
		.Gpio[27].name				= BSP_Gpio_name_led_internet_red,
		.Gpio[27].flags			= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio),
		.Gpio[28].name				= BSP_Gpio_name_led_internet_green,
		.Gpio[28].flags			= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio),
		.Gpio[29].name				= BSP_Gpio_name_led_phone_2,
		.Gpio[29].flags			= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio),
		.Gpio[30].name				= BSP_Gpio_name_led_wireless,
		.Gpio[30].flags			= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio),
		.Gpio[31].name				= BSP_Gpio_name_led_adsl,
		.Gpio[31].flags			= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio),
		.Lan.Mii[0].dev			= BSP_Lan_mii_dev_switch,
		.Lan.Mii[0].smiAddr			= 0xff,
		.Lan.Mii[0].Info.sw.miiNum	= -1,
		.Lan.Mii[1].dev			= BSP_Lan_mii_dev_switch,
		.Lan.Mii[1].smiAddr			= 0xff,
		.Lan.Mii[1].Info.sw.miiNum	= -1,
		.Lan.MiiDev.Switch.nb		= 1,
		.Lan.MiiDev.Phy.nb			= 0,
		.Lan.all[0]				= BSP_Gpio_name_switch_rst,
		.Lan.all[1]				= BSP_Gpio_name_reserved,
		.Usb.Hub.nb				= 1,
		.Usb.Hub.all[0]			= BSP_Gpio_name_usb_rst_hub,
		.Usb.Hub.all[1]			= BSP_Gpio_name_reserved,
		.Usb.Master.nb				= 2,
		.Usb.Master.all[0]			= BSP_Gpio_name_usb_pwr_en_master_1,
		.Usb.Master.all[1]			= BSP_Gpio_name_usb_pwr_en_master_2,
		.Usb.Master.all[2]			= BSP_Gpio_name_reserved,
		.Usb.Slave.nb				= 1,
		.Usb.Slave.all[0]			= BSP_Gpio_name_usb_pwr_en_slave,
		.Usb.Slave.all[1]			= BSP_Gpio_name_reserved,
		.Voice.Fxs.nb				= 2,
		.Voice.Fxs.all[0]			= BSP_Gpio_name_voice_rst_fxs_all_and_fxo,
		.Voice.Fxs.all[1]			= BSP_Gpio_name_reserved,
		.Voice.Fxo.nb				= 1,
		.Voice.Fxo.all[0]			= BSP_Gpio_name_reserved,
		.Relay.Fxs.nb				= 2,
		.Relay.Fxs.map				= ( BSP_MAP_RELAY_ON_FXS_1 | BSP_MAP_RELAY_ON_FXS_2 ),
		.Relay.Fxs.all[0]			= BSP_Gpio_name_relay_fxs_all,
		.Relay.Fxs.all[1]			= BSP_Gpio_name_reserved,
		.Relay.Fxo.nb				= 1,
		.Relay.Fxo.map				= ( BSP_MAP_RELAY_ON_FX0_1),
		.Relay.Fxo.all[0]			= BSP_Gpio_name_relay_fxo,
		.Relay.Fxo.all[1]			= BSP_Gpio_name_reserved,
	},
	/** Board All FTTH v2. */
	.Board.All[BSP_Board_info_lb_ftth_v2]					= {
		.Gpio[0].name				= BSP_Gpio_name_reserved,
		.Gpio[0].flags				= (BSP_Gpio_flags_dir_out),
		.Gpio[1].name				= BSP_Gpio_name_spi_cs_fxs_all,
		.Gpio[1].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_spi_cs),
		.Gpio[2].name				= BSP_Gpio_name_spi_cs_fxo,
		.Gpio[2].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_spi_cs),
		.Gpio[3].name				= BSP_Gpio_name_voice_rst_fxs_all_and_fxo,
		.Gpio[3].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio),
		.Gpio[4].name				= BSP_Gpio_name_config_1,
		.Gpio[4].flags				= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_used_as_config),
		.Gpio[5].name				= BSP_Gpio_name_config_2,
		.Gpio[5].flags				= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_used_as_config),
		.Gpio[6].name				= BSP_Gpio_name_config_3,
		.Gpio[6].flags				= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_used_as_config),
		.Gpio[7].name				= BSP_Gpio_name_led_wireless,
		.Gpio[7].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio),
		.Gpio[8].name				= BSP_Gpio_name_led_voip,
		.Gpio[8].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio),
		.Gpio[9].name				= BSP_Gpio_name_led_lan,
		.Gpio[9].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio),
		.Gpio[10].name				= BSP_Gpio_name_led_internet_red,
		.Gpio[10].flags			= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio),
		.Gpio[11].name				= BSP_Gpio_name_led_power,
		.Gpio[11].flags			= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio),
		.Gpio[12].name				= BSP_Gpio_name_reserved,
		.Gpio[12].flags			= (BSP_Gpio_flags_dir_out),
		.Gpio[13].name				= BSP_Gpio_name_cmd_nl,
		.Gpio[13].flags			= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio),
		.Gpio[14].name				= BSP_Gpio_name_reserved,
		.Gpio[14].flags			= (BSP_Gpio_flags_dir_out),
		.Gpio[15].name				= BSP_Gpio_name_usb_pwr_en_master_1,
		.Gpio[15].flags			= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio),
		.Gpio[16].name				= BSP_Gpio_name_adsl_rst_afe,
		.Gpio[16].flags			= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio),
		.Gpio[17].name				= BSP_Gpio_name_usb_overcurrent_master_1,
		.Gpio[17].flags			= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio),
		.Gpio[18].name				= BSP_Gpio_name_reserved,
		.Gpio[18].flags			= (BSP_Gpio_flags_dir_out),
		.Gpio[19].name				= BSP_Gpio_name_relay_fxs_all,
		.Gpio[19].flags			= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio),
		.Gpio[20].name				= BSP_Gpio_name_reserved,
		.Gpio[20].flags			= (BSP_Gpio_flags_dir_out),
		.Gpio[21].name				= BSP_Gpio_name_reserved,
		.Gpio[21].flags			= (BSP_Gpio_flags_dir_out),
		.Gpio[22].name				= BSP_Gpio_name_reserved,
		.Gpio[22].flags			= (BSP_Gpio_flags_dir_out),
		.Gpio[23].name				= BSP_Gpio_name_reserved,
		.Gpio[23].flags			= (BSP_Gpio_flags_dir_out),
		.Gpio[24].name				= BSP_Gpio_name_button_reset,
		.Gpio[24].flags			= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio),
		.Gpio[25].name				= BSP_Gpio_name_dying_gasp,
		.Gpio[25].flags			= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_active_low | BSP_Gpio_flags_intr | BSP_Gpio_flags_used_as_gpio),
		.Gpio[26].name				= BSP_Gpio_name_button_pairing,
		.Gpio[26].flags			= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio),
		.Gpio[27].name				= BSP_Gpio_name_led_internet_green,
		.Gpio[27].flags			= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio),
		.Gpio[28].name				= BSP_Gpio_name_switch_rst,
		.Gpio[28].flags			= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio),
		.Gpio[29].name				= BSP_Gpio_name_reserved,
		.Gpio[29].flags			= (BSP_Gpio_flags_dir_out),
		.Gpio[30].name				= BSP_Gpio_name_button_reboot,
		.Gpio[30].flags			= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio),
		.Gpio[31].name				= BSP_Gpio_name_led_upgrade,
		.Gpio[31].flags			= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio),
		.Lan.Mii[0].dev			= BSP_Lan_mii_dev_switch,
		.Lan.Mii[0].smiAddr			= 0xff,
		.Lan.Mii[0].Info.sw.miiNum	= -1,
		.Lan.Mii[1].dev			= BSP_Lan_mii_dev_switch,
		.Lan.Mii[1].smiAddr			= 0xff,
		.Lan.Mii[1].Info.sw.miiNum	= -1,
		.Lan.MiiDev.Switch.nb		= 1,
		.Lan.MiiDev.Phy.nb			= 0,
		.Lan.all[0]				= BSP_Gpio_name_switch_rst,
		.Lan.all[1]				= BSP_Gpio_name_reserved,
		.Usb.Hub.nb				= 0,
		.Usb.Hub.all[0]			= BSP_Gpio_name_reserved,
		.Usb.Master.nb				= 1,
		.Usb.Master.all[0]			= BSP_Gpio_name_usb_pwr_en_master_1,
		.Usb.Master.all[1]			= BSP_Gpio_name_reserved,
		.Usb.Slave.nb				= 0,
		.Usb.Slave.all[0]			= BSP_Gpio_name_reserved,
		.Voice.Fxs.nb				= 2,
		.Voice.Fxs.all[0]			= BSP_Gpio_name_voice_rst_fxs_all_and_fxo,
		.Voice.Fxs.all[1]			= BSP_Gpio_name_reserved,
		.Voice.Fxo.nb				= 1,
		.Voice.Fxo.all[0]			= BSP_Gpio_name_reserved,
		.Relay.Fxs.nb				= 2,
		.Relay.Fxs.map				= ( BSP_MAP_RELAY_ON_FXS_1 | BSP_MAP_RELAY_ON_FXS_2 ),
		.Relay.Fxs.all[0]			= BSP_Gpio_name_relay_fxs_all,
		.Relay.Fxs.all[1]			= BSP_Gpio_name_reserved,
		.Relay.Fxo.map				= 0,
		.Relay.Fxo.all[0]			= BSP_Gpio_name_reserved,
	},
	/** Board All V2_1. */
	.Board.All[BSP_Board_info_lb_v2_1]						= {
		.Gpio[0].name				= BSP_Gpio_name_reserved,
		.Gpio[0].flags				= (BSP_Gpio_flags_dir_out),
		.Gpio[1].name				= BSP_Gpio_name_spi_cs_fxs_all,
		.Gpio[1].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_spi_cs),
		.Gpio[2].name				= BSP_Gpio_name_spi_cs_fxo,
		.Gpio[2].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_spi_cs),
		.Gpio[3].name				= BSP_Gpio_name_voice_rst_fxs_all_and_fxo,
		.Gpio[3].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio),
		.Gpio[4].name				= BSP_Gpio_name_config_1,
		.Gpio[4].flags				= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_used_as_config),
		.Gpio[5].name				= BSP_Gpio_name_config_2,
		.Gpio[5].flags				= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_used_as_config),
		.Gpio[6].name				= BSP_Gpio_name_config_3,
		.Gpio[6].flags				= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_used_as_config),
		.Gpio[7].name				= BSP_Gpio_name_led_wireless,
		.Gpio[7].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio),
		.Gpio[8].name				= BSP_Gpio_name_led_voip,
		.Gpio[8].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio),
		.Gpio[9].name				= BSP_Gpio_name_led_lan,
		.Gpio[9].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio),
		.Gpio[10].name				= BSP_Gpio_name_led_internet_red,
		.Gpio[10].flags			= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio),
		.Gpio[11].name				= BSP_Gpio_name_led_power,
		.Gpio[11].flags			= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio),
		.Gpio[12].name				= BSP_Gpio_name_usb_rst_hub,
		.Gpio[12].flags			= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio),
		.Gpio[13].name				= BSP_Gpio_name_cmd_nl,
		.Gpio[13].flags			= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio),
		.Gpio[14].name				= BSP_Gpio_name_reserved,
		.Gpio[14].flags			= (BSP_Gpio_flags_dir_out),
		.Gpio[15].name				= BSP_Gpio_name_usb_pwr_en_master_1,
		.Gpio[15].flags			= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio),
		.Gpio[16].name				= BSP_Gpio_name_adsl_rst_afe,
		.Gpio[16].flags			= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio),
		.Gpio[17].name				= BSP_Gpio_name_usb_overcurrent_master_all,
		.Gpio[17].flags			= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_active_low | BSP_Gpio_flags_intr | BSP_Gpio_flags_used_as_gpio),
		.Gpio[18].name				= BSP_Gpio_name_usb_pwr_en_master_2,
		.Gpio[18].flags			= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio),
		.Gpio[19].name				= BSP_Gpio_name_relay_fxs_all,
		.Gpio[19].flags			= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio),
		.Gpio[20].name				= BSP_Gpio_name_reserved,
		.Gpio[20].flags			= (BSP_Gpio_flags_dir_out),
		.Gpio[21].name				= BSP_Gpio_name_reserved,
		.Gpio[21].flags			= (BSP_Gpio_flags_dir_out),
		.Gpio[22].name				= BSP_Gpio_name_reserved,
		.Gpio[22].flags			= (BSP_Gpio_flags_dir_out),
		.Gpio[23].name				= BSP_Gpio_name_reserved,
		.Gpio[23].flags			= (BSP_Gpio_flags_dir_out),
		.Gpio[24].name				= BSP_Gpio_name_button_reset,
		.Gpio[24].flags			= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio),
		.Gpio[25].name				= BSP_Gpio_name_dying_gasp,
		.Gpio[25].flags			= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_active_low | BSP_Gpio_flags_intr | BSP_Gpio_flags_used_as_gpio),
		.Gpio[26].name				= BSP_Gpio_name_button_pairing,
		.Gpio[26].flags			= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio),
		.Gpio[27].name				= BSP_Gpio_name_led_internet_green,
		.Gpio[27].flags			= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio),
		.Gpio[28].name				= BSP_Gpio_name_switch_rst,
		.Gpio[28].flags			= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio),
		.Gpio[29].name				= BSP_Gpio_name_reserved,
		.Gpio[29].flags			= (BSP_Gpio_flags_dir_out),
		.Gpio[30].name				= BSP_Gpio_name_reserved,
		.Gpio[30].flags			= (BSP_Gpio_flags_dir_out),
		.Gpio[31].name				= BSP_Gpio_name_led_upgrade,
		.Gpio[31].flags			= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio),
		.Lan.Mii[0].dev			= BSP_Lan_mii_dev_switch,
		.Lan.Mii[0].Info.sw.miiNum	= -1,
		.Lan.Mii[0].smiAddr			= 0xff,
		.Lan.Mii[1].dev			= BSP_Lan_mii_dev_switch,
		.Lan.Mii[1].smiAddr			= 0xff,
		.Lan.Mii[1].Info.sw.miiNum	= -1,
		.Lan.MiiDev.Switch.nb		= 1,
		.Lan.MiiDev.Phy.nb			= 0,
		.Lan.all[0]				= BSP_Gpio_name_switch_rst,
		.Lan.all[1]				= BSP_Gpio_name_reserved,
		.Usb.Hub.nb				= 1,
		.Usb.Hub.all[0]			= BSP_Gpio_name_usb_rst_hub,
		.Usb.Hub.all[1]			= BSP_Gpio_name_reserved,
		.Usb.Master.nb				= 2,
		.Usb.Master.all[0]			= BSP_Gpio_name_usb_pwr_en_master_1,
		.Usb.Master.all[1]			= BSP_Gpio_name_usb_pwr_en_master_2,
		.Usb.Master.all[2]			= BSP_Gpio_name_reserved,
		.Usb.Slave.nb				= 0,
		.Usb.Slave.all[0]			= BSP_Gpio_name_reserved,
		.Voice.Fxs.nb				= 2,
		.Voice.Fxs.all[0]			= BSP_Gpio_name_voice_rst_fxs_all_and_fxo,
		.Voice.Fxs.all[1]			= BSP_Gpio_name_reserved,
		.Voice.Fxo.nb				= 1,
		.Voice.Fxo.all[0]			= BSP_Gpio_name_reserved,
		.Relay.Fxs.nb				= 2,
		.Relay.Fxs.map				= ( BSP_MAP_RELAY_ON_FXS_1 | BSP_MAP_RELAY_ON_FXS_2 ),
		.Relay.Fxs.all[0]			= BSP_Gpio_name_relay_fxs_all,
		.Relay.Fxs.all[1]			= BSP_Gpio_name_reserved,
		.Relay.Fxo.nb				= 0,
		.Relay.Fxo.map				= 0,
		.Relay.Fxo.all[0]			= BSP_Gpio_name_reserved,
	},
	/** Board All V2_2. */
	.Board.All[BSP_Board_info_lb_v2_2]						= {
		.Gpio[0].name				= BSP_Gpio_name_switch_int,
		.Gpio[0].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_low | BSP_Gpio_flags_intr | BSP_Gpio_flags_used_as_gpio),
		.Gpio[1].name				= BSP_Gpio_name_spi_cs_fxs_all_and_fxo,
		.Gpio[1].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_spi_cs),
		.Gpio[2].name				= BSP_Gpio_name_reserved,
		.Gpio[2].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_used_as_spi_cs),
		.Gpio[3].name				= BSP_Gpio_name_voice_rst_fxs_1,
		.Gpio[3].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio),
		.Gpio[4].name				= BSP_Gpio_name_config_1,
		.Gpio[4].flags				= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_used_as_config),
		.Gpio[5].name				= BSP_Gpio_name_config_2,
		.Gpio[5].flags				= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_used_as_config),
		.Gpio[6].name				= BSP_Gpio_name_config_3,
		.Gpio[6].flags				= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_used_as_config),
		.Gpio[7].name				= BSP_Gpio_name_led_wireless,
		.Gpio[7].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio),
		.Gpio[8].name				= BSP_Gpio_name_led_voip,
		.Gpio[8].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio),
		.Gpio[9].name				= BSP_Gpio_name_led_lan,
		.Gpio[9].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio),
		.Gpio[10].name				= BSP_Gpio_name_led_internet_red,
		.Gpio[10].flags			= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio),
		.Gpio[11].name				= BSP_Gpio_name_led_power,
		.Gpio[11].flags			= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio),
		.Gpio[12].name				= BSP_Gpio_name_usb_rst_hub,
		.Gpio[12].flags			= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio),
		.Gpio[13].name				= BSP_Gpio_name_cmd_nl,
		.Gpio[13].flags			= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio),
		.Gpio[14].name				= BSP_Gpio_name_voice_rst_fxs_2_and_fxo,
		.Gpio[14].flags			= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio),
		.Gpio[15].name				= BSP_Gpio_name_usb_pwr_en_master_1,
		.Gpio[15].flags			= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio),
		.Gpio[16].name				= BSP_Gpio_name_adsl_rst_afe,
		.Gpio[16].flags			= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio),
		.Gpio[17].name				= BSP_Gpio_name_voice_int_fxs_1,
		.Gpio[17].flags			= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_active_low | BSP_Gpio_flags_intr | BSP_Gpio_flags_used_as_gpio),
		.Gpio[18].name				= BSP_Gpio_name_usb_pwr_en_master_2,
		.Gpio[18].flags			= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio),
		.Gpio[19].name				= BSP_Gpio_name_relay_fxs_all,
		.Gpio[19].flags			= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio),
		.Gpio[20].name				= BSP_Gpio_name_reserved,
		.Gpio[20].flags			= (BSP_Gpio_flags_dir_out),
		.Gpio[21].name				= BSP_Gpio_name_reserved,
		.Gpio[21].flags			= (BSP_Gpio_flags_dir_out),
		.Gpio[22].name				= BSP_Gpio_name_reserved,
		.Gpio[22].flags			= (BSP_Gpio_flags_dir_out),
		.Gpio[23].name				= BSP_Gpio_name_reserved,
		.Gpio[23].flags			= (BSP_Gpio_flags_dir_out),
		.Gpio[24].name				= BSP_Gpio_name_button_reset,
		.Gpio[24].flags			= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio),
		.Gpio[25].name				= BSP_Gpio_name_dying_gasp,
		.Gpio[25].flags			= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_active_low | BSP_Gpio_flags_intr | BSP_Gpio_flags_used_as_gpio),
		.Gpio[26].name				= BSP_Gpio_name_button_pairing,
		.Gpio[26].flags			= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio),
		.Gpio[27].name				= BSP_Gpio_name_led_internet_green,
		.Gpio[27].flags			= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio),
		.Gpio[28].name				= BSP_Gpio_name_switch_rst,
		.Gpio[28].flags			= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio),
		.Gpio[29].name				= BSP_Gpio_name_voice_int_fxs_2_and_fxo,
		.Gpio[29].flags			= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_active_low | BSP_Gpio_flags_intr | BSP_Gpio_flags_used_as_gpio),
		.Gpio[30].name				= BSP_Gpio_name_wifi_rst,
		.Gpio[30].flags			= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio),
		.Gpio[31].name				= BSP_Gpio_name_led_upgrade,
		.Gpio[31].flags			= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio),
		.Lan.Mii[0].dev			= BSP_Lan_mii_dev_switch,
		.Lan.Mii[0].Info.sw.miiNum	= -1,
		.Lan.Mii[0].smiAddr			= 0xff,
		.Lan.Mii[1].dev			= BSP_Lan_mii_dev_switch,
		.Lan.Mii[1].smiAddr			= 0xff,
		.Lan.Mii[1].Info.sw.miiNum	= -1,
		.Lan.MiiDev.Switch.nb		= 1,
		.Lan.MiiDev.Phy.nb			= 2,
		.Lan.all[0]				= BSP_Gpio_name_switch_rst,
		.Lan.all[1]				= BSP_Gpio_name_reserved,
		.Usb.Hub.nb				= 1,
		.Usb.Hub.all[0]			= BSP_Gpio_name_usb_rst_hub,
		.Usb.Hub.all[1]			= BSP_Gpio_name_reserved,
		.Usb.Master.nb				= 2,
		.Usb.Master.all[0]			= BSP_Gpio_name_usb_pwr_en_master_1,
		.Usb.Master.all[1]			= BSP_Gpio_name_usb_pwr_en_master_2,
		.Usb.Master.all[2]			= BSP_Gpio_name_reserved,
		.Usb.Slave.nb				= 0,
		.Usb.Slave.all[0]			= BSP_Gpio_name_reserved,
		.Voice.Fxs.nb				= 2,
		.Voice.Fxs.all[0]			= BSP_Gpio_name_voice_rst_fxs_1,
		.Voice.Fxs.all[1]			= BSP_Gpio_name_voice_rst_fxs_2_and_fxo,
		.Voice.Fxs.all[2]			= BSP_Gpio_name_reserved,
		.Voice.Fxo.nb				= 1,
		.Voice.Fxo.all[0]			= BSP_Gpio_name_reserved,
		.Relay.Fxs.nb				= 2,
		.Relay.Fxs.map				= ( BSP_MAP_RELAY_ON_FXS_1 | BSP_MAP_RELAY_ON_FXS_2 ),
		.Relay.Fxs.all[0]			= BSP_Gpio_name_relay_fxs_all,
		.Relay.Fxs.all[1]			= BSP_Gpio_name_reserved,
		.Relay.Fxo.map				= 0,
		.Relay.Fxo.nb				= 0,
		.Relay.Fxo.all[0]			= BSP_Gpio_name_reserved,
	},
	/** Board All Lb_mini. */
	.Board.All[BSP_Board_info_lb_mini]						= {
		.Gpio[0].name				= BSP_Gpio_name_voice_rst_fxs_all_and_fxo,
		.Gpio[0].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio),
		.Gpio[1].name				= BSP_Gpio_name_spi_cs_fxs_all,
		.Gpio[1].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_spi_cs),
		.Gpio[2].name				= BSP_Gpio_name_spi_cs_fxo,
		.Gpio[2].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_spi_cs),
		.Gpio[3].name				= BSP_Gpio_name_mini_led_act_eth_0,
		.Gpio[3].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio),
		.Gpio[4].name				= BSP_Gpio_name_mini_led_act_eth_1,
		.Gpio[4].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio),
		.Gpio[5].name				= BSP_Gpio_name_config_1,
		.Gpio[5].flags				= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_used_as_config),
		.Gpio[6].name				= BSP_Gpio_name_config_2,
		.Gpio[6].flags				= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_used_as_config),
		.Gpio[7].name				= BSP_Gpio_name_led_wireless,
		.Gpio[7].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio),
		.Gpio[8].name				= BSP_Gpio_name_led_voip,
		.Gpio[8].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio),
		.Gpio[9].name				= BSP_Gpio_name_led_lan,
		.Gpio[9].flags				= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio),
		.Gpio[10].name				= BSP_Gpio_name_led_internet_red,
		.Gpio[10].flags			= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio),
		.Gpio[11].name				= BSP_Gpio_name_led_power,
		.Gpio[11].flags			= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio),
		.Gpio[12].name				= BSP_Gpio_name_reserved,
		.Gpio[12].flags			= (BSP_Gpio_flags_dir_out),
		.Gpio[13].name				= BSP_Gpio_name_cmd_nl,
		.Gpio[13].flags			= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio),
		.Gpio[14].name				= BSP_Gpio_name_reserved,
		.Gpio[14].flags			= (BSP_Gpio_flags_dir_out),
		.Gpio[15].name				= BSP_Gpio_name_usb_pwr_en_master_1,
		.Gpio[15].flags			= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio),
		.Gpio[16].name				= BSP_Gpio_name_adsl_rst_afe,
		.Gpio[16].flags			= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio),
		.Gpio[17].name				= BSP_Gpio_name_usb_overcurrent_master_1,
		.Gpio[17].flags			= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio),
		.Gpio[18].name				= BSP_Gpio_name_button_reboot,
		.Gpio[18].flags			= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio),
		.Gpio[19].name				= BSP_Gpio_name_relay_fxs_all,
		.Gpio[19].flags			= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio),
		.Gpio[20].name				= BSP_Gpio_name_reserved,
		.Gpio[20].flags			= (BSP_Gpio_flags_dir_out),
		.Gpio[21].name				= BSP_Gpio_name_reserved,
		.Gpio[21].flags			= (BSP_Gpio_flags_dir_out),
		.Gpio[22].name				= BSP_Gpio_name_reserved,
		.Gpio[22].flags			= (BSP_Gpio_flags_dir_out),
		.Gpio[23].name				= BSP_Gpio_name_reserved,
		.Gpio[23].flags			= (BSP_Gpio_flags_dir_out),
		.Gpio[24].name				= BSP_Gpio_name_button_reset,
		.Gpio[24].flags			= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio),
		.Gpio[25].name				= BSP_Gpio_name_led_upgrade,
		.Gpio[25].flags			= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio),
		.Gpio[26].name				= BSP_Gpio_name_button_pairing,
		.Gpio[26].flags			= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio),
		.Gpio[27].name				= BSP_Gpio_name_led_internet_green,
		.Gpio[27].flags			= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio),
		.Gpio[28].name				= BSP_Gpio_name_phy_rst_1,
		.Gpio[28].flags			= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio),
		.Gpio[29].name				= BSP_Gpio_name_phy_rst_2,
		.Gpio[29].flags			= (BSP_Gpio_flags_dir_out | BSP_Gpio_flags_active_low | BSP_Gpio_flags_used_as_gpio),
		.Gpio[30].name				= BSP_Gpio_name_mini_pin_crs_eth_0,
		.Gpio[30].flags			= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio),
		.Gpio[31].name				= BSP_Gpio_name_mini_pin_crs_eth_1,
		.Gpio[31].flags			= (BSP_Gpio_flags_dir_in | BSP_Gpio_flags_active_high | BSP_Gpio_flags_used_as_gpio),
		.Lan.Mii[0].dev			= BSP_Lan_mii_dev_phy,
		.Lan.Mii[0].smiAddr			= 0x0f,
		.Lan.Mii[0].Info.phy.flags	= BSP_LAN_PHY_HANDLE_LEDS,
		.Lan.Mii[1].dev			= BSP_Lan_mii_dev_phy,
		.Lan.Mii[1].smiAddr			= 0x1f,
		.Lan.Mii[1].Info.phy.flags	= BSP_LAN_PHY_HANDLE_LEDS,
		.Lan.MiiDev.Switch.nb		= 0,
		.Lan.MiiDev.Phy.nb			= 2,
		.Lan.all[0]				= BSP_Gpio_name_phy_rst_1,
		.Lan.all[1]				= BSP_Gpio_name_phy_rst_2,
		.Lan.all[2]				= BSP_Gpio_name_reserved,
		.Usb.Hub.nb				= 0,
		.Usb.Hub.all[0]			= BSP_Gpio_name_usb_rst_hub,
		.Usb.Hub.all[1]			= BSP_Gpio_name_reserved,
		.Usb.Master.nb				= 1,
		.Usb.Master.all[0]			= BSP_Gpio_name_usb_pwr_en_master_1,
		.Usb.Master.all[1]			= BSP_Gpio_name_reserved,
		.Usb.Slave.nb				= 0,
		.Usb.Slave.all[0]			= BSP_Gpio_name_reserved,
		.Voice.Fxs.nb				= 1,
		.Voice.Fxs.all[0]			= BSP_Gpio_name_voice_rst_fxs_all_and_fxo,
		.Voice.Fxs.all[1]			= BSP_Gpio_name_reserved,
		.Voice.Fxo.nb				= 1,
		.Voice.Fxo.all[0]			= BSP_Gpio_name_reserved,
		.Relay.Fxs.nb				= 1,
		.Relay.Fxs.map				= BSP_MAP_RELAY_ON_FXS_1,
		.Relay.Fxs.all[0]			= BSP_Gpio_name_relay_fxs_all,
		.Relay.Fxs.all[1]			= BSP_Gpio_name_reserved,
		.Relay.Fxo.nb				= 0,
		.Relay.Fxo.map				= 0,
		.Relay.Fxo.all[0]			= BSP_Gpio_name_reserved,
	},
};
EXPORT_SYMBOL(bsp_base);
// vim:ts=5:
