/* ============================================================================
 *
 * Copyright : (c) 2005 SAGEM SA - Site Velizy - URD2
 *
 * The information and source code contained herein is the exclusive
 * property of SAGEM and may not be disclosed, examined, or
 * reproduced in whole or in part without explicit written authorization
 * of the company.
 *
 * ============================================================================
 */

/**
 * \file bsp_led.c Bsp Led functions
 *
 * This file contain all Bsp Led functions.
 */

/*
 * Includes
 * ~~~~~~~~
 */

#include "bsp_pri.h"

/*
 * Functions
 * ~~~~~~~~~
 */

/** Set Led state. */
int kSysLedCtrl(	BOARD_LED_NAME 	_nameUser,
				BOARD_LED_STATE	_stateUser)
{
	/* Check Led name*/
	if ( (_nameUser < 0) || (_nameUser >= kLedEnd) ){
		printk(	"%s: WARNING -> Led name >= Max(%u)\n",
				__FUNCTION__,
				kLedEnd);

		return(-1);
	}

	/* Check  Led state */
	if ( (_stateUser < 0) || (_stateUser >= kLedStateOn + 1) ){
		printk(	"%s: WARNING -> Led state >= Max(%u)\n",
				__FUNCTION__,
				kLedStateOn + 1);

		return(-1);
	}

	{
		bsp_gpio_name_e	_nameGpio;
		bsp_gpio_state_e	_stateGpio;

		/* Retrieve nameGpio */
		_nameGpio		= bsp_base.Index.Led.Name.user2gpio[_nameUser];

		/* Retrieve stateGpio */
		_stateGpio	= bsp_base.Index.Led.State.user2gpio[_stateUser];

		/* Set state */
		bsp_gpio_set_state(	_nameGpio,
						_stateGpio);

#ifdef BSP_DEBUG
		/* If bsp */
		if (bsp_base.debug & BSP_FLAGS_DEBUG_LED) {
			printk(	"%s: Led -> nameUser(%u) nameGpio(%u) stateUser(%u) stateGpio(%u)\n",
					__FUNCTION__,
					_nameUser,
					_nameGpio,
					_stateUser,
					_stateGpio);
		}
#endif
	}

	/* Ok */
	return(0);
}
EXPORT_SYMBOL(kSysLedCtrl);

/** Kernel read led state function */
BOARD_RELAY_STATE kSysGetLedState(BOARD_LED_NAME _nameUser)
{
	BOARD_LED_STATE	_stateUser;

	/* Check name. */
	if ( (_nameUser < 0) || (_nameUser >= kLedEnd) ) {
		printk(	"%s: WARNING -> Led >= Max(%u)\n",
				__FUNCTION__,
				kLedEnd);

		return(-1);
	}

	/* Retrieve state */
	{
		bsp_gpio_state_e	_stateGpio;
		bsp_gpio_name_e	_nameGpio;

		/* Retrieve nameGpio */
		_nameGpio = bsp_base.Index.Led.Name.user2gpio[_nameUser];

		/* Retrieve stateGpio */
		if ((_stateGpio = bsp_gpio_get_state(_nameGpio)) == BSP_Gpio_state_failure) {
			/* Error */
			return(-1);
		}

		/* Retrieve userState */
		_stateUser = bsp_base.Index.Led.State.gpio2user[_stateGpio];

#ifdef BSP_DEBUG
		/* If bsp */
		if (bsp_base.debug & BSP_FLAGS_DEBUG_RELAY) {
			printk(	"%s: Led -> nameUser(%u) nameGpio(%u) stateUser(%u) stateGpio(%u)\n",
					__FUNCTION__,
					_nameUser,
					_nameGpio,
					_stateUser,
					_stateGpio);
		}
#endif
	}

	return(_stateUser);
}
EXPORT_SYMBOL(kSysGetLedState);

/** sysctl bsp_led_maps_proc.
 *
 *   \par Description
 *	<b>This function allow to set connexion tampon.</b>\n
 *
 *	@param _pTable:	Sysctl table reference.
 *	@param _write:		If write.
 *	@param _pFile:		File reference.
 *	@param _pBuffer:	Buffer reference.
 *	@param _pLen:		Length reference.
 *	@param _pPpos:		Position reference.
 *
 *	@return proc_dostring return.
 */
int bsp_led_maps_proc(	ctl_table		*_pTable,
					int			_write,
					struct file	*_pFile,
					void __user	*_pBuffer,
					size_t		*_pLen,
					loff_t		*_pPpos)
{
	char				*_pTampon		= (char *)_pTable->data;
	BOARD_LED_STATE	_userState;

	sprintf(	_pTampon,
			"\n");

	/* Next */
	_pTampon += strlen(_pTampon);

	/* Retrieve tampon */
	if ((_pTampon = (char *)_pTable->data) != (char *)0) {
		sprintf(	_pTampon,
				"| Power | @ red | @ green | Voip | lan | Wireless | Upgrade |\n");

		/* Next */
		_pTampon += strlen(_pTampon);

		/* Adsl part */
		{
			/* Retrieve led state */
			_userState = kSysGetLedState(kLedPower);
	
			switch (_userState) {
				case kLedStateOn:
				{
					/* Print */
					sprintf(	_pTampon,
							"|   1   |");
				}
				break;
	
				case kLedStateOff:
				{
					/* Print */
					sprintf(	_pTampon,
							"|   0   |");
				}
				break;
	
				default:
				{
					/* Print */
					sprintf(	_pTampon,
							"|   -   |");
				}
				break;
			}
		}

		/* Next */
		_pTampon += strlen(_pTampon);

		/* Internet red part */
		{
			/* Retrieve led state */
			_userState = kSysGetLedState(kLedInternetRed);
	
			switch (_userState) {
				case kLedStateOn:
				{
					/* Print */
					sprintf(	_pTampon,
							"   1   |");
				}
				break;
	
				case kLedStateOff:
				{
					/* Print */
					sprintf(	_pTampon,
							"   0   |");
				}
				break;
	
				default:
				{
					/* Print */
					sprintf(	_pTampon,
							"   -   |");
				}
				break;
			}
		}

		/* Next */
		_pTampon += strlen(_pTampon);

		/* Internet green part */
		{
			/* Retrieve led state */
			_userState = kSysGetLedState(kLedInternetGreen);
	
			switch (_userState) {
				case kLedStateOn:
				{
					/* Print */
					sprintf(	_pTampon,
							"    1    |");
				}
				break;
	
				{
				case kLedStateOff:
					/* Print */
					sprintf(	_pTampon,
							"    0    |");
				}
				break;
	
				default:
				{
					/* Print */
					sprintf(	_pTampon,
							"    -    |");
				}
				break;
			}
		}

		/* Next */
		_pTampon += strlen(_pTampon);

		/* Wireless part */
		{
			/* Retrieve led state */
			_userState = kSysGetLedState(kLedVoip);
	
			switch (_userState) {
				case kLedStateOn:
				{
					/* Print */
					sprintf(	_pTampon,
							"  1   |");
				}
				break;
	
				case kLedStateOff:
				{
					/* Print */
					sprintf(	_pTampon,
							"  0   |");
				}
				break;
	
				default:
				{
					/* Print */
					sprintf(	_pTampon,
							"  -   |");
				}
				break;
			}
		}

		/* Next */
		_pTampon += strlen(_pTampon);

		/* Power part */
		{
			/* Retrieve led state */
			_userState = kSysGetLedState(kLedLan);
	
			switch (_userState) {
				case kLedStateOn:
				{
					/* Print */
					sprintf(	_pTampon,
							"  1  |");
				}
				break;
	
				case kLedStateOff:
				{
					/* Print */
					sprintf(	_pTampon,
							"  0  |");
				}
				break;
	
				default:
				{
					/* Print */
					sprintf(	_pTampon,
							"  -  |");
				}
				break;
			}
		}

		/* Next */
		_pTampon += strlen(_pTampon);

		/* Lan part */
		{
			/* Retrieve led state */
			_userState = kSysGetLedState(kLedWireless);
	
			switch (_userState) {
				case kLedStateOn:
				{
					/* Print */
					sprintf(	_pTampon,
							"    1     |");
				}
				break;
	
				case kLedStateOff:
				{
					/* Print */
					sprintf(	_pTampon,
							"    0     |");
				}
				break;
	
				default:
				{
					/* Print */
					sprintf(	_pTampon,
							"    -     |");
				}
				break;
			}
		}

		/* Next */
		_pTampon += strlen(_pTampon);

		/* Upgrade part */
		{
			/* Retrieve led state */
			_userState = kSysGetLedState(kLedUpgrade);
	
			switch (_userState) {
				case kLedStateOn:
				{
					/* Print */
					sprintf(	_pTampon,
							"    1    |");
				}
				break;
	
				case kLedStateOff:
				{
					/* Print */
					sprintf(	_pTampon,
							"    0    |");
				}
				break;
	
				default:
				{
					/* Print */
					sprintf(	_pTampon,
							"    -    |");
				}
				break;
			}
		}

		/* Next */
		_pTampon += strlen(_pTampon);
	}
	else {
		/* Print */
		sprintf(	_pTampon,
				"Process Error\n");

		/* Next */
		_pTampon += strlen(_pTampon);
	}

	sprintf(	_pTampon,
			"\n");

	/* Next */
	_pTampon += strlen(_pTampon);

	/* Call proc_dostring */
	return(proc_dostring(	_pTable,
						_write,
						_pFile,
						_pBuffer,
						_pLen,
						_pPpos));
}

// vim:ts=5:
