/*
 * ============================================================================
 *
 * Copyright : (c) 2005 SAGEM SA - Site Velizy - URD2
 *
 * The information and source code contained herein is the exclusive
 * property of SAGEM and may not be disclosed, examined, or
 * reproduced in whole or in part without explicit written authorization
 * of the company.
 *
 * ============================================================================
 */

/**
 * \file bsp_notify.c Bsp Notify functions
 *
 * This file contain all Bsp Notify functions.
 */

/*
 * Includes
 * ~~~~~~~~
 */

#include "bsp_pri.h"

/*
 * Functions
 * ~~~~~~~~~~
 */

/** Notify Evt. */
int bsp_notify_evt(	bsp_notify_evt_e	_evt)
{
	switch(_evt) {
		case BSP_Notify_evt_adsl_link_down:
		{
			/* Unset Flag */
			bsp_base.Env.Relay.flags &= ~BSP_Relay_flags_adsl_link_up;

			/* Update */
			bsp_update_all_fxs_relay();
		}
		break;

		case BSP_Notify_evt_adsl_link_up:
		{
			/* Set Flag */
			bsp_base.Env.Relay.flags |= BSP_Relay_flags_adsl_link_up;

			/* Update */
			bsp_update_all_fxs_relay();
		}
		break;

		case BSP_Notify_evt_voice_module_down:
		{
			/* Unset Flag */
			bsp_base.Env.Relay.flags &= ~BSP_Relay_flags_voice_module_up;

			/* Update */
			bsp_update_all_fxs_relay();
		}
		break;

		case BSP_Notify_evt_voice_module_up:
		{
			/* Set Flag */
			bsp_base.Env.Relay.flags |= BSP_Relay_flags_voice_module_up;

			/* Update */
			bsp_update_all_fxs_relay();
		}
		break;

		case BSP_Notify_evt_relay_mode_adsl:
		{
			/* Unset */
			bsp_base.Env.Relay.flags &= ~BSP_Relay_flags_mode_ftth;

			/* Set Flag */
			bsp_base.Env.Relay.flags |= BSP_Relay_flags_mode_adsl;

			/* Update */
			bsp_update_all_fxs_relay();
		}
		break;

		case BSP_Notify_evt_relay_mode_ftth:
		{
			/* Unset */
			bsp_base.Env.Relay.flags &= ~BSP_Relay_flags_mode_adsl;

			/* Set Flag */
			bsp_base.Env.Relay.flags |= BSP_Relay_flags_mode_ftth;

			/* Update */
			bsp_update_all_fxs_relay();
		}
		break;

		case BSP_Notify_evt_relay_mode_none:
		{
			/* Unset */
			bsp_base.Env.Relay.flags &= ~BSP_Relay_flags_mode_adsl;
			bsp_base.Env.Relay.flags &= ~BSP_Relay_flags_mode_ftth;

			/* Update */
			bsp_update_all_fxs_relay();
		}
		break;

		case BSP_Notify_evt_need_reset_gpio_slic:
		{
			/* Reset Gpio voice */
			bsp_reset_gpio_voice();
		}
		break;

		case BSP_Notify_evt_need_reset_gpio_usb:
		{
			/* Reset Gpio usb */
			bsp_reset_gpio_usb();
		}
		break;

		case BSP_Notify_evt_need_reset_gpio_lan:
		{
			/* Reset Gpio lan */
			bsp_reset_gpio_lan();
		}
		break;

		default:
		{
			printk(	"%s: FAILURE -> Evt(%u) unknown\n",
					__FUNCTION__,
					_evt);

			/* Error */
			return(-1);
		}
		break;
	}

	/* Ok */
	return(0);
}
EXPORT_SYMBOL(bsp_notify_evt);

// vim:ts=5:
