/*
 * ============================================================================
 *
 * Copyright : (c) 2005 SAGEM SA - Site Velizy - URD2
 *
 * The information and source code contained herein is the exclusive
 * property of SAGEM and may not be disclosed, examined, or
 * reproduced in whole or in part without explicit written authorization
 * of the company.
 *
 * ============================================================================
 */

#ifndef __BSP_PRI_H__
#define __BSP_PRI_H__

/**
 * \file bsp_pri.h Bsp Private description.
 *
 * This file allow to describe Bsp Private.
*/

#ifdef __KERNEL__

/*
 * Defines
 * ~~~~~~~
 */

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/version.h>
#include <linux/device.h>
#include <linux/fs.h>
//#include <linux/devfs_fs_kernel.h>
#include <linux/cdev.h>
#include <linux/sysctl.h>
//#include <asm/ddb5xxx/ddb5477.h>
#include <linux/swap.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/ioctl.h>
#include <linux/proc_fs.h>
#include <linux/delay.h>

//#include <asm/mach-adi_fusiv/adi6843.h>
#include <asm/mach-fusiv/adi6843.h>
#include <linux/interrupt.h>

#include <linux/bsp/bsp_krnopt.h>
#include <linux/bsp/bsp_sysCtl.h>
#include <linux/bsp/bsp_dev.h>
#include <linux/bsp/bsp_permParam.h>
#include <linux/bsp/bsp_firmware.h>
#include <linux/bsp/bsp_flash.h>
#include <linux/bsp/bsp_gpio.h>
#include <linux/bsp/bsp_board.h>
#include <linux/bsp/bsp_lan.h>
#include <linux/bsp/bsp_but.h>
#include <linux/bsp/bsp_led.h>
#include <linux/bsp/bsp_usb.h>
#include <linux/bsp/bsp_voice.h>
#include <linux/bsp/bsp_relay.h>
#include <linux/bsp/bsp_boot.h>
#include <linux/bsp/bsp_notify.h>

/*
 * Defines
 * ~~~~~~~
 */

/** Tampon max */
#define BSP_SYSCTL_TAMPON_MAX_SIZE				4096

/** \name Base Flags.
 */
/*@{*/
/** Pp Version 0.x */
#define BSP_FLAGS_BASE_PP_VERSION_0			(1 << 0)
/** Pp Version 1.x */
#define BSP_FLAGS_BASE_PP_VERSION_1			(1 << 1)
/*@}*/

/** \name Gpio Flags.
 */
/*@{*/
/** Memory segment Kseg0 -> Kseg1. */
#define BSP_K0_TO_K1(x)   					((unsigned)(x)|0xA0000000)

/** Gpio Dir1. */
#define BSP_GPIO_DIR1						(BSP_K0_TO_K1(0x19040000))
/** Gpio Flag Set1. */
#define BSP_GPIO_FLAG_SET1					(BSP_K0_TO_K1(0x19040006))
/** Gpio Flag Clear1. */
#define BSP_GPIO_FLAG_CLEAR1					(BSP_K0_TO_K1(0x19040004))
/** Gpio Int Set1. */
#define BSP_GPIO_INT_SET1					(BSP_K0_TO_K1(0x1904000A))
/** Gpio Int Clear1. */
#define BSP_GPIO_INT_CLEAR1					(BSP_K0_TO_K1(0x19040008))
/** Gpio Pol Set1. */
#define BSP_GPIO_POL_SET1					(BSP_K0_TO_K1(0x19040012))
/** Gpio Pol Clear1. */
#define BSP_GPIO_POL_CLEAR1					(BSP_K0_TO_K1(0x19040010))
/** Gpio Edge Set1. */
#define BSP_GPIO_EDGE_SET1					(BSP_K0_TO_K1(0x1904001A))
/** Gpio Edge Clear1. */
#define BSP_GPIO_EDGE_CLEAR1					(BSP_K0_TO_K1(0x19040018))

/** Gpio Dir2. */
#define BSP_GPIO_DIR2						(BSP_K0_TO_K1(0x190b0000))
/** Gpio Set2. */
#define BSP_GPIO_FLAG_SET2					(BSP_K0_TO_K1(0x190b0006))
/** Gpio Clear2. */
#define BSP_GPIO_FLAG_CLEAR2					(BSP_K0_TO_K1(0x190b0004))
/** Gpio Int Set2. */
#define BSP_GPIO_INT_SET2					(BSP_K0_TO_K1(0x190b000A))
/** Gpio Int Clear2. */
#define BSP_GPIO_INT_CLEAR2					(BSP_K0_TO_K1(0x190b0008))
/** Gpio Pol Set2. */
#define BSP_GPIO_POL_SET2					(BSP_K0_TO_K1(0x190b0012))
/** Gpio Pol Clear2. */
#define BSP_GPIO_POL_CLEAR2					(BSP_K0_TO_K1(0x190b0010))
/** Gpio Edge Set2. */
#define BSP_GPIO_EDGE_SET2					(BSP_K0_TO_K1(0x190b001A))
/** Gpio Edge Clear2. */
#define BSP_GPIO_EDGE_CLEAR2					(BSP_K0_TO_K1(0x190b0018))

/** Gpio Max. */
#define BSP_GPIO_MAX 						32
/*@}*/

/** \name Recovery Force.
 */
/*@{*/
/** Addr */
#define BSP_BOOT_RECOVERY_FORCE_ADDR			0x80001000
/** Normal. */
#define BSP_BOOT_RECOVERY_FORCE_NORMAL_VALUE		"__FoRCe ReCoVeRy NoRMaL__"
/** Rescue */
#define BSP_BOOT_RECOVERY_FORCE_RESCUE_VALUE		"__FoRCe ReCoVeRy ReSCue__"
/** Network */
#define BSP_BOOT_RECOVERY_FORCE_NETWORK_VALUE	"__FoRCe ReCoVeRy NeTWoRK_"
/*@}*/

/*
 * Typedefs
 * ~~~~~~~~
 */

/** Cmd PermParm CallBack. */
typedef void *(*bsp_getPermParamCb_f)(			pBsp_permParam_s);

/** \name Enums.
 */
/*@{*/
/** \link bsp_gpio_intr_enu bsp_gpio_intr enum \endlink typedef. */
typedef enum bsp_gpio_intr_enu				bsp_gpio_intr_e;
/** \link bsp_gpio_flags_enu bsp_gpio_flags enum \endlink typedef. */
typedef enum bsp_gpio_flags_enu				bsp_gpio_flags_e;
/*@}*/

/** \name bsp_board_gpio struct Typedefs.
 */
/*@{*/
/** \link bsp_board_gpio_str bsp_board_gpio struct \endlink typedef. */
typedef struct bsp_board_gpio_str				bsp_board_gpio_s;
/** \link bsp_board_gpio_str bsp_board_gpio struct \endlink pointer typedef. */
typedef struct bsp_board_gpio_str				*pBsp_board_gpio_s;
/*@}*/

/** \name bsp_board_index_led struct Typedefs.
 */
/*@{*/
/** \link bsp_board_index_led_str bsp_board_index_led struct \endlink typedef. */
typedef struct bsp_board_index_led_str			bsp_board_index_led_s;
/** \link bsp_board_index_led_str bsp_board_index_led struct \endlink pointer typedef. */
typedef struct bsp_board_index_led_str			*pBsp_board_index_led_s;
/*@}*/

/** \name bsp_board_index_but struct Typedefs.
 */
/*@{*/
/** \link bsp_board_index_but_str bsp_board_index_but struct \endlink typedef. */
typedef struct bsp_board_index_but_str			bsp_board_index_but_s;
/** \link bsp_board_index_but_str bsp_board_index_but struct \endlink pointer typedef. */
typedef struct bsp_board_index_but_str			*pBsp_board_index_but_s;
/*@}*/

/** \name bsp_board_index_relay struct Typedefs.
 */
/*@{*/
/** \link bsp_board_index_relay_str bsp_board_index_relay struct \endlink typedef. */
typedef struct bsp_board_index_relay_str		bsp_board_index_relay_s;
/** \link bsp_board_index_relay_str bsp_board_index_relay struct \endlink pointer typedef. */
typedef struct bsp_board_index_relay_str		*pBsp_board_index_relay_s;
/*@}*/

/** \name bsp_pp2board struct Typedefs.
 */
/*@{*/
/** \link bsp_pp2board_str bsp_pp2board struct \endlink typedef. */
typedef struct bsp_pp2board_str				bsp_pp2board_s;
/** \link bsp_pp2board_str bsp_pp2board struct \endlink pointer typedef. */
typedef struct bsp_pp2board_str				*pBsp_pp2board_s;
/*@}*/

/** \name bsp_board_info_all struct Typedefs.
 */
/*@{*/
/** \link bsp_board_info_all_str bsp_board_info_all struct \endlink typedef. */
typedef struct bsp_board_info_all_str			bsp_board_info_all_s;
/** \link bsp_board_info_all_str bsp_board_info_all struct \endlink pointer typedef. */
typedef struct bsp_board_info_all_str			*pBsp_board_info_all_s;
/*@}*/

/** \name bsp_board_current struct Typedefs.
 */
/*@{*/
/** \link bsp_board_current_str bsp_board_current struct \endlink typedef. */
typedef struct bsp_board_current_str			bsp_board_current_s;
/** \link bsp_board_current_str bsp_board_current struct \endlink pointer typedef. */
typedef struct bsp_board_current_str			*pBsp_board_current_s;
/*@}*/

/** \name bsp_baseDev struct Typedefs.
 */
/*@{*/
/** \link bsp_dev_str bsp_dev struct \endlink typedef. */
typedef struct bsp_dev_str					bsp_dev_s;
/** \link bsp_dev_str bsp_dev struct \endlink pointer typedef. */
typedef struct bsp_dev_str					*pBsp_dev_s;
/*@}*/

/** \name bsp_base struct Typedefs.
 */
/*@{*/
/** \link bsp_base_str bsp_base struct \endlink typedef. */
typedef struct bsp_base_str					bsp_base_s;
/** \link bsp_base_str bsp_base struct \endlink pointer typedef. */
typedef struct bsp_base_str					*pBsp_base_s;
/*@}*/

/*
 * Enums
 * ~~~~~
 */

/** Bsp_gpio_intr. */
enum bsp_gpio_intr_enu {
	/** Gpio 0 to 15. */
	BSP_Gpio_intr_0_to_15,

	/** Gpio 15 to 32. */
	BSP_Gpio_intr_16_to_31,

	/** End. */
	BSP_Gpio_intr_end,
};

/** Bsp_gpio_flags. */
enum bsp_gpio_flags_enu {
	/** Output. */
	BSP_Gpio_flags_dir_out		= (1 << 0),

	/** Input. */
	BSP_Gpio_flags_dir_in		= (1 << 1),

	/** Active Low. */
	BSP_Gpio_flags_active_low	= (1 << 2),

	/** Active High. */
	BSP_Gpio_flags_active_high	= (1 << 3),

	/** Intr. */
	BSP_Gpio_flags_intr			= (1 << 4),

	/** Used as Gpio. */
	BSP_Gpio_flags_used_as_gpio	= (1 << 5),

	/** Used as Gpio. */
	BSP_Gpio_flags_used_as_config	= (1 << 6),

	/** Used as Gpio. */
	BSP_Gpio_flags_used_as_spi_cs	= (1 << 7),

	/** End. */
	BSP_Gpio_flags_end			= (1 << 8),
};

/*
 * Externs
 * ~~~~~~~
 */

/** Board flash start */
extern int board_flash_start;

/** Board_flash size. */
extern int board_flash_size;

/*
 * Structs
 * ~~~~~~~
 */

/** Bsp_board_gpio_str.
 *
 * 	This struct manage Gpio Information.
 */
struct bsp_board_gpio_str {
	/* Name. */
	bsp_gpio_name_e				name;

	/* Direction. */
	bsp_gpio_flags_e				flags;
};

/** Bsp_led_str.
 *
 * 	This struct manage Led Information.
 */
struct bsp_board_index_led_str {
	/** Struct manage Name. */
	struct {
		/** Indirect array. */
		bsp_gpio_name_e			user2gpio[kLedEnd];
	}Name;

	/** Struct manage state. */
	struct {
		/** Indirect array. */
		BOARD_LED_STATE			gpio2user[BSP_Gpio_state_set + 1];

		/** Indirect array. */
		bsp_gpio_state_e			user2gpio[kLedStateOn + 1];
	}State;
};

/** Bsp_board_index_but_str.
 *
 * 	This struct manage Board index Button Information.
 */
struct bsp_board_index_but_str {
	/** Struct manage Name. */
	struct {
		/** Indirect array. */
		bsp_gpio_name_e			user2gpio[kButtonEnd];
	}Name;

	/** Struct manage state. */
	struct {
		/** Indirect array. */
		BOARD_BUTTON_STATE			gpio2user[BSP_Gpio_state_set + 1];

		/** Indirect array. */
		bsp_gpio_state_e			user2gpio[kButtonPushed + 1];
	}State;
};

/** Bsp_noard_index_relay_str.
 *
 * 	This struct manage Board index Relay Information.
 */
struct bsp_board_index_relay_str {
	/** Struct manage Name. */
	struct {
		/** Indirect array. */
		bsp_gpio_name_e			user2gpio[kRelayEnd];
	}Name;

	/** Struct manage state. */
	struct {
		/** Indirect array. */
		BOARD_RELAY_STATE			gpio2user[BSP_Gpio_state_set + 1];
		
		/** Indirect array. */
		bsp_gpio_state_e			user2gpio[kRelayOff + 1];
	}State;
};

/** Bsp_board_info_all_str.
 *
 * 	This struct manage Board Info All Information.
 */
struct bsp_board_info_all_str {
	/** Gpio Info. */
	bsp_board_gpio_s				Gpio[BSP_GPIO_MAX];

	/* Lan Info. */
	bsp_board_lan_s				Lan;

	/* Usb Info. */
	bsp_board_usb_s				Usb;

	/* Voice Info. */
	bsp_board_voice_s				Voice;

	/* Relay Info. */
	bsp_board_relay_s				Relay;
};

/** Bsp_board_current_str.
 *
 * 	This struct manage Current Board.
 */
struct bsp_board_current_str {
	/** Name2Board (used to convert Pp Info to board). */
	bsp_pp2board_e					pp2board;

	/** Board Type (used for userland and specify board type no info about version or revision). */
	bsp_board_type_e				boardType;

	/** Board Info (used to access on board_info array info about version or revision). */
	bsp_board_info_e				boardInfo;

	/** Gpio name access. */
	pBsp_board_gpio_s				pGpioName[BSP_Gpio_name_max];

	/** Current. */
	pBsp_board_info_all_s			pInfo;

	/* Gpio lock. */
	spinlock_t					gpio_lock;
};

/* Struct manage device. */
struct bsp_dev_str {
	/** Device. */
	struct cdev					cdev;

	/** File operations. */
	struct file_operations			fops;

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,16)
	/* Class. */
	struct class					*pClass;
#else
	/* Class simple. */
	struct class_simple				*pClass;
#endif

	/** Semaphore. */
	struct semaphore				sem;

	/** Name. */
	char							*name;

	/** Nb Open. */
	__u8							nbOpen;
};

/* Bsp_pp2board_str.
 *
 *	match pP To Board.
 */
struct bsp_pp2board_str {
	/** Pp info. */
	char							*pInfo[BSP_Pp_info_max + 1];
	
	/** Board Info. */
	bsp_board_info_e				boardInfo;

	/** Board Type. */
	bsp_board_type_e				boardType;
};

/* Bsp_base_str.
 *
 *	This struct manage Base Bsp.
 */
struct bsp_base_str {
	/** Struct managed Device. */
	bsp_dev_s						Dev[BSP_Dev_minor_max];

	/** Struct Sysctl. */
	struct {
		/** Tampon. */
		char						tampon[BSP_SYSCTL_TAMPON_MAX_SIZE];

		/** Sysctl header. */
		struct ctl_table_header		*head;
	
		/** Sagem Sysctl. */
		struct ctl_table			sagem[5];

		/** Sagem List Sysctl. */
		struct ctl_table			sagem_list[8];

		/** Sagem List Boot Sysctl. */
		struct ctl_table			sagem_list_boot[2];

		/** Sagem List Relay Sysctl. */
		struct ctl_table			sagem_list_relay[3];

		/** Sagem List Led Sysctl. */
		struct ctl_table			sagem_list_led[2];

		/** Sagem List But Sysctl. */
		struct ctl_table			sagem_list_but[2];

		/** Sagem List Gpio Sysctl. */
		struct ctl_table			sagem_list_gpio[2];

		/** Sagem List Gpio Maps Sysctl. */
		struct ctl_table			sagem_list_gpio_maps[3];
	}SysCtl;

	struct {
		/** Info on all board. */
		bsp_board_info_all_s		All[BSP_Board_info_max];
	
		/** Struct current. */
		bsp_board_current_s			Cur;

		/** Struct Gpio Intr. */
		struct {
			/** Gpio 0 to 15. */
			bsp_gpio_intr_e		low;

			/** Gpio 16 to 31. */
			bsp_gpio_intr_e		high;
		}GpioIntr;

		/** Txt For debug. */
		char						*pGpioNameTxt[BSP_Gpio_name_max];
	}Board;

	/** Struct Index. */
	struct {
		/** But Info. */
		bsp_board_index_but_s		But;
	
		/** Led Info. */
		bsp_board_index_led_s		Led;
	
		/** Relay Info. */
		bsp_board_index_relay_s		Relay;
	}Index;

	/** Manage struct Pp. */
	struct {
		/** Match Pp name and Type board. */
		bsp_pp2board_s				pp2board[BSP_Pp2board_max];

		/* Permanent Parameter. */
		pBsp_permParam_s			pFlashAddr;
	
		/** Get PermParm Cb. */
		bsp_getPermParamCb_f		getPpCb[BSP_Get_permParam_max];
	}Pp;

	/** Struct Env. */
	struct {
		/** Struct bsp_krnopt. */
		bsp_env_krnopt_s			Krnopt;

		/** Boot. */
		bsp_env_boot_s				Boot;

		/** Relay. */
		bsp_env_relay_s			Relay;

		/** Firmware. */
		bsp_env_firmware_s			Firmware;
	}Env;

	/** Flags. */
	__u8							flags;

	/** Debug. */
	__u8							debug;
};

/*
 * Externs
 * ~~~~~~~
 */

/* Base struct. */
extern bsp_base_s					bsp_base;

/*
 * Functions
 * ~~~~~~~~~
 */

/** Gpio initialize */
void	bsp_gpio_initialize(					void);

/* Update all fxs relay */
int bsp_update_all_fxs_relay(					void);

/** Recovery force sysctl proc_do int vec. */
int bsp_boot_recovery_force_proc_dointvec_minmax(	ctl_table		*_table,
										int			_write,
										struct file	*_filp,
										void __user	*_buffer,
										size_t		*_lenp,
										loff_t		*_ppos);
	
/** sysctl bsp_relay_maps_proc. */
int bsp_relay_maps_proc(						ctl_table		*_pTable,
										int			_write,
										struct file	*_pFile,
										void __user	*_pBuffer,
										size_t		*_pLen,
										loff_t		*_pPpos);

/** sysctl bsp_led_maps_proc. */
int bsp_led_maps_proc(						ctl_table		*_pTable,
										int			_write,
										struct file	*_pFile,
										void __user	*_pBuffer,
										size_t		*_pLen,
										loff_t		*_pPpos);

/** sysctl bsp_but_maps_proc. */
int bsp_but_maps_proc(						ctl_table		*_pTable,
										int			_write,
										struct file	*_pFile,
										void __user	*_pBuffer,
										size_t		*_pLen,
										loff_t		*_pPpos);

/** sysctl bsp_gpio_maps_proc. */
int bsp_gpio_maps_proc(						ctl_table		*_pTable,
										int			_write,
										struct file	*_pFile,
										void __user	*_pBuffer,
										size_t		*_pLen,
										loff_t		*_pPpos);

/** sysctl bsp_relay_mode_proc_dostring. */
int bsp_relay_mode_proc_dostring(				ctl_table		*_pTable,
										int			_write,
										struct file	*_pFile,
										void __user	*_pBuffer,
										size_t		*_pLen,
										loff_t		*_pPpos);

/* Interrupt */
irqreturn_t bsp_gpio_irq(					int			_irq,
										void			*_pParam,
		       							struct pt_regs *_pRegs);

/** Initialize. */
int bsp_phyLed_initialize(					void);

#else
#warning "This header is only for kernel land"
#endif

#endif

// vim:ts=5:
