/*
 * ============================================================================
 *
 * Copyright : (c) 2005 SAGEM SA - Site Velizy - URD2
 *
 * The information and source code contained herein is the exclusive
 * property of SAGEM and may not be disclosed, examined, or
 * reproduced in whole or in part without explicit written authorization
 * of the company.
 *
 * ============================================================================
 */

/**
 * \file bsp.c Bsp functions
 *
 * This file contain all Bsp functions.
 */

/*
 * Includes
 * ~~~~~~~~
 */
#include "bsp_pri.h"


/** Module info. */
MODULE_AUTHOR("Sagem");
/** Module License. */
MODULE_LICENSE("Sagem SA");
/** Module description. */
MODULE_DESCRIPTION("Bsp Sagem Module");

/*
 * Functions
 * ~~~~~~~~~
 */

/** Init module. */
static int __init bsp_init(void)
{
	printk("BSP: Load ...\n");

	/* Env */
	{
		/* Boot part */
		{
			/* Init Semaphore */
			init_rwsem(&(bsp_base.Env.Boot.Recovery.sem));

			/* Recovery force */
			bsp_base.Env.Boot.Recovery.force	= BSP_Boot_recovery_force_normal;
		}

		/* Relay part */
		{
			/* Init Semaphore */
			init_rwsem(&(bsp_base.Env.Relay.Mode.sem));

			/* Mode relay. */
			strncpy(	bsp_base.Env.Relay.Mode.Cur.name,
					"none",
					BSP_RELAY_MODE_NAME_MAX_SIZE);
		}
	}

	/* Board part */
	{
		/* Gpio lock */
		spin_lock_init(&bsp_base.Board.Cur.gpio_lock);
	}

	/* Permanent Parameters */
	{
		memcpy(bsp_base.Pp.pFlashAddr, BSP_FLASH_ADDR_PERMPARM, sizeof(bsp_permParam_s));
		/* Check Magic */
		if (bsp_base.Pp.pFlashAddr->magic != BSP_MAGIC_PERMPARAM) {
			printk(	"%s: FAILURE -> Bad Magic(%x-%x) in Permanent Parameter(%p)\n",
					__FUNCTION__,
					bsp_base.Pp.pFlashAddr->magic,
					BSP_MAGIC_PERMPARAM,
					bsp_base.Pp.pFlashAddr);
					
			/* Fail */
			return(-1);
		}

		/* Check version */
		{
			char *_pVersion;
			char	*_pVersion_0	= "0.";
			char	*_pVersion_1	= "1.";
	
			/* Retrieve version */
			_pVersion = (char *)bsp_get_permParam(BSP_Get_permParam_version);
	
			/* Check 0.x version */
			if (memcmp(	_pVersion,
						_pVersion_0,
						strlen(_pVersion_0)) == 0) {
				/* Set Flag */
				bsp_base.flags |= BSP_FLAGS_BASE_PP_VERSION_0;

				/* Panic */
				panic("Bad type Perm Param version 0.x");
			}
			else {
				/* Check 1.x version */
				if (memcmp(	_pVersion,
							_pVersion_1,
							strlen(_pVersion_1)) == 0) {
					/* Set Flag */
					bsp_base.flags |= BSP_FLAGS_BASE_PP_VERSION_1;
				}
				else {
					/* Panic */
					panic("Bad type Perm Param version ");
				}
			}
		}

		/* Retrieve typeBoard */
		{
			unsigned int	_pp2board	= 0;
			unsigned int	_match	= 0;

			/* All board */
			for (_pp2board = 0; _pp2board < BSP_Pp2board_max; _pp2board++) {
				/* Reset */
				_match = 0;

				/* If sagem product id */
				if (bsp_base.Pp.pp2board[_pp2board].pInfo[BSP_Pp_info_sagem_product_id] != (char *)0) {
					/* If doesn't Match */
					if (strcmp(	((char *)bsp_get_permParam(	BSP_Get_permParam_sagem_product_id)),
								bsp_base.Pp.pp2board[_pp2board].pInfo[BSP_Pp_info_sagem_product_id]) != 0) {
						/* Continue */
						continue;
					}
					else {
						/* Set */
						_match = 1;
					}
				}

				/* If sagem hardware version */
				if (bsp_base.Pp.pp2board[_pp2board].pInfo[BSP_Pp_info_sagem_hardware_version] != (char *)0) {
					/* If doesn't Match */
					if (strcmp(	((char *)bsp_get_permParam(	BSP_Get_permParam_sagem_hardware_version)),
								bsp_base.Pp.pp2board[_pp2board].pInfo[BSP_Pp_info_sagem_hardware_version]) != 0) {
						/* Continue */
						continue;
					}
					else {
						/* Set */
						_match = 1;
					}
				}

				/* If sagem hardware revision */
				if (bsp_base.Pp.pp2board[_pp2board].pInfo[BSP_Pp_info_sagem_hardware_revision] != (char *)0) {
					/* If doesn't Match */
					if (strcmp(	((char *)bsp_get_permParam(	BSP_Get_permParam_sagem_hardware_revision)),
								bsp_base.Pp.pp2board[_pp2board].pInfo[BSP_Pp_info_sagem_hardware_revision]) != 0) {
						/* Continue */
						continue;
					}
					else {
						/* Set */
						_match = 1;
					}
				}


				/* If customer product id */
				if (bsp_base.Pp.pp2board[_pp2board].pInfo[BSP_Pp_info_customer_product_id] != (char *)0) {
					/* If doesn't Match */
					if (strcmp(	((char *)bsp_get_permParam(	BSP_Get_permParam_customer_product_id)),
								bsp_base.Pp.pp2board[_pp2board].pInfo[BSP_Pp_info_customer_product_id]) != 0) {
						/* Continue */
						continue;
					}
					else {
						/* Set */
						_match = 1;
					}
				}

				/* If customer hardware version */
				if (bsp_base.Pp.pp2board[_pp2board].pInfo[BSP_Pp_info_customer_hardware_version] != (char *)0) {
					/* If doesn't Match */
					if (strcmp(	((char *)bsp_get_permParam(	BSP_Get_permParam_customer_hardware_version)),
								bsp_base.Pp.pp2board[_pp2board].pInfo[BSP_Pp_info_customer_hardware_version]) != 0) {
						/* Continue */
						continue;
					}
					else {
						/* Set */
						_match = 1;
					}
				}

				/* If customer hardware revision */
				if (bsp_base.Pp.pp2board[_pp2board].pInfo[BSP_Pp_info_customer_hardware_revision] != (char *)0) {
					/* If doesn't Match */
					if (strcmp(	((char *)bsp_get_permParam(	BSP_Get_permParam_customer_hardware_revision)),
								bsp_base.Pp.pp2board[_pp2board].pInfo[BSP_Pp_info_customer_hardware_revision]) != 0) {
						/* Continue */
						continue;
					}
					else {
						/* Set */
						_match = 1;
					}
				}

				/* If match */
				if (_match == 1) {
					/* Set Name2board*/
					bsp_base.Board.Cur.pp2board	= _pp2board;

					/* Set Board Info */
					bsp_base.Board.Cur.boardInfo	= bsp_base.Pp.pp2board[_pp2board].boardInfo;
	
					/* Set Board Type */
					bsp_base.Board.Cur.boardType	= bsp_base.Pp.pp2board[_pp2board].boardType;

					/* Out */
					break;
				}
			}
		}

		/* Check */
		if (bsp_base.Board.Cur.pp2board == BSP_Pp2board_unknown) {
			/* Set to unknwon */
			{
				bsp_base.Board.Cur.pp2board		= BSP_Pp2board_unknown;
				bsp_base.Board.Cur.boardInfo		= BSP_Board_info_unknown;
				bsp_base.Board.Cur.boardType		= BSP_Board_type_unknown;
			}

			printk("	Bad Pp -> pp2Board(%u) boardInfo(%u) boardType(%u) => Fix to Unknown board type !\n",
				bsp_base.Board.Cur.pp2board,
				bsp_base.Board.Cur.boardInfo,
				bsp_base.Board.Cur.boardType);
		}
		else {
			printk("	Manufacturer(%s) Serial Number(%s)\n",
					(char *)bsp_get_permParam(	BSP_Get_permParam_manufacturer),
					(char *)bsp_get_permParam(	BSP_Get_permParam_serial_number));
			printk("	Sagem Hardware ProductId(%s) Version(%s) Revision(%s)\n",
					(char *)bsp_get_permParam(	BSP_Get_permParam_sagem_product_id),
					(char *)bsp_get_permParam(	BSP_Get_permParam_sagem_hardware_version),
					(char *)bsp_get_permParam(	BSP_Get_permParam_sagem_hardware_revision));
			printk("	Customer Hardware ProductId(%s) Version(%s) Revision(%s)\n",
					(char *)bsp_get_permParam(	BSP_Get_permParam_customer_product_id),
					(char *)bsp_get_permParam(	BSP_Get_permParam_customer_hardware_version),
					(char *)bsp_get_permParam(	BSP_Get_permParam_customer_hardware_revision));
			printk("	Firmware Version(%s) Type(",
					(char *)bsp_get_permParam(	BSP_Get_permParam_firmware_version));
			switch (bsp_base.Env.Krnopt.typeFirmware) {
				case BSP_Firmware_type_operational:
				{
					printk("%s)\n", "OPERATIONAL");
				}
				break;
	
				case BSP_Firmware_type_lanrescue:
				{
					printk("%s)\n", "LANRESCUE");
				}
				break;
	
				case BSP_Firmware_type_rescue:
				{
					printk("%s)\n", "RESCUE");
				}
				break;
	
				default:
				{
					/* Panic */
					panic("Bad Type Firmware");
				}
				break;
			}
			printk("	Size {Ram -> [Pp(%d) Linux(%ld)] Flash ->[Pp(%d) Linux(%d)]}\n",
					(*(__u32 *)bsp_get_permParam(	BSP_Get_permParam_ram_size)) * 1024,
					totalram_pages << (PAGE_SHIFT-10),
					(*(__u32 *)bsp_get_permParam(	BSP_Get_permParam_flash_size)) * 1024,
					board_flash_size / 1024);
		}
	}
	
	/* Gpio */
	{
		/* Initialize Gpio */
		bsp_gpio_initialize();
	}

	/* Dev */
	{
		/* Base */
		{
			/* Register chrdev region */
			if (register_chrdev_region(	MKDEV(BSP_DEV_MAJOR, 0),
									BSP_Dev_minor_max,
									BSP_DEV_NAME) != 0) {
				printk(" KO\n\n");
	
				/* Error */
				return(-1);
			}
		}

		/* All minor */
		{
			unsigned char _minor;

			for (_minor = 0; _minor < BSP_Dev_minor_max; _minor++) {
				/* Various */
				init_MUTEX(&(bsp_base.Dev[_minor].sem));
				bsp_base.Dev[_minor].nbOpen		= 0;

				/* Cdev_init */
				cdev_init(	&(bsp_base.Dev[_minor].cdev),
							&(bsp_base.Dev[_minor].fops));

				/* Add cdev */
				if (cdev_add(	&(bsp_base.Dev[_minor].cdev),
							MKDEV(BSP_DEV_MAJOR, _minor),
							1) != 0) {
					printk(	"%s: FAILURE -> cdev_add.\n",
							__FUNCTION__);

					for (_minor = _minor - 1; _minor <= 0; _minor--) {
						/* Cdev_del */
						cdev_del(&(bsp_base.Dev[_minor].cdev));
					}

					/* Unregister chrdev */
					unregister_chrdev_region(	MKDEV(BSP_DEV_MAJOR, _minor),
											BSP_Dev_minor_max);
		
					/* Ko */
					return(-1);
				}

				/* Create Class */
				{
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,16)
					if (IS_ERR(bsp_base.Dev[_minor].pClass = class_create(	THIS_MODULE,
																bsp_base.Dev[_minor].name))) {
#else
					if (IS_ERR(bsp_base.Dev[_minor].pClass = class_simple_create(	THIS_MODULE,
																		bsp_base.Dev[_minor].name))) {
#endif
						printk(	"%s: FAILURE -> Creating bsp class.\n",
								__FUNCTION__);

						for (_minor = _minor; _minor <= 0; _minor--) {
							/* CLASS */
							{
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,16)
								/* Destroy */
								class_destroy(bsp_base.Dev[_minor].pClass);
#else
								/* Destroy */
								class_simple_destroy(bsp_base.Dev[_minor].pClass);
#endif
							}

							/* Cdev_del */
							cdev_del(&(bsp_base.Dev[_minor].cdev));
						}
		
						/* Unregister chrdev */
						unregister_chrdev_region(	MKDEV(BSP_DEV_MAJOR, _minor),
												BSP_Dev_minor_max);
		
						/* Ko */
						return(-1);
					}
				}

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,16)
				/* Device Create */
				device_create(	bsp_base.Dev[_minor].pClass,
									NULL,
									MKDEV(BSP_DEV_MAJOR, _minor),
									NULL,
									bsp_base.Dev[_minor].name);
#else
				/* Device Add */
				class_simple_device_add(	bsp_base.Dev[_minor].pClass,
									MKDEV(BSP_DEV_MAJOR, _minor),
									NULL,
									bsp_base.Dev[_minor].name);
#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,18)
				/* Mk Cdev */
				devfs_mk_cdev(	MKDEV(BSP_DEV_MAJOR, _minor),
							S_IFCHR | S_IRUGO | S_IWUGO,
							bsp_base.Dev[_minor].Dev.name) {
#endif
			}
		}
	}

	/* Register /sys/sagem/ SysCtl */
	{
		/* Register device */
		if ((bsp_base.SysCtl.head = register_sysctl_table(bsp_base.SysCtl.sagem)) == 0) {
			printk(	"%s: Error register_sysctl on /sys/sagem/\n",
					__FUNCTION__);
		
			/* Error */
			return(-1);
		}
	}

	printk("BSP: Ok ...\n");

	return(0);
}

static void __exit bsp_exit(void)
{
	printk(	"BSP(%s): Unload ...\n",
			(char *)bsp_get_permParam(	BSP_Get_permParam_manufacturer));

	/* If register */
	if (bsp_base.SysCtl.head != (struct ctl_table_header *)0) {
		printk("Unregister /sys/sagem/ SysCtl ...");

		/* Unregister sysctl */
		unregister_sysctl_table(bsp_base.SysCtl.head);

		/* Unset */
		bsp_base.SysCtl.head = (struct ctl_table_header *)0;

		printk(" OK\n");
	}

	/* All minor */
	{
		unsigned char _minor;

		for (_minor = 0; _minor < BSP_Dev_minor_max; _minor++) {
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,18)
			/* Rem DEVFS */
			devfs_remove(bsp_base.Dev[_minor].cdev.kobj.name);
#endif

			/* CLASS */
			{
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,16)
				/* Remove */
				device_destroy(	bsp_base.Dev[_minor].pClass,
									MKDEV(BSP_DEV_MAJOR, _minor));
		
				/* Destroy */
				class_destroy(bsp_base.Dev[_minor].pClass);
#else
				/* Remove */
				class_simple_device_remove(MKDEV(BSP_DEV_MAJOR, _minor));
		
				/* Destroy */
				class_simple_destroy(bsp_base.Dev[_minor].pClass);
#endif
			}

			/* Del Cdev */
			cdev_del(&(bsp_base.Dev[_minor].cdev));
		}

		/* Unregister chrdev region */
		unregister_chrdev_region(	MKDEV(BSP_DEV_MAJOR, _minor),
								BSP_Dev_minor_max);
	}

	printk(	"BSP(%s): Ok ...\n",
			(char *)bsp_get_permParam(	BSP_Get_permParam_manufacturer));
}

/** Retrieve Type Firmware. */
static int __init bsp_retrieve_typeFirmware(	char *_pStr)
{
	unsigned int	_type	= 0;

	/* All Type */
	for (_type = 0; _type < BSP_Firmware_type_max; _type++) {
		/* Match */
		if (strcmp(	_pStr,
					bsp_base.Env.Firmware.Type[_type].name) == 0) {
			/* Set */
			bsp_base.Env.Krnopt.typeFirmware	= _type;
		}
	}

	/* Check */
	if (_type == BSP_Firmware_type_max) {
		return(0);
	}

	/* Ok */
	return(1);
}

/* Retrieving myfs_start */
static int __init bsp_retrieve_myfs_start(char *_pStr)
{
	/* Get option */
	get_option(	&_pStr,
				&(bsp_base.Env.Krnopt.myfs_start));

	/* Ok */
	return 1;
}

/* Retrieving myfs_size */
static int __init bsp_retrieve_myfs_size(char *_pStr)
{
	/* Get option */
	get_option(	&_pStr,
				&(bsp_base.Env.Krnopt.myfs_size));

	/* Ok */
	return 1;
}

/* Retrieving operational_start_size */
static int __init bsp_retrieve_operational_start(char *_pStr)
{
	/* Get option */
	get_option(	&_pStr,
				&(bsp_base.Env.Krnopt.operational_start));

	/* Ok */
	return 1;
}

/* Retrieving rescue_start_size */
static int __init bsp_retrieve_rescue_start(char *_pStr)
{
	/* Get option */
	get_option(	&_pStr,
				&(bsp_base.Env.Krnopt.rescue_start));

	/* Ok */
	return 1;
}

/** Init module. */
module_init(bsp_init);
/** Exit module. */
module_exit(bsp_exit);

__setup(	"type=",
		bsp_retrieve_typeFirmware);
__setup(	"myfs_start=",
		bsp_retrieve_myfs_start);
__setup(	"myfs_size=",
		bsp_retrieve_myfs_size);
__setup(	"operational_start=",
		bsp_retrieve_operational_start);
__setup(	"rescue_start=",
		bsp_retrieve_rescue_start);

// vim:ts=5:
