/* ============================================================================
 *
 * Copyright : (c) 2005 SAGEM SA - Site Velizy - URD2
 *
 * The information and source code contained herein is the exclusive
 * property of SAGEM and may not be disclosed, examined, or
 * reproduced in whole or in part without explicit written authorization
 * of the company.
 *
 * ============================================================================
 */

/**
 * \file bsp_relay.c Bsp Relay functions
 *
 * This file contain all Bsp Relay functions.
 */

/*
 * Includes
 * ~~~~~~~~
 */

#include "bsp_pri.h"

/*
 * Functions
 * ~~~~~~~~~
 */

/** Get relay. */
pBsp_board_relay_s	bsp_get_board_relay(	void)
{
	/* Chech reference */
	if (bsp_base.Board.Cur.pInfo == (pBsp_board_info_all_s)0) {
		printk(	"%s: WARNING -> Current reference is NULL\n",
				__FUNCTION__);

		return((pBsp_board_relay_s)0);
	}

	/* Ok */
	return(&(bsp_base.Board.Cur.pInfo->Relay));
}
EXPORT_SYMBOL(bsp_get_board_relay);

/* Update all fxs relay */
int bsp_update_all_fxs_relay(void)
{
	/* If ok to activate all fxs relay */
	if ( ( (bsp_base.Env.Relay.flags & ( BSP_Relay_flags_mode_adsl | BSP_Relay_flags_adsl_link_up | BSP_Relay_flags_voice_module_up)) ==
								( BSP_Relay_flags_mode_adsl | BSP_Relay_flags_adsl_link_up | BSP_Relay_flags_voice_module_up) ) || 
		( (bsp_base.Env.Relay.flags & ( BSP_Relay_flags_mode_ftth | BSP_Relay_flags_voice_module_up)) ==
								( BSP_Relay_flags_mode_ftth | BSP_Relay_flags_voice_module_up) ) ) {
		/* Set relay state (relay2 match fxo) */
		kSysRelayCtrl(	kRelay1,
					kRelayOn);
	}
	else {
		/* Set relay state (relay2 match fxo) */
		kSysRelayCtrl(	kRelay1,
					kRelayOff);
	}

	/* Ok */
	return(0);
}

/** Kernel write relay state function */
BOARD_RELAY_STATE kSysRelayCtrl(	BOARD_RELAY_NAME	_nameUser,
							BOARD_RELAY_STATE	_stateUser)
{
	/* Check name. */
	if ( (_nameUser < 0) || (_nameUser >= kRelayEnd) ) {
		printk(	"%s: WARNING -> Relay >= Max(%u)\n",
				__FUNCTION__,
				kRelayEnd);

		return(-1);
	}

	/* Check state. */
	if ( (_stateUser < 0) || (_stateUser >= kRelayOff + 1) ) {
		printk(	"%s: WARNING -> Relay >= Max(%u)\n",
				__FUNCTION__,
				kRelayOff + 1);

		return(-1);
	}

	{
		bsp_gpio_name_e	_nameGpio;
		bsp_gpio_state_e	_stateGpio;

		/* Retrieve nameGpio */
		_nameGpio		= bsp_base.Index.Relay.Name.user2gpio[_nameUser];

		/* Retrieve stateGpio */
		_stateGpio	= bsp_base.Index.Relay.State.user2gpio[_stateUser];

		/* Set state */
		bsp_gpio_set_state(	_nameGpio,
						_stateGpio);

#ifdef BSP_DEBUG
		/* If bsp */
		if (bsp_base.debug & BSP_FLAGS_DEBUG_RELAY) {
			printk(	"%s: Relay -> nameUser(%u) nameGpio(%u) stateUser(%u) stateGpio(%u)\n",
					__FUNCTION__,
					_nameUser,
					_nameGpio,
					_stateUser,
					_stateGpio);
		}
#endif
	}

	return(0);
}
EXPORT_SYMBOL(kSysRelayCtrl);

/** Kernel read relay state function */
BOARD_RELAY_STATE kSysGetRelayState(BOARD_RELAY_NAME _nameUser)
{
	BOARD_RELAY_STATE	_stateUser;

	/* Check name. */
	if ( (_nameUser < 0) || (_nameUser >= kRelayEnd) ) {
		printk(	"%s: WARNING -> Relay >= Max(%u)\n",
				__FUNCTION__,
				kRelayEnd);

		return(-1);
	}

	/* Retrieve state */
	{
		bsp_gpio_state_e	_stateGpio;
		bsp_gpio_name_e	_nameGpio;

		/* Retrieve nameGpio */
		_nameGpio = bsp_base.Index.Relay.Name.user2gpio[_nameUser];

		/* Retrieve stateGpio */
		if ((_stateGpio = bsp_gpio_get_state(_nameGpio)) == BSP_Gpio_state_failure) {
			/* Error */
			return(-1);
		}

		/* Retrieve userState */
		_stateUser = bsp_base.Index.Relay.State.gpio2user[_stateGpio];

#ifdef BSP_DEBUG
		/* If bsp */
		if (bsp_base.debug & BSP_FLAGS_DEBUG_RELAY) {
			printk(	"%s: Relay -> nameUser(%u) nameGpio(%u) stateUser(%u) stateGpio(%u)\n",
					__FUNCTION__,
					_nameUser,
					_nameGpio,
					_stateUser,
					_stateGpio);
		}
#endif
	}

	return(_stateUser);
}
EXPORT_SYMBOL(kSysGetRelayState);

/** sysctl bsp_relay_maps_proc.
 *
 *   \par Description
 *	<b>This function allow to set connexion tampon.</b>\n
 *
 *	@param _pTable:	Sysctl table reference.
 *	@param _write:		If write.
 *	@param _pFile:		File reference.
 *	@param _pBuffer:	Buffer reference.
 *	@param _pLen:		Length reference.
 *	@param _pPpos:		Position reference.
 *
 *	@return proc_dostring return.
 */
int bsp_relay_maps_proc(	ctl_table		*_pTable,
					int			_write,
					struct file	*_pFile,
					void __user	*_pBuffer,
					size_t		*_pLen,
					loff_t		*_pPpos)
{
	pBsp_board_relay_s	_pBsp_relay	= bsp_get_board_relay();
	char				*_pTampon		= (char *)_pTable->data;
	BOARD_RELAY_STATE	_userState;

	sprintf(	_pTampon,
			"\n");

	/* Next */
	_pTampon += strlen(_pTampon);

	/* Retrieve tampon */
	if ((_pTampon = (char *)_pTable->data) != (char *)0) {
		sprintf(	_pTampon,
				"| Mode | Voice module | Adsl link | Nb | AllFxs | Fxo |\n");

		/* Next */
		_pTampon += strlen(_pTampon);

		{
			/* Mode */
			{
				/* Ftth */
				if (bsp_base.Env.Relay.flags & BSP_Relay_flags_mode_ftth) {
					sprintf(	_pTampon,
							"| ftth |");
				}
				/* Adsl */
				else if (bsp_base.Env.Relay.flags & BSP_Relay_flags_mode_adsl) {
					sprintf(	_pTampon,
							"| adsl |");
				}
				/* None */
				else {
					sprintf(	_pTampon,
							"| none |");
				}
			}

			/* Next */
			_pTampon += strlen(_pTampon);

			/* Voice Driver */
			{
				/* Up */
				if (bsp_base.Env.Relay.flags & BSP_Relay_flags_voice_module_up) {
					sprintf(	_pTampon,
							"       1      |");
				}
				/* Down */
				else {
					sprintf(	_pTampon,
							"       0      |");
				}
			}

			/* Next */
			_pTampon += strlen(_pTampon);

			/* Adsl */
			{
				/* Up */
				if (bsp_base.Env.Relay.flags & BSP_Relay_flags_adsl_link_up) {
					sprintf(	_pTampon,
							"     1     |");
				}
				/* Down */
				else {
					sprintf(	_pTampon,
							"     0     |");
				}
			}

			/* Next */
			_pTampon += strlen(_pTampon);
		}

#if 0
		/* If relay */
		if (_pBsp_relay->Fxs.nb + _pBsp_relay->Fxo.nb) {
#endif
			sprintf(	_pTampon,
					" %2u |",
					_pBsp_relay->Fxs.nb + _pBsp_relay->Fxo.nb);

			/* Next */
			_pTampon += strlen(_pTampon);

			/* Relay all Fxs */
			{
				/* Retrieve relay state (relay1 match all_fxs) */
				_userState = kSysGetRelayState(kRelay1);

				switch (_userState) {
					case kRelayOn:
					{
						/* Print */
						sprintf(	_pTampon,
								"   1    |");
					}
					break;
		
					case kRelayOff:
					{
						/* Print */
						sprintf(	_pTampon,
								"   0    |");
					}
					break;
		
					default:
					{
						/* Print */
						sprintf(	_pTampon,
								"   -    |");
					}
					break;
				}
			}
	
			/* Next */
			_pTampon += strlen(_pTampon);
			
			/* Relay FX0 */
			{
				/* Retrieve relay state (relay2 match fxo) */
				_userState = kSysGetRelayState(kRelay2);

				switch (_userState) {
					case kRelayOn:
					{
						/* Print */
						sprintf(	_pTampon,
								"  1  |\n");
					}
					break;
		
					case kRelayOff:
					{
						/* Print */
						sprintf(	_pTampon,
								"  0  |\n");
					}
					break;
		
					default:
					{
						/* Print */
						sprintf(	_pTampon,
								"  -  |\n");
					}
					break;
				}
			}
	
			/* Next */
			_pTampon += strlen(_pTampon);
#if 0
		}
		else {
			sprintf(	_pTampon,
					"  %02u  |\n",
					_pBsp_relay->Fxs.nb + _pBsp_relay->Fxo.nb);

			/* Next */
			_pTampon += strlen(_pTampon);
		}
#endif
	}
	else {
		/* Print */
		sprintf(	_pTampon,
				"Process Error\n");

		/* Next */
		_pTampon += strlen(_pTampon);
	}

	sprintf(	_pTampon,
			"\n");

	/* Next */
	_pTampon += strlen(_pTampon);

	/* Call proc_dostring */
	return(proc_dostring(	_pTable,
						_write,
						_pFile,
						_pBuffer,
						_pLen,
						_pPpos));
}

/** sysctl bsp_relay_mode_proc_dostring.
 *
 *   \par Description
 *	<b>This function allow to set connexion tampon.</b>\n
 *
 *	@param _pTable:	Sysctl table reference.
 *	@param _write:		If write.
 *	@param _pFile:		File reference.
 *	@param _pBuffer:	Buffer reference.
 *	@param _pLen:		Length reference.
 *	@param _pPpos:		Position reference.
 *
 *	@return proc_dostring return.
 */
int bsp_relay_mode_proc_dostring(	ctl_table		*_pTable,
							int			_write,
							struct file	*_pFile,
							void __user	*_pBuffer,
							size_t		*_pLen,
							loff_t		*_pPpos)
{
	int _res;

	/* Write */
	if (_write) {
		/* Take write sem */
		down_write(&(bsp_base.Env.Relay.Mode.sem));

		/* Process */
		_res = proc_dostring(	_pTable,
							_write,
							_pFile,
							_pBuffer,
							_pLen,
							_pPpos);

		/* If ok */
		if (_res == 0) {
			bsp_relay_mode_e	_relayMode;

			for (_relayMode = 0; _relayMode < BSP_Relay_mode_end; _relayMode++) {
				/* Match */
				if (strcmp(	_pTable->data,
							bsp_base.Env.Relay.Mode.All[_relayMode].name) == 0) {
					/* Out */
					break;
				}
			}
		
			/* Check */
			if (_relayMode == BSP_Relay_mode_end) {
				/* Notify */
				bsp_notify_evt(BSP_Notify_evt_relay_mode_none);

				printk(	"%s: Unknown mode name(%s) -> Fix to None !\n",
						__FUNCTION__,
						(char *)_pTable->data);
			}
			else {
				/* Notify */
				bsp_notify_evt(bsp_base.Env.Relay.Mode.All[_relayMode].notify);
			}
		}

		/* Release write sem */
		up_write(&(bsp_base.Env.Relay.Mode.sem));
	}
	/* read */
	else {
		/* Take read sem */
		down_read(&(bsp_base.Env.Relay.Mode.sem));

		/* Process */
		_res = proc_dostring(	_pTable,
							_write,
							_pFile,
							_pBuffer,
							_pLen,
							_pPpos);
		
		/* Release read sem */
		up_read(&(bsp_base.Env.Relay.Mode.sem));
	}

	/* out */
	return(_res);
}

// vim:ts=5:
