/*
 * Copyright (c) 2011 SoftAtHome
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 */

#ifndef __SAH_LINUXLED_WRAP
#define __SAH_LINUXLED_WRAP

#include "linux/khco_led_api.h"

struct sah_linuxled_desc {
#ifdef CONFIG_LEDS_CLASS
        const char      *name;
        BOARD_LED_NAME  sah_led_name;
#endif
};

#ifdef CONFIG_LEDS_CLASS

/* 
 * Default interface.
 *   create 'num_leds' linux led devices with names 'led-N' where N is [0..num_leds).
 * 
 * Expected use cases:
 *   ctx = sah_linuxleds_register(kLedEnd);
 *   ...
 *   sah_linuxleds_unregister(ctx);
 *
 * BOARD_LED_NAME values (which serve as internal 'led names') are assumed to start at 0
 * and then next = prev + 1, so all the numbers in the range [0..kLedEnd)
 * correspond to existing leds.
 *
 * To enforce the previous assumption (and as a safety measure), we don't allow to register
 * more than SAH_LINUXLEDS_MAXNUM devices with sah_linuxleds_register().
 * Note, that sah_linuxleds_register_desc() doesn't have this limitation.
 */

#define SAH_LINUXLEDS_MAXNUM	256

extern void *sah_linuxleds_register(int num_leds);
extern void  sah_linuxleds_unregister(void *opaque);

/* 
 * Extended interface.
 *  create 'num_leds' linux led devices with user-specified names.
 *
 * 'desc' structures given to sah_linuxleds_register_desc() must be kept in the memory
 * until sah_linuxleds_unregister_desc() call.
 */

extern void *sah_linuxleds_register_desc(int num_leds, struct sah_linuxled_desc *desc);
extern void  sah_linuxleds_unregister_desc(void *opaque);

#else /* !CONFIG_LEDS_CLASS */

static inline void *sah_linuxleds_register(int num_leds) { return NULL; }
static inline sah_linuxleds_unregister(void *opaque) {}
static inline void *sah_linuxleds_register_desc(int num_leds, struct sah_linuxled_desc *desc) { return NULL; }
static inline void sah_linuxleds_unregister_desc(void *opaque) {}

#endif /* CONFIG_LEDS_CLASS */

#endif /* __SAH_LINUXLED_WRAP */
