#ifndef __LED_LOCAL_H__
#define __LED_LOCAL_H__

#include "linux/khco_led_api.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#define LED_DRV_MAJOR	237
#define LED_DRV_DEVICE		"led"
#define BOARD_IOCTL_MAGIC       'B'
#define BOARD_IOCTL_LED_CTRL    _IOWR(BOARD_IOCTL_MAGIC, 7, kled_param_t)
#define BOARD_IOCTL_LED_STATUS	_IOWR(BOARD_IOCTL_MAGIC, 8, kled_param_t)

// to enum the states related to the leds
typedef enum _BOARD_LED_STATE
{
	kLedStateOff,			/* turn off led */
	kLedStateOn,			/* turn on led */
	kLedStateFail,			/* turn led on red */
	kLedStateBlinkOnce,		/* blink once, ~100ms and ignore the same call during the 100ms period */
	kLedStateSlowBlinkContinues,	/* slow blink continues at ~600ms interval */
	kLedStateFastBlinkContinues,	/* fast blink continues at ~200ms interval */
	kLedStateBlinkContinues,	/* blink with a user-defined frequency */
} BOARD_LED_STATE;

// ioctl parameter between kernel space and user space
typedef struct _kled_param_t
{
	BOARD_LED_NAME name;
	BOARD_LED_STATE state;
	int freq_on;
	int freq_off;
} kled_param_t;

// to record the states of leds
typedef struct _kled_generic_t
{
	struct list_head l_list;
	BOARD_LED_STATE state;		/* current led state */
	BOARD_LED_STATE state_saved;	/* used in blink once to restore the orignal led state */
	int blink_count_down;		/* if equals to 0, do blinking (toggle). assigned value and minus 1 each time */
	short blink_freq_on;		/* for kLedStateBlinkContinues only, to record how many 100ms to be on */
	short blink_freq_off;		/* for kLedStateBlinkContinues only, to record how many 100ms to be off */
	int led_on;			/* 1: ON, 0: OFF */
	int blink_mode;
	BOARD_LED_NAME name;		/* led name */
} kled_generic_t, *pkled_generic_t;

typedef void (*kled_ctrl_func_t)(BOARD_LED_NAME name, BOARD_LED_STATE state);
void kled_set_state(BOARD_LED_NAME led, BOARD_LED_STATE state, int freq_on, int freq_off);
void kled_get_state(BOARD_LED_NAME led, BOARD_LED_STATE *state);
void kled_init_state(BOARD_LED_NAME led, BOARD_LED_STATE state);
int kled_init(void);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* __LED_LOCAL_H__ */

