/*
 *
 * Handle mapping of the flash on Any ADI Boards
 *
 * Author:	Leo @ Analog Devices
 * Copyright:	(C) 2005 Analog Devices
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 */

#include <linux/module.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <asm/io.h>
#include <linux/mtd/mtd.h>
#include <linux/mtd/map.h>
#include <linux/mtd/partitions.h>
#include <linux/bsp/bsp_krnopt.h>

/** Extern */
extern int board_flash_start,board_flash_size;

/** Define */
#define FLASH_BLOCK_SIZE	(128 * 1024)
#define BOOT_SIZE		FLASH_BLOCK_SIZE
#define FACTORY_SIZE	FLASH_BLOCK_SIZE
#define BOOT_OFFSET		0


/* partition_info gives details on the logical partitions that the split the
 * single flash device into. If the size if zero we use up to the end of the
 * device. */
static struct mtd_partition partition_info[]={
	{ .name = "Bootloader Partition",
	.offset = BOOT_OFFSET,
	.size =   BOOT_SIZE },
	{ .name = "Factory Partition",
	.offset = BOOT_OFFSET + BOOT_SIZE,
	.size = FACTORY_SIZE },
	{ .name = "User Partition",
#ifdef CONFIG_MTD_SAGEM_MINI_PARTITIONS
	.offset = BOOT_OFFSET + BOOT_SIZE + FACTORY_SIZE },
#else
	.offset = 0x1E00000,
	.size = (2 * 1024 * 1024) },
#endif
	{ .name = "System Partition 1 "},
	{ .name = "System Partition 2 "},
	{ .name = "System Partition 3 "},
	{ .name = "System Partition 4 "},
#ifndef CONFIG_MTD_SAGEM_MINI_PARTITIONS
	{ .name = "RG Conf1",
	.offset = 0x1160000,
	.size = 0x20000 },
	{ .name = "RG Conf2",
	.offset = 0x1180000,
	.size = 0x20000 },
	{ .name = "Sagem Conf",
	.offset = 0x11A0000,
	.size = 0xA0000 }
#endif
};
				   
/* MTD Partitions */
static struct mtd_info *mymtd;

struct map_info mb_map = {
	.name = "ADI flash",
	.phys = 0,
	.size = 0,
	.bankwidth = 2,
};

int __init init_mb(void)
{
	pBsp_env_krnopt_s	pKrnopt;
	unsigned int		WINDOW_ADDR;
	unsigned int		WINDOW_SIZE;
	u32				tmp;

	/* Retrieve Kernel Options */
	{
		pKrnopt		= bsp_get_env_krnopt();

		WINDOW_ADDR	= board_flash_start & 0x1FFFFFFF;
		WINDOW_SIZE	= board_flash_size;
	}

	// Lce : detect flash size of 16/32 Mo and update partition offset for BOOT,FACTORY SETTING and USER
	if (board_flash_size == 0x2000000) 
		{
		partition_info[0].offset += 0x1000000;
		partition_info[1].offset += 0x1000000;
#ifdef CONFIG_MTD_SAGEM_MINI_PARTITIONS
		partition_info[2].offset += 0x1000000;
#endif

		// check for 32 Mo partition
		tmp =	pKrnopt->operational_start & 0x1FFFFFFF;

		/* Check operational_start for operationnal in first bank */
		if ( (tmp < (WINDOW_ADDR) || ((tmp > (WINDOW_ADDR+WINDOW_SIZE/2) && tmp < (WINDOW_ADDR+0x1040000))) || tmp > (WINDOW_ADDR+WINDOW_SIZE)) &&
			(pKrnopt->typeFirmware == BSP_Firmware_type_operational)) {
			panic("!!! Partition start address(operational_start) setting 0x%x is wrong. \n", pKrnopt->operational_start);
			}

		tmp =	pKrnopt->myfs_start & 0x1FFFFFFF;

		/* Check myfs_start for operationnal in first bank take care about boot, PP and Factory setting */
		if ( (tmp < (WINDOW_ADDR) || ((tmp > (WINDOW_ADDR+WINDOW_SIZE/2) && tmp < (WINDOW_ADDR+0x1040000))) || tmp > (WINDOW_ADDR+WINDOW_SIZE)) &&
			(pKrnopt->typeFirmware == BSP_Firmware_type_operational)) {
			panic("!!! File system start address(myfs_start) setting 0x%x is wrong. \n", pKrnopt->myfs_start);
			}

		/* Check myfs_start for rescue in second bank, */
		if ( (tmp < (WINDOW_ADDR+0x1040000) || (tmp > (WINDOW_ADDR+WINDOW_SIZE))) &&
			(pKrnopt->typeFirmware == BSP_Firmware_type_rescue)) {
			panic("!!! File system start address(myfs_start) setting 0x%x is wrong. \n", pKrnopt->myfs_start);
			}

		tmp =	pKrnopt->rescue_start & 0x1FFFFFFF;

		/* Check rescue in second bank take care about boot, PP and Factory setting */
		if ( (tmp < (WINDOW_ADDR+0x140000) || (tmp > (WINDOW_ADDR+WINDOW_SIZE))) ) {
			panic("!!! Partition start address(rescue_start) setting 0x%x is wrong. \n", pKrnopt->rescue_start);
			}

		}
	else
		{
		// check for 16 Mo partition: take care about boot, PP and Factory setting
		tmp =	pKrnopt->operational_start & 0x1FFFFFFF;

		/* Check */
		if ( (tmp < (WINDOW_ADDR+0x40000) || (tmp > (WINDOW_ADDR+WINDOW_SIZE))) &&
			(pKrnopt->typeFirmware == BSP_Firmware_type_operational)) {
			panic("!!! Partition start address(operational_start) setting 0x%x is wrong. \n", pKrnopt->operational_start);
			}

		tmp =	pKrnopt->myfs_start & 0x1FFFFFFF;

		/* Check */
		if ( (tmp < (WINDOW_ADDR+0x40000) || (tmp > (WINDOW_ADDR+WINDOW_SIZE))) &&
			(pKrnopt->typeFirmware != BSP_Firmware_type_lanrescue)) {
			panic("!!! File system start address(myfs_start) setting 0x%x is wrong. \n", pKrnopt->myfs_start);
			}

		tmp =	pKrnopt->rescue_start & 0x1FFFFFFF;

		/* Check */
		if ( (tmp < (WINDOW_ADDR+0x40000) || (tmp > (WINDOW_ADDR+WINDOW_SIZE))) ) {
			panic("!!! Partition start address(rescue_start) setting 0x%x is wrong. \n", pKrnopt->rescue_start);
			}
		}
	/* Registers flash chip with do_map_probe */
	mb_map.phys = WINDOW_ADDR;
	mb_map.size = WINDOW_SIZE;

	/* print */
	printk(KERN_NOTICE "On Board flash device: 0x%x at 0x%x\n", WINDOW_SIZE, WINDOW_ADDR);

	mb_map.virt = ioremap(WINDOW_ADDR, WINDOW_SIZE);

	/* Check */
	if (!mb_map.virt) {
	    printk("Failed to ioremap\n");
	    return -EIO;
	}
	simple_map_init(&mb_map);
	mymtd = do_map_probe("cfi_probe", &mb_map);

	if (mymtd) {
	    mymtd->owner = THIS_MODULE;

	    add_mtd_device(mymtd);

	    /* MTD mapping depends upon firmware type */
	    switch (pKrnopt->typeFirmware) {
		case BSP_Firmware_type_operational:
		{
			/* Partition name */
			{
				strncpy(partition_info[3].name, "Rescue FW",20);
				strncpy(partition_info[4].name, "Operational FW",20);
				strncpy(partition_info[5].name, "Operational SquashFS",21);
			}

			/* Set partition struct */
			{
				partition_info[3].offset = (pKrnopt->rescue_start & 0x1FFFFFFF) - WINDOW_ADDR;
#ifdef CONFIG_MTD_SAGEM_MINI_PARTITIONS
				partition_info[2].size = partition_info[3].offset - partition_info[2].offset;
#endif
				partition_info[4].offset = (pKrnopt->operational_start & 0x1FFFFFFF) - WINDOW_ADDR;
				partition_info[5].offset = (pKrnopt->myfs_start & 0x1FFFFFFF) - WINDOW_ADDR;
				partition_info[4].size = partition_info[5].offset - partition_info[4].offset;

				if (board_flash_size == 0x2000000)
				{
					partition_info[3].size = WINDOW_SIZE - partition_info[3].offset;

					/* Bank memory without boot */
					if (partition_info[5].offset < WINDOW_SIZE/2) {
						partition_info[5].size = WINDOW_SIZE/2 - partition_info[5].offset;
					}
					/* Bank memory with boot */
					else {
						partition_info[5].size = WINDOW_SIZE - partition_info[5].offset;
					}
				}
				else
				{
					partition_info[3].size = partition_info[4].offset - partition_info[3].offset;
					partition_info[5].size = WINDOW_SIZE - partition_info[5].offset;
				}
			}

			/* Register partition table */
#ifdef CONFIG_MTD_SAGEM_MINI_PARTITIONS
			add_mtd_partitions(mymtd, partition_info, 6);
#else
			add_mtd_partitions(mymtd, partition_info, 10);
#endif
		}
		break;

		case BSP_Firmware_type_lanrescue:
		{
			/* Partition name */
			{
				strncpy(partition_info[3].name, "Firmwares",20);
			}

			/* Set partition struct */
			{
				partition_info[3].offset = (pKrnopt->rescue_start & 0x1FFFFFFF) - WINDOW_ADDR;
#ifdef CONFIG_MTD_SAGEM_MINI_PARTITIONS
				partition_info[2].size = partition_info[3].offset - partition_info[2].offset;
#endif
				partition_info[3].size = WINDOW_SIZE - partition_info[3].offset;
			}

			if (board_flash_size == 0x2000000)
				{
				// partition for all first flash bank
				partition_info[4].offset = 0;
				partition_info[4].size = WINDOW_SIZE/2;
				strncpy(partition_info[4].name, "Extended Firmwares",20);

				/* Register partition table */
				add_mtd_partitions(mymtd, partition_info, 5);
				}
			else
				{
				/* Register partition table */
				add_mtd_partitions(mymtd, partition_info, 4);
				}
		}
		break;

		case BSP_Firmware_type_rescue:
		{
			/* Partition name */
			{
				strncpy(partition_info[3].name, "Rescue FW",20);
				strncpy(partition_info[4].name, "Rescue SquashFS",20);
			}

			/* Set partition struct */
			{
				partition_info[3].offset = (pKrnopt->rescue_start & 0x1FFFFFFF) - WINDOW_ADDR;
				partition_info[4].offset = (pKrnopt->myfs_start & 0x1FFFFFFF) - WINDOW_ADDR;
				partition_info[4].size = FLASH_BLOCK_SIZE*((pKrnopt->myfs_size+FLASH_BLOCK_SIZE-1)/FLASH_BLOCK_SIZE);
/*
				printk(	"1=%08x 2=%08x 3=%08x 4=%08x 5=%08x\n",
						pKrnopt->myfs_size+FLASH_BLOCK_SIZE,
						pKrnopt->myfs_size+FLASH_BLOCK_SIZE-1,
						(pKrnopt->myfs_size+FLASH_BLOCK_SIZE-1)/FLASH_BLOCK_SIZE,
						FLASH_BLOCK_SIZE*((pKrnopt->myfs_size+FLASH_BLOCK_SIZE-1)/FLASH_BLOCK_SIZE),
						partition_info[4].size);
*/
				partition_info[5].offset = partition_info[4].offset + partition_info[4].size;
#ifdef CONFIG_MTD_SAGEM_MINI_PARTITIONS
				partition_info[2].size = partition_info[3].offset - partition_info[2].offset;
#endif
				partition_info[3].size = partition_info[4].offset - partition_info[3].offset;
				partition_info[5].size = WINDOW_SIZE - partition_info[5].offset;
			}

			if (board_flash_size == 0x2000000)
				{
				// partition for all first flash bank
				partition_info[6].offset = 0;
				partition_info[6].size = WINDOW_SIZE/2;
				strncpy(partition_info[5].name, "Unused",20);
				strncpy(partition_info[6].name, "Operational FW",20);

				/* Register partition table */
#ifdef CONFIG_MTD_SAGEM_MINI_PARTITIONS
				add_mtd_partitions(mymtd, partition_info, 7);
#else
				add_mtd_partitions(mymtd, partition_info, 10);
#endif
				}
			else
				{
				strncpy(partition_info[5].name, "Operational FW",20);

				/* Register partition table */
#ifdef CONFIG_MTD_SAGEM_MINI_PARTITIONS
				add_mtd_partitions(mymtd, partition_info, 6);
#else
				add_mtd_partitions(mymtd, partition_info, 10);
#endif
				}
		}
		break;

		default:
		{
			panic("Bad type firmware");
		}
		break;
	    }

	    return 0;
	}

	iounmap((void *)mb_map.virt);
	return -ENXIO;
}

static void __exit cleanup_mb(void)
{
	if (mymtd) {
		del_mtd_device(mymtd);
		map_destroy(mymtd);
	}
	if (mb_map.virt) {
		iounmap((void *)mb_map.virt);
		mb_map.virt = 0;
	}
}

module_init(init_mb);
module_exit(cleanup_mb);

MODULE_AUTHOR("Leo @ Analog Devices");
MODULE_DESCRIPTION("MTD map driver ADI Fusiv board");
MODULE_LICENSE("GPL");
