// ===========================================================================
//
// Copyright (C) 2002 Analog Devices Inc. All rights reserved
//
// The information and source code contained herein is the exclusive property
// of Analog Devices and may not be disclosed, examined or reproduced in whole
// or in part without the explicit written authorization from Analog Devices.
//
// ===========================================================================

// ===========================================================================
//
// File name    : $RCSfile: apconf.h
// Author       : $Author: sreeni
// Version      : $Revision:
// Description  : Modules configuration to build AP binaries
// ===========================================================================

#ifndef _APCONF_H_
#define _APCONF_H_

#include <sysconf.h>

#if IS_VX185
#include <apautoconf_185.h>
#elif IS_VX180
#include <apautoconf_180.h>
#elif IS_VX160
#include <apautoconf_160.h>
#else
#include <apautoconf_200.h>
#endif

#define   PORT_MONITORING       0
#define   UART2                 0

#if CONFIG_FUSIV_KERNEL_PERI_AP || CONFIG_FUSIV_KERNEL_PERI_AP_MODULE
/* Following Macro is to enable support in host code for ETH-WLAN Hdr
*  conversion at AP level.
*  It's mandatory that WLAN_ETH_8023 macro in inc/netpro/apconf.h must be
*  defined to have this support at AP level */
#define WLAN_FASTPATH	1
#ifdef WLAN_ETH_8023
#define ETH_WLAN_HDR_CONVERT 1
#endif
#endif

/* To Enable IPCQ priority mechanism */
#if IS_VX180
#define IPCQOS	                0
#else
#define IPCQOS		        0
#endif

/* To Enable H/W tagging mechanism of incoming packets */
#if IS_VX180
#define PACKET_TAGGING 		0
#else
#define PACKET_TAGGING 		0
#endif
#define FASTPATH_FOR_FRAGMENTS  0

#if VENDOR_TAG
/* NOTE: Either REALTEK or MARVELL should be enabled but not together
*/
#define   MARVELL             0
#define   REALTEK             1

#if MARVELL && REALTEK
  Only one type of switch can be enabled.
#endif
#endif

// IPv4 into IPv6 can not be without IPv6 code
#if IPV6
#if IS_VX185
    #define   	IPV4_INTO_IPV6        		1	// This is maintained for vx180 compatibility, which
							// still refers to this flag. This should be 
							// removed later and the following 4 flags should
							// be used.
    #define	IPV4_TO_IPV6_TUNNELING		1
    #define	IPV4_TO_IPV6_TRANSLATION	1
    #define	IPV6_TO_IPV4_TUNNELING		1
    #define	IPV6_TO_IPV4_TRANSLATION	1
#else
/* Tunnel code: include by linux kernel option
   Translation: include by following Macros.
  IPV4_TO_IPV6_TRANSLATION
  IPV6_TO_IPV4_TRANSLATION
*/

#ifdef CONFIG_IPV6_TUNNEL_MODULE
    #define	IPV4_TO_IPV6_TUNNELING		1
#else
    #define	IPV4_TO_IPV6_TUNNELING		0
#endif
    #define	IPV4_TO_IPV6_TRANSLATION	1

#ifdef CONFIG_IPV6_SIT_MODULE 
    #define	IPV6_TO_IPV4_TUNNELING		1
#else
    #define	IPV6_TO_IPV4_TUNNELING		0
#endif
    #define	IPV6_TO_IPV4_TRANSLATION	1

#if (IPV4_TO_IPV6_TUNNELING) || (IPV4_TO_IPV6_TRANSLATION)
    #define   	IPV4_INTO_IPV6        		1	// This is maintained for vx180 AP code, which
							// still refers to this flag. This should be 
							// removed later and the following 4 flags should
							// be used.
#else
    #define   	IPV4_INTO_IPV6        		0	
#endif //(IPV4_TO_IPV6_TUNNELING) ||(IPV4_TO_IPV6_TRANSLATION)

#endif //IS_VX185
#endif //IPV6

#if !(IS_VX180 || IS_VX185)
#define   PERI_INQ2_ADDR        0x191A0610
#endif

// Protection from uncompatible combinations
// This should be at the end of file

#if IPV4_INTO_IPV6 && UART2
 Cannot coexist. .sbss+.sdata sections are too long
#endif

#if !(IS_VX180 || IS_VX185)
#if IPV4_INTO_IPV6 && VENDOR_TAG
 Flow entry address calculation is not correct for this combination
#endif

#if IPV6 &&  WLAN_ETH_8023
// Cannot coexist. pmem is in shortage for both options to be enabled.
#endif

#if POLICY_BASED_VLAN && VENDOR_TAG
 Cannot coexist .pmem and .dmem is in shortage for both options to be enabled.
#endif
#endif

#if IS_VX185
/* As such, not code is under this macro (except static_RFT), 
 * disabling this causes nothing; But still make sure that
 * if you disable at toolkit code; disable here also.
 * 16th Dec 2010.
 */
#define	MESH_NETWORK 	1

// This flag compiles in the software based IPQoS PQ/DWPFQ scheduler implementation
// for ATM interface
#define DWRR_SOFT_IPQOS_SCHEDULER       1

// This flag enables enqueueing at the head of the IPQoS queues. This is used by the
// egress AP for enqueuing high priority packets such as TCP ack, multicast, etc at
// at the head of the queue so that those packets are not dropped even if queue is 
// full and get prioritized over other packets in the queue
#define DWRR_HOQ_ENQUEUE_ENABLE		1
#endif

#endif /* _APCONF_H_ */
