/* Copyright (C) 2006 Ikanos Communications
 * See IKANOS_PROP_LICENSE.txt for license information. */

#ifndef __IPQOS_WFQ_LIB_H
#define __IPQOS_WFQ_LIB_H


#define  IPQOS_DEFAULT          0
#define  IPQOS_LOW              1
#define  IPQOS_MEDIUM           2
#define  IPQOS_HIGH             3
#define  IPQOS_CRITICAL         4
#define  IPQOS_REAL_TIME        5
#define  IPQOS_PREMIUM          6
#define  IPQOS_URGENT           7

#define DEFAULT_QSIZE           8

/* Type of the traffic scheduler. */
#define DEFICIT_WT_PRIORITY		  1
#define SIMPLE_PRIORITY    		  2


#if __KERNEL__
typedef struct weighted_fair_queue
{
  struct weighted_fair_queue *next;
  struct weighted_fair_queue *prev;
#if CONFIG_FUSIV_KERNEL_IPQOS_TC_RATE_CONTROL
	/* TO Use Rate Control information from the TC module */
  struct sk_buff *grean;          /* Assured BW, green packet. */
  struct sk_buff *yellow;         /* Average packet, yellow    */
  struct sk_buff *red;            /* Over limit packet, red    */
#else
	/* TO use when there is no rate control defined. */
	struct sk_buff_head skb_head;   /* SKB Head for enqueue list */
#endif
  int bytes;                      /* Bytes in this queue       */
  int quota_avail;                /* Difference of max and min, this sets limit max quota available from borrowed  */
 // int weight;                     /* Weight in percent         */
  int minbw;                        /* Minimum/Guaranteed Bandwidth */
  int maxbw;                        /* Maximum Bandwidth */
  unsigned char priority;                   /* Priority of the queue     */
  int deficit_counter;            /* Deficit Counter           */
  int hwm;                        /* High Water Mark           */
  unsigned char borrow;           /* Borrow Flag.              */
	unsigned char contribute;       /* Contribute Flag.          */
	unsigned char ackPriority;      /* TCP Ack Priority Flag.    */
#if CONFIG_FUSIV_KERNEL_IPQOS_STATISTICS
  unsigned int sent_packets;      /* packets sent in this queue*/
  unsigned int drop_packets;      /* dropped packets           */
  unsigned long long sent_bytes;  /* bytes sent in this queue  */
  unsigned long long drop_bytes;  /* bytes sent in this queue  */
#endif

}wfq;
#endif

typedef struct
{
	unsigned char  schedType;      /* Type of scheduler.         */
	int            bandwidth;      /* Scheduler Bandwidth        */
	unsigned char  borrow;         /* Borrow Mask.               */              
  unsigned char  ackPriority;    /* Ack Priority Mask          */
  unsigned char  contribute;     /* Contribute Mask            */
	struct 
	{
//		char         weight;         /* Weight of the Q.           */
                int         minbw;           /* Minimum/Guaranteed Bandwidth */
                int         maxbw;           /* Maximum Bandwidth */
		unsigned int sent_packets;   /* packets sent in this queue */
		unsigned int drop_packets;   /* dropped packets */
		unsigned long long sent_bytes;  /* bytes sent in this queue */
		unsigned long long drop_bytes;  /* bytes sent in this queue */
	}PerQ[DEFAULT_QSIZE];
}wfq_config_t;

#endif

