/*
 * ============================================================================
 *
 * Copyright : (c) 2005 SAGEM SA - Site Velizy - URD2
 *
 * The information and source code contained herein is the exclusive
 * property of SAGEM and may not be disclosed, examined, or
 * reproduced in whole or in part without explicit written authorization
 * of the company.
 *
 * ============================================================================
 */

#ifndef __BSP_FLASH_H
#define __BSP_FLASH_H

/**
 * \file bsp_flash.h Bsp Flash format description.
 *
 * This file allow to describe Bsp Flash format.
*/


/*
 * Includes
 * ~~~~~~~~
 */

#ifdef __KERNEL__
#include <asm/types.h>
#else
#include <stdint.h>
#endif

#include <linux/bsp/bsp_firmware.h>
#include <linux/bsp/bsp_common.h>

/*
 * Defines
 * ~~~~~~~
 */

/** Flash Address Begin. */
#define BSP_FLASH_ADDR_BEGIN				0xBF000000

/** Flash Address End. */
#define BSP_FLASH_ADDR_END				0xBFFFFFFF

/** Flash Offset BootLoader. */
#define BSP_FLASH_OFFSET_BOOTLOADER		0

/** Flash Address BootLoader. */
#define BSP_FLASH_ADDR_BOOTLOADER			(BSP_FLASH_ADDR_BEGIN + BSP_FLASH_OFFSET_BOOTLOADER)

/** Flash Offset BootLoader. */
#define BSP_FLASH_OFFSET_PERMPARM			0x18000

/** Flash Address BootLoader. */
#define BSP_FLASH_ADDR_PERMPARM			(BSP_FLASH_ADDR_BEGIN + BSP_FLASH_OFFSET_PERMPARM)

/** Flash Offset Factory. */
#define BSP_FLASH_OFFSET_FACTORY			0x20000

/** Flash Address Factory. */
#define BSP_FLASH_ADDR_FACTORY			(BSP_FLASH_ADDR_BEGIN + BSP_FLASH_OFFSET_FACTORY)

/** Flash Offset User Partition. */
#define BSP_FLASH_OFFSET_USERPART			0x40000

/** Flash Address User Partition. */
#define BSP_FLASH_ADDR_USERPART			(BSP_FLASH_ADDR_BEGIN + BSP_FLASH_ADDR_USERPART)

/** Header Version. */
#define BSP_FIRM_HEADER_VERSION			0x03

/** Block. */
#define BSP_FIRM_BLOCK					4096

/** Erase block. */
#define BSP_FIRM_ERASE_BLOCK				0x20000

/** Exec ram address. */
#define BSP_FIRM_EXEC_RAM_ADDR			0x80800000

/** Magic sagem partition. */
#define BSP_FIRM_SAGEM_PARTITION_MAGIC		0x604c51ea

/*
 * Typedefs
 * ~~~~~~~~
 */

/** \name Enum Typedefs.
 */
/*@{*/
/** \link bsp_firmSubPart_enu bsp_firmSubPart enum \endlink typedef. */
typedef enum bsp_firmSubPart_enu			bsp_firmSubPart_e;
/** \link bsp_firware_enu bsp_firware enum \endlink typedef. */
typedef enum bsp_firmware_enu				bsp_firmware_e;
/*@}*/

/** \name bsp_dsaPubKey struct Typedefs.
 */
/*@{*/
/** \link bsp_dsaPubKey_str bsp_pp struct \endlink typedef. */
typedef struct bsp_dsaPubKey_str			bsp_dsaPubKey_s;
/** \link bsp_dsaPubKey_str bsp_pp struct \endlink pointer typedef. */
typedef struct bsp_dsaPubKey_str			*pBsp_dsaPubKey_s;
/*@}*/

/** \name bsp_ffDsa struct Typedefs.
 */
/*@{*/
/** \link bsp_ffDsa_str bsp_ffDsa struct \endlink typedef. */
typedef struct bsp_ffDsa_str				bsp_ffDsa_s;
/** \link bsp_ffDsa_str bsp_ffDsa struct \endlink pointer typedef. */
typedef struct bsp_ffDsa_str				*pBsp_ffDsa_s;
/*@}*/

/** \name bsp_subPart struct Typedefs.
 */
/*@{*/
/** \link bsp_subPart_str bsp_subPart struct \endlink typedef. */
typedef struct bsp_subPart_str			bsp_subPart_s;
/** \link bsp_subPart_str bsp_subPart struct \endlink pointer typedef. */
typedef struct bsp_subPart_str			*pBsp_subPart_s;
/*@}*/

/** \name bsp_partMetaData struct Typedefs.
 */
/*@{*/
/** \link bsp_partMetaData_str bsp_partMetaData struct \endlink typedef. */
typedef struct bsp_partMetaData_str		bsp_partMetaData_s;
/** \link bsp_partMetaData_str bsp_partMetaData struct \endlink pointer typedef. */
typedef struct bsp_partMetaData_str		*pBsp_partMetaData_s;
/*@}*/

/** \name bsp_dsaMetaData struct Typedefs.
 */
/*@{*/
/** \link bsp_dsaMetaData_str bsp_dsaMetaData struct \endlink typedef. */
typedef struct bsp_dsaMetaData_str			bsp_dsaMetaData_s;
/** \link bsp_dsaMetaData_str bsp_dsaMetaData struct \endlink pointer typedef. */
typedef struct bsp_dsaMetaData_str			*pBsp_dsaMetaData_s;
/*@}*/


/*
 * Enums
 * ~~~~~
 */

/**
 * \name Sub Partition type Enum.
 */
/*@{*/
enum bsp_firmware_subPart_enu {
	/** Boot args. */
	BSP_Firmware_subPart_boot_args,

	/** Kernel Args. */
	BSP_Firmware_subPart_kernel_args,

	/** Kernel. */
	BSP_Firmware_subPart_kernel,

	/** Root Fs. */
	BSP_Firmware_subPart_root_fs,

	/** Max. */
	BSP_Firmware_subPart_max
};

/*
 * Structs
 * ~~~~~~~
 */

/* Bsp_dsaPubKey_str.
 *
 *	This struct manage Dsa Public Key Parameters.
 */
struct bsp_dsaPubKey_str {
	/** P. */
	__u8						p[128];

	/** Q. */
	__u8						q[20];

	/** G. */
	__u8						g[128];

	/** Public Key. */
	__u8						pub_key[128];
};

/* Bsp_ffDsa_str.
 *
 *	This struct manage FF Dsa Parameters.
 */
struct bsp_ffDsa_str {
	/** R. */
	__u8						r[20];
	/** S. */
	__u8						s[20];
};

/* Bsp_subPart_str.
 *
 *	This struct manage Firmware Sub partition Parameters.
 */
struct bsp_subPart_str {
	/** Start. */
	__u32					start;

	/** Size. */
	__u32					size;

	/** reserved. */
	__u32					reserved;
};

/* Bsp_partitionMetaData_str.
 *
 *	This struct manage partition meta data Parameters.
 */
struct bsp_partMetaData_str {
	/** Version of the Header. Correspond to the CVS cryptdsa version. */
	__u16					formatVersion;

	/** Flags reserved for various purpose. Not used yet. */
	__u16					flags;

	/* Identifies the firmware type (operational, rescue, lan-rescue) */
  	bsp_firmware_type_e			firmwareType;

	/** String identifying the firmware type. */
	char						firmwareVersion[64];

	/** Total size of firmware. */
	__u32					size;

	/** Information about each subparts of the partition : start address, size, and a reserved field. */
	bsp_subPart_s				part[BSP_Firmware_subPart_max];
};

/* Bsp_dsaMetaData_str.
 *
 *	This struct manage dsa meta data Parameters.
 */
struct bsp_dsaMetaData_str {
	/** Magic number used to identify quickly the beginning of a new SAGEM partition. */
	__u32					magic;

	/** Signature of the SAGEM partition. The DSA algorythm is not adsaPubKeylied on "magic" and "signature" fields. */
	bsp_ffDsa_s				signature;

	/** Checksum used to check quickly the validity of the header
	 *	Calculated on fields from "formatVersion" to the end of "PartitionMetaData"
	 */
	__u32					crc32;
};

/* Bsp_permData_str.
 *
 *	This struct manage permanent data Parameters.
 */
struct bsp_permData_str {
	/** Version. */
	__u16					version;

	/** Mac address. */
	__u8						mac_addr[6];

	/** Dsa Public Kr. */
	bsp_dsaPubKey_s			kr;

	/** Dsa Public Kf. */
	bsp_dsaPubKey_s			kf;
};
  
#endif

// vim:ts=5:
