/*
 * ============================================================================
 *
 * Copyright : (c) 2009 SAGEM SA - Site Velizy - URD2
 *
 * The information and source code contained herein is the exclusive
 * property of SAGEM and may not be disclosed, examined, or
 * reproduced in whole or in part without explicit written authorization
 * of the company.
 *
 * ============================================================================
 */

#ifndef __BSP_LAN_H__
#define __BSP_LAN_H__

/**
 * \file bsp_lan.h Bsp description for manage LAN.
 *
 * This file allow to describe Bsp Lan Public.
*/

#ifdef __KERNEL__

/*
 * Includes
 * ~~~~~~~~
 */

#include <linux/bsp/bsp_gpio.h>

/*
 * Defines
 * ~~~~~~~~
 */

/** \name Lan.
 */
/*@{*/
/** MII MAX. */
#define	BSP_LAN_MII_MAX			2

/** PHY MAX. */
#define	BSP_LAN_PHY_MAX			BSP_LAN_MII_MAX

/** SWITCH MAX. */
#define	BSP_LAN_SWITCH_MAX			1

/** Lan Phy Handle led. */
#define	BSP_LAN_PHY_HANDLE_LEDS		(1 << 0)
/*@}*/

/*
 * Typedefs
 * ~~~~~~~~
 */

/** \name Enums.
 */
/*@{*/
/** \link bsp_lan_mii_dev_enu bsp_lan_mii_dev enum \endlink typedef. */
typedef enum bsp_lan_mii_dev_enu		bsp_lan_mii_dev_e;
/*@}*/

/*
 * Enums
 * ~~~~~
 */

/** Bsp_Lan_mii_dev_cnx. */
enum bsp_lan_mii_dev_enu {
	/** Connect to Phy. */
	BSP_Lan_mii_dev_phy,

	/** Connect to Switch. */
	BSP_Lan_mii_dev_switch,

	/** Max. */
	BSP_Lan_mii_dev_max,

	/** Unknown. */
	BSP_lan_mii_dev_unknown	= 0xff
};

/*
 * Typedefs
 * ~~~~~~~~
 */

/** \name bsp_phy_info struct Typedefs.
 */
/*@{*/
/** \link bsp_lan_phy_info_str bsp_lan_phy_info struct \endlink typedef. */
typedef struct bsp_lan_phy_info_str		bsp_lan_phy_info_s;
/** \link bsp_phy_info_str bsp_lan_phy_info struct \endlink pointer typedef. */
typedef struct bsp_lan_phy_info_str		*pBsp_lan_phy_info_s;
/*@}*/

/** \name bsp_lan_switch_info struct Typedefs.
 */
/*@{*/
/** \link bsp_lan_switch_info_str bsp_lan_switch_info struct \endlink typedef. */
typedef struct bsp_lan_switch_info_str		bsp_lan_switch_info_s;
/** \link bsp_lan_switch_info_str bsp_lan_switch_info struct \endlink pointer typedef. */
typedef struct bsp_lan_switch_info_str		*pBsp_lan_switch_info_s;
/*@}*/

/** \name bsp_board_switch struct Typedefs.
 */
/*@{*/
/** \link bsp_board_switch_str bsp_board_switch struct \endlink typedef. */
typedef struct bsp_board_switch_str		bsp_board_switch_s;
/** \link bsp_board_switch_str bsp_board_switch struct \endlink pointer typedef. */
typedef struct bsp_board_switch_str		*pBsp_board_switch_s;
/*@}*/

/** \link bsp_board_phy_str bsp_board_phy struct \endlink typedef. */
typedef struct bsp_board_phy_str			bsp_board_phy_s;
/** \link bsp_board_phy_str bsp_board_phy struct \endlink pointer typedef. */
typedef struct bsp_board_phy_str			*pBsp_board_phy_s;
/*@}*/

/** \name bsp_board_mii struct Typedefs.
 */
/*@{*/
/** \link bsp_board_mii_str bsp_board_mii struct \endlink typedef. */
typedef struct bsp_board_mii_str			bsp_board_mii_s;
/** \link bsp_board_mii_str bsp_board_mii struct \endlink pointer typedef. */
typedef struct bsp_board_mii_str			*pBsp_board_mii_s;
/*@}*/

/** \name bsp_board_lan struct Typedefs.
 */
/*@{*/
/** \link bsp_board_lan_str bsp_board_lan struct \endlink typedef. */
typedef struct bsp_board_lan_str			bsp_board_lan_s;
/** \link bsp_board_lan_str bsp_board_lan struct \endlink pointer typedef. */
typedef struct bsp_board_lan_str			*pBsp_board_lan_s;
/*@}*/

/*
 * Structs
 * ~~~~~~~
 */

/** bsp_lan_phy_info.
 *
 * 	Struct manage Phy composant.
 */
struct bsp_lan_phy_info_str {
	/* Flags. */
	unsigned char					flags;
};

/** bsp_switch_info.
 *
 * 	Struct manage Switch composant.
 */
struct bsp_lan_switch_info_str {
	/** Mii Number. */
	unsigned char					miiNum;
};

/** Bsp_board_mii_str.
 *
 * 	This struct manage board Mii Information.
 */
struct bsp_board_mii_str {
	/** Lan Mii Connect. */
	bsp_lan_mii_dev_e				dev;

	union {
		/* Switch Info. */
		bsp_lan_switch_info_s		sw;

		/* Phy Info. */
		bsp_lan_phy_info_s			phy;
	}Info;

	/** Smi Addr.*/
	unsigned char					smiAddr;
};

/** Bsp_switch_str.
 *
 * 	This struct manage Switch Information.
 */
struct bsp_board_switch_str {
	/* Nb. */
	int							nb;
};

/** Bsp_phy_str.
 *
 * 	This struct manage Phy Information.
 */
struct bsp_board_phy_str {
	/* Nb. */
	int							nb;
};

/** Bsp_lan_str.
 *
 * 	This struct manage Eth Information.
 */
struct bsp_board_lan_str {
	/** Mii Info. */
	bsp_board_mii_s				Mii[BSP_LAN_MII_MAX];

	/* Struct lan dev. */
	struct {
		/* Switch Info. */
		bsp_board_switch_s			Switch;
	
		/* Phy Info. */
		bsp_board_phy_s			Phy;
	}MiiDev;

	/** All Name. */
	bsp_gpio_name_e				all[BSP_LAN_PHY_MAX + BSP_LAN_SWITCH_MAX + 1];
};

/*
 * Functions
 * ~~~~~~~~~
 */
/** Get lan cnx. */
pBsp_board_lan_s	bsp_get_board_lan(	void);

/* Reset lan */
int bsp_reset_gpio_lan(				void);

#endif

#endif

// vim:ts=5:
