/* ============================================================================
 *
 * Copyright : (c) 2005 SAGEM SA - Site Velizy - URD2
 *
 * The information and source code contained herein is the exclusive
 * property of SAGEM and may not be disclosed, examined, or
 * reproduced in whole or in part without explicit written authorization
 * of the company.
 *
 * ============================================================================
 */
#ifndef __BSP_RELAY_H__
#define __BSP_RELAY_H__

/**
 * \file bsp_relay.h Bsp Relay description.
 *
 * This file allow to describe Bsp Relay.
 */

/*
 * Includes
 * ~~~~~~~~
 */

#include <linux/rwsem.h>
#include <linux/bsp/bsp_notify.h>

/*
 * Defines
 * ~~~~~~~
 */

/** \name Usb Hub Max.
 */
/*@{*/
/** Relay Fxs MAX. */
#define BSP_RELAY_FXS_MAX				2

/** Relay Fxo MAX. */
#define BSP_RELAY_FXO_MAX				1

/** Relay mode max size. */
#define BSP_RELAY_MODE_NAME_MAX_SIZE		20
/*@}*/

/** \name Maps.
 */
/*@{*/
/** Map Relay On FXS 1 */
#define BSP_MAP_RELAY_ON_FXS_1			(1 << 0)
/** Map Relay On Fxs 2 */
#define BSP_MAP_RELAY_ON_FXS_2			(1 << 1)
/** Map Relay On Fxo 1 */
#define BSP_MAP_RELAY_ON_FX0_1			(1 << 0)
/*@}*/

/*
 * Typedefs
 * ~~~~~~~~
 */

/** \name bsp_board_relay struct Typedefs.
 */
/*@{*/
/** \link bsp_board_relay_str bsp_board_relay struct \endlink typedef. */
typedef struct bsp_board_relay_str			bsp_board_relay_s;
/** \link bsp_board_relay_str bsp_board_relay struct \endlink pointer typedef. */
typedef struct bsp_board_relay_str			*pBsp_board_relay_s;
/*@}*/

/** \name bsp_env_relay struct Typedefs.
 */
/*@{*/
/** \link bsp_env_relay_str bsp_env_relay struct \endlink typedef. */
typedef struct bsp_env_relay_str			bsp_env_relay_s;
/** \link bsp_env_relay_str bsp_env_relay struct \endlink pointer typedef. */
typedef struct bsp_env_relay_str			*pBsp_env_relay_s;
/*@}*/

/**
 * \name Enum typedefs.
 */
/*@{*/
/** Typedef on enum bsp_relay_mode_enu. */
typedef enum bsp_relay_mode_enu			bsp_relay_mode_e;
/** Typedef on enum bsp_relay_flags_enu. */
typedef enum bsp_relay_flags_enu			bsp_relay_flags_e;
/*@}*/


typedef enum
{
	kRelay1,
	kRelay2,
	kRelayEnd,
} BOARD_RELAY_NAME;

typedef enum
{
	kRelayOn, /* turn relay on */
	kRelayOff, /* turn relay off */
} BOARD_RELAY_STATE;

/*
 * Enums
 * ~~~~~
 */

/** Flags. */
enum bsp_relay_mode_enu {
	/** None. */
	BSP_Relay_mode_none,

	/** Adsl. */
	BSP_Relay_mode_adsl,

	/** Ftth. */
	BSP_Relay_mode_ftth,

	/** End. */
	BSP_Relay_mode_end,
};

/** Flags. */
enum bsp_relay_flags_enu {
	/** Adsl link up. */
	BSP_Relay_flags_adsl_link_up		= (1 << 0),

	/** Voice module up. */
	BSP_Relay_flags_voice_module_up	= (1 << 1),

	/** Mode ftth. */
	BSP_Relay_flags_mode_ftth		= (1 << 2),

	/** Mode adsl. */
	BSP_Relay_flags_mode_adsl		= (1 << 3),

	/** End. */
	BSP_Relay_flags_end,
};

/*
 * Structs
 * ~~~~~~~
 */

/** Bsp_board_relay_str.
 *
 * 	This struct manage Board Relay Information.
 */
struct bsp_board_relay_str {
	/** Struct manage Fxs. */
	struct {
		/* Nb. */
		int							nb;

		/* Map. */
		int							map;

		/** All Name. */
		bsp_gpio_name_e				all[BSP_RELAY_FXS_MAX + 1];
	}Fxs;

	/** Struct manage Fxo. */
	struct {
		/* Nb. */
		int							nb;

		/* Map. */
		int							map;

		/** All Name. */
		bsp_gpio_name_e				all[BSP_RELAY_FXO_MAX + 1];
	}Fxo;
};

/** Bsp_env_relay_str.
 *
 * 	This struct manage Environment Relay Information.
 */
struct bsp_env_relay_str {
	/** Struct manage mode. */
	struct {
		/** All */
		struct {
			/** Name. */
			char						*name;

			/* Motify. */
			bsp_notify_evt_e			notify;
		}All[BSP_Relay_mode_end];

		/** Cur */
		struct {
			/** Name. */
			char						name[BSP_RELAY_MODE_NAME_MAX_SIZE];
		}Cur;
	
		/** Read/write semaphore. */
		struct rw_semaphore				sem;
	}Mode;

	/** Flags. */
	unsigned char						flags;
};

/*
 * Functions
 * ~~~~~~~~~
 */
/** Get relay. */
pBsp_board_relay_s	bsp_get_board_relay(			void);

/** Relay control. */
BOARD_RELAY_STATE kSysRelayCtrl(	BOARD_RELAY_NAME	_relay,
							BOARD_RELAY_STATE	_state);

/** Get Relay State. */
BOARD_RELAY_STATE kSysGetRelayState(BOARD_RELAY_NAME	_relay);

#endif
