/******************************************************************************/
/*                                                                            */
/* Copyright : (c) 2006 SAGEM SA - Site Velizy                                */
/*                                                                            */
/* MODULE    : dect                                                           */
/* FILE NAME : dect-api-g.h                                                   */
/*                                                                            */
/* SUMMARY   : auto configuration generic actions                             */
/*                                                                            */
/* USAGE     :                                                                */
/*                                                                            */
/*                                                                            */
/* ORIGIN    : 22/02/2006                                                     */
/* REVISIONS :      								*/
/*										*/
/*	date    : 11/04/2006							*/							
/*	Thibault Gueslin Thomson						*/
/*										*/
/*	Date	: 14/03/2006							*/
/*	Remarks : Changed Receive voice method (callback replaces polling	*/
/*										*/
/*	Number	: 1.1								*/
/*	Date	: 08/03/2006							*/
/*	Remarks : Fixed to meet reference (ReceiveVoiceData)			*/
/*										*/
/*    Number       : 1.0                                                      */
/*    Date         : 22/02/2006                                               */
/*    Remarcs      : Creat                                                    */
/*                                                                            */
/******************************************************************************/
/******************************************************************************/


#ifndef _DECT_API_G
#define _DECT_API_G

/* VHo
 *
 * Description of the DECT API-G
 *
 * Ref: API Generalisee Livebox pour le projet dongle DECT v.1.8
 *
 */

#include <linux/types.h>

/* Type definitions */
#define DECT_RING_PATT_T dect_RingPatt_t



#define DECT_BOOL_T __u8
#define DECT_UINT8 __u8
#define DECT_UINT16 __u16
#define DECT_FUNC_STRUCT dect_func_struct_t

typedef struct DECT_TIME_T {
	DECT_UINT16 year;
	DECT_UINT16 month;
	DECT_UINT16 day;
	DECT_UINT16 hour;
	DECT_UINT16 minute;
} DECT_TIME_T ;

typedef struct dect_RingPatt {
	/* TO BE DEFINED BY THOMSON */
	DECT_UINT8 *private;
} dect_RingPatt_t;

#define DECT_BOOL_TRUE	1
#define DECT_BOOL_FALSE	0

/* Constant definitions */
#define DECT_EVENT_INS_BASE	1
#define DECT_EVENT_REM_BASE	2

#define DECT_VENDOR_SAGEM	0
#define DECT_VENDOR_THOMSON	1

#define DECT_HANDSET_ALL	0xff
#define DECT_HANDSET_0		0x01
#define DECT_HANDSET_1		0x02
#define DECT_HANDSET_2		0x04
#define DECT_HANDSET_3		0x08
#define DECT_HANDSET_4		0x10
#define DECT_HANDSET_5		0x20
#define DECT_HANDSET_6		0x40
#define DECT_HANDSET_7		0x80

#define DECT_HANDSET_REGISTERED		1
#define DECT_HANDSET_NOT_REGISTERED	0

#define DECT_RING_ID_0	0
#define DECT_RING_ID_1	1
#define DECT_RING_ID_2	2
#define DECT_RING_ID_3	3
#define DECT_RING_ID_4	4
#define DECT_RING_ID_5	5
#define DECT_RING_ID_6	6
#define DECT_RING_ID_7	7
#define DECT_RING_ID_8	8
#define DECT_RING_ID_9	9

#define DECT_LINE_ID_0	0
#define DECT_LINE_ID_1	1

#define DECT_REASON_PRIVATE	1
#define DECT_REASON_UNAVAIL	2
#define DECT_REASON_VOID	0


/* OTHER_CODECS_SUPPORTED (from HCI documentation)*/
#define DECT_SUPPORTED_CODEC_G726   0x01
#define DECT_SUPPORTED_CODEC_G711A  0x02
#define DECT_SUPPORTED_CODEC_G722   0x04
#define DECT_SUPPORTED_CODEC_G711U  0x08

#define DECT_CODEC_G711		0x01
#define DECT_CODEC_G722_64K	0x02
#define DECT_CODEC_G711U	0x04


/* Error code definitions */
#define DECT_ERR_OK		0
#define DECT_ERR_KO		1
#define DECT_ERR_NO_BASE	2
#define DECT_ERR_WG_HANDSET	3
#define DECT_ERR_WG_RING_PAT	4
#define DECT_ERR_WG_LINE	5
#define DECT_ERR_WG_CODEC	6
#define DECT_ERR_NOT_SUPPORT	7

#define DECT_REMOTE_RELEASE 0
#define DECT_FXS_OFFHOOK    1

/* Direction 2 : UPPER to LOWER */
typedef struct dect_func_struct {
	DECT_UINT8 (*InitDriver) (DECT_UINT8 lines);
	DECT_UINT8 (*ResetBase) (void);
	DECT_UINT8 (*GetVersion) (DECT_UINT16 *version, DECT_UINT8 *idVendor);
	DECT_UINT8 (*GetListRegisteredHandset) (DECT_UINT8 *listHandset);
	DECT_UINT8 (*GetHandsetStatus) (DECT_UINT8 handset, DECT_UINT8 *statusHandset, DECT_UINT8 *idVendor);
	DECT_UINT8 (*OpenRegistrationWindow) (void);
	DECT_UINT8 (*CloseRegistrationWindow) (void);
	DECT_UINT8 (*RemoveRegisteredHandset) (DECT_UINT8 handset);
	DECT_UINT8 (*StartHandsetResearch) (void);
	DECT_UINT8 (*StopHandsetResearch) (void);
	DECT_UINT8 (*SetRingPattern) (DECT_UINT8 ringID, DECT_RING_PATT_T *ringParameter);
	DECT_UINT8 (*SendClipInfo) (DECT_UINT8 handset, DECT_TIME_T date, DECT_BOOL_T mwi_on, 
			DECT_UINT8 numberLength, DECT_UINT8 reason, DECT_UINT8 *number, DECT_UINT8 nameLength, DECT_UINT8 *name);
	DECT_UINT8 (*IncomingCall) (DECT_UINT8 handset, DECT_UINT8 ringPattern);
	DECT_UINT8 (*ReleaseCall) (DECT_UINT8 handset,DECT_UINT8 ReleaseReason);
	DECT_UINT8 (*InitVoiceOption) (DECT_UINT8 handset, DECT_UINT8 codecType);
	DECT_UINT8 (*TransmitVoiceData) (DECT_UINT8 handset, DECT_UINT8 lengthData, DECT_UINT8 *dataBuffer);
	DECT_UINT8 (*SetWideBandIcon) (DECT_UINT8 handset, DECT_BOOL_T iconState);
} dect_func_struct_t;

/* Direction 1: LOWER to UPPER / Registration */
DECT_UINT8 dect_Register(DECT_UINT8 eventId, DECT_FUNC_STRUCT *func);

/* Direction 1: LOWER to UPPER / Management §3.2 */
DECT_UINT8 dect_IndicRegistrationHandset(DECT_UINT8 handset);
DECT_UINT8 dect_IndicUnRegistrationHandset(DECT_UINT8 handset);

/* Direction 1: LOWER to UPPER / Call §3.4 */
DECT_UINT8 dect_IndicOffHookHandset(DECT_UINT8 handset, DECT_UINT8 codecType);
DECT_UINT8 dect_IndicOnHookHandset(DECT_UINT8 handset);
DECT_UINT8 dect_IndicDtmfHandset(DECT_UINT8 handset, DECT_UINT8 dtmf);

DECT_UINT8 dect_ReceiveVoiceData(DECT_UINT8 handset, DECT_UINT8 lengthData, DECT_UINT8 *dataBuffer);



#endif 
