#ifndef _SAH_PROFILER_H_
#define _SAH_PROFILER_H_

#include <linux/types.h>
#include <linux/jiffies.h>
#include <asm/io.h>
#include <linux/interrupt.h>

typedef __u32 sah_prf_id_t;

#define SAH_PRF_INVERVALS 100			/* Number of seconds to buffer. */
#define SAH_PRF_MAXIDS 128			/* Max number of id registrations. */
#define SAH_PRF_ID_INVALID 0xffffffff

#if CONFIG_SAH_KERNEL_PROFILER
extern u32 sah_prf_run;
extern u32 sah_prf_in_ksoftirqd;
extern sah_prf_id_t sah_prf_ksoftirqd_id;

/* Register. Returns unique timer ID or SAH_PRF_ID_INVALID. */
extern sah_prf_id_t sah_prf_register(const char *name);

/* Log start of timed section. */
extern void __sah_prf_enter(sah_prf_id_t id);
static inline void sah_prf_enter(sah_prf_id_t id)
{
        if (unlikely(sah_prf_run))
                __sah_prf_enter(id);
}

/* Log end of timed section. */
extern void __sah_prf_leave(sah_prf_id_t id);
static inline void sah_prf_leave(sah_prf_id_t id)
{
        if (unlikely(sah_prf_run))
                __sah_prf_leave(id);
}

static inline void sah_prf_enter_ksoftirqd(void)
{
	sah_prf_in_ksoftirqd = 1;
	sah_prf_enter(sah_prf_ksoftirqd_id);
}

static inline void sah_prf_leave_ksoftirqd(void)
{
	sah_prf_in_ksoftirqd = 0;
	sah_prf_leave(sah_prf_ksoftirqd_id);
}

#define DECLARE_PRF_TASKLET_WRAPPER(name, func) \
static sah_prf_id_t sah_prf_##name##_tasklet_id = SAH_PRF_ID_INVALID; \
static sah_prf_id_t sah_prf_##name##_tasklet_id_int = SAH_PRF_ID_INVALID; \
static void sah_prf_##name##tasklet_func(unsigned long __sah_prf_data) \
{ \
	if (unlikely(sah_prf_##name##_tasklet_id == SAH_PRF_ID_INVALID)) { \
		sah_prf_##name##_tasklet_id = sah_prf_register("" #name ""); \
		sah_prf_##name##_tasklet_id_int = sah_prf_register("" #name "_int"); \
	} \
	sah_prf_enter(sah_prf_in_ksoftirqd ? sah_prf_##name##_tasklet_id : sah_prf_##name##_tasklet_id_int); \
	func(__sah_prf_data); \
	sah_prf_leave(sah_prf_in_ksoftirqd ? sah_prf_##name##_tasklet_id : sah_prf_##name##_tasklet_id_int); \
}

#define DECLARE_PRF_TASKLET(name, func, data) \
DECLARE_PRF_TASKLET_WRAPPER(name, func) \
DECLARE_TASKLET(name, sah_prf_##name##tasklet_func, data)

#define SAH_PRF_TASKLET_INIT(t, func, data) \
tasklet_init((t), sah_prf_##func##tasklet_func, (data))

#else /* CONFIG_SAH_KERNEL_PROFILER */
static inline sah_prf_id_t sah_prf_register(const char *name)
{
	return (sah_prf_id_t)SAH_PRF_ID_INVALID;
}

static inline void sah_prf_enter(sah_prf_id_t id)
{
}

static inline void sah_prf_leave(sah_prf_id_t id)
{
}

static inline void sah_prf_enter_ksoftirqd(void)
{
}

static inline void sah_prf_leave_ksoftirqd(void)
{
}

#define DECLARE_PRF_TASKLET_WRAPPER(name, func)

#define DECLARE_PRF_TASKLET(name, func, data) \
DECLARE_TASKLET(name, func, data)

#define SAH_PRF_TASKLET_INIT(t, func, data) \
tasklet_init((t), (func), (data))

#endif /* CONFIG_SAH_KERNEL_PROFILER */

#endif
