
#include <sys/types.h>
#include <sys/param.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>

#include <err.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <ctype.h>
#include <fcntl.h>

#include <pcb/pcb_client.h>
#include <pcb/core.h>
#include <pcb/utils.h>
#include <pcb/common.h>
connection_list_t connections;

#include <dnsres/pcb_dnsres.h>
#include "resolv.h"     
#include <arpa/inet.h>


int callback_expected = 0;
int callback_called = 0;

#ifndef   NI_MAXHOST
#define   NI_MAXHOST 1025
#endif

void TestCaseGetHostByAddr_Cb(struct dnsres_hostent *he, int res, void *arg) {
        callback_called++;

        SAH_TRACE_INFO("Callback called for %s", (char *)arg);
        if(he==NULL) {
          switch(res) {
          case DNSRES_HOST_NOT_FOUND:
            SAH_TRACE_INFO("DNSRES_HOST_NOT_FOUND");
            break;
          case DNSRES_TRY_AGAIN:
            SAH_TRACE_INFO("DNSRES_TRY_AGAIN");
            break;
          case DNSRES_NO_RECOVERY:
            SAH_TRACE_INFO("DNSRES_NO_RECOVERY");
            break;
          case DNSRES_NO_DATA:
            SAH_TRACE_INFO("DNSRES_NO_DATA");
            break;
          case DNSRES_NETDB_INTERNAL:
            SAH_TRACE_INFO("DNSRES_NETDB_INTERNAL");
            break;
          case DNSRES_NETDB_SUCCESS:
            SAH_TRACE_INFO("DNSRES_NETDB_SUCCESS");
            break;
          default:
            SAH_TRACE_INFO("default");
            break;
          }
      } else {
          SAH_TRACE_INFO("\tGot: %s -> %s", (char *)arg, he->h_name);
      }

}

void TestCaseGetHostByAddr(connection_info_t *con, char *address, int family) {
        SAH_TRACE_INFO("Running TestCaseGetHostByAddr");
        //initialize test context
        callback_called = 0;
        callback_expected = 1;

        //call the function
        struct in_addr a;
        inet_aton(address, &a);
        pcb_dnsres_gethostbyaddr(con,0,NULL,&a,sizeof(a), family,TestCaseGetHostByAddr_Cb,address);

        //simulation of pcb event loop
        int fdSet = 0;
        while (1) {
            /* handle connection events */
            struct timeval timeout;
            timerclear(&timeout);
            timeout.tv_sec = 5;
            fdSet = connection_waitForEvents(&connections, &timeout);
            if (fdSet < 0) {
                SAH_TRACE_FATAL("select failed: %m");
                continue;
            }
            connection_handleEvents(&connections);
            if(callback_called == callback_expected) break;
        }
}

int main(int argc, char **argv) {
    // initialize tracing and logging
    sahTraceOpen("Example client",TRACE_TYPE_STDOUT);
    sahTraceSetLevel(500);

    connection_initLibrary();

    if(argc<2) {
        SAH_TRACE_ERROR("usage: %s [address]",argv[0]);
        goto error;
    }

    connection_info_t *con = connection_create("DNSRES");
    if(con==NULL) {
        SAH_TRACE_ERROR("unable to initialize connection");
        goto error;
    }

    if (!llist_initialize(&connections)) {
        SAH_TRACE_ERROR("unable to initialize connection list %s", error_string(pcb_error));
        goto error;
    }
    llist_append(&connections, connection_getIterator(con));


    TestCaseGetHostByAddr(con,argv[1],AF_INET); // currently only ipv4 is supported

    connection_destroy(con);
    connection_exitLibrary();
    sahTraceClose();
    return 0;


  error:
    connection_exitLibrary();
    sahTraceClose();
    return -1;
}

