#include <sys/types.h>
#include <sys/param.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>

#include <err.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <ctype.h>
#include <fcntl.h>

#include <pcb/pcb_client.h>
#include <pcb/core.h>
#include <pcb/utils.h>
#include <pcb/common.h>
connection_list_t connections;

#include "pcb_dnsres.h"
#include "resolv.h"
#include <arpa/inet.h>

int callback_expected = 0;
int callback_called = 0;

#ifndef   NI_MAXHOST
#define   NI_MAXHOST 1025
#endif


static pcb_timer_t *cTimer = NULL;


/******************************************** COMMON ROUTINES **************************************************/
void *get_in_addr(struct sockaddr *sa) {
    if (sa->sa_family == AF_INET) {
        return &(((struct sockaddr_in*)sa)->sin_addr);
    }

    return &(((struct sockaddr_in6*)sa)->sin6_addr);
}


static void TestCaseCancel_TimerCb(pcb_timer_t *timer, void *userdata) {
    pcb_dnsres_cancelRequest((pcb_dnsres_t *)userdata);
    callback_called = 1;
    SAH_TRACE_INFO("====================================== CANCEL REQUEST ===============================");
}

void PrintAddrInfo(struct addrinfo *ai) {
    struct addrinfo *res;
    char buf[100];
    int n;

    SAH_TRACE_INFO("Addr info struct:");

    memset(buf, 0, sizeof(buf));
    n = 0;
    for (res = ai; res != NULL; res = res->ai_next) {
      SAH_TRACE_INFO(" item %d:",n++);
      SAH_TRACE_INFO("   ai_flags     = 0x%02X",res->ai_flags);
      SAH_TRACE_INFO("   ai_family    = %d (PF_INET = %d, PF_INET6 = %d)",res->ai_family,PF_INET,PF_INET6);
      SAH_TRACE_INFO("   ai_socktype  = %d (SOCK_STREAM = %d, SOCK_DGRAM = %d)",res->ai_socktype,SOCK_STREAM,SOCK_DGRAM);
      SAH_TRACE_INFO("   ai_protocol  = %d (IPPROTO_TCP = %d, IPPROTO_UDP = %d)",res->ai_protocol,IPPROTO_TCP,IPPROTO_UDP);
      SAH_TRACE_INFO("   ai_addrlen   = %d (sockaddr_in = %d, sockaddr_in6 = %d)",res->ai_addrlen,sizeof( struct sockaddr_in ),sizeof( struct sockaddr_in6 ));
      SAH_TRACE_INFO("   ai_canonname = %s",res->ai_canonname);
      SAH_TRACE_INFO("     ai_addr: got packet from %s",inet_ntop(res->ai_family,get_in_addr(res->ai_addr),buf, sizeof(buf)));
    }

}

void PrintAddrInfoExt(struct addrinfo_ext *ai) {
    struct addrinfo_ext *res;
    char buf[100];
    int n;

    SAH_TRACE_INFO("Addr info struct:");

    memset(buf, 0, sizeof(buf));
    n = 0;
    for (res = ai; res != NULL; res = res->ai_next) {
      SAH_TRACE_INFO(" item %d:",n++);
      SAH_TRACE_INFO("   ai_flags     = 0x%02X",res->ai_flags);
      SAH_TRACE_INFO("   ai_family    = %d (PF_INET = %d, PF_INET6 = %d)",res->ai_family,PF_INET,PF_INET6);
      SAH_TRACE_INFO("   ai_socktype  = %d (SOCK_STREAM = %d, SOCK_DGRAM = %d)",res->ai_socktype,SOCK_STREAM,SOCK_DGRAM);
      SAH_TRACE_INFO("   ai_protocol  = %d (IPPROTO_TCP = %d, IPPROTO_UDP = %d)",res->ai_protocol,IPPROTO_TCP,IPPROTO_UDP);
      SAH_TRACE_INFO("   ai_addrlen   = %d (sockaddr_in = %d, sockaddr_in6 = %d)",res->ai_addrlen,sizeof( struct sockaddr_in ),sizeof( struct sockaddr_in6 ));
      SAH_TRACE_INFO("   ai_canonname = %s",res->ai_canonname);
      SAH_TRACE_INFO("     aittl  = %d ",res->ai_ttl);
    }

}
/******************************************** BLOCKING ADDR INFO TEST **************************************************/
void TestCaseAddrinfoB(char *address, char *tcpudp, char *family, const char *retry) {
        SAH_TRACE_INFO("Running AddrinfoLookup%s...",tcpudp);
        //setup of test environment
        unsigned int options = 0;
        struct addrinfo hints;

        //set the options and call the function
        if (strcmp(tcpudp,"TCP")==0) {
                options = PCB_DNSRES_USE_TCP;
    } else {
                options &= ~PCB_DNSRES_USE_TCP;
        }

        memset(&hints, 0, sizeof(hints));
        if (strcmp(family,"IPV4")==0) {
                hints.ai_family = AF_INET;
        } else if (strcmp(family,"IPV6")==0) {
                hints.ai_family = AF_INET6;
        } else {
                hints.ai_family = AF_UNSPEC;
        }
        hints.ai_socktype = SOCK_STREAM;

        struct addrinfo *ai;
        int ret = pcb_dnsres_getaddrinfo_b(options,
                                           retry,
                                   address,
                                   NULL,
                                   &hints,
                                   &ai);
        if(ai==NULL) {
          switch(ret) {
          case DNSRES_HOST_NOT_FOUND:
            SAH_TRACE_INFO("DNSRES_HOST_NOT_FOUND");
            break;
          case DNSRES_TRY_AGAIN:
            SAH_TRACE_INFO("DNSRES_TRY_AGAIN");
            break;
          case DNSRES_NO_RECOVERY:
            SAH_TRACE_INFO("DNSRES_NO_RECOVERY");
            break;
          case DNSRES_NO_DATA:
            SAH_TRACE_INFO("DNSRES_NO_DATA");
            break;
          case DNSRES_NETDB_INTERNAL:
            SAH_TRACE_INFO("DNSRES_NETDB_INTERNAL");
            break;
          case DNSRES_NETDB_SUCCESS:
            SAH_TRACE_INFO("DNSRES_NETDB_SUCCESS");
            break;
          case DNSRES_EAI_NONAME:
            SAH_TRACE_INFO("DNSRES_EAI_NONAME - name or service is not known");
            break;
          case DNSRES_EAI_AGAIN:
            SAH_TRACE_INFO("DNSRES_EAI_AGAIN - temporary failure in name resolution");
            break;
          case DNSRES_EAI_FAIL:
            SAH_TRACE_INFO("DNSRES_EAI_FAIL - non-recoverable failure in name resolution");
            break;
          case DNSRES_EAI_NODATA:
            SAH_TRACE_INFO("DNSRES_EAI_NODATA - no address associated with name");
            break;
          case DNSRES_EAI_FAMILY:
            SAH_TRACE_INFO("DNSRES_EAI_FAMILY - ai_family not supported");
            break;
          case DNSRES_EAI_SOCKTYPE:
            SAH_TRACE_INFO("DNSRES_EAI_SOCKTYPE - ai_socktype not supported");
            break;
          case DNSRES_EAI_SERVICE:
            SAH_TRACE_INFO("DNSRES_EAI_SERVICE - service not supported for ai_socktype");
            break;
          case DNSRES_EAI_ADDRFAMILY:
            SAH_TRACE_INFO("DNSRES_EAI_ADDRFAMILY - address family for name not supported");
            break;
          case DNSRES_EAI_MEMORY:
            SAH_TRACE_INFO("DNSRES_EAI_MEMORY - memory allocation failure");
            break;
          case DNSRES_EAI_SYSTEM:
            SAH_TRACE_INFO("DNSRES_EAI_SYSTEM - system error (code indicated in errno)");
            break;
          case DNSRES_EAI_BADHINTS:
            SAH_TRACE_INFO("DNSRES_EAI_BADHINTS -  invalid value for hints");
            break;
          case DNSRES_EAI_PROTOCOL:
            SAH_TRACE_INFO("DNSRES_EAI_PROTOCOL -  resolved protocol is unknown");
            break;
          default:
            SAH_TRACE_INFO("default %d",ret);
            break;
          }
      } else {
         PrintAddrInfo(ai);
         freeaddrinfo(ai);
      }

}

void TestCaseAddrinfoBWrapper() {
    TestCaseAddrinfoB("localhost","TCP","IPV4","ONCE:1,2.3.4.5");
    TestCaseAddrinfoB("localhost","TCP","IPV6","ONCE:1,2.3.4.5");
    TestCaseAddrinfoB("localhost","TCP","BOTH","ONCE:1,2.3.4.5");
    TestCaseAddrinfoB("localhost","UDP","IPV4","ONCE:1,2.3.4.5");
    TestCaseAddrinfoB("localhost","UDP","IPV6","ONCE:1,2.3.4.5");
    TestCaseAddrinfoB("localhost","UDP","BOTH","ONCE:1,2.3.4.5");
    TestCaseAddrinfoB("www.google.be","TCP","IPV4","ONCE:1,2.3.4.5");
    TestCaseAddrinfoB("www.google.be","TCP","IPV6","ONCE:1,2.3.4.5");
    TestCaseAddrinfoB("www.google.be","TCP","BOTH","ONCE:1,2.3.4.5");
    TestCaseAddrinfoB("www.google.be","UDP","IPV4","ONCE:1,2.3.4.5");
    TestCaseAddrinfoB("www.google.be","UDP","IPV6","ONCE:1,2.3.4.5");
    TestCaseAddrinfoB("www.google.be","UDP","BOTH","ONCE:1,2.3.4.5");
    TestCaseAddrinfoB("www.google.be","TCP","IPV4","NONE:5");
    TestCaseAddrinfoB("www.google.be","TCP","IPV6","NONE:5");
    TestCaseAddrinfoB("www.google.be","TCP","BOTH","NONE:5");
    TestCaseAddrinfoB("www.google.be","UDP","IPV4","NONE:5");
    TestCaseAddrinfoB("www.google.be","UDP","IPV6","NONE:5");
    TestCaseAddrinfoB("www.google.be","TCP","BOTH","NONE:5");
}


/******************************************** NON BLOCKING ADDR INFO TEST **************************************************/


void AddrinfoLookup_cb(struct addrinfo *ai, int res, void *arg) {
        callback_called++;
        pcb_timer_destroy(cTimer);
        cTimer=NULL;
        SAH_TRACE_INFO("Callback called for %s", (char *)arg);
        if(ai==NULL) {
          switch(res) {
          case DNSRES_HOST_NOT_FOUND:
            SAH_TRACE_INFO("DNSRES_HOST_NOT_FOUND");
            break;
          case DNSRES_TRY_AGAIN:
            SAH_TRACE_INFO("DNSRES_TRY_AGAIN");
            break;
          case DNSRES_NO_RECOVERY:
            SAH_TRACE_INFO("DNSRES_NO_RECOVERY");
            break;
          case DNSRES_NO_DATA:
            SAH_TRACE_INFO("DNSRES_NO_DATA");
            break;
          case DNSRES_NETDB_INTERNAL:
            SAH_TRACE_INFO("DNSRES_NETDB_INTERNAL");
            break;
          case DNSRES_NETDB_SUCCESS:
            SAH_TRACE_INFO("DNSRES_NETDB_SUCCESS");
            break;
          case DNSRES_EAI_NONAME:
            SAH_TRACE_INFO("DNSRES_EAI_NONAME - name or service is not known");
            break;
          case DNSRES_EAI_AGAIN:
            SAH_TRACE_INFO("DNSRES_EAI_AGAIN - temporary failure in name resolution");
            break;
          case DNSRES_EAI_FAIL:
            SAH_TRACE_INFO("DNSRES_EAI_FAIL - non-recoverable failure in name resolution");
            break;
          case DNSRES_EAI_NODATA:
            SAH_TRACE_INFO("DNSRES_EAI_NODATA - no address associated with name");
            break;
          case DNSRES_EAI_FAMILY:
            SAH_TRACE_INFO("DNSRES_EAI_FAMILY - ai_family not supported");
            break;
          case DNSRES_EAI_SOCKTYPE:
            SAH_TRACE_INFO("DNSRES_EAI_SOCKTYPE - ai_socktype not supported");
            break;
          case DNSRES_EAI_SERVICE:
            SAH_TRACE_INFO("DNSRES_EAI_SERVICE - service not supported for ai_socktype");
            break;
          case DNSRES_EAI_ADDRFAMILY:
            SAH_TRACE_INFO("DNSRES_EAI_ADDRFAMILY - address family for name not supported");
            break;
          case DNSRES_EAI_MEMORY:
            SAH_TRACE_INFO("DNSRES_EAI_MEMORY - memory allocation failure");
            break;
          case DNSRES_EAI_SYSTEM:
            SAH_TRACE_INFO("DNSRES_EAI_SYSTEM - system error (code indicated in errno)");
            break;
          case DNSRES_EAI_BADHINTS:
            SAH_TRACE_INFO("DNSRES_EAI_BADHINTS -  invalid value for hints");
            break;
          case DNSRES_EAI_PROTOCOL:
            SAH_TRACE_INFO("DNSRES_EAI_PROTOCOL -  resolved protocol is unknown");
            break;
          default:
            SAH_TRACE_INFO("default %d",res);
            break;
          }
      } else {
         PrintAddrInfo(ai);
         freeaddrinfo(ai);
      }

}

void TestCaseAddrinfo(connection_info_t *con, char *address, char *tcpudp, char *family, char *retry, int cancelTimer) {

        SAH_TRACE_INFO("Running AddrinfoLookup%s...",tcpudp);
        //setup of test environment
        unsigned int options = 0;
        struct addrinfo hints;
        callback_expected = callback_called = 0;
        callback_expected ++;

        //set the options and call the function
        if (strcmp(tcpudp,"TCP")==0) {
                options = PCB_DNSRES_USE_TCP;
    } else {
                options &= ~PCB_DNSRES_USE_TCP;
        }

        memset(&hints, 0, sizeof(hints));
        if (strcmp(family,"IPV4")==0) {
                hints.ai_family = AF_INET;
        } else if (strcmp(family,"IPV6")==0) {
                hints.ai_family = AF_INET6;
        } else {
                hints.ai_family = AF_UNSPEC;
        }
        hints.ai_socktype = SOCK_STREAM;
        pcb_dnsres_t *c = pcb_dnsres_getaddrinfo(con,options,retry, address, NULL, &hints, AddrinfoLookup_cb, address);
        if(cancelTimer) {
            cTimer = pcb_timer_create();
            pcb_timer_setHandler(cTimer, TestCaseCancel_TimerCb);
            pcb_timer_setUserData(cTimer, c);
            pcb_timer_start(cTimer, cancelTimer);
        }

        //simulation of pcb event loop
        int fdSet = 0;
        while (1) {
            /* handle connection events */
            struct timeval timeout;
            timerclear(&timeout);
            timeout.tv_sec = 5;
            fdSet = connection_waitForEvents(&connections, &timeout);
            if (fdSet < 0) {
                SAH_TRACE_FATAL("select failed: %m");
                continue;
            }
            connection_handleEvents(&connections);
            if(callback_called == callback_expected) break;
        }

        pcb_timer_destroy(cTimer);
        cTimer=NULL;
        SAH_TRACE_INFO("OK");
}


void TestCaseAddrinfoWrapper(connection_info_t *con) {
    TestCaseAddrinfo(con,"localhost","TCP","IPV4","ONCE:1,2.3.4.5",0);
    TestCaseAddrinfo(con,"localhost","TCP","IPV6","ONCE:1,2.3.4.5",0);
    TestCaseAddrinfo(con,"localhost","TCP","BOTH","ONCE:1,2.3.4.5",0);
    TestCaseAddrinfo(con,"localhost","UDP","IPV4","ONCE:1,2.3.4.5",0);
    TestCaseAddrinfo(con,"localhost","UDP","IPV6","ONCE:1,2.3.4.5",0);
    TestCaseAddrinfo(con,"localhost","UDP","BOTH","ONCE:1,2.3.4.5",0);
    TestCaseAddrinfo(con,"localhost","TCP","IPV4","ONCE:1,2.3.4.5",1);
    TestCaseAddrinfo(con,"localhost","TCP","IPV6","ONCE:1,2.3.4.5",1);
    TestCaseAddrinfo(con,"localhost","TCP","BOTH","ONCE:1,2.3.4.5",1);
    TestCaseAddrinfo(con,"localhost","UDP","IPV4","ONCE:1,2.3.4.5",1);
    TestCaseAddrinfo(con,"localhost","UDP","IPV6","ONCE:1,2.3.4.5",1);
    TestCaseAddrinfo(con,"localhost","UDP","BOTH","ONCE:1,2.3.4.5",1);
    TestCaseAddrinfo(con,"localhost","TCP","IPV4","ONCE:1,2.3.4.5",100);
    TestCaseAddrinfo(con,"localhost","TCP","IPV6","ONCE:1,2.3.4.5",100);
    TestCaseAddrinfo(con,"localhost","TCP","BOTH","ONCE:1,2.3.4.5",100);
    TestCaseAddrinfo(con,"localhost","UDP","IPV4","ONCE:1,2.3.4.5",100);
    TestCaseAddrinfo(con,"localhost","UDP","IPV6","ONCE:1,2.3.4.5",100);
    TestCaseAddrinfo(con,"localhost","UDP","BOTH","ONCE:1,2.3.4.5",100);
    TestCaseAddrinfo(con,"localhost","TCP","IPV4","ONCE:1,2.3.4.5",500);
    TestCaseAddrinfo(con,"localhost","TCP","IPV6","ONCE:1,2.3.4.5",500);
    TestCaseAddrinfo(con,"localhost","TCP","BOTH","ONCE:1,2.3.4.5",500);
    TestCaseAddrinfo(con,"localhost","UDP","IPV4","ONCE:1,2.3.4.5",500);
    TestCaseAddrinfo(con,"localhost","UDP","IPV6","ONCE:1,2.3.4.5",500);
    TestCaseAddrinfo(con,"localhost","UDP","BOTH","ONCE:1,2.3.4.5",500);
    TestCaseAddrinfo(con,"www.chocolat1.be","UDP","BOTH","ONCE:10,10,10",600);
    TestCaseAddrinfo(con,"www.chocolat2.be","UDP","BOTH","ONCE:10,10,10",900);
    TestCaseAddrinfo(con,"www.chocolat3.be","UDP","BOTH","ONCE:10,10,10",1500);
}



void ExtAddrinfoLookup_cb(pcb_dnsres_t *request, struct addrinfo_ext *ai, int error, void *arg) {
        callback_called++;
        pcb_timer_destroy(cTimer);
        cTimer=NULL;
        SAH_TRACE_INFO("Callback called for %s", (char *)arg);
        if(ai==NULL) {
          switch(error) {
          case DNSRES_HOST_NOT_FOUND:
            SAH_TRACE_INFO("DNSRES_HOST_NOT_FOUND");
            break;
          case DNSRES_TRY_AGAIN:
            SAH_TRACE_INFO("DNSRES_TRY_AGAIN");
            break;
          case DNSRES_NO_RECOVERY:
            SAH_TRACE_INFO("DNSRES_NO_RECOVERY");
            break;
          case DNSRES_NO_DATA:
            SAH_TRACE_INFO("DNSRES_NO_DATA");
            break;
          case DNSRES_NETDB_INTERNAL:
            SAH_TRACE_INFO("DNSRES_NETDB_INTERNAL");
            break;
          case DNSRES_NETDB_SUCCESS:
            SAH_TRACE_INFO("DNSRES_NETDB_SUCCESS");
            break;
          case DNSRES_EAI_NONAME:
            SAH_TRACE_INFO("DNSRES_EAI_NONAME - name or service is not known");
            break;
          case DNSRES_EAI_AGAIN:
            SAH_TRACE_INFO("DNSRES_EAI_AGAIN - temporary failure in name resolution");
            break;
          case DNSRES_EAI_FAIL:
            SAH_TRACE_INFO("DNSRES_EAI_FAIL - non-recoverable failure in name resolution");
            break;
          case DNSRES_EAI_NODATA:
            SAH_TRACE_INFO("DNSRES_EAI_NODATA - no address associated with name");
            break;
          case DNSRES_EAI_FAMILY:
            SAH_TRACE_INFO("DNSRES_EAI_FAMILY - ai_family not supported");
            break;
          case DNSRES_EAI_SOCKTYPE:
            SAH_TRACE_INFO("DNSRES_EAI_SOCKTYPE - ai_socktype not supported");
            break;
          case DNSRES_EAI_SERVICE:
            SAH_TRACE_INFO("DNSRES_EAI_SERVICE - service not supported for ai_socktype");
            break;
          case DNSRES_EAI_ADDRFAMILY:
            SAH_TRACE_INFO("DNSRES_EAI_ADDRFAMILY - address family for name not supported");
            break;
          case DNSRES_EAI_MEMORY:
            SAH_TRACE_INFO("DNSRES_EAI_MEMORY - memory allocation failure");
            break;
          case DNSRES_EAI_SYSTEM:
            SAH_TRACE_INFO("DNSRES_EAI_SYSTEM - system error (code indicated in errno)");
            break;
          case DNSRES_EAI_BADHINTS:
            SAH_TRACE_INFO("DNSRES_EAI_BADHINTS -  invalid value for hints");
            break;
          case DNSRES_EAI_PROTOCOL:
            SAH_TRACE_INFO("DNSRES_EAI_PROTOCOL -  resolved protocol is unknown");
            break;
          default:
            SAH_TRACE_INFO("default %d",error);
            break;
          }
      } else {
         PrintAddrInfoExt(ai);
         pcb_dnsres_free_addrinfo_ext(ai);
      }

}

void TestCaseExtAddrinfo(connection_info_t *con, char *address, char *tcpudp, char *family, char *retry, int cancelTimer) {

        SAH_TRACE_INFO("Running ExtAddrinfoLookup%s...",tcpudp);
        //setup of test environment
        unsigned int options = 0;
        struct addrinfo hints;
        callback_expected = callback_called = 0;
        callback_expected ++;

        //set the options and call the function
        if (strcmp(tcpudp,"TCP")==0) {
                options = PCB_DNSRES_USE_TCP;
        } else {
                options &= ~PCB_DNSRES_USE_TCP;
        }

        memset(&hints, 0, sizeof(hints));
        if (strcmp(family,"IPV4")==0) {
                hints.ai_family = AF_INET;
        } else if (strcmp(family,"IPV6")==0) {
                hints.ai_family = AF_INET6;
        } else {
                hints.ai_family = AF_UNSPEC;
        }
        hints.ai_socktype = SOCK_STREAM;
        pcb_dnsres_t *c = pcb_dnsres_getaddrinfo_ext(con,options,retry, address, NULL, &hints, ExtAddrinfoLookup_cb, address);
        if(cancelTimer) {
            cTimer = pcb_timer_create();
            pcb_timer_setHandler(cTimer, TestCaseCancel_TimerCb);
            pcb_timer_setUserData(cTimer, c);
            pcb_timer_start(cTimer, cancelTimer);
        }

        //simulation of pcb event loop
        int fdSet = 0;
        while (1) {
            /* handle connection events */
            struct timeval timeout;
            timerclear(&timeout);
            timeout.tv_sec = 5;
            fdSet = connection_waitForEvents(&connections, &timeout);
            if (fdSet < 0) {
                SAH_TRACE_FATAL("select failed: %m");
                continue;
            }
            connection_handleEvents(&connections);
            if(callback_called == callback_expected) break;
        }

        pcb_timer_destroy(cTimer);
        cTimer=NULL;
        SAH_TRACE_INFO("OK");
}


void TestCaseExtAddrinfoWrapper(connection_info_t *con) {
    TestCaseExtAddrinfo(con,"localhost","TCP","IPV4","ONCE:1,2.3.4.5",0);
    TestCaseExtAddrinfo(con,"localhost","TCP","IPV6","ONCE:1,2.3.4.5",0);
    TestCaseExtAddrinfo(con,"localhost","TCP","BOTH","ONCE:1,2.3.4.5",0);
    TestCaseExtAddrinfo(con,"localhost","UDP","IPV4","ONCE:1,2.3.4.5",0);
    TestCaseExtAddrinfo(con,"localhost","UDP","IPV6","ONCE:1,2.3.4.5",0);
    TestCaseExtAddrinfo(con,"localhost","UDP","BOTH","ONCE:1,2.3.4.5",0);
    TestCaseExtAddrinfo(con,"localhost","TCP","IPV4","ONCE:1,2.3.4.5",1);
    TestCaseExtAddrinfo(con,"localhost","TCP","IPV6","ONCE:1,2.3.4.5",1);
    TestCaseExtAddrinfo(con,"localhost","TCP","BOTH","ONCE:1,2.3.4.5",1);
    TestCaseExtAddrinfo(con,"localhost","UDP","IPV4","ONCE:1,2.3.4.5",1);
    TestCaseExtAddrinfo(con,"localhost","UDP","IPV6","ONCE:1,2.3.4.5",1);
    TestCaseExtAddrinfo(con,"localhost","UDP","BOTH","ONCE:1,2.3.4.5",1);
    TestCaseExtAddrinfo(con,"localhost","TCP","IPV4","ONCE:1,2.3.4.5",100);
    TestCaseExtAddrinfo(con,"localhost","TCP","IPV6","ONCE:1,2.3.4.5",100);
    TestCaseExtAddrinfo(con,"localhost","TCP","BOTH","ONCE:1,2.3.4.5",100);
    TestCaseExtAddrinfo(con,"localhost","UDP","IPV4","ONCE:1,2.3.4.5",100);
    TestCaseExtAddrinfo(con,"localhost","UDP","IPV6","ONCE:1,2.3.4.5",100);
    TestCaseExtAddrinfo(con,"localhost","UDP","BOTH","ONCE:1,2.3.4.5",100);
    TestCaseExtAddrinfo(con,"localhost","TCP","IPV4","ONCE:1,2.3.4.5",500);
    TestCaseExtAddrinfo(con,"localhost","TCP","IPV6","ONCE:1,2.3.4.5",500);
    TestCaseExtAddrinfo(con,"localhost","TCP","BOTH","ONCE:1,2.3.4.5",500);
    TestCaseExtAddrinfo(con,"localhost","UDP","IPV4","ONCE:1,2.3.4.5",500);
    TestCaseExtAddrinfo(con,"localhost","UDP","IPV6","ONCE:1,2.3.4.5",500);
    TestCaseExtAddrinfo(con,"localhost","UDP","BOTH","ONCE:1,2.3.4.5",500);
    TestCaseExtAddrinfo(con,"www.chocolat1.be","UDP","BOTH","ONCE:10,10,10",600);
    TestCaseExtAddrinfo(con,"www.chocolat2.be","UDP","BOTH","ONCE:10,10,10",900);
    TestCaseExtAddrinfo(con,"www.chocolat3.be","UDP","BOTH","ONCE:10,10,10",1500);
    TestCaseExtAddrinfo(con,"www.google.be","TCP","IPV4","ONCE:1,2.3.4.5",500);
    TestCaseExtAddrinfo(con,"www.google.be","TCP","IPV6","ONCE:1,2.3.4.5",500);
    TestCaseExtAddrinfo(con,"www.google.be","TCP","BOTH","ONCE:1,2.3.4.5",500);
    TestCaseExtAddrinfo(con,"www.google.be","UDP","IPV4","ONCE:1,2.3.4.5",500);
    TestCaseExtAddrinfo(con,"www.google.be","UDP","IPV6","ONCE:1,2.3.4.5",500);
    TestCaseExtAddrinfo(con,"www.google.be","UDP","BOTH","ONCE:1,2.3.4.5",500);
}

/******************************************** BLOCKING HOSTBYNAME **************************************************/

void TestCaseGetHostByAddrB(char *address, int family, const char *retry) {
        SAH_TRACE_INFO("Running TestCaseGetHostByAddr");
        //initialize test context
        callback_called = 0;
        callback_expected = 1;

        //call the function
        struct in_addr a;
        inet_aton(address, &a);

        struct dnsres_hostent *he = NULL;

       int ret = pcb_dnsres_gethostbyaddr_b(0,
                                     retry,
                                     &a,
                                     sizeof(a),
                                     family,
                                     &he);
       if(ret==-1) {
           SAH_TRACE_ERROR("pcb_dnsres_gethostbyaddr_b failed");
       }else if(he==NULL) {
          switch(ret) {
          case DNSRES_HOST_NOT_FOUND:
            SAH_TRACE_INFO("DNSRES_HOST_NOT_FOUND");
            break;
          case DNSRES_TRY_AGAIN:
            SAH_TRACE_INFO("DNSRES_TRY_AGAIN");
            break;
          case DNSRES_NO_RECOVERY:
            SAH_TRACE_INFO("DNSRES_NO_RECOVERY");
            break;
          case DNSRES_NO_DATA:
            SAH_TRACE_INFO("DNSRES_NO_DATA");
            break;
          case DNSRES_NETDB_INTERNAL:
            SAH_TRACE_INFO("DNSRES_NETDB_INTERNAL");
            break;
          case DNSRES_NETDB_SUCCESS:
            SAH_TRACE_INFO("DNSRES_NETDB_SUCCESS");
            break;
          default:
            SAH_TRACE_INFO("default");
            break;
          }
      } else {
          SAH_TRACE_INFO("\tGot: %s", he->h_name);
          pcb_dnsres_gethostbyaddr_b_free_hostent(he);
      }
}


void TestCaseGetHostByAddrBWrapper() {
    TestCaseGetHostByAddrB("127.0.0.1",AF_INET,"ONCE:1,2.3.4.5");// currently only ipv4 is supported
    TestCaseGetHostByAddrB("173.194.35.104",AF_INET,"ONCE:1,2.3.4.5");// currently only ipv4 is supported
}

/******************************************** NON BLOCKING HOSTBYNAME **************************************************/


void TestCaseGetHostByAddr_Cb(struct dnsres_hostent *he, int res, void *arg) {
        callback_called++;
        pcb_timer_destroy(cTimer);
        cTimer=NULL;
        SAH_TRACE_INFO("Callback called for %s", (char *)arg);
        if(he==NULL) {
          switch(res) {
          case DNSRES_HOST_NOT_FOUND:
            SAH_TRACE_INFO("DNSRES_HOST_NOT_FOUND");
            break;
          case DNSRES_TRY_AGAIN:
            SAH_TRACE_INFO("DNSRES_TRY_AGAIN");
            break;
          case DNSRES_NO_RECOVERY:
            SAH_TRACE_INFO("DNSRES_NO_RECOVERY");
            break;
          case DNSRES_NO_DATA:
            SAH_TRACE_INFO("DNSRES_NO_DATA");
            break;
          case DNSRES_NETDB_INTERNAL:
            SAH_TRACE_INFO("DNSRES_NETDB_INTERNAL");
            break;
          case DNSRES_NETDB_SUCCESS:
            SAH_TRACE_INFO("DNSRES_NETDB_SUCCESS");
            break;
          default:
            SAH_TRACE_INFO("default");
            break;
          }
      } else {
          SAH_TRACE_INFO("\tGot: %s -> %s", (char *)arg, he->h_name);
      }

}
void TestCaseGetHostByAddr(connection_info_t *con, char *address, int family, const char *retry, int cancelTimer) {
    SAH_TRACE_INFO("Running TestCaseGetHostByAddr");

    //initialize test context
    callback_called = 0;
    callback_expected = 1;


    //call the function
    struct in_addr a;
    inet_aton(address, &a);
    pcb_dnsres_t *c = pcb_dnsres_gethostbyaddr(con,0,retry,&a,sizeof(a), family,TestCaseGetHostByAddr_Cb,address);
    if(cancelTimer) {
        cTimer = pcb_timer_create();
        pcb_timer_setHandler(cTimer, TestCaseCancel_TimerCb);
        pcb_timer_setUserData(cTimer, c);
        pcb_timer_start(cTimer, cancelTimer);
    }

    //simulation of pcb event loop
    int fdSet = 0;
    while (1) {
        /* handle connection events */
        struct timeval timeout;
        timerclear(&timeout);
        timeout.tv_sec = 5;
        fdSet = connection_waitForEvents(&connections, &timeout);
        if (fdSet < 0) {
            SAH_TRACE_FATAL("select failed: %m");
            continue;
        }
        connection_handleEvents(&connections);
        if(callback_called == callback_expected) break;
    }
    pcb_timer_destroy(cTimer);
    cTimer=NULL;
}




void TestCaseGetHostByAddrWrapper(connection_info_t *con) {
    TestCaseGetHostByAddr(con,"127.0.0.1",AF_INET,"ONCE:1,2.3.4.5",0); // currently only ipv4 is supported
    TestCaseGetHostByAddr(con,"173.194.35.104",AF_INET,"ONCE:1,2.3.4.5",0); // currently only ipv4 is supported
    TestCaseGetHostByAddr(con,"127.0.0.1",AF_INET,"ONCE:1,2.3.4.5",1); // currently only ipv4 is supported
    TestCaseGetHostByAddr(con,"173.194.35.104",AF_INET,"ONCE:1,2.3.4.5",1); // currently only ipv4 is supported
    TestCaseGetHostByAddr(con,"127.0.0.1",AF_INET,"ONCE:1,2.3.4.5",100); // currently only ipv4 is supported
    TestCaseGetHostByAddr(con,"173.194.35.104",AF_INET,"ONCE:1,2.3.4.5",100); // currently only ipv4 is supported
    TestCaseGetHostByAddr(con,"127.0.0.1",AF_INET,"ONCE:1,2.3.4.5",500); // currently only ipv4 is supported
    TestCaseGetHostByAddr(con,"173.194.35.104",AF_INET,"ONCE:1,2.3.4.5",500); // currently only ipv4 is supported
}


/******************************************** MAIN **************************************************/



int main(int argc, char **argv) {
    // initialize tracing and logging
    sahTraceOpen("Example client",TRACE_TYPE_STDOUT);
    sahTraceSetLevel(500);

    connection_initLibrary();

    connection_info_t *con = connection_create("DNSRES");
    if(con==NULL) {
        SAH_TRACE_ERROR("unable to initialize connection");
        goto error;
    }

    if (!llist_initialize(&connections)) {
        SAH_TRACE_ERROR("unable to initialize connection list %s", error_string(pcb_error));
        goto error;
    }
    llist_append(&connections, connection_getIterator(con));

    //call test routines
    TestCaseGetHostByAddrWrapper(con);
    TestCaseGetHostByAddrBWrapper();
    TestCaseAddrinfoWrapper(con);
    TestCaseAddrinfoBWrapper();
    TestCaseExtAddrinfoWrapper(con);


    connection_destroy(con);
    connection_exitLibrary();
    sahTraceClose();
    return 0;


  error:
    connection_exitLibrary();
    sahTraceClose();
    return -1;
}
