include $(STAGINGDIR)/components.config

INSTALL ?= install
STRIP ?= $(CROSS_COMPILE)strip

CFLAGS += -g -fPIC

ifneq (,$(findstring mips,$(CC)))
CFLAGS += -DMIPS
endif

all: all-lib mafia/mafia

ifneq ($(strip $(CONFIG_SAH_LIB_MAFIA_HEAP)),)

CFLAGS += -DCONFIG_MAFIA_HEAP

all-lib: libmafia_nothreads.so libmafia_withthreads.so

install-lib:
	$(INSTALL) -D -m 0755 libmafia_nothreads.so $(STAGINGDIR)/lib/libmafia_nothreads.so
	$(INSTALL) -D -m 0755 libmafia_withthreads.so $(STAGINGDIR)/lib/libmafia_withthreads.so

release-lib:
	$(INSTALL) -D -m 0755 $(STAGINGDIR)/lib/libmafia_nothreads.so $(ROOT_FS_DIR)/lib/libmafia_nothreads.so
	$(STRIP) $(ROOT_FS_DIR)/lib/libmafia_nothreads.so
	$(INSTALL) -D -m 0755 $(STAGINGDIR)/lib/libmafia_withthreads.so $(ROOT_FS_DIR)/lib/libmafia_withthreads.so
	$(STRIP) $(ROOT_FS_DIR)/lib/libmafia_withthreads.so

else

all-lib: libmafia.so

install-lib:
	$(INSTALL) -D -m 0755 libmafia.so $(STAGINGDIR)/lib/libmafia.so
	ln -sf libmafia.so $(STAGINGDIR)/lib/libmafia_nothreads.so
	ln -sf libmafia.so $(STAGINGDIR)/lib/libmafia_withthreads.so

release-lib:
	$(INSTALL) -D -m 0755 $(STAGINGDIR)/lib/libmafia_nothreads.so $(ROOT_FS_DIR)/lib/libmafia.so
	$(STRIP) $(ROOT_FS_DIR)/lib/libmafia.so
	ln -sf libmafia.so $(ROOT_FS_DIR)/lib/libmafia_nothreads.so
	ln -sf libmafia.so $(ROOT_FS_DIR)/lib/libmafia_withthreads.so

endif

clean:
	rm -f *.so glibcmalloc/*.o libmafia/*.o mafia/mafia

install: all install-lib
	$(INSTALL) -D -p -m 0644 include/mafia.h $(STAGINGDIR)/include/mafia.h
	$(INSTALL) -D -p -m 0644 include/mafia/backtrace.h $(STAGINGDIR)/include/mafia/backtrace.h
	$(INSTALL) -D -p -m 0644 include/mafia/heap.h $(STAGINGDIR)/include/mafia/heap.h
	$(INSTALL) -D -m 0755 mafia/mafia $(STAGINGDIR)/bin/mafia
	$(INSTALL) -D -m 0644 mafia_nothreads.pc $(PKG_CONFIG_LIBDIR)/mafia_nothreads.pc
	$(INSTALL) -D -m 0644 mafia_withthreads.pc $(PKG_CONFIG_LIBDIR)/mafia_withthreads.pc

release: release-lib
	$(INSTALL) -D -m 0755 $(STAGINGDIR)/bin/mafia $(ROOT_FS_DIR)/bin/mafia
	$(STRIP) $(ROOT_FS_DIR)/bin/mafia

doc:
	doxygen mafia.doxy
	for FILE in $$(find html -type f); do \
		$(INSTALL) -m 0644 -D $${FILE} $(DOCDIR)/html/$${FILE} || exit 1; \
	done

libmafia_nothreads.so: glibcmalloc/malloc_nothreads.o libmafia/backtrace_nothreads.o libmafia/heap_nothreads.o libmafia/mafia_nothreads.o
	$(CC) $(CFLAGS) -O3 -Wall -Wextra -shared -o $@ glibcmalloc/malloc_nothreads.o libmafia/backtrace_nothreads.o libmafia/heap_nothreads.o libmafia/mafia_nothreads.o

libmafia_withthreads.so: glibcmalloc/malloc_withthreads.o libmafia/backtrace_withthreads.o libmafia/heap_withthreads.o libmafia/mafia_withthreads.o
	$(CC) $(CFLAGS) -O3 -Wall -Wextra  -shared -o $@ glibcmalloc/malloc_withthreads.o libmafia/backtrace_withthreads.o libmafia/heap_withthreads.o libmafia/mafia_withthreads.o -pthread

libmafia.so: libmafia/backtrace.o libmafia/mafia.o
	$(CC) $(CFLAGS) -O3 -Wall -Wextra  -shared -o $@ libmafia/backtrace.o libmafia/mafia.o

glibcmalloc/malloc_nothreads.o: glibcmalloc/malloc.c
	$(CC) -c $(CFLAGS) -O3 -D_GNU_SOURCE -o $@ glibcmalloc/malloc.c

glibcmalloc/malloc_withthreads.o: glibcmalloc/malloc.c
	$(CC) -c $(CFLAGS) -O3 -D_GNU_SOURCE -o $@ -pthread glibcmalloc/malloc.c

libmafia/backtrace_nothreads.o libmafia/heap_nothreads.o libmafia/mafia_nothreads.o:
	$(CC) -c $(CFLAGS) -O0 -Wall -Wextra -Iinclude -DNOTHREADS -o $@ $(subst _nothreads.o,.c,$@)

libmafia/backtrace_withthreads.o libmafia/heap_withthreads.o libmafia/mafia_withthreads.o:
	$(CC) -c $(CFLAGS) -O0 -Wall -Wextra -Iinclude -DWITHTHREADS -o $@ -pthread $(subst _withthreads.o,.c,$@)

libmafia/backtrace.o libmafia/mafia.o:
	$(CC) -c $(CFLAGS) -O0 -Wall -Wextra -Iinclude -DNOTHREADS -DHEAP_DISABLED -o $@ $(subst .o,.c,$@)

libmafia/backtrace_nothreads.o libmafia/backtrace_withthreads.o libmafia/backtrace.o: libmafia/backtrace.c include/mafia/backtrace.h libmafia/common.h
libmafia/heap_nothreads.o libmafia/heap_withthreads.o: libmafia/heap.c include/mafia.h include/mafia/heap.h include/mafia/backtrace.h libmafia/common.h
libmafia/mafia_nothreads.o libmafia/mafia_withthreads.o libmafia/mafia.o: libmafia/mafia.c include/mafia.h include/mafia/heap.h include/mafia/backtrace.h libmafia/common.h

mafia/mafia: mafia/main.c
	$(CC) $(CFLAGS) -O3 -Wall -Wextra -o mafia/mafia mafia/main.c

all clean install release doc: FORCE

FORCE:

