/*
 * Copyright (c) 2011 SoftAtHome
 *
 * The information and source code contained herein is the exclusive
 * property of SoftAtHome and may not be disclosed, examined, or
 * reproduced in whole or in part without explicit written authorization
 * of the copyright owner.
 *
 * Author : Dries De Winter (dries.dewinter@softathome.com)
 *
 */
#ifndef __MAFIA_HEAP_H__
#define __MAFIA_HEAP_H__

/**
@ingroup mafia
@{
*/

/**
@brief
Dump the complete heap

@details
All currently allocated blocks are grouped together per matching backtrace and for each occuring backtrace, the payload
(amount of memory allocated, excluding bookkeeping or fragmentation overhead) and the number of blocks is printed to <i>fd</i>,
followed by the backtrace. Notice that (1) this only works if heap debugging is enabled and
(2) the heap is dumped implicitly at application exit to reveal memory leaks. See also @ref environment.

@param fd File descriptor to dump the heap to.
@param attributes Attributes to be passed to backtrace_dump().
*/
void heap_dump(int fd, int attributes);

/**
@brief
Count the complete heap

@details
Stores the total payload of the entire heap (sum of payloads of all currently allocated blocks) into <i>payload</i> and
counts the total number of all currently allocated blocks, to be stored into <i>blocks</i>.
Notice that this only works if heap debugging is enabled. See also @ref environment.

@param fd File descriptor to dump the heap to.
@param attributes Attributes to be passed to backtrace_dump().
*/
void heap_count(size_t *payload, int *blocks);

/**
@}
*/

#endif
