/*
 * Copyright (c) 2011 SoftAtHome
 *
 * The information and source code contained herein is the exclusive
 * property of SoftAtHome and may not be disclosed, examined, or
 * reproduced in whole or in part without explicit written authorization
 * of the copyright owner.
 *
 */
#define _GNU_SOURCE
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>

extern char **environ;

static const char *usage =
	"Usage: mafia [OPTION]... COMMAND [ARG]...\n"
	"Run command with libmafia preloaded.\n"
	"Options are:\n"
	"    --nothreads           Preload libmafia_nothreads.so for single-threaded\n"
	"                          applications. This is the default.\n"
	"    --withthreads         Preload libmafia_withthreads.so for multi-threaded\n"
	"                          applications.\n"
	"    --heapdebug           Enable heap debugging.\n"
	"    --exitbacktrace {never|always|nonzero}\n"
	"                          Specify when to print a backtrace on exit. Never,\n"
	"                          always or only if the exit code is non-zero.\n"
	"                          Never is the default.\n"
	"    --mafiasink <file>    Specify a file to redirect libmafia's output to.\n"
	"                          The default is stderr.\n"
	;

void preload(const char *file) {
	static const char *origpreload = NULL;
	static char newpreload[1024];
	static int boot = 1;
	if (boot) {
		origpreload = getenv("LD_PRELOAD");
		boot = 0;
	}
	snprintf(newpreload, 1024, "%s%s%s", origpreload?:"", origpreload?" ":"", file);
	setenv("LD_PRELOAD", newpreload, 1);
}

int main(int argc, char **argv) {
	preload("libmafia_nothreads.so");
	while (++argv, --argc) {
		if (!strcmp(*argv, "--help") || !strcmp(*argv, "-h")) {
			fprintf(stderr, "%s\n", usage);
			exit(EXIT_SUCCESS);
		} else if (!strcmp(*argv, "--nothreads")) {
			preload("libmafia_nothreads.so");
		} else if (!strcmp(*argv, "--withthreads")) {
			preload("libmafia_withthreads.so");
		} else if (!strcmp(*argv, "--heapdebug")) {
			setenv("HEAPDEBUG", "yes", 1);
		} else if (!strcmp(*argv, "--exitbacktrace")) {
			if (!(++argv, --argc)) {
				fprintf(stderr, "%s\n", usage);
				exit(EXIT_FAILURE);
			}
			if (!strcmp(*argv, "never") || !strcmp(*argv, "always") || !strcmp(*argv, "nonzero")) {
				setenv("EXITBACKTRACE", *argv, 1);
			} else {
				fprintf(stderr, "%s\n", usage);
				exit(EXIT_FAILURE);
			}
		} else if (!strcmp(*argv, "--mafiasink")) {
			if (!(++argv, --argc)) {
				fprintf(stderr, "%s\n", usage);
				exit(EXIT_FAILURE);
			}
			setenv("MAFIASINK", *argv, 1);
		} else {
			break;
		}
	}
	if (argc) {
		execvp(argv[0], argv);
		fprintf(stderr, "mafia: %s: %m\n", argv[0]);
		exit(EXIT_FAILURE);
	} else {
		fprintf(stderr, "%s\n", usage);
		exit(EXIT_FAILURE);
	}
	return 0;
}

