include $(STAGINGDIR)/components.config

CFLAGS += -fPIC -g -Os -Wall -Wextra -I. -I./include -I$(STAGINGDIR)/include $(shell pkg-config --cflags sahtrace pcb)
LDFLAGS += -L$(STAGINGDIR)/lib $(shell pkg-config --libs sahtrace pcb)

LIBS = libnemo_client.so

OBJS-libnemo_client.so = client/csi.o client/util.o client/common_api.o client/dhcp-api.o client/wlan.o client/netdev.o

OBJS = $(OBJS-libnemo_client.so)

CONFIG = nemo-defaults.odl nemo-config.odl

INSTALL-lib=/lib/libnemo_client.so
INSTALL-header=/include/nemo/client.h /include/nemo/common_api.h \
	/include/nemo/client/csi.h /include/nemo/client/dhcp.h \
	/include/nemo/client/netdev.h /include/nemo/client/wlan.h
INSTALL-script=/etc_ro_fs/init.d/nemo-clients
INSTALL-config=/etc_ro_fs/nemo-defaults.odl /etc_ro_fs/nemo-config.odl
INSTALL-link=etc_ro_fs/rc1.d/S23nemo-clients
INSTALLLINKSRC-/etc_ro_fs/rc1.d/S23nemo-clients=../init.d/nemo-clients
INSTALL-pkg-config=nemo_client.pc

INSTALL-all = $(INSTALL-lib) $(INSTALL-script) $(INSTALL-data) $(INSTALL-config) $(INSTALL-header)
RELEASE-all = $(INSTALL-lib) $(INSTALL-script) $(INSTALL-data) $(INSTALL-config) $(INSTALL-link)

all: $(LIBS) $(addprefix $(STAGINGDIR)/config/,$(CONFIG))

libnemo_client.so: $(OBJS-libnemo_client.so)

clean:
	rm -f $(LIBS) $(OBJS) $(OBJS:.o=.d)

install: all $(addprefix $(STAGINGDIR),$(INSTALL-all)) $(addprefix $(PKG_CONFIG_LIBDIR)/,$(INSTALL-pkg-config))

release: $(addprefix $(ROOT_FS_DIR),$(RELEASE-all))

doc:
	doxygen nemo.doxy
	$(INSTALL) -d $(STAGINGDIR)/../doc/datamodel/sah_lib_nemo
	pcb_docgen --xml $(STAGINGDIR)/usr/lib/nemo/nemo.odl > nemo.xml
	$(INSTALL) -m 0644 nemo.xml $(STAGINGDIR)/../doc/datamodel/sah_lib_nemo/nemo.xml
	pcb_docgen --html --title "NeMo" --output $(STAGINGDIR)/../doc/datamodel/sah_lib_nemo nemo.xml

$(OBJS): %.o: %.c Component.mk $(STAGINGDIR)/components.config
	$(CC) $(CFLAGS) $(CFLAGS-$@) -o $@ -c $< 
	$(CC) $(CFLAGS) $(CFLAGS-$@) -MM -MP -MT '$(@) $(@:.o=.d)' -MF $(@:.o=.d) $<

-include $(OBJS:.o=.d)

$(LIBS):
	$(CC) $(CFLAGS) -Wl,-soname,$@ -shared -o $@ $(OBJS-$@) $(LDFLAGS) $(LDFLAGS-$@)

$(addprefix $(STAGINGDIR)/config/,$(CONFIG)):
	touch $@

$(addprefix $(STAGINGDIR),$(INSTALL-lib)) $(addprefix $(STAGINGDIR),$(INSTALL-script)):
	$(INSTALL) -p -D -m 0750 $(notdir $@) $@

$(addprefix $(STAGINGDIR),$(INSTALL-data)):
	$(INSTALL) -p -D -m 0640 $(notdir $@) $@

$(addprefix $(STAGINGDIR),$(INSTALL-config)):
	$(INSTALL) -p -D -m 0640 $(STAGINGDIR)/config/$(notdir $@) $@

$(addprefix $(STAGINGDIR),$(INSTALL-header)):
	$(INSTALL) -p -D -m 0640 $(subst $(STAGINGDIR)/,,$@) $@

$(addprefix $(PKG_CONFIG_LIBDIR)/,$(INSTALL-pkg-config)):
	$(INSTALL) -p -D -m 0640 $(subst $(PKG_CONFIG_LIBDIR)/,,$@) $@

$(addprefix $(ROOT_FS_DIR),$(INSTALL-lib)):
	$(INSTALL) -p -D -m 0750 $(subst $(ROOT_FS_DIR),$(STAGINGDIR),$@) $@
	$(STRIP) $@

$(addprefix $(ROOT_FS_DIR),$(INSTALL-script)):
	$(INSTALL) -p -D -m 0750 $(subst $(ROOT_FS_DIR),$(STAGINGDIR),$@) $@

$(addprefix $(ROOT_FS_DIR),$(INSTALL-data)):
	$(INSTALL) -p -D -m 0640 $(subst $(ROOT_FS_DIR),$(STAGINGDIR),$@) $@

$(addprefix $(ROOT_FS_DIR),$(INSTALL-config)):
	$(INSTALL) -p -D -m 0640 $(STAGINGDIR)/config/$(notdir $@) $@

$(addprefix $(ROOT_FS_DIR),$(INSTALL-link)):
	$(INSTALL) -p -d $(dir $@)
	ln -sf $(INSTALLLINKSRC-$(subst $(ROOT_FS_DIR),,$@)) $@

$(addprefix $(STAGINGDIR),$(INSTALL-all)) $(addprefix $(ROOT_FS_DIR),$(RELEASE-all)) all clean install release: FORCE
$(addprefix $(PKG_CONFIG_LIBDIR)/,$(INSTALL-pkg-config)): FORCE

FORCE:
