/*
 * Copyright (c) 2011 SoftAtHome
 *
 * The information and source code contained herein is the exclusive
 * property of SoftAtHome and may not be disclosed, examined, or
 * reproduced in whole or in part without explicit written authorization
 * of the copyright owner.
 *
 */
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <signal.h>
#include <string.h>

#include <debug/sahtrace.h>

#include <pcb/utils.h>
#include <pcb/core.h>

#include "nemo/client.h"
#include "client/util.h"

const char *nemo_traverse_this = "this";
const char *nemo_traverse_down = "down";
const char *nemo_traverse_up = "up";
const char *nemo_traverse_down_exclusive = "down exclusive";
const char *nemo_traverse_up_exclusive = "up exclusive";
const char *nemo_traverse_one_level_down = "one level down";
const char *nemo_traverse_one_level_up = "one level up";
const char *nemo_traverse_all = "all";

bool nemo_isUp(const char *intf, const char *flag, const char *traverse) {
	request_t *req = request_create_executeFunction(nemo_intf2path(intf), "isUp", request_common_path_key_notation | request_function_args_by_name);
	if (flag)
		request_addCharParameter(req, "flag", flag);
	if (traverse)
		request_addCharParameter(req, "traverse", traverse);
	nemo_request_invoke(req);
	bool retval = variant_bool(nemo_request_returnValue(req));
	request_destroy(req);
	return retval;
}

bool nemo_hasFlag(const char *intf, const char *flag, const char *condition, const char *traverse) {
	request_t *req = request_create_executeFunction(nemo_intf2path(intf), "hasFlag", request_common_path_key_notation | request_function_args_by_name);
	if (flag)
		request_addCharParameter(req, "flag", flag);
	if (condition)
		request_addCharParameter(req, "condition", condition);
	if (traverse)
		request_addCharParameter(req, "traverse", traverse);
	nemo_request_invoke(req);
	bool retval = variant_bool(nemo_request_returnValue(req));
	request_destroy(req);
	return retval;
}

void nemo_setFlag(const char *intf, const char *flag, const char *condition, const char *traverse) {
	request_t *req = request_create_executeFunction(nemo_intf2path(intf), "setFlag", request_common_path_key_notation | request_function_args_by_name);
	if (flag)
		request_addCharParameter(req, "flag", flag);
	if (condition)
		request_addCharParameter(req, "condition", condition);
	if (traverse)
		request_addCharParameter(req, "traverse", traverse);
	nemo_request_invoke(req);
	request_destroy(req);
}

void nemo_clearFlag(const char *intf, const char *flag, const char *condition, const char *traverse) {
	request_t *req = request_create_executeFunction(nemo_intf2path(intf), "clearFlag", request_common_path_key_notation | request_function_args_by_name);
	if (flag)
		request_addCharParameter(req, "flag", flag);
	if (condition)
		request_addCharParameter(req, "condition", condition);
	if (traverse)
		request_addCharParameter(req, "traverse", traverse);
	nemo_request_invoke(req);
	request_destroy(req);
}

bool nemo_isLinkedTo(const char *intf, const char *target, const char *traverse) {
	request_t *req = request_create_executeFunction(nemo_intf2path(intf), "isLinkedTo", request_common_path_key_notation | request_function_args_by_name);
	if (target)
		request_addCharParameter(req, "target", target);
	if (traverse)
		request_addCharParameter(req, "traverse", traverse);
	nemo_request_invoke(req);
	bool retval = variant_bool(nemo_request_returnValue(req));
	request_destroy(req);
	return retval;
}

variant_list_t *nemo_getIntfs(const char *intf, const char *flag, const char *traverse) {
	request_t *req = request_create_executeFunction(nemo_intf2path(intf), "getIntfs", request_common_path_key_notation | request_function_args_by_name);
	if (flag)
		request_addCharParameter(req, "flag", flag);
	if (traverse)
		request_addCharParameter(req, "traverse", traverse);
	nemo_request_invoke(req);
	variant_list_t *retval = variant_array(nemo_request_returnValue(req));
	request_destroy(req);
	return retval;
}

char *nemo_luckyIntf(const char *intf, const char *flag, const char *traverse) {
	request_t *req = request_create_executeFunction(nemo_intf2path(intf), "luckyIntf", request_common_path_key_notation | request_function_args_by_name);
	if (flag)
		request_addCharParameter(req, "flag", flag);
	if (traverse)
		request_addCharParameter(req, "traverse", traverse);
	nemo_request_invoke(req);
	char *retval = variant_char(nemo_request_returnValue(req));
	request_destroy(req);
	return retval;
}

variant_t *nemo_getFirstParameter(const char *intf, const char *name, const char *flag, const char *traverse) {
	request_t *req = request_create_executeFunction(nemo_intf2path(intf), "getFirstParameter", request_common_path_key_notation | request_function_args_by_name);
	if (name)
		request_addCharParameter(req, "name", name);
	if (flag)
		request_addCharParameter(req, "flag", flag);
	if (traverse)
		request_addCharParameter(req, "traverse", traverse);
	nemo_request_invoke(req);
	variant_t *retval = calloc(1, sizeof(variant_t));
	variant_initialize(retval, variant_type_unknown);
	variant_copy(retval, nemo_request_returnValue(req));
	request_destroy(req);
	return retval;
}

variant_map_t *nemo_getParameters(const char *intf, const char *name, const char *flag, const char *traverse) {
	request_t *req = request_create_executeFunction(nemo_intf2path(intf), "getParameters", request_common_path_key_notation | request_function_args_by_name);
	if (name)
		request_addCharParameter(req, "name", name);
	if (flag)
		request_addCharParameter(req, "flag", flag);
	if (traverse)
		request_addCharParameter(req, "traverse", traverse);
	nemo_request_invoke(req);
	variant_map_t *retval = variant_map(nemo_request_returnValue(req));
	request_destroy(req);
	return retval;
}

void nemo_setFirstParameter(const char *intf, const char *name, const variant_t *value, const char *flag, const char *traverse) {
	request_t *req = request_create_executeFunction(nemo_intf2path(intf), "setFirstParameter", request_common_path_key_notation | request_function_args_by_name);
	if (name)
		request_addCharParameter(req, "name", name);
	if (value)
		request_addParameter(req, "value", value);
	if (flag)
		request_addCharParameter(req, "flag", flag);
	if (traverse)
		request_addCharParameter(req, "traverse", traverse);
	nemo_request_invoke(req);
	request_destroy(req);
}

void nemo_setParameters(const char *intf, const char *name, const variant_t *value, const char *flag, const char *traverse) {
	request_t *req = request_create_executeFunction(nemo_intf2path(intf), "setParameters", request_common_path_key_notation | request_function_args_by_name);
	if (name)
		request_addCharParameter(req, "name", name);
	if (value)
		request_addParameter(req, "value", value);
	if (flag)
		request_addCharParameter(req, "flag", flag);
	if (traverse)
		request_addCharParameter(req, "traverse", traverse);
	nemo_request_invoke(req);
	request_destroy(req);
}

variant_map_t *nemo_getMIBs(const char *intf, const char *mibs, const char *flag, const char *traverse) {
	request_t *req = request_create_executeFunction(nemo_intf2path(intf), "getMIBs", request_common_path_key_notation | request_function_args_by_name);
	if (mibs)
		request_addCharParameter(req, "mibs", mibs);
	if (flag)
		request_addCharParameter(req, "flag", flag);
	if (traverse)
		request_addCharParameter(req, "traverse", traverse);
	nemo_request_invoke(req);
	variant_map_t *retval = variant_map(nemo_request_returnValue(req));
	request_destroy(req);
	return retval;
}

void nemo_setMIBs(const char *intf, variant_map_t *mibs) {
	request_t *req = request_create_executeFunction(nemo_intf2path(intf), "setMIBs", request_common_path_key_notation | request_function_args_by_name);
	if (mibs)
		request_addMapParameter(req, "mibs", mibs);
	nemo_request_invoke(req);
	request_destroy(req);
}

void nemo_linkIntfs(const char *ulintf, const char *llintf) {
	request_t *req = request_create_executeFunction("NeMo", "linkIntfs", request_common_path_key_notation | request_function_args_by_name);
	if (ulintf)
		request_addCharParameter(req, "ulintf", ulintf);
	if (llintf)
		request_addCharParameter(req, "llintf", llintf);
	nemo_request_invoke(req);
	request_destroy(req);
}

void nemo_unlinkIntfs(const char *ulintf, const char *llintf) {
	request_t *req = request_create_executeFunction("NeMo", "unlinkIntfs", request_common_path_key_notation | request_function_args_by_name);
	if (ulintf)
		request_addCharParameter(req, "ulintf", ulintf);
	if (llintf)
		request_addCharParameter(req, "llintf", llintf);
	nemo_request_invoke(req);
	request_destroy(req);	
}

typedef struct _query_request {
	llist_iterator_t it;
	request_t *request;
	uint32_t query_id;
	int refcount;
} query_request_t;

struct _nemo_query {
	llist_iterator_t it;
	char *intf;
	char *subscriber;
	char *class;
	argument_value_list_t args;
	uint32_t attributes;
	nemo_result_handler_t handler;
	void *userdata;
	query_request_t *request;
};

static llist_t queries = {NULL, NULL};
static llist_t requests = {NULL, NULL};

static bool queryReplyHandler(request_t *req, pcb_t *pcb, peer_info_t *from, void *userdata) {
	(void)pcb; (void)from;
	query_request_t *request = (query_request_t *)userdata;
	reply_item_t *item;
	for (item = reply_firstItem(request_reply(req)); item; item = reply_nextItem(item)) {
		if (reply_item_type(item) != reply_type_notification)
			continue;
		notification_t *notification = reply_item_notification(item);
		variant_t *result = notification_parameter_variant(notification_getParameter(notification, "Result"));
		if (!result)
			continue;
		nemo_query_t *query;
		for (query = (nemo_query_t *)llist_first(&queries); query; query = (nemo_query_t *)llist_iterator_next(&query->it)) {
			if (query->request == request && query->handler)
				query->handler(result, query->userdata);
		}
	}
	return true;
}

static void query_request_destroy(query_request_t *request) {
	if (!request)
		return;
	if (request->request) {
		SAH_TRACEZ_INFO("nemo", "cancel request %s?&", request_path(request->request));
		request_destroy(request->request);
	}
	llist_iterator_take(&request->it);
	free(request);
}

static query_request_t *query_request_create(const char *intf, uint32_t query_id) {
	query_request_t *request = calloc(1, sizeof(query_request_t));
	if (!request) {
		SAH_TRACEZ_ERROR("nemo", "calloc failed");
		goto error;
	}
	request->query_id = query_id;
	static char querypath[64];
	snprintf(querypath, 64, "NeMo.Intf.%s.Query.%u", intf, query_id);
	request->request = request_create_getObject(querypath, 0, request_no_object_caching | request_common_path_key_notation | request_notify_custom | request_notify_only);
	request_setData(request->request, request);
	request_setReplyHandler(request->request, queryReplyHandler);
	SAH_TRACEZ_INFO("nemo", "send request %s?&", request_path(request->request));
	if (!pcb_sendRequest(nemo_client_pcb(), nemo_client_peer(), request->request)) {
		SAH_TRACEZ_ERROR("nemo", "Failed to send request: %d (%s)", pcb_error, error_string(pcb_error));
		goto error;
	}
	llist_append(&requests, &request->it);
	return request;
error:
	query_request_destroy(request);
	return NULL;
}

static query_request_t *query_request_find(uint32_t query_id) {
	query_request_t *request;
	for (request = (query_request_t *)llist_first(&requests); request; request = (query_request_t *)llist_iterator_next(&request->it)) {
		if (request->query_id == query_id)
			return request;
	}
	return NULL;
}

static void nemo_query_destroy(nemo_query_t *query) {
	if (!query)
		return;
	if (query->request) {
		query->request->refcount--;
		if (!query->request->refcount)
			query_request_destroy(query->request);
	}
	free(query->intf);
	free(query->subscriber);
	free(query->class);
	argument_valueClear(&query->args);
	free(query);
}

static nemo_query_t *nemo_query_create(const char *intf, const char *subscriber, const char *class, argument_value_list_t *args, uint32_t attributes, nemo_result_handler_t handler, void *userdata) {
	nemo_query_t *query = calloc(1, sizeof(nemo_query_t));
	if (!query) {
		SAH_TRACEZ_ERROR("nemo", "calloc failed");
		goto error;
	}
	query->intf = strdup(intf);
	if (!query->intf) {
		SAH_TRACEZ_ERROR("nemo", "strdup(%s) failed", intf);
		goto error;
	}
	query->subscriber = strdup(subscriber);
	if (!query->subscriber) {
		SAH_TRACEZ_ERROR("nemo", "strdup(%s) failed", subscriber);
		goto error;
	}
	query->class = strdup(class);
	if (!query->class) {
		SAH_TRACEZ_ERROR("nemo", "strdup(%s) failed", class);
		goto error;
	}
	argument_value_list_initialize(&query->args);
	argument_value_t *arg;
	for (arg = argument_valueFirstArgument(args); arg; arg = argument_valueNextArgument(arg))
		argument_valueAdd(&query->args, argument_valueName(arg), argument_value(arg));
	query->attributes = attributes;
	query->handler = handler;
	query->userdata = userdata;
	return query;
error:
	nemo_query_destroy(query);
	return NULL;

}

nemo_query_t *nemo_openQuery(const char *intf, const char *subscriber, const char *class, argument_value_list_t *args, uint32_t attributes, nemo_result_handler_t handler, void *userdata) {
	if (!intf)
		intf = "lo";
	if (!subscriber) {
		SAH_TRACEZ_ERROR("nemo", "mandatory argument subscriber is NULL");
		return NULL;
	}
	if (!class) {
		SAH_TRACEZ_ERROR("nemo", "mandatory argument class is NULL");
		return NULL;
	}
	request_t *req = NULL;
	nemo_query_t *query = nemo_query_create(intf, subscriber, class, args, attributes, handler, userdata);
	if (!query)
		goto error;
	req = request_create_executeFunction(nemo_intf2path(intf), "openQuery", request_common_path_key_notation | attributes);
	request_addCharParameter(req, "subscriber", subscriber);
	request_addCharParameter(req, "class", class);
	argument_value_t *arg;
	while ((arg = argument_valueTakeArgumentFirst(args)))
		request_addArgument(req, arg);
	if (!nemo_request_invoke(req))
		goto error;
	uint32_t query_id = variant_uint32(nemo_request_returnValue(req));
	if (!query_id) {
		SAH_TRACEZ_ERROR("nemo", "openQuery(intf=%s, subscriber=%s, class=%s) failed", intf, subscriber, class);
		goto error;
	}
	request_destroy(req);
	req = NULL;
	query->request = query_request_find(query_id);
	if (!query->request)
		query->request = query_request_create(intf, query_id);
	if (!query->request)
		goto error;
	query->request->refcount++;

	// get initial result
	request_t *getResult = request_create_executeFunction(request_path(query->request->request), "getResult", request_common_path_key_notation);
	if (!nemo_request_invoke(getResult)) {
		request_destroy(getResult);
		goto error;
	}
	if (query->handler)
		query->handler(nemo_request_returnValue(getResult), query->userdata);
	request_destroy(getResult);
	
	llist_append(&queries, &query->it);
	return query;
error:
	if (req)
		request_destroy(req);
	nemo_query_destroy(query);	
	return NULL;
}

void nemo_closeQuery(nemo_query_t *query) {
	if (!query)
		return;
	request_t *req = request_create_executeFunction(nemo_intf2path(query->intf), "closeQuery", request_common_path_key_notation | (query->attributes & request_function_args_by_name));
	request_addCharParameter(req, "subscriber", query->subscriber);
	request_addCharParameter(req, "class", query->class);
	argument_value_t *arg;
	while ((arg = argument_valueTakeArgumentFirst(&query->args)))
		request_addArgument(req, arg);
	nemo_request_invoke(req);
	request_destroy(req);
	llist_iterator_take(&query->it);
	nemo_query_destroy(query);
}

nemo_query_t *nemo_openQuery_isUp(const char *intf, const char *subscriber, const char *flag, const char *traverse, nemo_result_handler_t handler, void *userdata) {
	argument_value_list_t args;
	argument_value_list_initialize(&args);
	if (flag) nemo_argument_valueAddChar(&args, "flag", flag);
	if (traverse) nemo_argument_valueAddChar(&args, "traverse", traverse);
	nemo_query_t *query = nemo_openQuery(intf, subscriber, "isUp", &args, request_function_args_by_name, handler, userdata);
	argument_valueClear(&args);
	return query;
}
nemo_query_t *nemo_openQuery_hasFlag(const char *intf, const char *subscriber, const char *flag, const char *condition, const char *traverse, nemo_result_handler_t handler, void *userdata) {
	argument_value_list_t args;
	argument_value_list_initialize(&args);
	if (flag) nemo_argument_valueAddChar(&args, "flag", flag);
	if (condition) nemo_argument_valueAddChar(&args, "condition", condition);
	if (traverse) nemo_argument_valueAddChar(&args, "traverse", traverse);
	nemo_query_t *query = nemo_openQuery(intf, subscriber, "hasFlag", &args, request_function_args_by_name, handler, userdata);
	argument_valueClear(&args);
	return query;
}
nemo_query_t *nemo_openQuery_isLinkedTo(const char *intf, const char *subscriber, const char *target, const char *traverse, nemo_result_handler_t handler, void *userdata) {
	argument_value_list_t args;
	argument_value_list_initialize(&args);
	if (target) nemo_argument_valueAddChar(&args, "target", target);
	if (traverse) nemo_argument_valueAddChar(&args, "traverse", traverse);
	nemo_query_t *query = nemo_openQuery(intf, subscriber, "isLinkedTo", &args, request_function_args_by_name, handler, userdata);
	argument_valueClear(&args);
	return query;
}
nemo_query_t *nemo_openQuery_getIntfs(const char *intf, const char *subscriber, const char *flag, const char *traverse, nemo_result_handler_t handler, void *userdata) {
	argument_value_list_t args;
	argument_value_list_initialize(&args);
	if (flag) nemo_argument_valueAddChar(&args, "flag", flag);
	if (traverse) nemo_argument_valueAddChar(&args, "traverse", traverse);
	nemo_query_t *query = nemo_openQuery(intf, subscriber, "getIntfs", &args, request_function_args_by_name, handler, userdata);
	argument_valueClear(&args);
	return query;
}
nemo_query_t *nemo_openQuery_luckyIntf(const char *intf, const char *subscriber, const char *flag, const char *traverse, nemo_result_handler_t handler, void *userdata) {
	argument_value_list_t args;
	argument_value_list_initialize(&args);
	if (flag) nemo_argument_valueAddChar(&args, "flag", flag);
	if (traverse) nemo_argument_valueAddChar(&args, "traverse", traverse);
	nemo_query_t *query = nemo_openQuery(intf, subscriber, "luckyIntf", &args, request_function_args_by_name, handler, userdata);
	argument_valueClear(&args);
	return query;
}
nemo_query_t *nemo_openQuery_getFirstParameter(const char *intf, const char *subscriber, const char *name, const char *flag, const char *traverse, nemo_result_handler_t handler, void *userdata) {
	argument_value_list_t args;
	argument_value_list_initialize(&args);
	if (name) nemo_argument_valueAddChar(&args, "name", name);
	if (flag) nemo_argument_valueAddChar(&args, "flag", flag);
	if (traverse) nemo_argument_valueAddChar(&args, "traverse", traverse);
	nemo_query_t *query = nemo_openQuery(intf, subscriber, "getFirstParameter", &args, request_function_args_by_name, handler, userdata);
	argument_valueClear(&args);
	return query;
}
nemo_query_t *nemo_openQuery_getParameters(const char *intf, const char *subscriber, const char *name, const char *flag, const char *traverse, nemo_result_handler_t handler, void *userdata) {
	argument_value_list_t args;
	argument_value_list_initialize(&args);
	if (name) nemo_argument_valueAddChar(&args, "name", name);
	if (flag) nemo_argument_valueAddChar(&args, "flag", flag);
	if (traverse) nemo_argument_valueAddChar(&args, "traverse", traverse);
	nemo_query_t *query = nemo_openQuery(intf, subscriber, "getParameters", &args, request_function_args_by_name, handler, userdata);
	argument_valueClear(&args);
	return query;
}
