/*
 * Copyright (c) 2011 SoftAtHome
 *
 * The information and source code contained herein is the exclusive
 * property of SoftAtHome and may not be disclosed, examined, or
 * reproduced in whole or in part without explicit written authorization
 * of the copyright owner.
 *
 */
#include <debug/sahtrace.h>

#include <pcb/utils.h>
#include <pcb/core.h>

#include "nemo/client.h"
#include "client/util.h"

variant_list_t *nemo_getAddrs(const char *intf, const char *flag, const char *traverse) {
	request_t *req = request_create_executeFunction(nemo_intf2path(intf), "getAddrs", request_common_path_key_notation | request_function_args_by_name);
	if (flag)
		request_addCharParameter(req, "flag", flag);
	if (traverse)
		request_addCharParameter(req, "traverse", traverse);
	nemo_request_invoke(req);
	variant_list_t *retval = variant_array(nemo_request_returnValue(req));
	request_destroy(req);
	return retval;
}

variant_map_t *nemo_luckyAddr(const char *intf, const char *flag, const char *traverse) {
	request_t *req = request_create_executeFunction(nemo_intf2path(intf), "luckyAddr", request_common_path_key_notation | request_function_args_by_name);
	if (flag)
		request_addCharParameter(req, "flag", flag);
	if (traverse)
		request_addCharParameter(req, "traverse", traverse);
	nemo_request_invoke(req);
	variant_map_t *retval = variant_map(nemo_request_returnValue(req));
	request_destroy(req);
	return retval;
}

char *nemo_luckyAddrAddress(const char *intf, const char *flag, const char *traverse) {
	request_t *req = request_create_executeFunction(nemo_intf2path(intf), "luckyAddrAddress", request_common_path_key_notation | request_function_args_by_name);
	if (flag)
		request_addCharParameter(req, "flag", flag);
	if (traverse)
		request_addCharParameter(req, "traverse", traverse);
	nemo_request_invoke(req);
	char *retval = variant_char(nemo_request_returnValue(req));
	request_destroy(req);
	return retval;
}

nemo_query_t *nemo_openQuery_getAddrs(const char *intf, const char *subscriber, const char *flag, const char *traverse, nemo_result_handler_t handler, void *userdata) {
	argument_value_list_t args;
	argument_value_list_initialize(&args);
	if (flag) nemo_argument_valueAddChar(&args, "flag", flag);
	if (traverse) nemo_argument_valueAddChar(&args, "traverse", traverse);
	nemo_query_t *query = nemo_openQuery(intf, subscriber, "getAddrs", &args, request_function_args_by_name, handler, userdata);
	argument_valueClear(&args);
	return query;
}

nemo_query_t *nemo_openQuery_luckyAddr(const char *intf, const char *subscriber, const char *flag, const char *traverse, nemo_result_handler_t handler, void *userdata) {
	argument_value_list_t args;
	argument_value_list_initialize(&args);
	if (flag) nemo_argument_valueAddChar(&args, "flag", flag);
	if (traverse) nemo_argument_valueAddChar(&args, "traverse", traverse);
	nemo_query_t *query = nemo_openQuery(intf, subscriber, "luckyAddr", &args, request_function_args_by_name, handler, userdata);
	argument_valueClear(&args);
	return query;
}

nemo_query_t *nemo_openQuery_luckyAddrAddress(const char *intf, const char *subscriber, const char *flag, const char *traverse, nemo_result_handler_t handler, void *userdata) {
	argument_value_list_t args;
	argument_value_list_initialize(&args);
	if (flag) nemo_argument_valueAddChar(&args, "flag", flag);
	if (traverse) nemo_argument_valueAddChar(&args, "traverse", traverse);
	nemo_query_t *query = nemo_openQuery(intf, subscriber, "luckyAddrAddress", &args, request_function_args_by_name, handler, userdata);
	argument_valueClear(&args);
	return query;
}
