/*
 * Copyright (c) 2011 SoftAtHome
 *
 * The information and source code contained herein is the exclusive
 * property of SoftAtHome and may not be disclosed, examined, or
 * reproduced in whole or in part without explicit written authorization
 * of the copyright owner.
 *
 */
#ifndef __NEMO_CLIENT_WLAN_H__
#define __NEMO_CLIENT_WLAN_H__

/**
@ingroup client
@defgroup wlan wlan.h - WLANConfig
@{

@brief
WLAN is an extension on top of the Common API (see @ref common_api) to facilitate out-of-process access to the WLAN Config MIB.
*/

#include <pcb/utils.h>

/**
@brief
This function is a wrapper for nemo_setMIBs() intended for wlan reconfigurations.

@details
Except for calling nemo_setMIBs(), it also calls
the wland's edit() before setting the MIBs and commit() after setting the MIBs for all affected Radios to guarantee atomic
reconfiguration, minimizing the performance penalty of it. See also documentation of WLAN daemon.

@param intf Dummy Intf. It does not affect the behavior of the function but it must exist. Default is "lo".
@param mibs A variant_map in the same format as the return value of nemo_getMIBs() [ i.e.
                A variant_map that maps MIB names to MIB maps. A MIB map is a variant_map that maps Intf names to object maps.
                An object map is a variant_map that maps parameter names to parameter values (variants) and child object names
                (including instances of templates) to object maps recursively.
            ]. However, the MIB maps don't have to be complete. Parameters and objects of the MIB that are missing (or discarded
            from the return value of nemo_getMIBs()) are left untouched, except for the instances of template objects:
            for each template object included by a MIB map, any unexisting instance included by the MIB map is created implicitly
            and any existing instance not included by MIB map is deleted implicitly. Mandatory.
*/
void nemo_setWLANConfig(const char *intf, variant_map_t *mibs);

/**
@}
*/

#endif
