CONCURRENCY_LEVEL ?= $(shell /bin/grep ^processor /proc/cpuinfo | /usr/bin/wc -l)

all:
	@echo "-------------------------------------------------------------"
	@echo "- building libcrypto"
	@echo "-------------------------------------------------------------"
	make -j$(CONCURRENCY_LEVEL) -C ./crypto -f ./Makefile libcrypto.so
	@echo "-------------------------------------------------------------"
	@echo "- building libssl"
	@echo "-------------------------------------------------------------"
	make -j$(CONCURRENCY_LEVEL) -C ./ssl -f ./Makefile libssl.so

clean:
	@echo "-------------------------------------------------------------"
	@echo "- cleaning libcrypto"
	@echo "-------------------------------------------------------------"
	make -C ./crypto -f ./Makefile clean
	@echo "-------------------------------------------------------------"
	@echo "- cleaning libssl"
	@echo "-------------------------------------------------------------"
	make -C ./ssl -f ./Makefile clean

install:
	@echo "-------------------------------------------------------------"
	@echo "- installing openssl headers to: " $(STAGINGDIR)
	@echo "-------------------------------------------------------------"
	$(INSTALL) -d $(STAGINGDIR)/include/openssl
	$(INSTALL) -p ./include/openssl/*h $(STAGINGDIR)/include/openssl
	@echo "-------------------------------------------------------------"
	@echo "- installing libcrypto to: " $(STAGINGDIR)
	@echo "-------------------------------------------------------------"
	$(INSTALL) -d $(STAGINGDIR)/lib
	$(INSTALL) -m 755 ./crypto/libcrypto.so $(STAGINGDIR)/lib
	ln -sf libcrypto.so $(STAGINGDIR)/lib/libcrypto.so.0.9.8	
	$(INSTALL) -m 755 ./crypto/libcrypto.a $(STAGINGDIR)/lib
	#ln -sf libcrypto.a $(STAGINGDIR)/lib/libcrypto_pic.a # see bug 15138
	@echo "-------------------------------------------------------------"
	@echo "- installing libssl to: " $(STAGINGDIR)
	@echo "-------------------------------------------------------------"
	$(INSTALL) -d $(STAGINGDIR)/lib
	$(INSTALL) -m 755 ./ssl/libssl.so $(STAGINGDIR)/lib	
	$(INSTALL) -m 755 ./ssl/libssl.a $(STAGINGDIR)/lib
	#ln -sf libssl.a $(STAGINGDIR)/lib/libssl_pic.a # see bug 15138
	$(INSTALL) -D -m 0644 pkg-config.pc $(PKG_CONFIG_LIBDIR)/openssl.pc
	$(INSTALL) -D -m 0644 pkg-config-crypto.pc $(PKG_CONFIG_LIBDIR)/libcrypto.pc

release:
	@echo "-------------------------------------------------------------"
	@echo "- make a libcrypto release in: " $(ROOT_FS_DIR)
	@echo "-------------------------------------------------------------"
	$(INSTALL) -d $(ROOT_FS_DIR)/lib
	cp -dp $(STAGINGDIR)/lib/libcrypto.so* $(ROOT_FS_DIR)/lib
	$(STRIP) $(ROOT_FS_DIR)/lib/libcrypto.so	
	@echo "-------------------------------------------------------------"
	@echo "- make a libssl release in: " $(ROOT_FS_DIR)
	@echo "-------------------------------------------------------------"
	$(INSTALL) -d $(ROOT_FS_DIR)/lib
	cp -dp $(STAGINGDIR)/lib/libssl.so* $(ROOT_FS_DIR)/lib
	$(STRIP) $(ROOT_FS_DIR)/lib/libssl.so

.PHONY: clean all install release
