################################################################
# project.lst build flags
#
# SAH_DEBUG..........: No optimalization, trace level set to 500
# SAH_DEBUG_NO_STRIP.: No stripping during release step
#
################################################################

ifndef STAGINGDIR
  $(error STAGINGDIR not defined, stopping build)
endif

include $(STAGINGDIR)/components.config

INSTALL ?= install
STRIP ?= strip
PKG_CONFIG_LIBDIR ?= $(STAGINGDIR)/build/pkg-config

all:
	$(MAKE) -f Makefile.utils -C src all
	$(MAKE) -f Makefile.sl -C src all
ifeq ($(CONFIG_PCB_OPEN_SSL_SUPPORT),y)
	$(MAKE) -f Makefile.ssl -C src all
endif
ifneq ($(CONFIG_PCB_UTILS_ONLY),y)
	$(MAKE) -f Makefile.dm -C src all
endif

clean:
	-$(MAKE) -C src clean
	-$(MAKE) -f Makefile.utils -C src clean
	-$(MAKE) -f Makefile.sl -C src clean
	-$(MAKE) -f Makefile.ssl -C src clean
	-$(MAKE) -f Makefile.dm -C src clean
	-$(MAKE) -C test clean
	rm -rf html
	rm -f Makefile
	rm -f pcb.pro.user
	rm -f pkg-config.pc

pkg-config.pc: pkg-config.pc.m4 $(STAGINGDIR)/components.config
	cat $(STAGINGDIR)/components.config.m4 $< | m4 > $@

pkg-config-ssl.pc: pkg-config-ssl.pc.m4 $(STAGINGDIR)/components.config
	cat $(STAGINGDIR)/components.config.m4 $< | m4 > $@

install: pkg-config.pc pkg-config-ssl.pc
	$(INSTALL) -d $(STAGINGDIR)/include
ifneq ($(CONFIG_PCB_UTILS_ONLY),y)
	for FILE in $$(find include/pcb/core -type f); do \
		$(INSTALL) -p -m 0644 -D $${FILE} $(STAGINGDIR)/$${FILE} || exit 1; \
	done
	$(INSTALL) -p -m 0644 -D include/pcb/core.h $(STAGINGDIR)/include/pcb/core.h
	$(INSTALL) -p -m 0644 -D include/pcb/pcb_client.h $(STAGINGDIR)/include/pcb/pcb_client.h
endif
	for FILE in $$(find include/pcb/common/ -type f); do \
		$(INSTALL) -p -m 0644 -D $${FILE} $(STAGINGDIR)/$${FILE} || exit 1; \
	done
	for FILE in $$(find include/pcb/utils/ -type f); do \
		$(INSTALL) -p -m 0644 -D $${FILE} $(STAGINGDIR)/$${FILE} || exit 1; \
	done
	for FILE in $$(find include/pcb/plugin/ -type f); do \
		$(INSTALL) -p -m 0644 -D $${FILE} $(STAGINGDIR)/$${FILE} || exit 1; \
	done
	for FILE in $$(find include/pcb/client/ -type f); do \
		$(INSTALL) -p -m 0644 -D $${FILE} $(STAGINGDIR)/$${FILE} || exit 1; \
	done
	$(INSTALL) -p -m 0644 -D include/pcb/common.h $(STAGINGDIR)/include/pcb/common.h
	$(INSTALL) -p -m 0644 -D include/pcb/utils.h $(STAGINGDIR)/include/pcb/utils.h
	$(INSTALL) -d $(STAGINGDIR)/lib
ifeq ($(CONFIG_PCB_UTILS_ONLY),y)
	$(INSTALL) -m 0755 src/libpcb_utils.so $(STAGINGDIR)/lib
	$(INSTALL) -m 0755 src/libpcb_sl.so $(STAGINGDIR)/lib
	$(INSTALL) -m 0755 src/libpcb_utils.a $(STAGINGDIR)/lib
	$(INSTALL) -m 0755 src/libpcb_sl.a $(STAGINGDIR)/lib
ifeq ($(CONFIG_PCB_OPEN_SSL_SUPPORT),y)
	$(INSTALL) -m 0755 src/libpcb_ssl.so $(STAGINGDIR)/lib
	$(INSTALL) -m 0755 src/libpcb_ssl.a $(STAGINGDIR)/lib
	ln -sf libpcb_ssl.a $(STAGINGDIR)/lib/libpcb_ssl_pic.a
endif
else
	$(INSTALL) -m 0755 src/libpcb_utils.so $(STAGINGDIR)/lib
	$(INSTALL) -m 0755 src/libpcb_sl.so $(STAGINGDIR)/lib
	$(INSTALL) -m 0755 src/libpcb_utils.a $(STAGINGDIR)/lib
	$(INSTALL) -m 0755 src/libpcb_sl.a $(STAGINGDIR)/lib
ifeq ($(CONFIG_PCB_OPEN_SSL_SUPPORT),y)
	$(INSTALL) -m 0755 src/libpcb_ssl.so $(STAGINGDIR)/lib
	$(INSTALL) -m 0755 src/libpcb_ssl.a $(STAGINGDIR)/lib
endif
	$(INSTALL) -m 0755 src/libpcb_dm.so $(STAGINGDIR)/lib
	$(INSTALL) -m 0755 src/libpcb_dm.a $(STAGINGDIR)/lib
endif
	$(INSTALL) -D -m 0644 pkg-config.pc $(PKG_CONFIG_LIBDIR)/pcb.pc
	$(INSTALL) -D -m 0644 pkg-config-ssl.pc $(PKG_CONFIG_LIBDIR)/pcbssl.pc

release:
ifndef ROOT_FS_DIR
	$(error ROOT_FS_DIR not defined, stopping release)
endif
	$(INSTALL) -d $(ROOT_FS_DIR)/lib
ifeq ($(CONFIG_PCB_UTILS_ONLY),y)
	$(INSTALL) -m 0755 $(STAGINGDIR)/lib/libpcb_utils.so $(ROOT_FS_DIR)/lib
ifndef SAH_DEBUG_NO_STRIP
	$(STRIP) $(ROOT_FS_DIR)/lib/libpcb_utils.so
endif
	$(INSTALL) -m 0755 $(STAGINGDIR)/lib/libpcb_sl.so $(ROOT_FS_DIR)/lib
ifndef SAH_DEBUG_NO_STRIP
	$(STRIP) $(ROOT_FS_DIR)/lib/libpcb_sl.so
endif
ifeq ($(CONFIG_PCB_OPEN_SSL_SUPPORT),y)
	$(INSTALL) -m 0755 $(STAGINGDIR)/lib/libpcb_ssl.so $(ROOT_FS_DIR)/lib
ifndef SAH_DEBUG_NO_STRIP
	$(STRIP) $(ROOT_FS_DIR)/lib/libpcb_ssl.so
endif
endif
else
	$(INSTALL) -m 0755 $(STAGINGDIR)/lib/libpcb_utils.so $(ROOT_FS_DIR)/lib
ifndef SAH_DEBUG_NO_STRIP
	$(STRIP) $(ROOT_FS_DIR)/lib/libpcb_utils.so
endif
	$(INSTALL) -m 0755 $(STAGINGDIR)/lib/libpcb_sl.so $(ROOT_FS_DIR)/lib
ifndef SAH_DEBUG_NO_STRIP
	$(STRIP) $(ROOT_FS_DIR)/lib/libpcb_sl.so
endif
ifeq ($(CONFIG_PCB_OPEN_SSL_SUPPORT),y)
	$(INSTALL) -m 0755 $(STAGINGDIR)/lib/libpcb_ssl.so $(ROOT_FS_DIR)/lib
ifndef SAH_DEBUG_NO_STRIP
	$(STRIP) $(ROOT_FS_DIR)/lib/libpcb_ssl.so
endif
endif
	$(INSTALL) -m 0755 $(STAGINGDIR)/lib/libpcb_dm.so $(ROOT_FS_DIR)/lib
ifndef SAH_DEBUG_NO_STRIP
	$(STRIP) $(ROOT_FS_DIR)/lib/libpcb_dm.so
endif
endif

doc:
ifndef DOCDIR
	$(error DOCDIR not defined, stop generating documentation)
endif
	doxygen documentation/pcb.doxy

test:
	@echo "Creating pcb library unit tests"
	@echo " Variable  COMPONENTNAME=$(COMPONENTNAME)"
	@echo " Variable  LOCALTESTDIR=$(LOCALTESTDIR)"
	$(MAKE) -C test
	$(INSTALL) -d $(LOCALTESTDIR)/
	$(INSTALL) -m 755 test/run $(LOCALTESTDIR)/
	$(INSTALL) -m 755 test/pcb_utils_testplan.sh $(LOCALTESTDIR)
	$(INSTALL) -m 755 test/pcb_dm_testplan.sh $(LOCALTESTDIR)
	$(INSTALL) -m 755 test/pcb_utils_r_testplan.sh $(LOCALTESTDIR)
	$(INSTALL) -m 755 test/pcb_dm_r_testplan.sh $(LOCALTESTDIR)
	$(INSTALL) -d $(LOCALTESTDIR)/unittests
	$(INSTALL) -m 755 test/test_circular_buffer/*.so $(LOCALTESTDIR)/unittests
	$(INSTALL) -m 644 test/test_circular_buffer/*.so.list.xml $(LOCALTESTDIR)/unittests
	$(INSTALL) -m 755 test/test_connection/*.so $(LOCALTESTDIR)/unittests
	$(INSTALL) -m 644 test/test_connection/*.so.list.xml $(LOCALTESTDIR)/unittests
	$(INSTALL) -m 755 test/test_connection/*.pem $(LOCALTESTDIR)/
	$(INSTALL) -m 755 test/test_connection/memcheck_ssl.supp $(LOCALTESTDIR)/
	$(INSTALL) -m 755 test/test_linked_list/*.so $(LOCALTESTDIR)/unittests
	$(INSTALL) -m 644 test/test_linked_list/*.so.list.xml $(LOCALTESTDIR)/unittests
	$(INSTALL) -m 755 test/test_string/*.so $(LOCALTESTDIR)/unittests
	$(INSTALL) -m 644 test/test_string/*.so.list.xml $(LOCALTESTDIR)/unittests
	$(INSTALL) -m 755 test/test_tree/*.so $(LOCALTESTDIR)/unittests
	$(INSTALL) -m 644 test/test_tree/*.so.list.xml $(LOCALTESTDIR)/unittests
	$(INSTALL) -m 755 test/test_variant/*.so $(LOCALTESTDIR)/unittests
	$(INSTALL) -m 644 test/test_variant/*.so.list.xml $(LOCALTESTDIR)/unittests
	$(INSTALL) -m 755 test/test_object/*.so $(LOCALTESTDIR)/unittests
	$(INSTALL) -m 644 test/test_object/*.so.list.xml $(LOCALTESTDIR)/unittests
	@echo "Done!"

.PHONY: all clean install release doc test

