/**
@file
@brief
Header file that includes all utility header files
*/

/**
@defgroup pcb_utils Utilities
@{
@brief
Collection of utilities.

@details
The pcb utilities is a collection of functions intended to make development of common tasks easier.\n
The uitilities includes:
    - @ref pcb_utils_containers
    - @ref pcb_utils_buffers
    - @ref pcb_utils_uri
@}
*/

/**
@ingroup pcb_utils
@defgroup pcb_utils_buffers Buffering
@{
@brief
Collection of buffering utilities.

@details
Buffering methods, helps in allocating dynamic buffers and managing the buffers
@}
*/
#include <pcb/utils/circular_buffer.h>

/**
@ingroup pcb_utils
@defgroup pcb_utils_containers Containers
@{
@brief
Implementation of some simple containers.

@details
Implementation of some simple containers. The following containers are implemented:
    - doubly linked list: @ref pcb_utils_linked_list
    - string: string helper functions: @ref pcb_utils_string
    - string list: linked list contaning strings: @ref pcb_utils_string_list
    - variant: a container that contain a lot of different data types: @ref pcb_utils_variant
    - variant list: linked list of variants: @ref pcb_utils_variant_list
    - variant map: linked list of key-value pairs, where the value is represented by a variant: @ref pcb_utils_variant_map
@}
*/
#include <pcb/utils/linked_list.h>
#include <pcb/utils/string.h>
#include <pcb/utils/string_list.h>
#include <pcb/utils/tree.h>
#include <pcb/utils/variant.h>
#include <pcb/utils/variant_list.h>
#include <pcb/utils/variant_map.h>

/**
@ingroup pcb_utils
@defgroup pcb_utils_uri URI Parsing
@{
@brief
Helper functions to parse a URI.\n

@details
A set of functions to parse a URI from a string.\n
@}
*/
#include <pcb/utils/uri.h>

/**
@defgroup pcb_utils_socket_events Sockets, Files and Eventing
@{
@brief
Filed descriptor and event handling

@details
Collection of functions to help in creating and managing sockets and file descriptors, dispatch events and
building event loops.\n
This includes:
    - Timers: @ref pcb_utils_timers
    - Socket creation and event handling: @ref pcb_socket_layer_connections
    - Managing sockets and file descriptors: @ref pcb_socket_layer_peer
@}
*/
#include <pcb/utils/connection.h>
#include <pcb/utils/peer.h>
#include <pcb/utils/timer.h>
