ifndef STAGINGDIR
  $(error STAGINGDIR not defined, stopping build)
endif

include $(STAGINGDIR)/components.config

OBJECTS_CORE   = core/datamodel.o \
                 core/object.o \
                 core/parameter.o \
                 core/function.o \
                 core/reply.o \
                 core/request.o \
                 core/requesthandlers.o \
                 core/serialize.o \
                 core/validator.o \
                 core/notification.o \
                 core/pcb_main.o \
                 core/pcb_wait.o \
                 core/pcb_client.o \
                 core/mapping.o \
                 core/acl.o

OBJECTS_PRIV = core/object_priv.o \
               core/parameter_priv.o \
               core/local_object.o \
               core/remote_object.o \
               core/core_priv.o

OBJECTS = $(OBJECTS_CORE) $(OBJECTS_PRIV)

ifdef SAH_DEBUG
  CFLAGS += -DSAHTRACES_LEVEL=500
else
  ifneq ($(CONFIG_PCB_NO_OPTIMIZE),y)
    CFLAGS += -O2
  endif
  CFLAGS += -DNDEBUG
endif

CFLAGS += -Wall -Wextra -Werror -fPIC -g -I../include -I$(STAGINGDIR)/include
LDFLAGS += -L$(STAGINGDIR)/lib -ldl

ifeq ($(CONFIG_PCB_ACL_USERMNGT),y)
  CFLAGS += $(shell pkg-config --cflags sahtrace usermngt)
  LDFLAGS += $(shell pkg-config --libs sahtrace usermngt)
else
  CFLAGS += $(shell pkg-config --cflags sahtrace)
  LDFLAGS += $(shell pkg-config --libs sahtrace)
endif

ifeq ($(CONFIG_PCB_HELP_SUPPORT),y)
CFLAGS += -DPCB_HELP_SUPPORT
endif

ifeq ($(CONFIG_SAH_TOOLS_UNITTESTS_GCOVENABLED),y)
  CFLAGS += -fprofile-arcs -ftest-coverage
  LDFLAGS += -fprofile-arcs
endif

all: libpcb_dm.so

libpcb_dm.so: $(OBJECTS_CORE) $(OBJECTS_PRIV) libpcb_dm.a
	$(CC) -Wl,-soname,$(@) -shared -fPIC -o $(@) $(OBJECTS_CORE) $(OBJECTS_PRIV) $(LDFLAGS)

-include $(OBJECTS:.o=.d)

########################
# This was added to enable the packaging scripts te reduce the library (see bug 13956)
########################
libpcb_dm.a: $(OBJECTS_CORE) $(OBJECTS_PRIV)
	$(AR) rcs $(@) $^

######################################################################################

%.o: %.c
	$(CC) $(CFLAGS) -c -o $@ $<
	@$(CC) $(CFLAGS) -MM -MP -MT '$(@) $(@:.o=.d)' -MF $(@:.o=.d) $(<)

clean:
	rm -f libpcb_dm.so $(OBJECTS_CORE) $(OBJECTS_PRIV) libpcb_dm.a
	rm -f core/*.d
	rm -f gcno core/*.gcno
	rm -f gcda core/*.gcda
	rm -f *.gcov

.PHONY: all clean
