ifndef STAGINGDIR
  $(error STAGINGDIR not defined, stopping build)
endif

include $(STAGINGDIR)/components.config

OBJECTS_SOCKET = utils/connection.o \
                 utils/peer.o \

ifdef SAH_DEBUG
  CFLAGS += -DSAHTRACES_LEVEL=500
else
  ifneq ($(CONFIG_PCB_NO_OPTIMIZE),y)
    CFLAGS += -O2
  endif
  CFLAGS += -DNDEBUG
endif

CFLAGS += -Wall -Wextra -fPIC -g -I../include -I$(STAGINGDIR)/include $(shell pkg-config --cflags sahtrace)
LDFLAGS += -L$(STAGINGDIR)/lib $(shell pkg-config --libs sahtrace)

ifeq ($(CONFIG_SAH_TOOLS_UNITTESTS_GCOVENABLED),y)
  CFLAGS += -fprofile-arcs -ftest-coverage
  LDFLAGS += -fprofile-arcs
endif

all: libpcb_sl.so

libpcb_sl.so: $(OBJECTS_SOCKET) libpcb_sl.a
	$(CC) -Wl,-soname,$(@) -shared -fPIC -o $(@) $(OBJECTS_SOCKET) $(LDFLAGS)


########################
# This was added to enable the packaging scripts
# te reduce the library (see bug 13956)
########################
libpcb_sl.a: $(OBJECTS_SOCKET)
	$(AR) rcs $(@) $^

######################################################################################

-include $(OBJECTS_SOCKET:.o=.d)

%.o: %.c
	$(CC) $(CFLAGS) -c -o $@ $<
	@$(CC) $(CFLAGS) -MM -MP -MT '$(@) $(@:.o=.d)' -MF $(@:.o=.d) $(<)

clean:
	rm -f libpcb_sl.so $(OBJECTS_SOCKET) libpcb_sl.a
	rm -f utils/*.d
	rm -f utils/*.gcno
	rm -f utils/*.gcda
	rm -f *.gcov

.PHONY: all clean
