/****************************************************************************
**
** Copyright (C) 2011 SoftAtHome. All rights reserved.
**
** SoftAtHome reserves all rights not expressly granted herein.
**
** - DISCLAIMER OF WARRANTY -
**
** THIS FILE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
** EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE.
**
** THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOURCE
** CODE IS WITH YOU. SHOULD THE SOURCE CODE PROVE DEFECTIVE, YOU
** ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
**
** - LIMITATION OF LIABILITY -
**
** IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN
** WRITING WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES
** AND/OR DISTRIBUTES THE SOURCE CODE, BE LIABLE TO YOU FOR DAMAGES,
** INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
** ARISING OUT OF THE USE OR INABILITY TO USE THE SOURCE CODE
** (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED
** INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE
** OF THE SOURCE CODE TO OPERATE WITH ANY OTHER PROGRAM), EVEN IF SUCH
** HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
** DAMAGES.
**
****************************************************************************/

#if !defined(PCB_LOCAL_OBJECT_H)
#define PCB_LOCAL_OBJECT_H

#ifdef __cplusplus
extern "C"
{
#endif

#include <pcb/core/object.h>
#include <pcb/core/requesthandlers.h>

/**
@file
@brief
Header file with private function prototypes for handling local objects

@todo
    - copy functions in the local_object_copy function
*/

#if !defined(NO_DOXYGEN)
// cleanup functions
bool __attribute__ ((visibility("hidden"))) local_object_delete(object_t *object);

// tree functions
object_t __attribute__ ((visibility("hidden"))) *local_object_parent(const object_t *object);
object_t __attribute__ ((visibility("hidden"))) *local_object_getObject(object_t *object, const char *path, const uint32_t pathAttr, bool *ExactMatch);
bool __attribute__ ((visibility("hidden"))) local_object_hasChildren(const object_t *object);
uint32_t __attribute__ ((visibility("hidden"))) local_object_childCount(object_t *object);
void __attribute__ ((visibility("hidden"))) local_object_path(const object_t *object, string_t *path, const uint32_t pathAttributes);

// instance functions
object_t __attribute__ ((visibility("hidden"))) *local_object_createInstance(object_t *object, uint32_t index, const char *key);
bool __attribute__ ((visibility("hidden"))) local_object_hasInstances(const object_t *object);
uint32_t __attribute__ ((visibility("hidden"))) local_object_instanceCount(object_t *object);

// parameter functions
parameter_t __attribute__ ((visibility("hidden"))) *local_object_getParameter(const object_t *object, const char *parameterName);
bool __attribute__ ((visibility("hidden"))) local_object_hasParameters(const object_t *object);
uint32_t __attribute__ ((visibility("hidden"))) local_object_parameterCount(object_t *object);

// functions
function_t __attribute__ ((visibility("hidden"))) *local_object_getFunction(object_t *object, const char *functionName);
bool __attribute__ ((visibility("hidden"))) local_object_hasFunctions(const object_t *object);
uint32_t __attribute__ ((visibility("hidden"))) local_object_functionCount(object_t *object);

// operators
bool __attribute__ ((visibility("hidden"))) local_object_update(object_t *object, llist_t *parameters);
bool __attribute__ ((visibility("hidden"))) local_object_commit(object_t *object);
void __attribute__ ((visibility("hidden"))) local_object_rollback(object_t *object);
bool __attribute__ ((visibility("hidden"))) local_object_validate(object_t *object);
#else
// cleanup functions
bool local_object_delete(object_t *object);

// tree functions
object_t *local_object_parent(const object_t *object);
object_t *local_object_getObject(object_t *object, const char *path, const uint32_t pathAttr, bool *ExactMatch);
bool local_object_hasChildren(const object_t *object);
uint32_t local_object_childCount(const object_t *object);
void local_object_path(const object_t *object, string_t *path, const uint32_t pathAttributes);

// instance functions
object_t *local_object_createInstance(object_t *object, uint32_t index, const char *key);
bool local_object_hasInstances(const object_t *object);
uint32_t local_object_instanceCount(const object_t *object);

// parameters
parameter_t *local_object_getParameter(object_t *object, const char *parameterName);
bool local_object_hasParameters(const object_t *object);
uint32_t local_object_parameterCount(const object_t *object);

// functions
function_t *local_object_getFunction(const object_t *object, const char *functionName);
bool local_object_hasFunctions(const object_t *object);
uint32_t local_object_functionCount(object_t *object);

// operators
bool local_object_update(object_t *object, llist_t *parameters);
bool local_object_commit(object_t *object);
void local_object_rollback(object_t *object);
bool local_object_validate(object_t *object);
#endif

#ifdef __cplusplus
}
#endif

#endif
