/****************************************************************************
**
** Copyright (C) 2011 SoftAtHome. All rights reserved.
**
** SoftAtHome reserves all rights not expressly granted herein.
**
** - DISCLAIMER OF WARRANTY -
**
** THIS FILE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
** EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE.
**
** THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOURCE
** CODE IS WITH YOU. SHOULD THE SOURCE CODE PROVE DEFECTIVE, YOU
** ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
**
** - LIMITATION OF LIABILITY -
**
** IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN
** WRITING WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES
** AND/OR DISTRIBUTES THE SOURCE CODE, BE LIABLE TO YOU FOR DAMAGES,
** INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
** ARISING OUT OF THE USE OR INABILITY TO USE THE SOURCE CODE
** (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED
** INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE
** OF THE SOURCE CODE TO OPERATE WITH ANY OTHER PROGRAM), EVEN IF SUCH
** HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
** DAMAGES.
**
****************************************************************************/

#if !defined(PCB_NOTIFICATION_PRIV_H)
#define PCB_NOTIFICATION_PRIV_H

#ifdef __cplusplus
extern "C"
{
#endif

#include <pcb/utils/variant.h>
#include <pcb/core/notification.h>
#include <pcb/utils/linked_list.h>
#include <pcb/utils/string.h>

/**
@file
@brief
Header file with notification type definitions and private notification functions
*/

/**
@ingroup pcb_core_notification
@{
*/

/**
@brief
Definition of the notification parameter structure
*/
struct _notification_parameter {
    string_t name;          /**< Parameter name */
    variant_t value;        /**< Parameter value */
    llist_iterator_t it;    /**< Linked list iterator, used to store the parameter in the linked list of a notification */
};

/**
@brief
Definition of the notification structure
*/
struct _notification {
    string_t objectPath;    /**< Object path, if not set (empty) the notification is not related to a data model or object */
    uint32_t type;          /**< Notification type or identifier (identifiers below 100 or reserved)*/
    string_t name;          /**< Notification name */
    llist_t parameters;     /**< Linked list of parameters */
};

/**
@}
*/

#ifdef __cplusplus
}
#endif

#endif


