/****************************************************************************
**
** Copyright (C) 2011 SoftAtHome. All rights reserved.
**
** SoftAtHome reserves all rights not expressly granted herein.
**
** - DISCLAIMER OF WARRANTY -
**
** THIS FILE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
** EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE.
**
** THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOURCE
** CODE IS WITH YOU. SHOULD THE SOURCE CODE PROVE DEFECTIVE, YOU
** ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
**
** - LIMITATION OF LIABILITY -
**
** IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN
** WRITING WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES
** AND/OR DISTRIBUTES THE SOURCE CODE, BE LIABLE TO YOU FOR DAMAGES,
** INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
** ARISING OUT OF THE USE OR INABILITY TO USE THE SOURCE CODE
** (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED
** INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE
** OF THE SOURCE CODE TO OPERATE WITH ANY OTHER PROGRAM), EVEN IF SUCH
** HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
** DAMAGES.
**
****************************************************************************/

#if !defined(PCB_MAIN_PRIV_H)
#define PCB_MAIN_PRIV_H

#ifdef __cplusplus
extern "C"
{
#endif

/**
@file
@brief
*/

#include <pcb/core/pcb_main.h>

#include "request_priv.h"
#include "datamodel_priv.h"

typedef struct _pcb_client_data {
    uint32_t attributes;
    uint32_t error;
    uint32_t timeout;
    uint32_t uid;
    llist_t  errors;
} pcb_client_data_t;

/**
@brief

@details
*/
struct _pcb {
    connection_info_t connection;
    // data model related stuff
    datamodel_t datamodel;                   /**< The connection's datamodel */
    datamodel_t cache;                       /**< Remote object cache */
    // the request handlers
    request_handlers_t *reqHandlers;         /**< The connection's request handlers, set of functions which are called when there is an incomming request */
    // default serializer
    serialize_handlers_t *defaultSerializer; /**< The default serializer */

    // arguments passed with pcb_create or pcb_initialize
    int argc;
    char **argv;

    // user data
    void *userdata;

    // plugin config data
    plugin_config_t *plugin_config;

    // plugin config data
    client_config_t *client_config;

    // client config data
    pcb_client_data_t client_data;

    peer_info_t *peer;                       /**< When set only replies are handled for this peer */
    uint32_t request_id;                     /**< When set only replies for requests with this id are handled */

    pcb_cleanup_handler_t cleanup;
};

typedef struct _pcb_peer_data {
    serialize_handlers_t *serializers;      /**< Serialization functions */
    request_handlers_t *reqhandlers;        /**< The request handler functions */
    request_t zero_request;                 /**< All notifications send with request id = 0, will be captured here */
    uint32_t last_used_req_id;              /**< Last used request id */
    uint32_t last_used_nreq_id;             /**< Last used notify request id */
    llist_t pendingRequests;                /**< List of sended pending requests */
    llist_t receivedRequests;               /**< List of received requests */
    llist_t notifyRequests;                 /**< List of persistent requests aka notify requests */
    llist_t forwardedRequests;              /**< List of forwarded requests */
    void *deserializeInfo;                  /**< Information stucture for deserializer */
    void *serializeInfo;                    /**< Information stucture for serializer */
    peer_info_t *peer;                      /**< Peer where this data is attached to */
    uint32_t UID;                           /**< User ID */
} pcb_peer_data_t;

#ifdef __cplusplus
}
#endif

#endif
