/****************************************************************************
**
** Copyright (C) 2011 SoftAtHome. All rights reserved.
**
** SoftAtHome reserves all rights not expressly granted herein.
**
** - DISCLAIMER OF WARRANTY -
**
** THIS FILE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
** EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE.
**
** THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOURCE
** CODE IS WITH YOU. SHOULD THE SOURCE CODE PROVE DEFECTIVE, YOU
** ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
**
** - LIMITATION OF LIABILITY -
**
** IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN
** WRITING WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES
** AND/OR DISTRIBUTES THE SOURCE CODE, BE LIABLE TO YOU FOR DAMAGES,
** INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
** ARISING OUT OF THE USE OR INABILITY TO USE THE SOURCE CODE
** (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED
** INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE
** OF THE SOURCE CODE TO OPERATE WITH ANY OTHER PROGRAM), EVEN IF SUCH
** HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
** DAMAGES.
**
****************************************************************************/

#if !defined(PCB_REMOTE_OBJECT_H)
#define PCB_REMOTE_OBJECT_H

#ifdef __cplusplus
extern "C"
{
#endif

#include <pcb/utils/string.h>
#include <pcb/core/serialize.h>

typedef struct _remote_object_info {
    string_t parentPathKey;
    string_t parentPathIndex;
    string_t key;
    uint32_t parameterCount;
    uint32_t childCount;
    uint32_t instanceCount;
    uint32_t functionCount;
    void *userdata;
    serialization_object_destroy_handler_t destroy;
    int fd;
} remote_object_info_t;

// cleanup functions
bool __attribute__ ((visibility("hidden"))) remote_object_delete(object_t *object);

// tree functions
object_t __attribute__ ((visibility("hidden"))) *remote_object_root(object_t *object);
object_t __attribute__ ((visibility("hidden"))) *remote_object_getObject(object_t *object, const char *path, const uint32_t pathAttr, bool *ExactMatch, remote_object_info_t *ri);
bool __attribute__ ((visibility("hidden"))) remote_object_hasChildren(const object_t *object);
uint32_t __attribute__ ((visibility("hidden"))) remote_object_childCount(const object_t *object);
void __attribute__ ((visibility("hidden"))) remote_object_path(const object_t *object, string_t *path, const uint32_t pathAttributes);

// instance functions
object_t __attribute__ ((visibility("hidden"))) *remote_object_createInstance(object_t *object, uint32_t index, const char *key);
bool __attribute__ ((visibility("hidden"))) remote_object_hasInstances(const object_t *object);
uint32_t __attribute__ ((visibility("hidden"))) remote_object_instanceCount(const object_t *object);

// parameters
parameter_t __attribute__ ((visibility("hidden"))) *remote_object_getParameter(const object_t *object, const char *parameterName);
bool __attribute__ ((visibility("hidden"))) remote_object_hasParameters(const object_t *object);
uint32_t __attribute__ ((visibility("hidden"))) remote_object_parameterCount(const object_t *object);

// functions
function_t __attribute__ ((visibility("hidden"))) *remote_object_getFunction(object_t *object, const char *functionName);
bool __attribute__ ((visibility("hidden"))) remote_object_hasFunctions(const object_t *object);
uint32_t __attribute__ ((visibility("hidden"))) remote_object_functionCount(const object_t *object);

object_t __attribute__ ((visibility("hidden"))) *object_searchCache(object_t *object, string_t *fullpath, uint32_t pathAttr, bool *exactMatch);

#ifdef __cplusplus
}
#endif

#endif
