/****************************************************************************
**
** Copyright (C) 2011 SoftAtHome. All rights reserved.
**
** SoftAtHome reserves all rights not expressly granted herein.
**
** - DISCLAIMER OF WARRANTY -
**
** THIS FILE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
** EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE.
**
** THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOURCE
** CODE IS WITH YOU. SHOULD THE SOURCE CODE PROVE DEFECTIVE, YOU
** ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
**
** - LIMITATION OF LIABILITY -
**
** IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN
** WRITING WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES
** AND/OR DISTRIBUTES THE SOURCE CODE, BE LIABLE TO YOU FOR DAMAGES,
** INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
** ARISING OUT OF THE USE OR INABILITY TO USE THE SOURCE CODE
** (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED
** INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE
** OF THE SOURCE CODE TO OPERATE WITH ANY OTHER PROGRAM), EVEN IF SUCH
** HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
** DAMAGES.
**
****************************************************************************/

#if !defined(PCB_REQUEST_PRIV_H)
#define PCB_REQUEST_PRIV_H

#ifdef __cplusplus
extern "C"
{
#endif

#include <pcb/core/request.h>

#include "reply_priv.h"
#include "datamodel_priv.h"

typedef struct _param_value {
    char *parameterName;
    variant_t value;
    llist_iterator_t it;
} param_value_t;

struct _request {
    uint32_t request_id;
    request_type_t type;
    request_states_t state;
    reply_t reply;
    request_replyHandler replyHandler;
    request_doneHandler doneHandler;
    request_cancelHandler cancelHandler;
    request_destroyHandler destroyHandler;
    void *userdata;
    void *serdata;                      /**< Data stored by (de)serializer */
    void *waitdata;                     /**< Data stored by wait mechanizm */
    request_cleanupSerializationData cleanupSerializationHandler;
    llist_iterator_t it;                /**< Generic list iterator, is used to store notify request in objects, pending request list or received request list */
    llist_iterator_t forwarded_it;      /**< List iterator used to store the request in the peer_info_t forwarded list */
    llist_iterator_t notify_it;         /**< List iterator used to store the request in the peer_info_t structure  */
    llist_iterator_t list_it;           /**< List iterator used to store the request in a request list */
    tree_item_t child_requests;         /**< Child requests */
};

typedef struct _request_object {
    request_t request;
    uint32_t UID;
    uint32_t sourcePID;                /** Process ID of the process originally created the request */
    char *objectPath;
    char *pattern;
    uint32_t attributes;
    uint32_t depth;
    uint32_t instanceIndex;
    char *instanceKey;
    char *functionName;
    llist_t parameters;
    datamodel_t *datamodel;
} request_object_t;

typedef struct _request_close {
    request_t request;
    uint32_t id;
} request_close_t;

typedef struct _request_open_session {
    request_t request;
    char *username;
} request_open_session_t;

#ifdef __cplusplus
}
#endif

#endif


