/****************************************************************************
**
** Copyright (C) 2011 SoftAtHome. All rights reserved.
**
** SoftAtHome reserves all rights not expressly granted herein.
**
** - DISCLAIMER OF WARRANTY -
**
** THIS FILE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
** EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE.
**
** THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOURCE
** CODE IS WITH YOU. SHOULD THE SOURCE CODE PROVE DEFECTIVE, YOU
** ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
**
** - LIMITATION OF LIABILITY -
**
** IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN
** WRITING WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES
** AND/OR DISTRIBUTES THE SOURCE CODE, BE LIABLE TO YOU FOR DAMAGES,
** INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
** ARISING OUT OF THE USE OR INABILITY TO USE THE SOURCE CODE
** (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED
** INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE
** OF THE SOURCE CODE TO OPERATE WITH ANY OTHER PROGRAM), EVEN IF SUCH
** HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
** DAMAGES.
**
****************************************************************************/

#if !defined(PCB_VALIDATOR_PRIV_H)
#define PCB_VALIDATOR_PRIV_H

#ifdef __cplusplus
extern "C"
{
#endif

/**
@file
@brief
Header file with private validator definitions
*/

#include <pcb/utils/variant.h>
#include <pcb/core/validator.h>

/**
@brief
Structure used to store the minimum and maximum of a range validator
*/
typedef struct _parameter_range {
    variant_t minimum;          /**< The minimum of the range */
    variant_t maximum;          /**< The maximum of the range */
} parameter_range_t;

/**
@brief
Structure used to store the parameter validator
*/
struct _parameter_validator {
    parameter_validator_type_t type; /**< The validation type */
    union {
        void *data;                  /**< Used for custom validation */
        variant_list_t *values;      /**< Used for enum validations */
        parameter_range_t range;     /**< Used for range validation */
        variant_t minimum;           /**< Used for minimum validation */
        variant_t maximum;           /**< Used for maximum validation */
    } data;
    param_validation_handler_t validate;
};

/**
@brief
Structure used to store the object validator
*/
struct _object_validator {
    void * data;
    object_validation_handler_t validate;
};

#ifdef __cplusplus
}
#endif

#endif
