#include <stdlib.h>
#include <stdio.h>
#include <pcb/utils/linked_list.h>
#include <unittests/unittest.h>

typedef struct _int32_list_item {
    uint32_t data;
    llist_iterator_t item;
} int32_list_item_t;

static int32_list_item_t *int32_list_item_create(uint32_t value) {
    int32_list_item_t *item = (int32_list_item_t *)calloc(1,sizeof(int32_list_item_t));
    llist_iterator_initialize(&item->item);
    if (item) {
        item->data = value;
    }
    return item;
}

static int create_test_items(uint32_t nr, int32_list_item_t *items[]) {
    T_TESTCASE_BEGIN();
    uint32_t i = 0;
    for(i = 0; i < nr; i++) {
        items[i] = int32_list_item_create(i);
        T_VERIFY(items[i] != NULL);
    }
cleanup:
    T_TESTCASE_END();
}

int test_ll_insertBefore() {
    T_TESTCASE_BEGIN();
    llist_t list;
    int32_list_item_t *items[5];
    llist_iterator_t *verify = NULL;
    int32_list_item_t *data = NULL;

    T_VERIFY(llist_initialize(&list) == true);
    T_VERIFY(llist_size(&list) == 0);
    T_VERIFY(llist_isEmpty(&list) == true);

    create_test_items(5, items);
    llist_append(&list,&items[0]->item);
    for(uint32_t i = 1; i < 5; i++) {
        T_VERIFY(llist_insertBefore(&list, llist_first(&list),&items[i]->item) == true);
    }

    uint32_t i = 4;
    llist_for_each(verify,&list) {
        data = llist_item_data(verify,int32_list_item_t,item);
        printf("index = %d",i);
        T_VERIFY(data == items[i]);
        T_VERIFY(data->data == i);
        i--;
    }

    llist_cleanup(&list);
    for(uint32_t i = 0; i < 5; i++) {
        free(items[i]);
    }
cleanup:
    T_TESTCASE_END();
}

int test_ll_insertAfter() {
    T_TESTCASE_BEGIN();
    llist_t list;
    int32_list_item_t *items[5];
    llist_iterator_t *verify = NULL;
    int32_list_item_t *data = NULL;

    T_VERIFY(llist_initialize(&list) == true);
    T_VERIFY(llist_size(&list) == 0);
    T_VERIFY(llist_isEmpty(&list) == true);

    create_test_items(5, items);
    llist_append(&list,&items[0]->item);
    for(uint32_t i = 0; i < 5; i++) {
        T_VERIFY(llist_insertAfter(&list,llist_last(&list),&items[i]->item) == true);
    }

    uint32_t i = 0;
    llist_for_each(verify,&list) {
        data = llist_item_data(verify,int32_list_item_t,item);
        T_VERIFY(data == items[i]);
        T_VERIFY(data->data == i);
        i++;
    }

    llist_cleanup(&list);
    for(uint32_t i = 0; i < 5; i++) {
        free(items[i]);
    }
cleanup:
    T_TESTCASE_END();
}

int test_ll_append() {
    T_TESTCASE_BEGIN();
    llist_t list;
    int32_list_item_t *items[5];
    llist_iterator_t *verify = NULL;
    int32_list_item_t *data = NULL;

    T_VERIFY(llist_initialize(&list) == true);
    T_VERIFY(llist_size(&list) == 0);
    T_VERIFY(llist_isEmpty(&list) == true);

    create_test_items(5, items);
    for(uint32_t i = 0; i < 5; i++) {
        T_VERIFY(llist_append(&list,&items[i]->item) == true);
    }
    T_VERIFY(llist_size(&list) == 5);
    T_VERIFY(llist_isEmpty(&list) == false);

    uint32_t i = 0;
    llist_for_each(verify,&list) {
        data = llist_item_data(verify,int32_list_item_t,item);
        T_VERIFY(data == items[i]);
        T_VERIFY(data->data == i);
        i++;
    }

    llist_cleanup(&list);
    for(uint32_t i = 0; i < 5; i++) {
        free(items[i]);
    }
cleanup:
    T_TESTCASE_END();
}

int test_ll_prepend() {
    T_TESTCASE_BEGIN();
    llist_t list;
    int32_list_item_t *items[5];
    llist_iterator_t *verify = NULL;
    int32_list_item_t *data = NULL;

    T_VERIFY(llist_initialize(&list) == true);
    T_VERIFY(llist_size(&list) == 0);
    T_VERIFY(llist_isEmpty(&list) == true);

    create_test_items(5, items);
    for(uint32_t i = 0; i < 5; i++) {
        T_VERIFY(llist_prepend(&list,&items[i]->item) == true);
    }
    T_VERIFY(llist_size(&list) == 5);
    T_VERIFY(llist_isEmpty(&list) == false);

    uint32_t i = 0;
    llist_for_each_reverse(verify,&list) {
        data = llist_item_data(verify,int32_list_item_t,item);
        T_VERIFY(data == items[i]);
        T_VERIFY(data->data == i);
        i++;
    }

    llist_cleanup(&list);
    for(uint32_t i = 0; i < 5; i++) {
        free(items[i]);
    }
cleanup:
    T_TESTCASE_END();
}

int test_ll_at() {
    T_TESTCASE_BEGIN();
    llist_t list;
    int32_list_item_t *items[5];
    llist_iterator_t *verify = NULL;
    int32_list_item_t *data = NULL;

    T_VERIFY(llist_initialize(&list) == true);
    T_VERIFY(llist_size(&list) == 0);
    T_VERIFY(llist_isEmpty(&list) == true);

    create_test_items(5, items);
    for(uint32_t i = 0; i < 5; i++) {
        T_VERIFY(llist_append(&list,&items[i]->item) == true);
    }
    T_VERIFY(llist_size(&list) == 5);
    T_VERIFY(llist_isEmpty(&list) == false);

    for(uint32_t i = 0; i < 5; i++) {
        verify = llist_at(&list,i);
        data = llist_item_data(verify,int32_list_item_t,item);
        T_VERIFY(data == items[i]);
        T_VERIFY(data->data == i);
    }

    llist_cleanup(&list);
    for(uint32_t i = 0; i < 5; i++) {
        free(items[i]);
    }
cleanup:
    T_TESTCASE_END();
}

int test_ll_insertAt() {
    T_TESTCASE_BEGIN();
    llist_t list;
    int32_list_item_t *items[5];
    llist_iterator_t *verify = NULL;
    int32_list_item_t *data = NULL;

    T_VERIFY(llist_initialize(&list) == true);
    T_VERIFY(llist_size(&list) == 0);
    T_VERIFY(llist_isEmpty(&list) == true);

    create_test_items(5, items);
    for(uint32_t i = 0; i < 5; i++) {
        T_VERIFY(llist_insertAt(&list,i,&items[i]->item) == true);
    }
    T_VERIFY(llist_size(&list) == 5);
    T_VERIFY(llist_isEmpty(&list) == false);

    for(uint32_t i = 0; i < 5; i++) {
        verify = llist_at(&list,i);
        data = llist_item_data(verify,int32_list_item_t,item);
        T_VERIFY(data == items[i]);
        T_VERIFY(data->data == i);
    }

    llist_cleanup(&list);
    for(uint32_t i = 0; i < 5; i++) {
        free(items[i]);
    }
cleanup:
    T_TESTCASE_END();
}

int test_ll_take() {
    T_TESTCASE_BEGIN();
    llist_t list;
    int32_list_item_t *items[5];
    llist_iterator_t *verify = NULL;
    int32_list_item_t *data = NULL;

    T_VERIFY(llist_initialize(&list) == true);
    T_VERIFY(llist_size(&list) == 0);
    T_VERIFY(llist_isEmpty(&list) == true);

    create_test_items(5, items);
    for(uint32_t i = 0; i < 5; i++) {
        T_VERIFY(llist_append(&list,&items[i]->item) == true);
    }
    T_VERIFY(llist_size(&list) == 5);
    T_VERIFY(llist_isEmpty(&list) == false);

    for(uint32_t i = 0; i < 5; i++) {
        verify = llist_iterator_take(llist_first(&list));
        data = llist_item_data(verify,int32_list_item_t,item);
        T_VERIFY(data == items[i]);
        T_VERIFY(data->data == i);
    }
    T_VERIFY(llist_size(&list) == 0);
    T_VERIFY(llist_isEmpty(&list) == true);

    llist_cleanup(&list);
    for(uint32_t i = 0; i < 5; i++) {
        free(items[i]);
    }
cleanup:
    T_TESTCASE_END();
}

int test_ll_takeFirst() {
    T_TESTCASE_BEGIN();
    llist_t list;
    int32_list_item_t *items[5];
    llist_iterator_t *verify = NULL;
    int32_list_item_t *data = NULL;

    T_VERIFY(llist_initialize(&list) == true);
    T_VERIFY(llist_size(&list) == 0);
    T_VERIFY(llist_isEmpty(&list) == true);

    create_test_items(5, items);
    for(uint32_t i = 0; i < 5; i++) {
        T_VERIFY(llist_append(&list,&items[i]->item) == true);
    }
    T_VERIFY(llist_size(&list) == 5);
    T_VERIFY(llist_isEmpty(&list) == false);

    for(uint32_t i = 0; i < 5; i++) {
        verify = llist_takeFirst(&list);
        data = llist_item_data(verify,int32_list_item_t,item);
        T_VERIFY(data == items[i]);
        T_VERIFY(data->data == i);
    }
    T_VERIFY(llist_size(&list) == 0);
    T_VERIFY(llist_isEmpty(&list) == true);

    llist_cleanup(&list);
    for(uint32_t i = 0; i < 5; i++) {
        free(items[i]);
    }
cleanup:
    T_TESTCASE_END();
}

int test_ll_takeLast() {
    T_TESTCASE_BEGIN();
    llist_t list;
    int32_list_item_t *items[5];
    llist_iterator_t *verify = NULL;
    int32_list_item_t *data = NULL;

    T_VERIFY(llist_initialize(&list) == true);
    T_VERIFY(llist_size(&list) == 0);
    T_VERIFY(llist_isEmpty(&list) == true);

    create_test_items(5, items);
    for(uint32_t i = 0; i < 5; i++) {
        T_VERIFY(llist_append(&list,&items[i]->item) == true);
    }
    T_VERIFY(llist_size(&list) == 5);
    T_VERIFY(llist_isEmpty(&list) == false);

    for(uint32_t i = 5; i > 0; i--) {
        verify = llist_takeLast(&list);
        data = llist_item_data(verify,int32_list_item_t,item);
        T_VERIFY(data == items[i - 1]);
        T_VERIFY(data->data == (i - 1));
    }
    T_VERIFY(llist_size(&list) == 0);
    T_VERIFY(llist_isEmpty(&list) == true);

    llist_cleanup(&list);
    for(uint32_t i = 0; i < 5; i++) {
        free(items[i]);
    }
cleanup:
    T_TESTCASE_END();
}

int test_ll_null() {
    T_TESTCASE_BEGIN();
    llist_t list;
    llist_t *test = NULL;
    T_VERIFY(llist_initialize(&list) == true);

    T_VERIFY(llist_initialize(NULL) == false);
    T_VERIFY(llist_iterator_initialize(NULL) == false);
    llist_cleanup(test);
    T_VERIFY(llist_first(NULL) == NULL);
    T_VERIFY(llist_last(NULL) == NULL);
    T_VERIFY(llist_at(NULL,100) == NULL);
    T_VERIFY(llist_iterator_next(NULL) == NULL);
    T_VERIFY(llist_iterator_prev(NULL) == NULL);
    T_VERIFY(llist_insertBefore(NULL,NULL,NULL) == false);
    T_VERIFY(llist_insertAfter(NULL, NULL,NULL) == false);
    T_VERIFY(llist_insertAfter(&list, llist_last(&list),NULL) == false);
    T_VERIFY(llist_insertBefore(&list, llist_first(&list),NULL) == false);
    T_VERIFY(llist_append(NULL,NULL) == false);
    T_VERIFY(llist_append(&list,NULL) == false);
    T_VERIFY(llist_prepend(NULL,NULL) == false);
    T_VERIFY(llist_prepend(&list,NULL) == false);
    T_VERIFY(llist_insertAt(NULL,100,NULL) == false);
    T_VERIFY(llist_insertAt(&list,100,NULL) == false);
    T_VERIFY(llist_iterator_take(NULL) == NULL);
    T_VERIFY(llist_takeFirst(NULL) == NULL);
    T_VERIFY(llist_takeLast(NULL) == NULL);
    T_VERIFY(llist_size(NULL) == 0);
    T_VERIFY(llist_isEmpty(NULL) == true);

    llist_cleanup(&list);
cleanup:
    T_TESTCASE_END();
}

int test_ll_twoLists() {
    T_TESTCASE_BEGIN();
    llist_t list1;
    llist_t list2;
    int32_list_item_t *items1[5];
    int32_list_item_t *items2[5];
    llist_iterator_t *verify = NULL;
    int32_list_item_t *data = NULL;

    //create the lists
    T_VERIFY(llist_initialize(&list1) == true);
    T_VERIFY(llist_initialize(&list2) == true);

    T_VERIFY(llist_size(&list1) == 0);
    T_VERIFY(llist_size(&list2) == 0);

    T_VERIFY(llist_isEmpty(&list1) == true);
    T_VERIFY(llist_isEmpty(&list2) == true);

    //create the list items and append them to the lists
    create_test_items(5, items1);
    for(uint32_t i = 0; i < 5; i++) {
        T_VERIFY(llist_append(&list1,&items1[i]->item) == true);
    }
    T_VERIFY(llist_size(&list1) == 5);
    T_VERIFY(llist_isEmpty(&list1) == false);

    create_test_items(5, items2);
    for(uint32_t i = 0; i < 5; i++) {
        T_VERIFY(llist_append(&list2,&items2[i]->item) == true);
    }
    T_VERIFY(llist_size(&list2) == 5);
    T_VERIFY(llist_isEmpty(&list2) == false);

    //move one item out of the first list and place it at the end of the first list
    T_VERIFY(llist_append(&list1,&items1[0]->item) == true);
    T_VERIFY(llist_size(&list1) == 5);
    T_VERIFY(llist_isEmpty(&list1) == false);

    //check the tree (go one item 2 far!)
    for(uint32_t i = 0; i < 5; i++) {
        verify = llist_at(&list1,i);
        data = llist_item_data(verify,int32_list_item_t,item);
        if(i < 4) {
          T_VERIFY(data == items1[i+1]);
          T_VERIFY(data->data == i+1);
        }
        else {
          T_VERIFY(data == items1[0]);
          T_VERIFY(data->data == 0);
        }
    }

    //move one item of the first list to the second list
    T_VERIFY(llist_append(&list2,&items1[0]->item) == true);
    T_VERIFY(llist_size(&list1) == 4);
    T_VERIFY(llist_isEmpty(&list1) == false);
    T_VERIFY(llist_size(&list2) == 6);
    T_VERIFY(llist_isEmpty(&list2) == false);

    //check both trees
    for(uint32_t i = 0; i < 4; i++) {
        verify = llist_at(&list1,i);
        data = llist_item_data(verify,int32_list_item_t,item);
        T_VERIFY(data == items1[i+1]);
        T_VERIFY(data->data == i+1);
    }

    for(uint32_t i = 0; i < 6; i++) {
        verify = llist_at(&list2,i);
        data = llist_item_data(verify,int32_list_item_t,item);
        if(i < 5) {
          T_VERIFY(data == items2[i]);
          T_VERIFY(data->data == i);
        }
        else {
          T_VERIFY(data == items1[0]);
          T_VERIFY(data->data == 0);
        }
    }

    llist_cleanup(&list1);
    llist_cleanup(&list2);
    //cleanup
    for(uint32_t i = 0; i < 5; i++) {
        free(items1[i]);
        free(items2[i]);
    }
cleanup:
    T_TESTCASE_END();
}

T_TESTCASES_BEGIN()
    T_TESTCASE(test_ll_insertBefore, "Test inserting before a reference item")
    T_TESTCASE(test_ll_insertAfter, "Test inserting after a reference item")
    T_TESTCASE(test_ll_append, "Test appending items")
    T_TESTCASE(test_ll_prepend, "Test prepending items")
    T_TESTCASE(test_ll_at, "Test get an item at a specific index")
    T_TESTCASE(test_ll_insertAt, "Test inserting an item at a specific index")
    T_TESTCASE(test_ll_take, "Test removing an item from the list")
    T_TESTCASE(test_ll_takeFirst, "Test removing of the first item")
    T_TESTCASE(test_ll_takeLast, "Test removing of the last item")
    T_TESTCASE(test_ll_null, "Test passing null pointer to the linked list functions")
    T_TESTCASE(test_ll_twoLists, "Test moving items from one list to another")
T_TESTCASES_END()
