#define _XOPEN_SOURCE
#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <unittests/unittest.h>

#include <pcb/utils/string.h>
#include <pcb/utils/string_list.h>

int test_string_append() {
    T_TESTCASE_BEGIN();
    string_t string;
    string_t string2;

    string_initialize(&string,64);
    string_initialize(&string2,64);

    string_appendChar(&string,"hallo");
    T_VERIFY (string_compareChar(&string,"hallo",CaseSensitive) == 0);
    string_appendChar(&string," world");
    T_VERIFY (string_compareChar(&string,"hallo world",CaseSensitive) == 0);

    string_cleanup(&string);
    string_fromChar(&string, "Hallo");
    T_VERIFY (string_compareChar(&string,"Hallo",CaseSensitive) == 0);
    string_appendChar(&string," World");
    T_VERIFY (string_compareChar(&string,"Hallo World",CaseSensitive) == 0);

    string_fromChar(&string, "Star");
    T_VERIFY (string_compareChar(&string,"Star",CaseSensitive) == 0);
    string_fromChar(&string2, " Trek");
    T_VERIFY (string_compareChar(&string2," Trek",CaseSensitive) == 0);
    string_append(&string,&string2);
    T_VERIFY (string_compareChar(&string,"Star Trek",CaseSensitive) == 0);

cleanup:
    string_cleanup(&string);
    string_cleanup(&string2);
    T_TESTCASE_END();
}

int test_string_prepend() {
    T_TESTCASE_BEGIN();
    string_t string;
    string_t string2;

    string_initialize(&string,64);
    string_initialize(&string2,64);

    string_prependChar(&string,"world");
    T_VERIFY (string_compareChar(&string,"world",CaseSensitive) == 0);
    string_prependChar(&string,"hallo ");
    T_VERIFY (string_compareChar(&string,"hallo world",CaseSensitive) == 0);

    string_cleanup(&string);
    string_fromChar(&string, "World");
    T_VERIFY (string_compareChar(&string,"World",CaseSensitive) == 0);
    string_prependChar(&string,"Peter's ");
    T_VERIFY (string_compareChar(&string,"Peter's World",CaseSensitive) == 0);

    string_fromChar(&string, "Stars");
    T_VERIFY (string_compareChar(&string,"Stars",CaseSensitive) == 0);
    string_fromChar(&string2, "A World Of ");
    T_VERIFY (string_compareChar(&string2,"A World Of ",CaseSensitive) == 0);
    string_prepend(&string,&string2);
    T_VERIFY (string_compareChar(&string,"A World Of Stars",CaseSensitive) == 0);

cleanup:
    string_cleanup(&string);
    string_cleanup(&string2);
    T_TESTCASE_END();
}

int test_string_left_mid_right() {
    T_TESTCASE_BEGIN();
    string_t string;
    string_t test;

    string_initialize(&string,64);
    string_initialize(&test,64);

    string_fromChar(&string,"0123456789ABCDEFGHIJ0123456789");
    string_mid(&test, &string,5,6);
    T_VERIFY (string_compareChar(&test,"56789a",CaseInsensitive) == 0);

    string_mid(&test, &string,9,12);
    T_VERIFY (string_compareChar(&test,"9abcdefghij0",CaseInsensitive) == 0);

    string_mid(&test, &string,19,-1);
    T_VERIFY (string_compareChar(&test,"J0123456789",CaseSensitive) == 0);

    string_left(&test, &string,11);
    T_VERIFY (string_compareChar(&test,"0123456789A",CaseSensitive) == 0);
    T_VERIFY (string_compareChar(&test,"0123456789a",CaseInsensitive) == 0);

    string_right(&test, &string,11);
    T_VERIFY (string_compareChar(&test,"j0123456789",CaseInsensitive) == 0);
    T_VERIFY (string_compareChar(&test,"J0123456789",CaseSensitive) == 0);

cleanup:
    string_cleanup(&string);
    string_cleanup(&test);
    T_TESTCASE_END();
}

int test_string_split_join() {
    T_TESTCASE_BEGIN();
    string_t string;
    string_list_iterator_t *str_it = NULL;
    string_list_t strlist;
    string_t test;
    string_t *strpart = NULL;
    string_t empty;

    string_initialize(&empty,0);
    string_initialize(&string,64);
    string_initialize(&test,64);
    string_list_initialize(&strlist);

    string_fromChar(&string, "1,2,3,4,5,,6,7,8,9,10,11");

    string_list_split(&strlist, &string,",",KeepEmptyParts,CaseInsensitive);
    T_VERIFY (string_list_size(&strlist) == 12);

    uint32_t index = 1;
    string_list_for_each(str_it, &strlist) {
        strpart = string_list_iterator_data(str_it);
        if (!string_isEmpty(strpart)) {
            T_VERIFY (string_toUInt32(strpart) == index);
            index++;
        }
    }
    string_list_clear(&strlist);

    string_list_split(&strlist, &string,",",SkipEmptyParts,CaseInsensitive);
    T_VERIFY (llist_size(&strlist) == 11);

    index = 1;
     string_list_for_each(str_it, &strlist) {
        strpart = string_list_iterator_data(str_it);
        T_VERIFY (string_toUInt32(strpart) == index);
        index++;
    }

    string_join(&test, &strlist,"+-+");
    T_VERIFY (string_compareChar(&test,"1+-+2+-+3+-+4+-+5+-+6+-+7+-+8+-+9+-+10+-+11",CaseInsensitive) == 0);

    string_list_clear(&strlist);
    T_VERIFY(string_list_isEmpty(&strlist));

    string_join(&empty, &strlist,"+-+");
    T_VERIFY(string_isEmpty(&empty));

cleanup:
    string_cleanup(&empty);
    string_list_cleanup(&strlist);
    string_cleanup(&test);
    string_cleanup(&string);
    T_TESTCASE_END();
}

int test_string_find_contains() {
    T_TESTCASE_BEGIN();
    string_t string;
    string_t needle;

    string_initialize(&string,64);
    string_initialize(&needle,64);
    //                       01234567890123456789012345678901234567890123456789
    string_fromChar(&string,"HaystackHaystackHaystackNeedleHaystackHaystackGold");
    string_fromChar(&needle,"GOLD");
    T_VERIFY (string_containsChar(&string,"Needle",CaseSensitive) == true);
    T_VERIFY (string_containsChar(&string,"needle",CaseSensitive) == false);
    T_VERIFY (string_containsChar(&string,"needle",CaseInsensitive) == true);
    T_VERIFY (string_containsChar(&string,"stars",CaseInsensitive) == false);
    T_VERIFY (string_contains(&string,&needle,CaseInsensitive) == true);
    T_VERIFY (string_contains(&string,&needle,CaseSensitive) == false);
    T_VERIFY (string_findChar(&string,0,"Needle",CaseSensitive) == 24);
    T_VERIFY (string_findChar(&string,0,"needle",CaseInsensitive) == 24);
    T_VERIFY (string_findChar(&string,25,"Needle",CaseSensitive) == 0xffffffff);
    T_VERIFY (string_findChar(&string,25,"needle",CaseInsensitive) == 0xffffffff);
    T_VERIFY (string_find(&string,0,&needle,CaseSensitive) == 0xffffffff);
    T_VERIFY (string_find(&string,0,&needle,CaseInsensitive) == 46);
    T_VERIFY (string_find(&string,47,&needle,CaseSensitive) == 0xffffffff);
    T_VERIFY (string_find(&string,47,&needle,CaseInsensitive) == 0xffffffff);

cleanup:
    string_cleanup(&string);
    string_cleanup(&needle);
    T_TESTCASE_END();
}

int test_string_replace() {
    T_TESTCASE_BEGIN();
    string_t string1;
    string_t string2;
    string_t string3;

    string_initialize(&string1,64);
    string_initialize(&string2,64);
    string_initialize(&string3,64);

    //string_replaceChar
    string_replaceChar(NULL,"","");
    string_replaceChar(&string1,NULL,"");
    string_replaceChar(&string1,"",NULL);

    string_fromChar(&string1,"");
    string_replaceChar(&string1,"needle","replaced");
    T_VERIFY (string_compareChar(&string1,"",CaseSensitive)==0);

    string_fromChar(&string1,"In this string all spaces should be replaced with dots");
    string_replaceChar(&string1," ",".");
    T_VERIFY (string_compareChar(&string1,"In.this.string.all.spaces.should.be.replaced.with.dots",CaseSensitive)==0);

    string_fromChar(&string1,"In this string \"In\" should be replaced with shrek");
    string_replaceChar(&string1,"In","shrek");
    T_VERIFY (string_compareChar(&string1,"shrek this string \"shrek\" should be replaced with shrek",CaseSensitive)==0);

    string_fromChar(&string1,"In this string Shrek should replace \r\n \"into the cage!\"");
    string_replaceChar(&string1,"\"into the cage!\"","Shrek");
    T_VERIFY (string_compareChar(&string1,"In this string Shrek should replace \r\n Shrek",CaseSensitive)==0);


    string_fromChar(&string1,"Try to pronounce this: When shot at, the dove dove into the bushes.");
    string_fromChar(&string2,"pronounce");
    string_fromChar(&string3,"repeat");
    string_replace(&string1,&string2,&string3);
    T_VERIFY (string_compareChar(&string1,"Try to repeat this: When shot at, the dove dove into the bushes.",CaseSensitive)==0);

cleanup:
    string_cleanup(&string1);
    string_cleanup(&string2);
    string_cleanup(&string3);
    T_TESTCASE_END();
}

int test_string_compare() {
    T_TESTCASE_BEGIN();
    string_t string1;
    string_t string2;

    string_initialize(&string1,64);
    string_initialize(&string2,64);

    //string_compareChar
    T_VERIFY (string_compareChar(NULL,"test",CaseSensitive)==-1);
    T_VERIFY (string_compareChar(&string1,NULL,CaseSensitive)==1);
    T_VERIFY (string_compareChar(NULL,NULL,CaseSensitive)==0);

    string_fromChar(&string1,"test");
    T_VERIFY (string_compareChar(&string1,"test",CaseSensitive)==0);
    T_VERIFY (string_compareChar(&string1,"Test",CaseSensitive)!=0);
    T_VERIFY (string_compareChar(&string1,"a",CaseSensitive)!=0);

    string_fromChar(&string1,"test");
    T_VERIFY (string_compareChar(&string1,"test",CaseInsensitive)==0);
    T_VERIFY (string_compareChar(&string1,"Test",CaseInsensitive)==0);
    T_VERIFY (string_compareChar(&string1,"best",CaseInsensitive)!=0);

    string_fromChar(&string1,"");
    T_VERIFY (string_compareChar(&string1,"test",CaseSensitive)!=0);
    T_VERIFY (string_compareChar(&string1,"",CaseInsensitive)==0);

    //string_compare
    string_fromChar(&string1,"string1");
    string_fromChar(&string2,"string2");

    T_VERIFY (string_compare(NULL,NULL,CaseInsensitive)==0);
    T_VERIFY (string_compare(&string1,NULL,CaseInsensitive)==1);
    T_VERIFY (string_compare(NULL,&string2,CaseInsensitive)==-1);

    T_VERIFY (string_compare(&string1,&string2,CaseInsensitive)!=0);

    string_fromChar(&string2,"String1");
    T_VERIFY (string_compare(&string1,&string2,CaseInsensitive)==0);
    T_VERIFY (string_compare(&string1,&string2,CaseSensitive)!=0);

cleanup:
    string_cleanup(&string1);
    string_cleanup(&string2);
    T_TESTCASE_END();
}

int test_string_toUpper_toLower() {
    T_TESTCASE_BEGIN();
    string_t string;

    string_initialize(&string,64);
    string_fromChar(&string,"abcdefghijklmnopqrstuvwxyz0123456789/*-+!@#$%^&*()");
    string_toUpper(&string);
    T_VERIFY (string_compareChar(&string,"ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789/*-+!@#$%^&*()",CaseSensitive) == 0);
    string_toLower(&string);
    T_VERIFY (string_compareChar(&string,"abcdefghijklmnopqrstuvwxyz0123456789/*-+!@#$%^&*()",CaseSensitive) == 0);

cleanup:
    string_cleanup(&string);
    T_TESTCASE_END();
}

int test_string_conversions() {
    T_TESTCASE_BEGIN();
    string_t string;

    string_initialize(&string,16);

    //test bool conversions
    string_fromChar(&string,"1");
    T_VERIFY (string_toBool(&string)==true);
    string_fromChar(&string,"true");
    T_VERIFY (string_toBool(&string)==true);
    string_fromChar(&string,"TrUe");
    T_VERIFY (string_toBool(&string)==true);
    string_fromChar(&string,"yes");
    T_VERIFY (string_toBool(&string)==true);
    string_fromChar(&string,"YES");
    T_VERIFY (string_toBool(&string)==true);

    T_VERIFY (string_toBool(NULL)==false);
    string_fromChar(&string,"");
    T_VERIFY (string_toBool(&string)==false);
    string_fromChar(&string,"0");
    T_VERIFY (string_toBool(&string)==false);
    string_fromChar(&string,"tru");
    T_VERIFY (string_toBool(&string)==false);
    string_fromChar(&string,"TrUee");
    T_VERIFY (string_toBool(&string)==false);
    string_fromChar(&string,"NO");
    T_VERIFY (string_toBool(&string)==false);
    string_fromChar(&string,"yess");
    T_VERIFY (string_toBool(&string)==false);
    string_fromChar(&string,"YYES");
    T_VERIFY (string_toBool(&string)==false);

    T_VERIFY (string_fromBool(&string,true)==1);
    T_VERIFY (string_compareChar(&string,"1",CaseSensitive) == 0);
    T_VERIFY (string_fromBool(&string,false)==1);
    T_VERIFY (string_compareChar(&string,"0",CaseSensitive) == 0);
    T_VERIFY (string_fromBool(NULL,true)==0);

    //test char conversions
    char *c=NULL;
    T_VERIFY (string_fromChar(&string,NULL)==0);
    T_VERIFY (string_fromChar(&string,"CHARACTERSTRING")==15);
    T_VERIFY (string_compareChar(&string,"CHARACTERSTRING",CaseSensitive) == 0);
    c = string_toChar(&string);
    T_VERIFY (strcmp(c,"CHARACTERSTRING") == 0);
    free(c);
    T_VERIFY (string_fromChar(&string,"CHARACTERSTRINGCHARACTERSTRING")==30);
    T_VERIFY (string_compareChar(&string,"CHARACTERSTRINGCHARACTERSTRING",CaseSensitive) == 0);
    c = string_toChar(&string);
    T_VERIFY (strcmp(c,"CHARACTERSTRINGCHARACTERSTRING") == 0);
    free(c);

    //test int8 conversions
    T_VERIFY (string_fromInt8(&string,0)==1);
    T_VERIFY (string_toInt8(&string)==0);
    T_VERIFY (string_compareChar(&string,"0",CaseSensitive) == 0);
    T_VERIFY (string_fromInt8(&string,127)==3);
    T_VERIFY (string_toInt8(&string)==127);
    T_VERIFY (string_compareChar(&string,"127",CaseSensitive) == 0);
    T_VERIFY (string_fromInt8(&string,-126)==4);
    T_VERIFY (string_toInt8(&string)==-126);
    T_VERIFY (string_compareChar(&string,"-126",CaseSensitive) == 0);

    //test int16 conversions
    T_VERIFY (string_fromInt16(&string,0)==1);
    T_VERIFY (string_toInt16(&string)==0);
    T_VERIFY (string_compareChar(&string,"0",CaseSensitive) == 0);
    T_VERIFY (string_fromInt16(&string,32767)==5);
    T_VERIFY (string_toInt16(&string)==32767);
    T_VERIFY (string_compareChar(&string,"32767",CaseSensitive) == 0);
    T_VERIFY (string_fromInt16(&string,-32766)==6);
    T_VERIFY (string_toInt16(&string)==-32766);
    T_VERIFY (string_compareChar(&string,"-32766",CaseSensitive) == 0);

    //test int32 conversions
    T_VERIFY (string_fromInt32(&string,0)==1);
    T_VERIFY (string_toInt32(&string)==0);
    T_VERIFY (string_compareChar(&string,"0",CaseSensitive) == 0);
    T_VERIFY (string_fromInt32(&string,2147483647)==10);
    T_VERIFY (string_toInt32(&string)==2147483647);
    T_VERIFY (string_compareChar(&string,"2147483647",CaseSensitive) == 0);
    T_VERIFY (string_fromInt32(&string,-2147483646)==11);
    T_VERIFY (string_toInt32(&string)==-2147483646);
    T_VERIFY (string_compareChar(&string,"-2147483646",CaseSensitive) == 0);

    //test int64 conversions
    T_VERIFY (string_fromInt64(&string,0)==1);
    T_VERIFY (string_toInt64(&string)==0);
    T_VERIFY (string_compareChar(&string,"0",CaseSensitive) == 0);
    T_VERIFY (string_fromInt64(&string,9223372036854775807LL)==19);
    T_VERIFY (string_toInt64(&string)==9223372036854775807LL);
    T_VERIFY (string_compareChar(&string,"9223372036854775807",CaseSensitive) == 0);
    T_VERIFY (string_fromInt64(&string,-9223372036854775806LL)==20);
    T_VERIFY (string_toInt64(&string)==-9223372036854775806LL);
    T_VERIFY (string_compareChar(&string,"-9223372036854775806",CaseSensitive) == 0);

    //test uint8 conversions
    T_VERIFY (string_fromUInt8(&string,0)==1);
    T_VERIFY (string_toUInt8(&string)==0);
    T_VERIFY (string_compareChar(&string,"0",CaseSensitive) == 0);
    T_VERIFY (string_fromUInt8(&string,255)==3);
    T_VERIFY (string_toUInt8(&string)==255);
    T_VERIFY (string_compareChar(&string,"255",CaseSensitive) == 0);

    //test uint16 conversions
    T_VERIFY (string_fromUInt16(&string,0)==1);
    T_VERIFY (string_toUInt16(&string)==0);
    T_VERIFY (string_compareChar(&string,"0",CaseSensitive) == 0);
    T_VERIFY (string_fromUInt16(&string,65535)==5);
    T_VERIFY (string_toUInt16(&string)==65535);
    T_VERIFY (string_compareChar(&string,"65535",CaseSensitive) == 0);


    //test uint32 conversions
    T_VERIFY (string_fromUInt32(&string,0)==1);
    T_VERIFY (string_toUInt32(&string)==0);
    T_VERIFY (string_compareChar(&string,"0",CaseSensitive) == 0);
    T_VERIFY (string_fromUInt32(&string,4294967295UL)==10);
    T_VERIFY (string_toUInt32(&string)==4294967295UL);
    T_VERIFY (string_compareChar(&string,"4294967295",CaseSensitive) == 0);


    //test uint64 conversions
    T_VERIFY (string_fromInt64(&string,0)==1);
    T_VERIFY (string_toInt64(&string)==0);
    T_VERIFY (string_compareChar(&string,"0",CaseSensitive) == 0);

    //THIS SHOULD WORK, BUT FAILS FOR AN UNKNOWN REASON
    //qDebug() << string_fromInt64(&string,18446744073709551615ULL) << endl;
    //                                 12354678901235467890
    //T_VERIFY (string_fromInt64(&string,18446744073709551615ULL)==20);
    //T_VERIFY (string_toUInt64(&string)==18446744073709551615ULL);
    //T_VERIFY (string_compareChar(&string,"18446744073709551615",CaseSensitive) == 0);

    //test datetime conversions
    struct tm *datetime = (struct tm *)malloc(sizeof(struct tm));
    char buf[255];

    strptime("2001-11-12 18:31:01Z", "%Y-%m-%d %H:%M:%SZ", datetime);

    T_VERIFY (string_fromTime(NULL,datetime,NULL)==0);
    T_VERIFY (string_fromTime(&string,NULL,NULL)==0);
    T_VERIFY (string_toTime(NULL,NULL)==NULL);

    T_VERIFY (string_fromTime(&string,datetime,NULL)==20);
    free(datetime);
    datetime = string_toTime(&string,NULL);
    strftime(buf, sizeof(buf), "%Y-%m-%d %H:%M:%SZ", datetime);
    T_VERIFY (strcmp("2001-11-12 18:31:01Z",buf)==0);
    free(datetime);

    datetime = (struct tm *)malloc(sizeof(struct tm));
    strptime("2002-12-11 19:43:48", "%Y-%m-%d %H:%M:%S", datetime);
    T_VERIFY (string_fromTime(&string,datetime,"%Y-%m-%d %H:%M:%S")==19);
    free(datetime);
    datetime = string_toTime(&string,"%Y-%m-%d %H:%M:%S");
    strftime(buf, sizeof(buf), "%Y-%m-%d %H:%M:%S", datetime);
    T_VERIFY (strcmp("2002-12-11 19:43:48",buf)==0);
    free(datetime);

    datetime = (struct tm *)calloc(1,sizeof(struct tm));
    T_VERIFY (string_fromTime(&string,datetime,NULL)==20);
    printf("%s\n",string_buffer(&string));
    T_VERIFY (strcmp("0001-01-01T00:00:00Z",string_buffer(&string))==0);
    free(datetime);
    
cleanup:
    string_cleanup(&string);
    T_TESTCASE_END();
}

int test_string_various() {
    T_TESTCASE_BEGIN();
    string_t string1;
    string_t string2;

    string_initialize(&string1,16);
    string_initialize(&string2,16);

    //string_copy
    string_fromChar(&string1,"testString1");
    string_fromChar(&string2,"testString2");

    T_VERIFY (string_copy(NULL,&string2)==0);
    T_VERIFY (string_copy(&string1,NULL)==0);
    T_VERIFY (string_length(&string1)==0);

    string_fromChar(&string1,"testString1");
    T_VERIFY (string_copy(&string1,&string2)==11);
    T_VERIFY (string_compareChar(&string1,"testString2",CaseSensitive) == 0);

    string_fromChar(&string2,"testString2testString2");
    T_VERIFY (string_copy(&string1,&string2)==22);
    T_VERIFY (string_compareChar(&string1,"testString2testString2",CaseSensitive) == 0);

    //string_Buffer
    T_VERIFY (string_buffer(&string2)==string2.buffer);

    //string_isEmpty
    T_VERIFY (string_isEmpty(&string1)==false);
    string_fromChar(&string1,"");
    T_VERIFY (string_isEmpty(&string1)==true);

    //string_isNumeric
    string_fromChar(&string1,"");
    T_VERIFY (string_isNumeric(&string1)==true);
    string_fromChar(&string1,"01234567890123456789");
    T_VERIFY (string_isNumeric(&string1)==true);
    string_fromChar(&string1,".j123456789");
    T_VERIFY (string_isNumeric(&string1)==false);
    string_fromChar(&string1,"123456.123546");
    T_VERIFY (string_isNumeric(&string1)==false);
    string_fromChar(&string1,"123456.12354s");
    T_VERIFY (string_isNumeric(&string1)==false);

    //string_isNull
    T_VERIFY (string_isNull(&string1)==false);
    string_cleanup(&string1);
    T_VERIFY (string_isNull(&string1)==true);

    //create string list & iterators
    string_list_t mylist;
    string_list_initialize(&mylist);
    string_list_iterator_t *it = string_list_iterator_createFromChar("A String");
    T_VERIFY(it);
    T_VERIFY(string_list_prepend(&mylist,it));
    it = string_list_iterator_createFromChar("Other String");
    T_VERIFY(it);
    T_VERIFY(string_list_prepend(&mylist,it));
    it = string_list_iterator_createFromChar("Last String");
    T_VERIFY(it);
    T_VERIFY(string_list_append(&mylist,it));

    it = string_list_at(&mylist,1);
    T_VERIFY(it);
    string_t *strpart = string_list_iterator_data(it);
    T_VERIFY(string_compareChar(strpart,"A String",0) == 0);
    it = string_list_last(&mylist);
    T_VERIFY(it);
    strpart = string_list_iterator_data(it);
    T_VERIFY(string_compareChar(strpart,"Last String",0) == 0);
    it = string_list_iterator_prev(it);
    T_VERIFY(it);
    strpart = string_list_iterator_data(it);
    T_VERIFY(string_compareChar(strpart,"A String",0) == 0);
    it = string_list_takeLast(&mylist);
    T_VERIFY(it);
    strpart = string_list_iterator_data(it);
    T_VERIFY(string_compareChar(strpart,"Last String",0) == 0);
    string_list_iterator_destroy(it);
    string_list_cleanup(&mylist);


cleanup:
    string_cleanup(&string2);
    T_TESTCASE_END();
}

int test_string_trim() {
    T_TESTCASE_BEGIN();
    string_t string;
    string_initialize(&string,64);

    //string without any trim character
    string_fromChar(&string,"0123456789ABCDEFGHIJ0123456789");
    string_trimLeft(&string);
    T_VERIFY (string_compareChar(&string,"0123456789ABCDEFGHIJ0123456789",CaseSensitive) == 0);
    string_trimRight(&string);
    T_VERIFY (string_compareChar(&string,"0123456789ABCDEFGHIJ0123456789",CaseSensitive) == 0);
    string_trim(&string);
    T_VERIFY (string_compareChar(&string,"0123456789ABCDEFGHIJ0123456789",CaseSensitive) == 0);

    //empty string
    string_fromChar(&string,"");
    string_trimLeft(&string);
    T_VERIFY (string_compareChar(&string,"",CaseSensitive) == 0);
    string_trimRight(&string);
    T_VERIFY (string_compareChar(&string,"",CaseSensitive) == 0);
    string_trim(&string);
    T_VERIFY (string_compareChar(&string,"",CaseSensitive) == 0);

    //null string
    string_trimLeft(NULL);
    string_trimRight(NULL);
    string_trim(NULL);

    //trim left
    string_fromChar(&string," \t\v\n\r trim left");
    string_trimLeft(&string);
    T_VERIFY (string_compareChar(&string,"trim left",CaseSensitive) == 0);
    string_fromChar(&string," \t\v\n\r trim left");
    string_trimRight(&string);
    T_VERIFY (string_compareChar(&string," \t\v\n\r trim left",CaseSensitive) == 0);
    string_fromChar(&string," \t\v\n\r trim left");
    string_trim(&string);
    T_VERIFY (string_compareChar(&string,"trim left",CaseSensitive) == 0);

    //trim right
    string_fromChar(&string,"trim right \t\v\n\r ");
    string_trimLeft(&string);
    T_VERIFY (string_compareChar(&string,"trim right \t\v\n\r ",CaseSensitive) == 0);
    string_fromChar(&string,"trim right \t\v\n\r ");
    string_trimRight(&string);
    T_VERIFY (string_compareChar(&string,"trim right",CaseSensitive) == 0);
    string_fromChar(&string,"trim right \t\v\n\r ");
    string_trim(&string);
    T_VERIFY (string_compareChar(&string,"trim right",CaseSensitive) == 0);

    //trim both
    string_fromChar(&string," \t\v\n\r trim both \t\v\n\r ");
    string_trimLeft(&string);
    T_VERIFY (string_compareChar(&string,"trim both \t\v\n\r ",CaseSensitive) == 0);
    string_fromChar(&string," \t\v\n\r trim both \t\v\n\r ");
    string_trimRight(&string);
    T_VERIFY (string_compareChar(&string," \t\v\n\r trim both",CaseSensitive) == 0);
    string_fromChar(&string," \t\v\n\r trim both \t\v\n\r ");
    string_trim(&string);
    T_VERIFY (string_compareChar(&string,"trim both",CaseSensitive) == 0);

cleanup:
    string_cleanup(&string);
    T_TESTCASE_END();
}

int test_string_null() {
    T_TESTCASE_BEGIN();
    string_t string;
    string_initialize(&string,64);

    T_VERIFY (string_initialize(NULL,0) == false);
    string_cleanup(NULL);
    string_clear(NULL);
    T_VERIFY (string_copy(NULL,NULL) == 0);
    T_VERIFY (string_copy(&string,NULL) == 0);
    T_VERIFY (string_copy(NULL,&string) == 0);
    T_VERIFY (string_fromBool(NULL,true) == 0);
    T_VERIFY (string_fromChar(NULL,"abcdef") == 0);
    T_VERIFY (string_fromChar(NULL,NULL) == 0);
    T_VERIFY (string_fromInt8(NULL,15) == 0);
    T_VERIFY (string_fromInt16(NULL,15) == 0);
    T_VERIFY (string_fromInt32(NULL,51) == 0);
    T_VERIFY (string_fromInt64(NULL,51) == 0);
    T_VERIFY (string_fromUInt8(NULL,15) == 0);
    T_VERIFY (string_fromUInt16(NULL,15) == 0);
    T_VERIFY (string_fromUInt32(NULL,51) == 0);
    T_VERIFY (string_fromUInt64(NULL,51) == 0);
    T_VERIFY (string_fromTime(NULL,NULL,NULL) == 0);
    T_VERIFY (string_buffer(NULL) == NULL);
    T_VERIFY (string_toBool(NULL) == false);
    T_VERIFY (string_toChar(NULL) == NULL);
    T_VERIFY (string_toInt8(NULL) == 0);
    T_VERIFY (string_toInt16(NULL) == 0);
    T_VERIFY (string_toInt32(NULL) == 0);
    T_VERIFY (string_toInt64(NULL) == 0);
    T_VERIFY (string_toUInt8(NULL) == 0);
    T_VERIFY (string_toUInt16(NULL) == 0);
    T_VERIFY (string_toUInt32(NULL) == 0);
    T_VERIFY (string_toUInt64(NULL) == 0);
    T_VERIFY (string_toTime(NULL,NULL) == NULL);
    string_toUpper(NULL);
    string_toLower(NULL);
    T_VERIFY (string_appendChar(NULL,NULL) == 0);
    T_VERIFY (string_append(NULL,NULL) == 0);
    T_VERIFY (string_prependChar(NULL,NULL) == 0);
    T_VERIFY (string_prepend(NULL,NULL) == 0);
    T_VERIFY (string_left(NULL,NULL,100) == 0);
    T_VERIFY (string_right(NULL,NULL,100) == 0);
    T_VERIFY (string_mid(NULL,NULL,50,100) == 0);
    T_VERIFY (string_findChar(NULL,50,NULL,CaseSensitive) == 0xffffffff);
    T_VERIFY (string_find(NULL,50,NULL,CaseSensitive) == 0xffffffff);
    T_VERIFY (string_containsChar(NULL,NULL,CaseSensitive) == 0);
    T_VERIFY (string_contains(NULL,NULL,CaseSensitive) == 0);
    string_replaceChar(NULL,NULL,NULL);
    string_replace(NULL,NULL,NULL);
    T_VERIFY (string_compareChar(NULL,NULL,CaseSensitive) == 0);
    T_VERIFY (string_compareChar(&string,NULL,CaseSensitive) > 0);
    T_VERIFY (string_compare(NULL,NULL,CaseSensitive) == 0);
    T_VERIFY (string_compare(&string,NULL,CaseSensitive) > 0);
    T_VERIFY (string_compare(NULL,&string,CaseSensitive) < 0);
    T_VERIFY (string_isNull(NULL) == true);
    T_VERIFY (string_isEmpty(NULL) == true);
    string_trimLeft(NULL);
    string_trimRight(NULL);
    string_trim(NULL);
    T_VERIFY (string_isNumeric(NULL) == false);
    string_list_iterator_destroy(NULL);
    T_VERIFY(!string_list_split(NULL,NULL,NULL,0,0));
    T_VERIFY(!string_join(NULL,NULL,NULL));
    T_VERIFY(!string_list_iterator_data(NULL));
    T_VERIFY(!string_list_first(NULL));
    T_VERIFY(!string_list_last(NULL));
    T_VERIFY(!string_list_at(NULL,0));
    T_VERIFY(!string_list_iterator_next(NULL));
    T_VERIFY(!string_list_iterator_prev(NULL));
    T_VERIFY(!string_list_append(NULL,NULL));
    T_VERIFY(!string_list_prepend(NULL,NULL));
    T_VERIFY(!string_list_insertAt(NULL,0,NULL));
    T_VERIFY(!string_list_iterator_take(NULL));
    T_VERIFY(!string_list_takeFirst(NULL));
    T_VERIFY(!string_list_takeLast(NULL));
    T_VERIFY(!string_list_size(NULL));
    T_VERIFY(string_list_isEmpty(NULL));
cleanup:
    string_cleanup(&string);
    T_TESTCASE_END();
}

T_TESTCASES_BEGIN()
    T_TESTCASE(test_string_append, "Test string appending")
    T_TESTCASE(test_string_prepend, "Test string prepending")
    T_TESTCASE(test_string_left_mid_right, "Test left mid and right functions")
    T_TESTCASE(test_string_split_join, "Test split and join functions")
    T_TESTCASE(test_string_find_contains, "Test find and contains functions")
    T_TESTCASE(test_string_replace, "Test string replace functions")
    T_TESTCASE(test_string_compare, "Test string compare functions")
    T_TESTCASE(test_string_toUpper_toLower, "Test string toupper and tolower functions")
    T_TESTCASE(test_string_conversions, "Test conversions of strings to other types")
    T_TESTCASE(test_string_various, "Test other string functions")
    T_TESTCASE(test_string_trim, "Test trim functions")
    T_TESTCASE(test_string_null, "Test passing null pointers to string functions")
T_TESTCASES_END()
