#include <stdlib.h>
#include <stdio.h>
#define _XOPEN_SOURCE /* glibc2 needs this */
#include <time.h>
#include <unistd.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <pcb/utils/variant.h>
#include <pcb/utils/variant_list.h>
#include <pcb/utils/string.h>
#include <pcb/utils/variant_map.h>
#include <unittests/unittest.h>

int test_var_string() {
    T_TESTCASE_BEGIN();
    variant_t variant;

    // Create the variant
    T_VERIFY (variant_initialize(NULL    ,variant_type_string)==false);
    T_VERIFY (variant_initialize(&variant,variant_type_string)==true);
    T_VERIFY (variant_type(&variant)==variant_type_string);

    // Test the conversion functions
    T_VERIFY (variant_setChar(NULL,"Test string")==false);
    T_VERIFY (variant_setChar(&variant,"Test string")==true);
    string_t string;
    string_initialize(&string,64);
    char *txt = variant_char(&variant);
    T_VERIFY (txt);
    T_VERIFY (variant_toString(&string, &variant)==true);
    T_VERIFY (string_compareChar(&string,"Test string",CaseSensitive) == 0);
    T_VERIFY (strcmp(txt,"Test string") == 0);
    T_VERIFY (variant_da_string(&variant));
    T_VERIFY (!variant_da_list(&variant));
    free(txt);

    string_fromChar(&string,"Text,text,text");
    T_VERIFY (variant_setString(&variant,&string) == true);
    txt = variant_char(&variant);
    T_VERIFY (txt);
    T_VERIFY (strcmp(txt,"Text,text,text") == 0);
    string_cleanup(&string);
    free(txt);

    T_VERIFY(variant_setChar(&variant,NULL)==true);
    T_VERIFY(variant_setString(&variant,NULL) == true);
    int8_t int8;
    T_VERIFY (variant_setChar(&variant,"38")==true);
    T_VERIFY (variant_toInt8(&int8, &variant)==true);
    T_VERIFY (int8==38);
    int8 = variant_int8(&variant);
    T_VERIFY (int8==38);

    int16_t int16;
    T_VERIFY (variant_setChar(&variant,"378")==true);
    T_VERIFY (variant_toInt16(&int16, &variant)==true);
    T_VERIFY (int16==378);
    int16 = variant_int16(&variant);
    T_VERIFY (int16==378);

    int32_t int32;
    T_VERIFY (variant_setChar(&variant,"-3888")==true);
    T_VERIFY (variant_toInt32(&int32, &variant)==true);
    T_VERIFY (int32==-3888);
    int32 = variant_int32(&variant);
    T_VERIFY (int32==-3888);

    int64_t int64;
    T_VERIFY (variant_setChar(&variant,"-9999")==true);
    T_VERIFY (variant_toInt64(&int64, &variant)==true);
    T_VERIFY (int64==-9999);
    int64 = variant_int64(&variant);
    T_VERIFY (int64==-9999);

    uint8_t uint8;
    T_VERIFY (variant_setChar(&variant,"38")==true);
    T_VERIFY (variant_toUInt8(&uint8, &variant)==true);
    T_VERIFY (uint8==38);
    uint8 = variant_uint8(&variant);
    T_VERIFY (uint8==38);

    uint16_t uint16;
    T_VERIFY (variant_setChar(&variant,"378")==true);
    T_VERIFY (variant_toUInt16(&uint16, &variant)==true);
    T_VERIFY (uint16==378);
    uint16 = variant_uint16(&variant);
    T_VERIFY (uint16==378);

    uint32_t uint32;
    T_VERIFY (variant_setChar(&variant,"3888")==true);
    T_VERIFY (variant_toUInt32(&uint32, &variant)==true);
    T_VERIFY (uint32==3888);
    uint32 = variant_uint32(&variant);
    T_VERIFY (uint16==378);

    uint64_t uint64;
    T_VERIFY (variant_setChar(&variant,"9999")==true);
    T_VERIFY (variant_toUInt64(&uint64, &variant)==true);
    T_VERIFY (uint64==9999);
    uint64 = variant_uint64(&variant);
    T_VERIFY (uint64==9999);

    bool boolean;
    T_VERIFY (variant_setChar(&variant,"yes")==true);
    T_VERIFY (variant_toBool(&boolean, &variant)==true);
    T_VERIFY (boolean==true);
    boolean = variant_bool(&variant);
    T_VERIFY (boolean==true);

    double dbl;
    T_VERIFY (variant_setChar(&variant,"112.15")==true);
    T_VERIFY (!variant_toDouble(&dbl, &variant));
    dbl = variant_double(&variant);
    T_VERIFY (dbl==0);

    struct tm datetime;
    struct tm *datetime2 = NULL;
    strptime("2008-10-11T16:12:54Z", "%Y-%m-%dT%H:%M:%SZ", &datetime);
    char buf[255];
    T_VERIFY (variant_setChar(&variant,"2001-11-12T18:31:01Z")==true);
    T_VERIFY (variant_toDateTime(&datetime,&variant));
    datetime2 = variant_dateTime(&variant);
    strftime(buf, sizeof(buf), "%Y-%m-%dT%H:%M:%SZ", &datetime);
    T_VERIFY (strcmp("2001-11-12T18:31:01Z",buf)==0);
    strftime(buf, sizeof(buf), "%Y-%m-%dT%H:%M:%SZ", datetime2);
    T_VERIFY (strcmp("2001-11-12T18:31:01Z",buf)==0);
    free(datetime2);

cleanup:
    variant_cleanup(&variant);
    T_TESTCASE_END();
}

int test_var_int8() {
    T_TESTCASE_BEGIN();
    variant_t variant;

    // Create the variant
    T_VERIFY (variant_initialize(NULL    ,variant_type_int8)==false);
    T_VERIFY (variant_initialize(&variant,variant_type_int8)==true);
    T_VERIFY (variant_type(&variant)==variant_type_int8);

    // Test the conversion functions
    T_VERIFY (variant_setInt8(NULL,-45)==false);
    T_VERIFY (variant_setInt8(&variant,-45)==true);
    string_t string;
    string_initialize(&string,64);
    T_VERIFY (variant_toString(&string, &variant)==true);
    T_VERIFY (string_compareChar(&string,"-45",CaseSensitive) == 0);
    string_cleanup(&string);
    T_VERIFY (!variant_da_string(&variant));
    T_VERIFY (!variant_da_list(&variant));

    int8_t int8;
    T_VERIFY (variant_setInt8(&variant,38)==true);
    T_VERIFY (variant_toInt8(&int8, &variant)==true);
    T_VERIFY (int8==38);

    int16_t int16;
    T_VERIFY (variant_setInt8(&variant,45)==true);
    T_VERIFY (variant_toInt16(&int16, &variant)==true);
    T_VERIFY (int16==45);

    int32_t int32;
    T_VERIFY (variant_setInt8(&variant,-12)==true);
    T_VERIFY (variant_toInt32(&int32, &variant)==true);
    T_VERIFY (int32==-12);

    int64_t int64;
    T_VERIFY (variant_setInt8(&variant,-44)==true);
    T_VERIFY (variant_toInt64(&int64, &variant)==true);
    T_VERIFY (int64==-44);

    uint8_t uint8;
    T_VERIFY (variant_setInt8(&variant,38)==true);
    T_VERIFY (variant_toUInt8(&uint8, &variant)==true);
    T_VERIFY (uint8==38);

    uint16_t uint16;
    T_VERIFY (variant_setInt8(&variant,12)==true);
    T_VERIFY (variant_toUInt16(&uint16, &variant)==true);
    T_VERIFY (uint16==12);

    uint32_t uint32;
    T_VERIFY (variant_setInt8(&variant,85)==true);
    T_VERIFY (variant_toUInt32(&uint32, &variant)==true);
    T_VERIFY (uint32==85);

    uint64_t uint64;
    T_VERIFY (variant_setInt8(&variant,44)==true);
    T_VERIFY (variant_toUInt64(&uint64, &variant)==true);
    T_VERIFY (uint64==44);

    bool boolean;
    T_VERIFY (variant_setInt8(&variant,1)==true);
    T_VERIFY (variant_toBool(&boolean, &variant)==true);
    T_VERIFY (boolean==true);

    //Currently not implemented
    //bool variant_toDouble(double *data, const variant_t *variant);

    struct tm *datetime = NULL;
    T_VERIFY (variant_setInt8(&variant,1)==true);
    T_VERIFY (variant_toDateTime(datetime,&variant)==false);
    T_VERIFY (datetime==NULL);

cleanup:
    variant_cleanup(&variant);
    T_TESTCASE_END();
}

int test_var_int16() {
    T_TESTCASE_BEGIN();
    variant_t variant;

    // Create the variant
    T_VERIFY (variant_initialize(NULL    ,variant_type_int16)==false);
    T_VERIFY (variant_initialize(&variant,variant_type_int16)==true);
    T_VERIFY (variant_type(&variant)==variant_type_int16);

    // Test the conversion functions
    T_VERIFY (variant_setInt16(NULL,-45)==false);
    T_VERIFY (variant_setInt16(&variant,-45)==true);
    string_t string;
    string_initialize(&string,64);
    T_VERIFY (variant_toString(&string, &variant)==true);
    T_VERIFY (string_compareChar(&string,"-45",CaseSensitive) == 0);
    string_cleanup(&string);
    T_VERIFY (!variant_da_string(&variant));
    T_VERIFY (!variant_da_list(&variant));

    int8_t int8;
    T_VERIFY (variant_setInt16(&variant,38)==true);
    T_VERIFY (variant_toInt8(&int8, &variant)==true);
    T_VERIFY (int8==38);
    T_VERIFY (variant_setInt16(&variant,0x0FFF)==true);
    T_VERIFY (variant_toInt8(&int8, &variant)==true);
    T_EQUAL (int8_t,int8,0x7F);

    int16_t int16;
    T_VERIFY (variant_setInt16(&variant,45)==true);
    T_VERIFY (variant_toInt16(&int16, &variant)==true);
    T_VERIFY (int16==45);

    int32_t int32;
    T_VERIFY (variant_setInt16(&variant,-12)==true);
    T_VERIFY (variant_toInt32(&int32, &variant)==true);
    T_VERIFY (int32==-12);

    int64_t int64;
    T_VERIFY (variant_setInt64(&variant,-77)==true);
    T_VERIFY (variant_toInt64(&int64, &variant)==true);
    T_VERIFY (int64==-77);

    uint8_t uint8;
    T_VERIFY (variant_setInt16(&variant,38)==true);
    T_VERIFY (variant_toUInt8(&uint8, &variant)==true);

    uint16_t uint16;
    T_VERIFY (variant_setInt16(&variant,12)==true);
    T_VERIFY (variant_toUInt16(&uint16, &variant)==true);
    T_VERIFY (uint16==12);

    uint32_t uint32;
    T_VERIFY (variant_setInt16(&variant,85)==true);
    T_VERIFY (variant_toUInt32(&uint32, &variant)==true);
    T_VERIFY (uint32==85);

    uint64_t uint64;
    T_VERIFY (variant_setInt16(&variant,85)==true);
    T_VERIFY (variant_toUInt64(&uint64, &variant)==true);
    T_VERIFY (uint64==85);

    bool boolean;
    T_VERIFY (variant_setInt16(&variant,1)==true);
    T_VERIFY (variant_toBool(&boolean, &variant)==true);
    T_VERIFY (boolean==true);

    //Currently not implemented
    //bool variant_toDouble(double *data, const variant_t *variant);

    struct tm *datetime = NULL;
    T_VERIFY (variant_setInt16(&variant,1)==true);
    T_VERIFY (variant_toDateTime(datetime,&variant)==false);
    T_VERIFY (datetime==NULL);

cleanup:
    variant_cleanup(&variant);
    T_TESTCASE_END();
}

int test_var_int32() {
    T_TESTCASE_BEGIN();
    variant_t variant;

    // Create the variant
    T_VERIFY (variant_initialize(NULL    ,variant_type_int32)==false);
    T_VERIFY (variant_initialize(&variant,variant_type_int32)==true);
    T_VERIFY (variant_type(&variant)==variant_type_int32);

    // Test the conversion functions
    T_VERIFY (variant_setInt32(NULL,-45)==false);
    T_VERIFY (variant_setInt32(&variant,-45)==true);
    string_t string;
    string_initialize(&string,64);
    T_VERIFY (variant_toString(&string, &variant)==true);
    T_VERIFY (string_compareChar(&string,"-45",CaseSensitive) == 0);
    string_cleanup(&string);
    T_VERIFY (!variant_da_string(&variant));
    T_VERIFY (!variant_da_list(&variant));

    int8_t int8;
    T_VERIFY (variant_setInt32(&variant,38)==true);
    T_VERIFY (variant_toInt8(&int8, &variant)==true);
    T_VERIFY (int8==38);

    int16_t int16;
    T_VERIFY (variant_setInt32(&variant,45)==true);
    T_VERIFY (variant_toInt16(&int16, &variant)==true);
    T_VERIFY (int16==45);

    int32_t int32;
    T_VERIFY (variant_setInt32(&variant,-12)==true);
    T_VERIFY (variant_toInt32(&int32, &variant)==true);
    T_VERIFY (int32==-12);

    int64_t int64;
    T_VERIFY (variant_setInt64(&variant,-88)==true);
    T_VERIFY (variant_toInt64(&int64, &variant)==true);
    T_VERIFY (int64==-88);

    uint8_t uint8;
    T_VERIFY (variant_setInt32(&variant,38)==true);
    T_VERIFY (variant_toUInt8(&uint8, &variant)==true);

    uint16_t uint16;
    T_VERIFY (variant_setInt32(&variant,12)==true);
    T_VERIFY (variant_toUInt16(&uint16, &variant)==true);

    uint32_t uint32;
    T_VERIFY (variant_setInt32(&variant,85)==true);
    T_VERIFY (variant_toUInt32(&uint32, &variant)==true);
    T_VERIFY (uint32==85);

    uint64_t uint64;
    T_VERIFY (variant_setUInt32(&variant,88)==true);
    T_VERIFY (variant_toUInt64(&uint64, &variant)==true);
    T_VERIFY (uint64==88);

    bool boolean;
    T_VERIFY (variant_setInt32(&variant,1)==true);
    T_VERIFY (variant_toBool(&boolean, &variant)==true);
    T_VERIFY (boolean==true);

    //Currently not implemented
    //bool variant_toDouble(double *data, const variant_t *variant);

    struct tm *datetime = NULL;
    T_VERIFY (variant_setInt32(&variant,1)==true);
    T_VERIFY (variant_toDateTime(datetime,&variant)==false);
    T_VERIFY (datetime==NULL);

cleanup:
    variant_cleanup(&variant);
    T_TESTCASE_END();
}

int test_var_int64() {
    T_TESTCASE_BEGIN();
    variant_t variant;

    // Create the variant
    T_VERIFY (variant_initialize(NULL    ,variant_type_int64)==false);
    T_VERIFY (variant_initialize(&variant,variant_type_int64)==true);
    T_VERIFY (variant_type(&variant)==variant_type_int64);

    // Test the conversion functions
    T_VERIFY (variant_setInt64(NULL,-45)==false);
    T_VERIFY (variant_setInt64(&variant,-45)==true);
    string_t string;
    string_initialize(&string,64);
    T_VERIFY (variant_toString(&string, &variant)==true);
    T_VERIFY (string_compareChar(&string,"-45",CaseSensitive) == 0);
    string_cleanup(&string);
    T_VERIFY (!variant_da_string(&variant));
    T_VERIFY (!variant_da_list(&variant));

    int8_t int8;
    T_VERIFY (variant_setInt64(&variant,38)==true);
    T_VERIFY (variant_toInt8(&int8, &variant)==true);
    T_VERIFY (int8==38);

    int16_t int16;
    T_VERIFY (variant_setInt64(&variant,45)==true);
    T_VERIFY (variant_toInt16(&int16, &variant)==true);
    T_VERIFY (int16==45);

    int32_t int32;
    T_VERIFY (variant_setInt64(&variant,-12)==true);
    T_VERIFY (variant_toInt32(&int32, &variant)==true);
    T_VERIFY (int32==-12);

    int64_t int64;
    T_VERIFY (variant_setInt64(&variant,-88)==true);
    T_VERIFY (variant_toInt64(&int64, &variant)==true);
    T_VERIFY (int64==-88);

    uint8_t uint8;
    T_VERIFY (variant_setInt64(&variant,38)==true);
    T_VERIFY (variant_toUInt8(&uint8, &variant)==true);

    uint16_t uint16;
    T_VERIFY (variant_setInt64(&variant,12)==true);
    T_VERIFY (variant_toUInt16(&uint16, &variant)==true);

    uint32_t uint32;
    T_VERIFY (variant_setInt64(&variant,85)==true);
    T_VERIFY (variant_toUInt32(&uint32, &variant)==true);
    T_VERIFY (uint32==85);

    uint64_t uint64;
    T_VERIFY (variant_setInt64(&variant,88)==true);
    T_VERIFY (variant_toUInt64(&uint64, &variant)==true);
    T_VERIFY (uint64==88);

    bool boolean;
    T_VERIFY (variant_setInt64(&variant,1)==true);
    T_VERIFY (variant_toBool(&boolean, &variant)==true);
    T_VERIFY (boolean==true);

    //Currently not implemented
    //bool variant_toDouble(double *data, const variant_t *variant);

    struct tm *datetime = NULL;
    T_VERIFY (variant_setInt64(&variant,1)==true);
    T_VERIFY (variant_toDateTime(datetime,&variant)==false);
    T_VERIFY (datetime==NULL);

cleanup:
    variant_cleanup(&variant);
    T_TESTCASE_END();
}

int test_var_uint8() {
    T_TESTCASE_BEGIN();
    variant_t variant;

    // Create the variant
    T_VERIFY (variant_initialize(NULL    ,variant_type_uint8)==false);
    T_VERIFY (variant_initialize(&variant,variant_type_uint8)==true);
    T_VERIFY (variant_type(&variant)==variant_type_uint8);

    // Test the conversion functions
    T_VERIFY (variant_setUInt8(NULL,45)==false);
    T_VERIFY (variant_setUInt8(&variant,45)==true);
    string_t string;
    string_initialize(&string,64);
    T_VERIFY (variant_toString(&string, &variant)==true);
    T_VERIFY (string_compareChar(&string,"45",CaseSensitive) == 0);
    string_cleanup(&string);
    T_VERIFY (!variant_da_string(&variant));
    T_VERIFY (!variant_da_list(&variant));

    int8_t int8;
    T_VERIFY (variant_setUInt8(&variant,38)==true);
    T_VERIFY (variant_toInt8(&int8, &variant)==true);
    T_VERIFY (int8==38);

    int16_t int16;
    T_VERIFY (variant_setUInt8(&variant,45)==true);
    T_VERIFY (variant_toInt16(&int16, &variant)==true);
    T_VERIFY (int16==45);

    int32_t int32;
    T_VERIFY (variant_setUInt8(&variant,12)==true);
    T_VERIFY (variant_toInt32(&int32, &variant)==true);
    T_VERIFY (int32==12);

    uint8_t uint8;
    T_VERIFY (variant_setUInt8(&variant,38)==true);
    T_VERIFY (variant_toUInt8(&uint8, &variant)==true);
    T_VERIFY (uint8==38);

    uint16_t uint16;
    T_VERIFY (variant_setUInt8(&variant,12)==true);
    T_VERIFY (variant_toUInt16(&uint16, &variant)==true);
    T_VERIFY (uint16==12);

    uint32_t uint32;
    T_VERIFY (variant_setUInt8(&variant,85)==true);
    T_VERIFY (variant_toUInt32(&uint32, &variant)==true);
    T_VERIFY (uint32==85);

    bool boolean;
    T_VERIFY (variant_setUInt8(&variant,1)==true);
    T_VERIFY (variant_toBool(&boolean, &variant)==true);
    T_VERIFY (boolean==true);

    //Currently not implemented
    //bool variant_toDouble(double *data, const variant_t *variant);

    struct tm *datetime = NULL;
    T_VERIFY (variant_setUInt8(&variant,1)==true);
    T_VERIFY (variant_toDateTime(datetime,&variant)==false);
    T_VERIFY (datetime==NULL);

cleanup:
    variant_cleanup(&variant);
    T_TESTCASE_END();
}

int test_var_uint16() {
    T_TESTCASE_BEGIN();
    variant_t variant;

    // Create the variant
    T_VERIFY (variant_initialize(NULL    ,variant_type_uint8)==false);
    T_VERIFY (variant_initialize(&variant,variant_type_uint8)==true);
    T_VERIFY (variant_type(&variant)==variant_type_uint8);

    // Test the conversion functions
    T_VERIFY (variant_setUInt16(NULL,45)==false);
    T_VERIFY (variant_setUInt16(&variant,45)==true);
    string_t string;
    string_initialize(&string,64);
    T_VERIFY (variant_toString(&string, &variant)==true);
    T_VERIFY (string_compareChar(&string,"45",CaseSensitive) == 0);
    string_cleanup(&string);
    T_VERIFY (!variant_da_string(&variant));
    T_VERIFY (!variant_da_list(&variant));

    int8_t int8;
    T_VERIFY (variant_setUInt16(&variant,38)==true);
    T_VERIFY (variant_toInt8(&int8, &variant)==true);
    T_VERIFY (int8==38);

    int16_t int16;
    T_VERIFY (variant_setUInt16(&variant,45)==true);
    T_VERIFY (variant_toInt16(&int16, &variant)==true);
    T_VERIFY (int16==45);

    int32_t int32;
    T_VERIFY (variant_setUInt16(&variant,12)==true);
    T_VERIFY (variant_toInt32(&int32, &variant)==true);
    T_VERIFY (int32==12);

    uint8_t uint8;
    T_VERIFY (variant_setUInt16(&variant,38)==true);
    T_VERIFY (variant_toUInt8(&uint8, &variant)==true);
    T_VERIFY (uint8==38);

    uint16_t uint16;
    T_VERIFY (variant_setUInt16(&variant,12)==true);
    T_VERIFY (variant_toUInt16(&uint16, &variant)==true);
    T_VERIFY (uint16==12);

    uint32_t uint32;
    T_VERIFY (variant_setUInt16(&variant,85)==true);
    T_VERIFY (variant_toUInt32(&uint32, &variant)==true);
    T_VERIFY (uint32==85);

    bool boolean;
    T_VERIFY (variant_setUInt16(&variant,1)==true);
    T_VERIFY (variant_toBool(&boolean, &variant)==true);
    T_VERIFY (boolean==true);

    //Currently not implemented
    //bool variant_toDouble(double *data, const variant_t *variant);

    struct tm *datetime = NULL;
    T_VERIFY (variant_setUInt16(&variant,1)==true);
    T_VERIFY (variant_toDateTime(datetime,&variant)==false);
    T_VERIFY (datetime==NULL);

cleanup:
    variant_cleanup(&variant);
    T_TESTCASE_END();
}

int test_var_uint32() {
    T_TESTCASE_BEGIN();
    variant_t variant;

    // Create the variant
    T_VERIFY (variant_initialize(NULL    ,variant_type_uint8)==false);
    T_VERIFY (variant_initialize(&variant,variant_type_uint8)==true);
    T_VERIFY (variant_type(&variant)==variant_type_uint8);

    // Test the conversion functions
    T_VERIFY (variant_setUInt32(NULL,45)==false);
    T_VERIFY (variant_setUInt32(&variant,45)==true);
    string_t string;
    string_initialize(&string,64);
    T_VERIFY (variant_toString(&string, &variant)==true);
    T_VERIFY (string_compareChar(&string,"45",CaseSensitive) == 0);
    string_cleanup(&string);
    T_VERIFY (!variant_da_string(&variant));
    T_VERIFY (!variant_da_list(&variant));

    int8_t int8;
    T_VERIFY (variant_setUInt32(&variant,38)==true);
    T_VERIFY (variant_toInt8(&int8, &variant)==true);
    T_VERIFY (int8==38);

    int16_t int16;
    T_VERIFY (variant_setUInt32(&variant,45)==true);
    T_VERIFY (variant_toInt16(&int16, &variant)==true);
    T_VERIFY (int16==45);

    int32_t int32;
    T_VERIFY (variant_setUInt32(&variant,12)==true);
    T_VERIFY (variant_toInt32(&int32, &variant)==true);
    T_VERIFY (int32==12);

    uint8_t uint8;
    T_VERIFY (variant_setUInt32(&variant,38)==true);
    T_VERIFY (variant_toUInt8(&uint8, &variant)==true);
    T_VERIFY (uint8==38);

    uint16_t uint16;
    T_VERIFY (variant_setUInt32(&variant,12)==true);
    T_VERIFY (variant_toUInt16(&uint16, &variant)==true);
    T_VERIFY (uint16==12);

    uint32_t uint32;
    T_VERIFY (variant_setUInt32(&variant,85)==true);
    T_VERIFY (variant_toUInt32(&uint32, &variant)==true);
    T_VERIFY (uint32==85);

    bool boolean;
    T_VERIFY (variant_setUInt32(&variant,1)==true);
    T_VERIFY (variant_toBool(&boolean, &variant)==true);
    T_VERIFY (boolean==true);

    //Currently not implemented
    //bool variant_toDouble(double *data, const variant_t *variant);

    struct tm *datetime = NULL;
    T_VERIFY (variant_setUInt32(&variant,1)==true);
    T_VERIFY (variant_toDateTime(datetime,&variant)==false);
    T_VERIFY (datetime==NULL);

cleanup:
    variant_cleanup(&variant);
    T_TESTCASE_END();
}

int test_var_uint64() {
    T_TESTCASE_BEGIN();
    variant_t variant;

    // Create the variant
    T_VERIFY (variant_initialize(NULL    ,variant_type_uint64)==false);
    T_VERIFY (variant_initialize(&variant,variant_type_uint64)==true);
    T_VERIFY (variant_type(&variant)==variant_type_uint64);

    // Test the conversion functions
    T_VERIFY (variant_setUInt64(NULL,45)==false);
    T_VERIFY (variant_setUInt64(&variant,45)==true);
    string_t string;
    string_initialize(&string,64);
    T_VERIFY (variant_toString(&string, &variant)==true);
    T_VERIFY (string_compareChar(&string,"45",CaseSensitive) == 0);
    string_cleanup(&string);
    T_VERIFY (!variant_da_string(&variant));
    T_VERIFY (!variant_da_list(&variant));

    int8_t int8;
    T_VERIFY (variant_setUInt64(&variant,38)==true);
    T_VERIFY (variant_toInt8(&int8, &variant)==true);
    T_VERIFY (int8==38);

    int16_t int16;
    T_VERIFY (variant_setUInt64(&variant,45)==true);
    T_VERIFY (variant_toInt16(&int16, &variant)==true);
    T_VERIFY (int16==45);

    int32_t int32;
    T_VERIFY (variant_setUInt64(&variant,12)==true);
    T_VERIFY (variant_toInt32(&int32, &variant)==true);
    T_VERIFY (int32==12);

    int64_t int64;
    T_VERIFY (variant_setUInt64(&variant,128)==true);
    T_VERIFY (variant_toInt64(&int64, &variant)==true);
    T_VERIFY (int64 == 128);

    uint8_t uint8;
    T_VERIFY (variant_setUInt64(&variant,38)==true);
    T_VERIFY (variant_toUInt8(&uint8, &variant)==true);
    T_VERIFY (uint8==38);

    uint16_t uint16;
    T_VERIFY (variant_setUInt64(&variant,12)==true);
    T_VERIFY (variant_toUInt16(&uint16, &variant)==true);
    T_VERIFY (uint16==12);

    uint32_t uint32;
    T_VERIFY (variant_setUInt64(&variant,85)==true);
    T_VERIFY (variant_toUInt32(&uint32, &variant)==true);
    T_VERIFY (uint32==85);

    uint64_t uint64;
    T_VERIFY (variant_setUInt64(&variant,9999)==true);
    T_VERIFY (variant_toUInt64(&uint64, &variant)==true);
    T_VERIFY (uint64 == 9999);

    bool boolean;
    T_VERIFY (variant_setUInt32(&variant,1)==true);
    T_VERIFY (variant_toBool(&boolean, &variant)==true);
    T_VERIFY (boolean==true);

    //Currently not implemented
    //bool variant_toDouble(double *data, const variant_t *variant);

    struct tm *datetime = NULL;
    T_VERIFY (variant_setUInt64(&variant,1)==true);
    T_VERIFY (variant_toDateTime(datetime,&variant)==false);
    T_VERIFY (datetime==NULL);

cleanup:
    variant_cleanup(&variant);
    T_TESTCASE_END();
}

int test_var_bool() {
    T_TESTCASE_BEGIN();
    variant_t variant;

    // Create the variant
    T_VERIFY (variant_initialize(NULL    ,variant_type_bool)==false);
    T_VERIFY (variant_initialize(&variant,variant_type_bool)==true);
    T_VERIFY (variant_type(&variant)==variant_type_bool);

    // Test the conversion functions
    T_VERIFY (variant_setBool(NULL,1)==false);
    T_VERIFY (variant_setBool(&variant,1)==true);
    string_t string;
    string_initialize(&string,64);
    T_VERIFY (variant_toString(&string, &variant)==true);
    T_VERIFY (string_compareChar(&string,"1",CaseSensitive) == 0);
    string_cleanup(&string);
    T_VERIFY (!variant_da_string(&variant));
    T_VERIFY (!variant_da_list(&variant));

    int8_t int8;
    T_VERIFY (variant_setBool(&variant,1)==true);
    T_VERIFY (variant_toInt8(&int8, &variant)==true);
    T_VERIFY (int8==1);

    int16_t int16;
    T_VERIFY (variant_setBool(&variant,0)==true);
    T_VERIFY (variant_toInt16(&int16, &variant)==true);
    T_VERIFY (int16==0);

    int32_t int32;
    T_VERIFY (variant_setBool(&variant,1)==true);
    T_VERIFY (variant_toInt32(&int32, &variant)==true);
    T_VERIFY (int32==1);

    uint8_t uint8;
    T_VERIFY (variant_setBool(&variant,0)==true);
    T_VERIFY (variant_toUInt8(&uint8, &variant)==true);
    T_VERIFY (uint8==0);

    uint16_t uint16;
    T_VERIFY (variant_setBool(&variant,1)==true);
    T_VERIFY (variant_toUInt16(&uint16, &variant)==true);
    T_VERIFY (uint16==1);

    uint32_t uint32;
    T_VERIFY (variant_setBool(&variant,0)==true);
    T_VERIFY (variant_toUInt32(&uint32, &variant)==true);
    T_VERIFY (uint32==0);

    bool boolean;
    T_VERIFY (variant_setBool(&variant,1)==true);
    T_VERIFY (variant_toBool(&boolean, &variant)==true);
    T_VERIFY (boolean==true);

    //Currently not implemented
    //bool variant_toDouble(double *data, const variant_t *variant);

    struct tm *datetime = NULL;
    T_VERIFY (variant_setBool(&variant,1)==true);
    T_VERIFY (variant_toDateTime(datetime,&variant)==false);
    T_VERIFY (datetime==NULL);

cleanup:
    variant_cleanup(&variant);
    T_TESTCASE_END();
}

int test_var_datetime() {
    T_TESTCASE_BEGIN();
    variant_t variant;
    struct tm *datetime = NULL;

    // Create the variant
    T_VERIFY (variant_initialize(NULL    ,variant_type_date_time)==false);
    T_VERIFY (variant_initialize(&variant,variant_type_date_time)==true);
    T_VERIFY (variant_type(&variant)==variant_type_date_time);

    // Test the conversion functions
    T_VERIFY (variant_setDateTime(NULL,datetime)==false);
    T_VERIFY (variant_setDateTime(&variant,datetime)==false);
    datetime = (struct tm *)malloc(sizeof(struct tm));
    strptime("2008-10-11T16:12:54Z", "%Y-%m-%dT%H:%M:%SZ", datetime);
    T_VERIFY (variant_setDateTime(&variant,datetime)==true);
    string_t string;
    string_initialize(&string,64);
    T_VERIFY (variant_toString(&string, &variant)==true);
    T_VERIFY (string_compareChar(&string,"2008-10-11T16:12:54Z",CaseSensitive) == 0);
    string_cleanup(&string);
    T_VERIFY (!variant_da_string(&variant));
    T_VERIFY (!variant_da_list(&variant));

    int8_t int8;
    T_VERIFY (variant_setDateTime(&variant,datetime)==true);
    T_VERIFY (variant_toInt8(&int8, &variant)==false);

    int16_t int16;
    T_VERIFY (variant_setDateTime(&variant,datetime)==true);
    T_VERIFY (variant_toInt16(&int16, &variant)==false);

    int32_t int32;
    T_VERIFY (variant_setDateTime(&variant,datetime)==true);
    T_VERIFY (variant_toInt32(&int32, &variant)==false);

    uint8_t uint8;
    T_VERIFY (variant_setDateTime(&variant,datetime)==true);
    T_VERIFY (variant_toUInt8(&uint8, &variant)==false);

    uint16_t uint16;
    T_VERIFY (variant_setDateTime(&variant,datetime)==true);
    T_VERIFY (variant_toUInt16(&uint16, &variant)==false);

    uint32_t uint32;
    T_VERIFY (variant_setDateTime(&variant,datetime)==true);
    T_VERIFY (variant_toUInt32(&uint32, &variant)==false);

    bool boolean;
    T_VERIFY (variant_setDateTime(&variant,datetime)==true);
    T_VERIFY (variant_toBool(&boolean, &variant)==false);

    //Currently not implemented
    //bool variant_toDouble(double *data, const variant_t *variant);

    char buf[256];
    T_VERIFY (variant_setDateTime(&variant,datetime)==true);
    T_VERIFY (variant_toDateTime(datetime,&variant)==true);
    strftime(buf, sizeof(buf), "%Y-%m-%dT%H:%M:%SZ", datetime);
    T_VERIFY (strcmp("2008-10-11T16:12:54Z",buf)==0);

    free(datetime);
cleanup:
    variant_cleanup(&variant);
    T_TESTCASE_END();
}

int test_var_copy() {
    T_TESTCASE_BEGIN();
    variant_t variant1;
    variant_t variant2;

    // Create the variant
    T_VERIFY (variant_initialize(&variant1,variant_type_string)==true);
    T_VERIFY (variant_initialize(&variant2,variant_type_string)==true);

    // Run the tests
    T_VERIFY (variant_setChar(&variant1, "What is the magic number?")==true);
    variant_copy(&variant2,&variant1);
    string_t string;
    string_initialize(&string,64);
    T_VERIFY (variant_toString(&string, &variant2)==true);
    T_VERIFY (string_compareChar(&string,"What is the magic number?",CaseSensitive) == 0);
    string_cleanup(&string);

    int8_t int8;
    T_VERIFY (variant_setInt8(&variant1,38)==true);
    variant_copy(&variant2,&variant1);
    T_VERIFY (variant_toInt8(&int8, &variant2)==true);
    T_VERIFY (int8==38);

    int16_t int16;
    T_VERIFY (variant_setInt16(&variant1,45)==true);
    variant_copy(&variant2,&variant1);
    T_VERIFY (variant_toInt16(&int16, &variant2)==true);
    T_VERIFY (int16==45);

    int32_t int32;
    T_VERIFY (variant_setInt32(&variant1,12)==true);
    variant_copy(&variant2,&variant1);
    T_VERIFY (variant_toInt32(&int32, &variant2)==true);
    T_VERIFY (int32==12);

    int64_t int64;
    T_VERIFY (variant_setInt64(&variant1,7851)==true);
    variant_copy(&variant2,&variant1);
    T_VERIFY (variant_toInt64(&int64, &variant2)==true);
    T_VERIFY (int64==7851);

    uint8_t uint8;
    T_VERIFY (variant_setUInt8(&variant1,38)==true);
    variant_copy(&variant2,&variant1);
    T_VERIFY (variant_toUInt8(&uint8, &variant2)==true);
    T_VERIFY (uint8==38);

    uint16_t uint16;
    T_VERIFY (variant_setUInt16(&variant1,12)==true);
    variant_copy(&variant2,&variant1);
    T_VERIFY (variant_toUInt16(&uint16, &variant2)==true);
    T_VERIFY (uint16==12);

    uint32_t uint32;
    T_VERIFY (variant_setUInt32(&variant1,85)==true);
    variant_copy(&variant2,&variant1);
    T_VERIFY (variant_toUInt32(&uint32, &variant2)==true);
    T_VERIFY (uint32==85);

    uint64_t uint64;
    T_VERIFY (variant_setUInt64(&variant1,7777)==true);
    variant_copy(&variant2,&variant1);
    T_VERIFY (variant_toUInt64(&uint64, &variant2)==true);
    T_VERIFY (uint64==7777);

    bool boolean;
    T_VERIFY (variant_setBool(&variant1,1)==true);
    variant_copy(&variant2,&variant1);
    T_VERIFY (variant_toBool(&boolean, &variant2)==true);
    T_VERIFY (boolean==true);

    double db;
    T_VERIFY (variant_setDouble(&variant1,1.5)==true);
    variant_copy(&variant2,&variant1);
    T_VERIFY (variant_toDouble(&db, &variant2)==true);
    T_VERIFY (db==1.5);

    struct tm *datetime = NULL;

    char buf[256];
    datetime = (struct tm *)malloc(sizeof(struct tm));
    strptime("2008-10-11T16:12:54Z", "%Y-%m-%dT%H:%M:%SZ", datetime);
    T_VERIFY (variant_setDateTime(&variant1,datetime)==true);
    variant_copy(&variant2,&variant1);
    T_VERIFY (variant_toDateTime(datetime,&variant2)==true);
    strftime(buf, sizeof(buf), "%Y-%m-%dT%H:%M:%SZ", datetime);
    T_VERIFY (strcmp("2008-10-11T16:12:54Z",buf)==0);

    free(datetime);
cleanup:
    variant_cleanup(&variant1);
    variant_cleanup(&variant2);
    T_TESTCASE_END();
}

int test_var_null() {
    T_TESTCASE_BEGIN();
    T_VERIFY(!variant_initialize(NULL,0));
    variant_copy(NULL,NULL);
    variant_cleanup(NULL);
    T_VERIFY(!variant_setChar(NULL,NULL));
    T_VERIFY(!variant_setString(NULL,NULL));
    T_VERIFY(!variant_setInt8(NULL,0));
    T_VERIFY(!variant_setInt16(NULL,0));
    T_VERIFY(!variant_setInt32(NULL,0));
    T_VERIFY(!variant_setUInt8(NULL,0));
    T_VERIFY(!variant_setUInt16(NULL,0));
    T_VERIFY(!variant_setUInt32(NULL,0));
    T_VERIFY(!variant_setBool(NULL,false));
    T_VERIFY(!variant_setDouble(NULL,0.0));
    T_VERIFY(!variant_setDateTime(NULL,NULL));
    T_VERIFY(!variant_setArrayMove(NULL,NULL));
    T_VERIFY(!variant_toString(NULL,NULL));
    T_VERIFY(!variant_char(NULL));
    T_VERIFY(!variant_toInt8(NULL,NULL));
    T_VERIFY(!variant_toInt16(NULL,NULL));
    T_VERIFY(!variant_toInt32(NULL,NULL));
    T_VERIFY(!variant_toUInt8(NULL,NULL));
    T_VERIFY(!variant_toUInt16(NULL,NULL));
    T_VERIFY(!variant_toUInt32(NULL,NULL));
    T_VERIFY(!variant_toBool(NULL,NULL));
    T_VERIFY(!variant_int8(NULL));
    T_VERIFY(!variant_int16(NULL));
    T_VERIFY(!variant_int32(NULL));
    T_VERIFY(!variant_uint8(NULL));
    T_VERIFY(!variant_uint16(NULL));
    T_VERIFY(!variant_uint32(NULL));
    T_VERIFY(!variant_bool(NULL));
    T_VERIFY(!variant_double(NULL));
    T_VERIFY(!variant_dateTime(NULL));
    T_VERIFY(!variant_array(NULL));
    T_VERIFY(!variant_da_string(NULL));
    T_VERIFY(!variant_da_list(NULL));
    T_VERIFY(!variant_type(NULL));

cleanup:
    T_TESTCASE_END();
}

int test_var_map() {
    T_TESTCASE_BEGIN();
    variant_t variant;
    variant_map_t map;
    variant_map_iterator_t *it = NULL;

    T_VERIFY(variant_map_initialize(&map));
    T_VERIFY(variant_map_isEmpty(&map) == true);
    T_VERIFY(variant_map_takeLast(&map) == NULL);
    T_VERIFY(variant_map_last(&map) == NULL);
    T_VERIFY(variant_map_first(&map) == NULL);
    T_VERIFY(variant_map_takeFirst(&map) == NULL);

    T_VERIFY(variant_map_addChar(&map,"Key1","Value") == true);
    T_VERIFY(variant_map_addInt32(&map,"Key3",123) == true);
    T_VERIFY(variant_map_addInt8(&map,"Key2",1) == true);
    T_VERIFY(variant_map_isEmpty(&map) == false);
    T_VERIFY(variant_map_isEmpty(NULL) == true);

    it = variant_map_at(&map,1);
    T_VERIFY(it);
    it = variant_map_at(NULL,1);
    T_VERIFY(it == NULL);
    it = variant_map_at(&map,5);
    T_VERIFY(it == NULL);

    it = variant_map_find(&map,"Key3");
    T_VERIFY(it);
    T_VERIFY(variant_map_iterator_setKey(it,"Key33") == true);
    T_VERIFY(variant_map_iterator_setKey(NULL,"Key33") == false);
    T_VERIFY(variant_map_iterator_setKey(it,NULL) == false);
    it = variant_map_find(&map,"BlaBla");
    T_VERIFY(it == NULL);
    it = variant_map_find(NULL,"Key3");
    T_VERIFY(it == NULL);
    T_VERIFY(variant_map_contains(&map,"Key33") == true);
    T_VERIFY(variant_map_contains(&map,"BlaBla") == false);
    T_VERIFY(variant_map_contains(NULL,"Key3") == false);

    T_VERIFY(variant_map_iterator_key(NULL) == NULL);
    T_VERIFY(variant_map_iterator_data(NULL) == NULL);

    T_VERIFY(variant_map_takeFirst(NULL) == NULL);
    T_VERIFY(variant_map_takeLast(NULL) == NULL);

    it = variant_map_takeFirst(&map);
    T_VERIFY(variant_map_size(&map) == 2);
    T_VERIFY(it);
    T_VERIFY(strcmp(variant_map_iterator_key(it),"Key1") == 0);
    char *value = variant_char(variant_map_iterator_data(it));
    T_VERIFY(value);
    T_VERIFY(strcmp(value,"Value") == 0);
    free(value);

    T_VERIFY(variant_map_prepend(&map,it) == true);
    it = variant_map_takeLast(&map);
    T_VERIFY(variant_map_size(&map) == 2);
    T_VERIFY(it);
    T_VERIFY(strcmp(variant_map_iterator_key(it),"Key2") == 0);
    value = variant_char(variant_map_iterator_data(it));
    T_VERIFY(value);
    T_VERIFY(strcmp(value,"1") == 0);
    free(value);
    T_VERIFY(variant_map_append(&map,it) == true);

    T_VERIFY(variant_map_iterator_take(NULL) == NULL);
    it = variant_map_takeLast(&map);
    T_VERIFY(variant_map_iterator_take(it) == it);
    T_VERIFY(variant_map_append(&map,it) == true);
    T_VERIFY(variant_map_append(&map,NULL) == false);
    T_VERIFY(variant_map_append(NULL,it) == false);
    T_VERIFY(variant_map_prepend(&map,NULL) == false);
    T_VERIFY(variant_map_prepend(NULL,it) == false);

    it = variant_map_iterator_create("test",NULL);
    T_VERIFY(it);
    T_VERIFY(variant_map_insertAt(&map,2,it) == true);
    T_VERIFY(variant_map_size(&map) == 4);
    T_VERIFY(variant_map_iterator_take(it) == it);
    T_VERIFY(variant_map_insertAt(NULL,2,it) == false);
    T_VERIFY(variant_map_insertAt(&map,2,NULL) == false);
    variant_map_iterator_destroy(it);

    it = variant_map_last(NULL);
    T_VERIFY(it == NULL);
    it = variant_map_last(&map);
    T_VERIFY(it);
    it = variant_map_iterator_prev(it);
    T_VERIFY(it);
    it = variant_map_iterator_prev(it);
    T_VERIFY(it == variant_map_first(&map));
    T_VERIFY(variant_map_iterator_prev(it) == NULL);

    it = variant_map_first(NULL);
    T_VERIFY(it == NULL);
    it = variant_map_first(&map);
    T_VERIFY(it);
    it = variant_map_iterator_next(it);
    T_VERIFY(it);
    it = variant_map_iterator_next(it);
    T_VERIFY(it == variant_map_last(&map));

    T_VERIFY(variant_map_iterator_next(NULL) == NULL);
    T_VERIFY(variant_map_iterator_prev(NULL) == NULL);

    // Create the variant
    T_VERIFY (variant_initialize(NULL    ,variant_type_map)==false);
    T_VERIFY (variant_initialize(&variant,variant_type_map)==true);
    T_VERIFY (variant_type(&variant)==variant_type_map);

    T_VERIFY (variant_setMapCopy(NULL,&map)==false);
    T_VERIFY (variant_setMapCopy(&variant,&map)==true);
    T_VERIFY (variant_map_size(&map) == 3);
    T_VERIFY (variant_da_map(&variant));
    T_VERIFY (variant_map_size(variant_da_map(&variant)) == 3);
    string_t string;
    string_initialize(&string,64);
    T_VERIFY (variant_toString(&string, &variant)==true);
    T_VERIFY (string_compareChar(&string,"{Key1:Value,Key33:123,Key2:1}",CaseSensitive) == 0);
    string_clear(&string);
    T_VERIFY (variant_toJSON(&string, &variant)==true);
    printf("%s",string_buffer(&string));
    T_VERIFY (string_compareChar(&string,"{\"Key1\":\"Value\",\"Key33\":123,\"Key2\":1}",CaseSensitive) == 0);
    string_cleanup(&string);

    T_VERIFY (variant_setMapMove(NULL,&map)==false);
    T_VERIFY (variant_setMapMove(&variant,&map)==true);
    T_VERIFY (variant_map_size(&map) == 0);
    T_VERIFY (variant_da_map(&variant));
    T_VERIFY (variant_map_size(variant_da_map(&variant)) == 3);
    string_initialize(&string,64);
    T_VERIFY (variant_toString(&string, &variant)==true);
    T_VERIFY (string_compareChar(&string,"{Key1:Value,Key33:123,Key2:1}",CaseSensitive) == 0);
    string_clear(&string);
    T_VERIFY (variant_toJSON(&string, &variant)==true);
    printf("%s",string_buffer(&string));
    T_VERIFY (string_compareChar(&string,"{\"Key1\":\"Value\",\"Key33\":123,\"Key2\":1}",CaseSensitive) == 0);
    string_cleanup(&string);

    T_VERIFY (!variant_da_string(&variant));
    T_VERIFY (!variant_da_list(&variant));

    int8_t int8 = 0;
    T_VERIFY (variant_toInt8(&int8, &variant)==false);
    T_VERIFY (int8==0);

    int16_t int16  = 0;
    T_VERIFY (variant_toInt16(&int16, &variant)==false);
    T_VERIFY (int16==0);

    int32_t int32  = 1;
    T_VERIFY (variant_toInt32(&int32, &variant)==false);
    T_VERIFY (int32==1);

    uint8_t uint8 = 0;
    T_VERIFY (variant_toUInt8(&uint8, &variant)==false);
    T_VERIFY (uint8==0);

    uint16_t uint16 = 123;
    T_VERIFY (variant_toUInt16(&uint16, &variant)==false);
    T_VERIFY (uint16==123);

    uint32_t uint32 = 456;
    T_VERIFY (variant_toUInt32(&uint32, &variant)==false);
    T_VERIFY (uint32==456);

    bool boolean = true;
    T_VERIFY (variant_toBool(&boolean, &variant)==false);
    T_VERIFY (boolean==true);

    //Currently not implemented
    //bool variant_toDouble(double *data, const variant_t *variant);

    struct tm *datetime = NULL;
    T_VERIFY (variant_toDateTime(datetime,&variant)==false);
    T_VERIFY (datetime==NULL);

    variant_list_t list;
    variant_list_initialize(&list);
    T_VERIFY (variant_toArray(&list,&variant) == true);
    T_VERIFY (variant_list_size(&list) == 3);
    T_VERIFY (variant_map_size(variant_da_map(&variant)) == 3);
    variant_list_cleanup(&list);

    variant_t copy;
    T_VERIFY(variant_initialize(&copy,variant_type_unknown) == true);
    variant_copy(&copy,&variant);
    T_VERIFY(variant_type(&copy) == variant_type_map);
    T_VERIFY(variant_map_size(variant_da_map(&copy)) == 3);
    variant_cleanup(&copy);

cleanup:
    variant_map_cleanup(&map);
    variant_cleanup(&variant);
    T_TESTCASE_END();
}

int test_var_JSON() {
    T_TESTCASE_BEGIN();

    variant_map_t map;
    variant_list_t list;
    variant_t variant;

    T_VERIFY(variant_map_initialize(&map) == true);
    T_VERIFY(variant_list_initialize(&list) == true);
    T_VERIFY(variant_initialize(&variant,variant_type_unknown) == true);

    T_VERIFY(variant_map_addChar(&map,"K1","Text") == true);
    T_VERIFY(variant_map_addUInt8(&map,"K2",88) == true);
    T_VERIFY(variant_list_addMap(&list,&map) == true);

    T_VERIFY(variant_list_addInt16(&list,-12) == true);
    T_VERIFY(variant_list_addInt32(&list,-24) == true);
    T_VERIFY(variant_list_addInt64(&list,-36) == true);

    T_VERIFY(variant_map_addUInt8(&map,"A",8) == true);
    T_VERIFY(variant_map_addUInt16(&map,"B",16) == true);
    T_VERIFY(variant_map_addUInt32(&map,"C",32) == true);
    T_VERIFY(variant_map_addUInt64(&map,"D",64) == true);
    struct tm *datetime = NULL;
    datetime = (struct tm *)malloc(sizeof(struct tm));
    strptime("2008-10-11T16:12:54Z", "%Y-%m-%dT%H:%M:%SZ", datetime);
    T_VERIFY(variant_map_addDateTime(&map,"time",datetime) == true);
    T_VERIFY(variant_map_addDateTime(&map,"time2",NULL) == true);
    free(datetime);

    T_VERIFY(variant_list_addMap(&list,&map) == true);

    T_VERIFY(variant_list_addBool(&list,true) == true);
    T_VERIFY(variant_list_addBool(&list,false) == true);
    T_VERIFY(variant_list_addDouble(&list,3.1415) == true);

    T_VERIFY(variant_setArrayMove(&variant,&list) == true);

    string_t string;
    string_initialize(&string,64);
    T_VERIFY (variant_toJSON(&string, &variant)==true);
    printf("%s",string_buffer(&string));

    T_VERIFY (variant_toJSON(NULL, &variant)==false);
    T_VERIFY (variant_toJSON(&string, NULL)==false);
    string_cleanup(&string);


cleanup:
    variant_map_cleanup(&map);
    variant_list_cleanup(&list);
    variant_cleanup(&variant);
    T_TESTCASE_END();
}

int test_var_fd() {
    T_TESTCASE_BEGIN();

    char buffer[46];
    variant_t varfd;
    T_VERIFY(variant_initialize(&varfd,variant_type_unknown));

    int fd = open("data.txt",O_RDWR | O_CREAT | O_TRUNC);
    T_VERIFY(fd != -1);

    T_VERIFY(write(fd,"ABCDEFGHIJKLMNOPQRSTUVWXYZ",26) != -1);

    T_VERIFY(variant_setFd(&varfd,fd));
    T_VERIFY(variant_bool(&varfd));
    T_EQUAL(int,variant_fd(&varfd),fd);

    T_VERIFY(write(variant_fd(&varfd),"0123456789",10) != -1);

    lseek(variant_fd(&varfd), 0, SEEK_SET);

    int bytes = read(variant_fd(&varfd),buffer,36);
    T_VERIFY(bytes == 36);
    buffer[bytes] = 0;

    T_CMP(buffer,"ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789");

    close(fd);
    unlink("data.txt");

    T_VERIFY(write(variant_fd(&varfd),"0123456789",10) == -1);
    T_VERIFY(read(variant_fd(&varfd),buffer,10) == -1);

    T_VERIFY(variant_bool(&varfd) == false);

cleanup:
    variant_cleanup(&varfd);
    T_TESTCASE_END();
}

typedef struct _data {
    bool bval;
    uint32_t ival;
    char strval[256];
    int64_t ival2;
} data_t;

int test_var_byte_array() {
    T_TESTCASE_BEGIN();
    data_t somedata = { true, 125, "Hello World", -4096 };
    data_t *result = NULL;
    uint32_t size = 0;
    void *data = NULL;
    
    variant_t varba;
    T_VERIFY(variant_initialize(&varba,variant_type_byte_array));
    T_VERIFY(variant_type(&varba) == variant_type_byte_array);
    
    data = variant_byteArray(&varba,&size);
    T_VERIFY(size == 0);
    T_VERIFY(data == NULL);
    
    T_VERIFY(variant_setByteArray(&varba, &somedata, sizeof(data_t)));
    T_VERIFY(variant_toByteArray(&data,&size,&varba));
    
    T_EQUAL(uint32_t,size,sizeof(data_t));
    T_VERIFY(data);
    T_VERIFY(memcmp(&somedata,data,size) == 0);
    
    free(data);
    
    data = variant_byteArray(&varba,&size);
    T_VERIFY(data);
    T_VERIFY(size == sizeof(data_t));
    
    T_VERIFY(memcmp(&somedata,data,size) == 0);
    
    result = (data_t *)data;
    
    free(data);
    
cleanup:
    variant_cleanup(&varba);
    T_TESTCASE_END();
}


T_TESTCASES_BEGIN()
    T_TESTCASE(test_var_string, "Test string variant")
    T_TESTCASE(test_var_int8, "Test int8 variant")
    T_TESTCASE(test_var_int16, "Test int16 variant")
    T_TESTCASE(test_var_int32, "Test int32 variant")
    T_TESTCASE(test_var_int64, "Test int64 variant")
    T_TESTCASE(test_var_uint8, "Test unit8 variant")
    T_TESTCASE(test_var_uint16, "Test unit16 variant")
    T_TESTCASE(test_var_uint32, "Test unit32 variant")
    T_TESTCASE(test_var_uint64, "Test unit64 variant")
    T_TESTCASE(test_var_bool, "Test bool variant")
    T_TESTCASE(test_var_datetime, "Test datetime variant")
    T_TESTCASE(test_var_copy, "Test variant copy function")
    T_TESTCASE(test_var_null, "Test passing null pointers to variant functions")
    T_TESTCASE(test_var_map, "Test variant map")
    T_TESTCASE(test_var_JSON, "Test converting variant to a json string")
    T_TESTCASE(test_var_fd, "Test variant type file descriptor")
    T_TESTCASE(test_var_byte_array, "Test variant type byte array")
T_TESTCASES_END()
