################################################################
# project.lst build flags
#
# SAH_DEBUG..........: No optimalization, trace level set to 500
# SAH_DEBUG_NO_STRIP.: No stripping during release step
#
################################################################

ifndef STAGINGDIR
  $(error STAGINGDIR not defined, stopping build)
endif

include $(STAGINGDIR)/components.config

INSTALL ?= install
STRIP ?= strip
PKG_CONFIG_LIBDIR ?= $(STAGINGDIR)/build/pkg-config

all:
	$(MAKE) -C src all

clean:
	$(MAKE) -C src clean
	$(MAKE) -C test clean
	rm -rf html
	rm -rf pkg-config.pc

pkg-config.pc: pkg-config.pc.m4 $(STAGINGDIR)/components.config
	m4 -I$(STAGINGDIR) $< > $@ 

install: all pkg-config.pc
	$(INSTALL) -D -p -m 0644 include/debug/sahtrace.h $(STAGINGDIR)/include/debug/sahtrace.h
	$(INSTALL) -D -m 0755 src/libsahtrace.so $(STAGINGDIR)/lib/libsahtrace.so
	$(INSTALL) -D -m 0755 src/libsahtrace.a $(STAGINGDIR)/lib/libsahtrace.a
	$(INSTALL) -D -m 0644 pkg-config.pc $(PKG_CONFIG_LIBDIR)/sahtrace.pc

release:
ifndef ROOT_FS_DIR
	$(error ROOT_FS_DIR not defined, stopping release)
endif
	$(INSTALL) -d $(ROOT_FS_DIR)/lib
	$(INSTALL) -m 755 $(STAGINGDIR)/lib/libsahtrace.so $(ROOT_FS_DIR)/lib
ifndef SAH_DEBUG_NO_STRIP
	$(STRIP) $(ROOT_FS_DIR)/lib/libsahtrace.so
endif

doc:
ifndef DOCDIR
	$(error DOCDIR not defined, stop generating documentation)
endif
	doxygen src/sahtrace.doxy
	for FILE in $$(find html -type f); do \
		$(INSTALL) -m 0644 -D $${FILE} $(DOCDIR)/html/$${FILE} || exit 1; \
	done

test:
	make -C test

runtest:
	make -C test test

.PHONY: all clean install release doc test runtest

