#include <unistd.h>
#include <syslog.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdarg.h>
#include <syslog.h>

#include "mock.h"

static bool openlog_unexpected_input = false;

mock_struct_declare(getenv,50);
mock_struct_declare(openlog,50);
mock_struct_declare(open,50);
mock_struct_declare(dup2,50);
mock_struct_declare(vsyslog,50);

typedef int (*open_func_t)(const char *pathname, int flags, ...);
typedef char *(*getenv_func_t)(const char *name);

char *getenv(const char *name) {
    if (sahtest_isRunning()) {
        if (strcmp(mock_get_expectation(getenv)->varname,name) != 0) {
            return NULL;
        }
        const char *value = mock_get_expectation(getenv)->value;
        return value?strdup(value):NULL;
    }

    getenv_func_t real_getenv = mock_real_function("libc.so.6","getenv");
    if (!real_getenv) {
        return NULL;
    }

    return real_getenv(name);
}

void openlog(const char *ident, int option, int facility) {
    openlog_unexpected_input = false;
    if (strcmp(mock_get_expectation(openlog)->ident,ident) != 0) {
        openlog_unexpected_input = true;
    }
    if (mock_get_expectation(openlog)->field != option) {
        openlog_unexpected_input = true;
    }
    if (mock_get_expectation(openlog)->field != facility) {
        openlog_unexpected_input = true;
    }
}

bool open_log_hasUnexpectedInput() {
    return openlog_unexpected_input;
}

int open(const char *pathname, int flags, ...) {
    if (sahtest_isRunning()) {
        T_CMP(mock_get_expectation(open)->pathname,pathname);
        T_VERIFY(mock_get_expectation(open)->flags == flags);
        return mock_get_expectation(open)->fd;
    }
    
    open_func_t real_open = mock_real_function("libc.so.6","open");
    if (!real_open) {
        return -1;
    }

    va_list arguments;
    va_start ( arguments, flags ); 
    int fd = real_open(pathname, flags, va_arg ( arguments, mode_t ));
    va_end ( arguments );

    return fd;
}

int dup2(int oldfd, int newfd) {
    T_VERIFY(mock_get_expectation(dup2)->fd == oldfd);
    T_VERIFY(mock_get_expectation(dup2)->fd == newfd);
    return mock_get_expectation(dup2)->fd;
}

void vsyslog(int priority, const char *format, va_list ap) {
    (void)format;
    (void)ap;
    if (mock_get_expectation(vsyslog)->priority != priority) {
        SAHTEST_ERROR("Unexpected priority given");
    }
}
