#include <dlfcn.h>

#include <debug/sahtrace.h>
#include <sahtest/sahtest.h>

#include "mock.h"

void test_parse_env_setup() {
    mock_reset(openlog);
    mock_reset(open);
    mock_reset(dup2);
    mock_reset(getenv);
}

void test_parse_env_teardown() {
    sahTraceClose();
}

int test_trace_zones_without_levels() {
    mock_add_expectation(getenv)->varname = "SAH_TRACE_ZONES";
    mock_add_expectation(getenv)->value = "zone1,zone2";
    mock_add_expectation(getenv)->varname = "zone1";
    mock_add_expectation(getenv)->value = NULL;
    mock_add_expectation(getenv)->varname = "zone2";
    mock_add_expectation(getenv)->value = NULL;

    sahTraceOpen("test",TRACE_TYPE_STDOUT);

    sah_trace_zone_it *it = sahTraceFirstZone();
    T_VERIFY(it != NULL);
    T_CMP(sahTraceZoneName(it),"zone2");
    T_VERIFY(sahTraceZoneLevel(it) == 500);

    it = sahTraceNextZone(it);
    T_VERIFY(it != NULL);
    T_CMP(sahTraceZoneName(it),"zone1");
    T_VERIFY(sahTraceZoneLevel(it) == 500);

    it = sahTraceNextZone(it);
    T_VERIFY(it == NULL);
    T_VERIFY(sahTraceZoneName(it) ==NULL);
    T_VERIFY(sahTraceZoneLevel(it) == 0);

    sahTraceClose();
    return 0;
}

int test_trace_zones_with_levels() {
    mock_add_expectation(getenv)->varname = "SAH_TRACE_ZONES";
    mock_add_expectation(getenv)->value = "zone1:100,zone2:200,zone3:300,zone4:400";
    mock_add_expectation(getenv)->varname = "zone1";
    mock_add_expectation(getenv)->value = NULL;
    mock_add_expectation(getenv)->varname = "zone2";
    mock_add_expectation(getenv)->value = NULL;
    mock_add_expectation(getenv)->varname = "zone3";
    mock_add_expectation(getenv)->value = NULL;
    mock_add_expectation(getenv)->varname = "zone4";
    mock_add_expectation(getenv)->value = NULL;
    sahTraceOpen("test",TRACE_TYPE_STDOUT);
    
    sah_trace_zone_it *it = sahTraceFirstZone();
    T_VERIFY(it != NULL);
    T_CMP(sahTraceZoneName(it),"zone4");
    T_VERIFY(sahTraceZoneLevel(it) == 400);

    it = sahTraceNextZone(it);
    T_VERIFY(it != NULL);
    T_CMP(sahTraceZoneName(it),"zone3");
    T_VERIFY(sahTraceZoneLevel(it) == 300);

    it = sahTraceNextZone(it);
    T_VERIFY(it != NULL);
    T_CMP(sahTraceZoneName(it),"zone2");
    T_VERIFY(sahTraceZoneLevel(it) == 200);

    it = sahTraceNextZone(it);
    T_VERIFY(it != NULL);
    T_CMP(sahTraceZoneName(it),"zone1");
    T_VERIFY(sahTraceZoneLevel(it) == 100);

    it = sahTraceNextZone(it);
    T_VERIFY(it == NULL);
    T_VERIFY(sahTraceZoneName(it) ==NULL);
    T_VERIFY(sahTraceZoneLevel(it) == 0);

    return 0;
}

int test_trace_zones_with_levels_in_separate_env() {
    mock_add_expectation(getenv)->varname = "SAH_TRACE_ZONES";
    mock_add_expectation(getenv)->value = "zone1,zone2";
    mock_add_expectation(getenv)->varname = "zone1";
    mock_add_expectation(getenv)->value = "100";
    mock_add_expectation(getenv)->varname = "zone2";
    mock_add_expectation(getenv)->value = "200";

    sahTraceOpen("test",TRACE_TYPE_STDOUT);
    
    sah_trace_zone_it *it = sahTraceFirstZone();
    T_VERIFY(it != NULL);
    T_CMP(sahTraceZoneName(it),"zone2");
    T_VERIFY(sahTraceZoneLevel(it) == 200);

    it = sahTraceNextZone(it);
    T_VERIFY(it != NULL);
    T_CMP(sahTraceZoneName(it),"zone1");
    T_VERIFY(sahTraceZoneLevel(it) == 100);

    it = sahTraceNextZone(it);
    T_VERIFY(it == NULL);
    T_VERIFY(sahTraceZoneName(it) ==NULL);
    T_VERIFY(sahTraceZoneLevel(it) == 0);

    return 0;
}
