#include <syslog.h>

#include <debug/sahtrace.h>
#include <sahtest/sahtest.h>

#include "mock.h"

void test_level_setup() {
    mock_reset(openlog);
    mock_reset(open);
    mock_reset(dup2);
    mock_reset(getenv);
    mock_reset(vsyslog);

    mock_add_expectation(getenv)->varname = "SAH_TRACE_ZONES";
    mock_add_expectation(getenv)->value = NULL;

    sahTraceOpen("test",TRACE_TYPE_STDERR);

    if (mock_verify_expectations(getenv) != true) {
        SAHTEST_ERROR("Not all expected values are taken from 'getenv'");
    }
}

void test_level_teardown() {
    sahTraceClose();
}

int test_set_and_verify_level() {
    sahTraceSetLevel(500);
    T_VERIFY(sahTraceLevel() == 500);

    sahTraceSetLevel(100);
    T_VERIFY(sahTraceLevel() == 100);

    return 0;
}

int test_set_and_verify_level_using_zone() {
    sahTraceAddZone(500,"zone1");
    sah_trace_zone_it *z = sahTraceGetZone("zone1");
    T_VERIFY(sahTraceZoneLevel(z) == 500);

    sahTraceZoneSetLevel(z,100);
    T_VERIFY(sahTraceZoneLevel(z) == 100);

    return 0;
}

int test_set_level_for_not_actvated_zone() {
    sah_trace_zone_it *z = sahTraceGetZone("zone1");
    T_VERIFY(z == NULL);
    T_VERIFY(sahTraceZoneLevel(z) == 0);

    return 0;
}
