#include <syslog.h>

#include <debug/sahtrace.h>
#include <sahtest/sahtest.h>

#include "mock.h"

void test_trace_zones_setup() {
    mock_reset(openlog);
    mock_reset(open);
    mock_reset(dup2);
    mock_reset(getenv);
    mock_reset(vsyslog);

    mock_add_expectation(getenv)->varname = "SAH_TRACE_ZONES";
    mock_add_expectation(getenv)->value = NULL;

    sahTraceOpen("test",TRACE_TYPE_STDERR);

    if (mock_verify_expectations(getenv) != true) {
        SAHTEST_ERROR("Not all expected values are taken from 'getenv'");
    }
    if (mock_verify_expectations(openlog) != true) {
        SAHTEST_ERROR("Not all expected values are taken from 'openlog'");
    }
}

void test_trace_zones_teardown() {
    sahTraceClose();
}

int test_adding_trace_zone() {
    sahTraceAddZone(500,"zone1");
    sah_trace_zone_it *z = sahTraceGetZone("zone1");
    T_VERIFY(sahTraceZoneLevel(z) == 500);

    return 0;
}

int test_removing_trace_zone() {
    sahTraceAddZone(500,"zone1");
    sah_trace_zone_it *z = sahTraceGetZone("zone1");
    T_VERIFY(sahTraceZoneLevel(z) == 500);

    sahTraceRemoveZone("zone1");
    T_VERIFY(sahTraceGetZone("zone1") == NULL);

    return 0;
}

int test_trace_zone_iteration() {
    sahTraceAddZone(500,"zone1");
    sahTraceAddZone(500,"zone2");
    sahTraceAddZone(500,"zone3");

    sah_trace_zone_it *it = sahTraceFirstZone();
    T_VERIFY(it != NULL);
    T_CMP(sahTraceZoneName(it),"zone3");

    it = sahTraceNextZone(it);
    T_VERIFY(it != NULL);
    T_CMP(sahTraceZoneName(it),"zone2");

    it = sahTraceNextZone(it);
    T_VERIFY(it != NULL);
    T_CMP(sahTraceZoneName(it),"zone1");

    it = sahTraceNextZone(it);
    T_VERIFY(it == NULL);

    it = sahTraceNextZone(it);
    T_VERIFY(it == NULL);

    return 0;
}

int test_add_3_zones_remove_first() {
    sahTraceAddZone(500,"zone1");
    sahTraceAddZone(500,"zone2");
    sahTraceAddZone(500,"zone3");

    sah_trace_zone_it *it = sahTraceFirstZone();
    T_VERIFY(it != NULL);
    T_CMP(sahTraceZoneName(it),"zone3");

    it = sahTraceNextZone(it);
    T_VERIFY(it != NULL);
    T_CMP(sahTraceZoneName(it),"zone2");

    it = sahTraceNextZone(it);
    T_VERIFY(it != NULL);
    T_CMP(sahTraceZoneName(it),"zone1");

    it = sahTraceNextZone(it);
    T_VERIFY(it == NULL);

    sahTraceRemoveZone("zone1");

    it = sahTraceFirstZone();
    T_VERIFY(it != NULL);
    T_CMP(sahTraceZoneName(it),"zone3");

    it = sahTraceNextZone(it);
    T_VERIFY(it != NULL);
    T_CMP(sahTraceZoneName(it),"zone2");

    it = sahTraceNextZone(it);
    T_VERIFY(it == NULL);

    return 0;
}

int test_add_3_zones_remove_last() {
    sahTraceAddZone(500,"zone1");
    sahTraceAddZone(500,"zone2");
    sahTraceAddZone(500,"zone3");

    sah_trace_zone_it *it = sahTraceFirstZone();
    T_VERIFY(it != NULL);
    T_CMP(sahTraceZoneName(it),"zone3");

    it = sahTraceNextZone(it);
    T_VERIFY(it != NULL);
    T_CMP(sahTraceZoneName(it),"zone2");

    it = sahTraceNextZone(it);
    T_VERIFY(it != NULL);
    T_CMP(sahTraceZoneName(it),"zone1");

    it = sahTraceNextZone(it);
    T_VERIFY(it == NULL);

    sahTraceRemoveZone("zone3");

    it = sahTraceFirstZone();
    T_VERIFY(it != NULL);
    T_CMP(sahTraceZoneName(it),"zone2");

    it = sahTraceNextZone(it);
    T_VERIFY(it != NULL);
    T_CMP(sahTraceZoneName(it),"zone1");

    it = sahTraceNextZone(it);
    T_VERIFY(it == NULL);
    return 0;
}

int test_add_3_zones_remove_middle() {
    sahTraceAddZone(500,"zone1");
    sahTraceAddZone(500,"zone2");
    sahTraceAddZone(500,"zone3");

    sah_trace_zone_it *it = sahTraceFirstZone();
    T_VERIFY(it != NULL);
    T_CMP(sahTraceZoneName(it),"zone3");

    it = sahTraceNextZone(it);
    T_VERIFY(it != NULL);
    T_CMP(sahTraceZoneName(it),"zone2");

    it = sahTraceNextZone(it);
    T_VERIFY(it != NULL);
    T_CMP(sahTraceZoneName(it),"zone1");

    it = sahTraceNextZone(it);
    T_VERIFY(it == NULL);

    sahTraceRemoveZone("zone2");

    it = sahTraceFirstZone();
    T_VERIFY(it != NULL);
    T_CMP(sahTraceZoneName(it),"zone3");

    it = sahTraceNextZone(it);
    T_VERIFY(it != NULL);
    T_CMP(sahTraceZoneName(it),"zone1");

    it = sahTraceNextZone(it);
    T_VERIFY(it == NULL);

    return 0;
}

int test_add_same_zone_twice() {
    sahTraceAddZone(500,"zone1");
    sahTraceAddZone(500,"zone1");

    sah_trace_zone_it *it = sahTraceFirstZone();
    T_VERIFY(it != NULL);
    T_CMP(sahTraceZoneName(it),"zone1");

    it = sahTraceNextZone(it);
    T_VERIFY(it == NULL);

    return 0;
}

int test_remove_same_zone_twice() {
    sahTraceAddZone(500,"zone1");

    sah_trace_zone_it *it = sahTraceFirstZone();
    T_VERIFY(it != NULL);
    T_CMP(sahTraceZoneName(it),"zone1");

    it = sahTraceNextZone(it);
    T_VERIFY(it == NULL);

    sahTraceRemoveZone("zone1");
    sahTraceRemoveZone("zone1");

    it = sahTraceFirstZone(it);
    T_VERIFY(it == NULL);

    return 0;
}
