#include <syslog.h>

#include <debug/sahtrace.h>
#include <sahtest/sahtest.h>

#include "mock.h"

void test_write_msg_setup() {
    mock_reset(openlog);
    mock_reset(open);
    mock_reset(dup2);
    mock_reset(getenv);
    mock_reset(vsyslog);

    mock_add_expectation(getenv)->varname = "SAH_TRACE_ZONES";
    mock_add_expectation(getenv)->value = NULL;

    mock_add_expectation(openlog)->ident = "test";
    mock_add_expectation(openlog)->field = LOG_CONS;
    mock_add_expectation(openlog)->field = LOG_USER;

    sahTraceOpen("test",TRACE_TYPE_SYSLOG);

    if (mock_verify_expectations(getenv) != true) {
        SAHTEST_ERROR("Not all expected values are taken from 'getenv'");
    }
    if (mock_verify_expectations(openlog) != true) {
        SAHTEST_ERROR("Not all expected values are taken from 'openlog'");
    }
}

void test_write_msg_teardown() {
    sahTraceClose();
}

int test_write_msgs_level_100() {
    mock_add_expectation(vsyslog)->priority = LOG_CRIT;
    mock_add_expectation(vsyslog)->priority = LOG_ERR;

    sahTraceSetLevel(100);

    sahTrace(TRACE_LEVEL_FATAL_ERROR,"%s","Test text FATAL_ERROR");
    sahTrace(TRACE_LEVEL_ERROR,"%s","Test text ERROR");
    sahTrace(TRACE_LEVEL_WARNING,"%s","Test text WARNING");
    sahTrace(TRACE_LEVEL_NOTICE,"%s","Test text NOTICE");
    sahTrace(TRACE_LEVEL_APP_INFO,"%s","Test text APP_INFO");
    sahTrace(TRACE_LEVEL_INFO,"%s","Test text INFO");
    sahTrace(TRACE_LEVEL_CALLSTACK,"%s","Test text TRACE_LEVEL_CALLSTACK");

    T_VERIFY(mock_verify_expectations(vsyslog) == true);

    return 0;
}

int test_write_msgs_level_200() {
    mock_add_expectation(vsyslog)->priority = LOG_CRIT;
    mock_add_expectation(vsyslog)->priority = LOG_ERR;
    mock_add_expectation(vsyslog)->priority = LOG_WARNING;

    sahTraceSetLevel(200);

    sahTrace(TRACE_LEVEL_FATAL_ERROR,"%s","Test text FATAL_ERROR");
    sahTrace(TRACE_LEVEL_ERROR,"%s","Test text ERROR");
    sahTrace(TRACE_LEVEL_WARNING,"%s","Test text WARNING");
    sahTrace(TRACE_LEVEL_NOTICE,"%s","Test text NOTICE");
    sahTrace(TRACE_LEVEL_APP_INFO,"%s","Test text APP_INFO");
    sahTrace(TRACE_LEVEL_INFO,"%s","Test text INFO");
    sahTrace(TRACE_LEVEL_CALLSTACK,"%s","Test text TRACE_LEVEL_CALLSTACK");

    T_VERIFY(mock_verify_expectations(vsyslog) == true);

    return 0;
}

int test_write_msgs_level_300() {
    mock_add_expectation(vsyslog)->priority = LOG_CRIT;
    mock_add_expectation(vsyslog)->priority = LOG_ERR;
    mock_add_expectation(vsyslog)->priority = LOG_WARNING;
    mock_add_expectation(vsyslog)->priority = LOG_NOTICE;

    sahTraceSetLevel(300);

    sahTrace(TRACE_LEVEL_FATAL_ERROR,"%s","Test text FATAL_ERROR");
    sahTrace(TRACE_LEVEL_ERROR,"%s","Test text ERROR");
    sahTrace(TRACE_LEVEL_WARNING,"%s","Test text WARNING");
    sahTrace(TRACE_LEVEL_NOTICE,"%s","Test text NOTICE");
    sahTrace(TRACE_LEVEL_APP_INFO,"%s","Test text APP_INFO");
    sahTrace(TRACE_LEVEL_INFO,"%s","Test text INFO");
    sahTrace(TRACE_LEVEL_CALLSTACK,"%s","Test text TRACE_LEVEL_CALLSTACK");

    T_VERIFY(mock_verify_expectations(vsyslog) == true);

    return 0;
}

int test_write_msgs_level_400() {
    mock_add_expectation(vsyslog)->priority = LOG_CRIT;
    mock_add_expectation(vsyslog)->priority = LOG_ERR;
    mock_add_expectation(vsyslog)->priority = LOG_WARNING;
    mock_add_expectation(vsyslog)->priority = LOG_NOTICE;
    mock_add_expectation(vsyslog)->priority = LOG_INFO;
    mock_add_expectation(vsyslog)->priority = LOG_INFO;

    sahTraceSetLevel(400);

    sahTrace(TRACE_LEVEL_FATAL_ERROR,"%s","Test text FATAL_ERROR");
    sahTrace(TRACE_LEVEL_ERROR,"%s","Test text ERROR");
    sahTrace(TRACE_LEVEL_WARNING,"%s","Test text WARNING");
    sahTrace(TRACE_LEVEL_NOTICE,"%s","Test text NOTICE");
    sahTrace(TRACE_LEVEL_APP_INFO,"%s","Test text APP_INFO");
    sahTrace(TRACE_LEVEL_INFO,"%s","Test text INFO");
    sahTrace(TRACE_LEVEL_CALLSTACK,"%s","Test text TRACE_LEVEL_CALLSTACK");

    T_VERIFY(mock_verify_expectations(vsyslog) == true);

    return 0;
}

int test_write_msgs_level_500() {
    mock_add_expectation(vsyslog)->priority = LOG_CRIT;
    mock_add_expectation(vsyslog)->priority = LOG_ERR;
    mock_add_expectation(vsyslog)->priority = LOG_WARNING;
    mock_add_expectation(vsyslog)->priority = LOG_NOTICE;
    mock_add_expectation(vsyslog)->priority = LOG_INFO;
    mock_add_expectation(vsyslog)->priority = LOG_INFO;
    mock_add_expectation(vsyslog)->priority = LOG_DEBUG;

    sahTraceSetLevel(500);

    sahTrace(TRACE_LEVEL_FATAL_ERROR,"%s","Test text FATAL_ERROR");
    sahTrace(TRACE_LEVEL_ERROR,"%s","Test text ERROR");
    sahTrace(TRACE_LEVEL_WARNING,"%s","Test text WARNING");
    sahTrace(TRACE_LEVEL_NOTICE,"%s","Test text NOTICE");
    sahTrace(TRACE_LEVEL_APP_INFO,"%s","Test text APP_INFO");
    sahTrace(TRACE_LEVEL_INFO,"%s","Test text INFO");
    sahTrace(TRACE_LEVEL_CALLSTACK,"%s","Test text TRACE_LEVEL_CALLSTACK");

    T_VERIFY(mock_verify_expectations(vsyslog) == true);

    return 0;
}

int test_write_msgs_using_added_zone_500() {
    mock_add_expectation(vsyslog)->priority = LOG_CRIT;
    mock_add_expectation(vsyslog)->priority = LOG_ERR;
    mock_add_expectation(vsyslog)->priority = LOG_DEBUG;

    sahTraceSetLevel(100);
    sahTraceAddZone(500,"zone1");

    sahTraceZ(TRACE_LEVEL_FATAL_ERROR,"zone1","%s","Test text FATAL_ERROR");
    sahTraceZ(TRACE_LEVEL_ERROR,"zone1","%s","Test text ERROR");
    sahTraceZ(TRACE_LEVEL_CALLSTACK,"zone1","%s","Test text TRACE_LEVEL_CALLSTACK");

    T_VERIFY(mock_verify_expectations(vsyslog) == true);

    return 0;
}

int test_write_msgs_using_added_zone_0() {
    mock_add_expectation(vsyslog)->priority = LOG_CRIT;

    sahTraceSetLevel(100);
    sahTraceAddZone(0,"zone1");

    sahTraceZ(TRACE_LEVEL_FATAL_ERROR,"zone1","%s","Test text FATAL_ERROR");
    sahTraceZ(TRACE_LEVEL_ERROR,"zone1","%s","Test text ERROR");
    sahTraceZ(TRACE_LEVEL_CALLSTACK,"zone1","%s","Test text TRACE_LEVEL_CALLSTACK");

    T_VERIFY(mock_verify_expectations(vsyslog) == true);

    return 0;
}

int test_write_msgs_using_zone() {
    mock_add_expectation(vsyslog)->priority = LOG_CRIT;

    sahTraceSetLevel(100);

    sahTraceZ(TRACE_LEVEL_FATAL_ERROR,"zone1","%s","Test text FATAL_ERROR");
    sahTraceZ(TRACE_LEVEL_ERROR,"zone1","%s","Test text ERROR");
    sahTraceZ(TRACE_LEVEL_CALLSTACK,"zone1","%s","Test text TRACE_LEVEL_CALLSTACK");

    T_VERIFY(mock_verify_expectations(vsyslog) == false);

    return 0;
}
