# ebtables Makefile

#do silent make
#Q=@

include $(STAGINGDIR)/components.config

PROGNAME:=ebtables
PROGVERSION:=2.0.6
PROGDATE:=November\ 2003

KERNELRELEASE := $(shell $(MAKE) --silent --no-print-directory -C $(HARDCO_KERNEL_DIR) kernelrelease  | sed  -n "s/.[0-9]*\.[0-9]*\.\([0-9]*\).*/\1/p")

CFLAGS:=-Wall -Wunused -Os -DKERNEL_VERSION_SUBLEVEL=$(KERNELRELEASE)

ifeq ($(shell uname -m),sparc64)
CFLAGS+=-DEBT_MIN_ALIGN=8 -DKERNEL_64_USERSPACE_32
endif

-include extensions/Makefile

OBJS:=getethertype.o ebtables.o communication.o $(EXT_OBJS)

KERNEL_INCLUDES=$(HARDCO_KERNEL_DIR)/include

ETHERTYPESFILE:=$(STAGINGDIR)/etc/ethertypes
BINFILE:=$(STAGINGDIR)/usr/sbin/$(PROGNAME)

PROGSPECS:=-DPROGVERSION=\"$(PROGVERSION)\" -DPROGNAME=\"$(PROGNAME)\" \
	-DPROGDATE=\"$(PROGDATE)\" -D_PATH_ETHERTYPES=\"$(ETHERTYPESFILE)\"

all: $(PROGNAME)
	@echo KERNELRELEASE $(KERNELRELEASE)

communication.o: communication.c include/ebtables_u.h
	@echo [CC] $<
	$(Q)$(CC) $(CFLAGS) $(PROGSPECS) -c -o $@ $< -I$(KERNEL_INCLUDES)

getethertype.o: getethertype.c include/ethernetdb.h
	@echo [CC] $<
	$(Q)$(CC) $(CFLAGS) $(PROGSPECS) -c -o $@ $< -Iinclude/

ebtables.o: ebtables.c include/ebtables_u.h
	@echo [CC] $<
	$(Q)$(CC) $(CFLAGS) $(PROGSPECS) -c -o $@ $< -I$(KERNEL_INCLUDES)

$(PROGNAME): $(OBJS)
	@echo [LINKING] $(PROGNAME)
	$(Q)$(CC) $(CFLAGS) -o $@ $^ 

# TODO CHECK THIS $(ETHERTYPESFILE): ethertypes
#	mkdir -p $(@D)
#	$(INSTALL) -m 0644 -o root -g root $< $@

release:
	@echo [RELEASE] $(PROGNAME)
	$(Q)$(INSTALL) -d $(ROOT_FS_DIR)/usr/sbin/
	$(Q)$(INSTALL) -m 0755 $(STAGINGDIR)/usr/sbin/$(PROGNAME) $(ROOT_FS_DIR)/usr/sbin/
	$(Q)$(STRIP) $(ROOT_FS_DIR)/usr/sbin/$(PROGNAME)

install: 
	@echo [INSTALL] $(PROGNAME)
	$(Q)$(INSTALL) -d $(STAGINGDIR)/usr/sbin/
	$(Q)$(INSTALL) -m 0755 $(PROGNAME) $(BINFILE)

clean:
	@echo [CLEAN] $(PROGNAME)
	$(Q)rm -f $(PROGNAME)
	$(Q)rm -f *.o *.c~
	$(Q)rm -f extensions/*.o extensions/*.c~

.PHONY: all clean install release
