
include $(STAGINGDIR)/components.config

ifeq ($(notdir $(CC)),gcc)
  FUSE_FULL_CONFIGURATION:=$(FUSE_FULL_CONFIGURATION) --disable-kernel-module --disable-static --enable-util --enable-shared --disable-mtab --exec-prefix=/ --with-gnu-ld
else
  FUSE_FULL_CONFIGURATION:=$(FUSE_FULL_CONFIGURATION) --host=mips-linux-uclibc --disable-kernel-module --disable-static --enable-util --exec-prefix=/ --disable-mtab
endif

CPPFLAGS = -I$(STAGINGDIR)/include
ifeq ($(CONFIG_FUSE_FORCE_FUSERMOUNT),y)
CPPFLAGS += -DFORCE_FUSERMOUNT
endif

all: build

src/Makefile:
	cd src && ./configure $(FUSE_FULL_CONFIGURATION) CPPFLAGS="$(CPPFLAGS)"

clean:
	@echo "Cleaning fuse"
	if [ -e src/Makefile ]; then  $(MAKE) -C src -f Makefile clean ; fi
	-find src -name .deps -exec rm -rf {} \;
	-find src -name .libs -exec rm -rf {} \;
	-find src -name Makefile -exec rm -f {} \;
	rm -rf src/autom4te.cache src/config.log src/config.status src/*.pc
	rm -f src/include/stamp-* src/include/config.h
	rm -rf install
	-rm -rf src/libtool

patch:
	@echo "Patching fuse"
	-patch -Np1 -d src < patches/fuse-2.7.4_dcache_bug.patch
	-patch -Np1 -d src < patches/fuse-2.7.4_init_script.patch

build: src/Makefile
	@echo "Building fuse"
	$(MAKE) -C src -f Makefile
	rm -rf install/
	mkdir install
	fakeroot $(MAKE) -C src -f Makefile install DESTDIR=`pwd`/install

install:
	$(INSTALL) -d $(STAGINGDIR)/bin
	$(INSTALL) -d $(STAGINGDIR)/sbin
	$(INSTALL) -d $(STAGINGDIR)/lib
	$(INSTALL) -d $(STAGINGDIR)/include/fuse
	$(INSTALL) -d $(STAGINGDIR)/etc_ro_fs/init.d
	$(INSTALL) -d $(STAGINGDIR)/etc_ro_fs/rc1.d

	cp -dp install/lib/libfuse.so* $(STAGINGDIR)/lib/
	cp -a  install/usr/local/include/* $(STAGINGDIR)/include/
	$(INSTALL) -m 0755 install/bin/fusermount $(STAGINGDIR)/bin/
	$(INSTALL) -m 0755 install/sbin/mount.fuse $(STAGINGDIR)/sbin/
	$(INSTALL) -m 0755 install/etc/init.d/fuse $(STAGINGDIR)/etc_ro_fs/init.d/
	ln -sf ../init.d/fuse $(STAGINGDIR)/etc_ro_fs/rc1.d/S420fuse

release:
	$(INSTALL) -d $(ROOT_FS_DIR)/bin
	$(INSTALL) -d $(ROOT_FS_DIR)/sbin
	$(INSTALL) -d $(ROOT_FS_DIR)/dev
	$(INSTALL) -d $(ROOT_FS_DIR)/lib
	$(INSTALL) -d $(ROOT_FS_DIR)/etc_ro_fs/init.d
	$(INSTALL) -d $(ROOT_FS_DIR)/etc_ro_fs/rc1.d

	cp -dp $(STAGINGDIR)/lib/libfuse.so* $(ROOT_FS_DIR)/lib/
	$(INSTALL) -m 0755 $(STAGINGDIR)/bin/fusermount $(ROOT_FS_DIR)/bin/
	$(STRIP) $(ROOT_FS_DIR)/bin/fusermount 
	$(INSTALL) -m 0755 $(STAGINGDIR)/sbin/mount.fuse $(ROOT_FS_DIR)/sbin/
	$(STRIP) $(ROOT_FS_DIR)/sbin/mount.fuse
	$(INSTALL) -m 0755 $(STAGINGDIR)/etc_ro_fs/init.d/fuse $(ROOT_FS_DIR)/etc_ro_fs/init.d/
	ln -sf ../init.d/fuse $(ROOT_FS_DIR)/etc_ro_fs/rc1.d/S420fuse
	mknod $(ROOT_FS_DIR)/dev/fuse -m 0666 c 10 229

.PHONY: clean build install release patch all


