# include components.config
ifdef STAGINGDIR
include $(STAGINGDIR)/components.config
endif

INSTALL ?= install
STRIP ?= strip

ifndef	CONFIG_SAH_SERVICES_FW_SOCKET
$(error CONFIG_SAH_SERVICES_FW_SOCKET not defined => fix defconfig! )
endif

all:
	@echo -en "\033[1m  ## Make firewall client ##\n\033[0m"
	@$(MAKE) -C client all
	-m4 -I $(STAGINGDIR) $(STAGINGDIR)/config/firewall_map.odl.m4 > firewall_map.odl
	-m4 -DSTAGINGDIR=$(STAGINGDIR) fw.odl.m4 > fw.odl
	-m4 -D PLUGIN_PRIORITY=$(CONFIG_SAH_SERVICES_FW_PRIORITY) -D FW_SOCKET=$(CONFIG_SAH_SERVICES_FW_SOCKET) scripts/firewall.m4 > scripts/firewall

clean:
	@echo -en "\033[1m  ## Clean firewall files ##\n\033[0m"
	$(MAKE) -C client clean
	rm -rf firewall_map.odl fw.odl

install:
	@echo -en "\033[1m  ## Install firewall files ##\n\033[0m"
	$(INSTALL) -d $(STAGINGDIR)/usr/lib/firewall
	$(INSTALL) -d $(STAGINGDIR)/lib
	$(INSTALL) -d $(STAGINGDIR)/include/fw
	$(INSTALL) -d $(PKG_CONFIG_LIBDIR)
	$(INSTALL) -d $(STAGINGDIR)/usr/lib/hgwcfg/backup
	$(INSTALL) -d $(STAGINGDIR)/usr/lib/hgwcfg/restore
	$(INSTALL) -d $(STAGINGDIR)/usr/lib/debuginfo
	$(INSTALL) -d $(STAGINGDIR)/usr/lib/igd_mapping/firewall
	$(INSTALL) -m 0644 client/libfw_client.a $(STAGINGDIR)/lib
	$(INSTALL) -p -m 0644 client/fw_client.h $(STAGINGDIR)/include/fw
	$(INSTALL) -p -m 0644 common/fw_common.h $(STAGINGDIR)/include/fw
	$(INSTALL) -m 0644 client/fw_client.pc $(PKG_CONFIG_LIBDIR)
	@$(INSTALL) -d $(STAGINGDIR)/etc/init.d
	@$(INSTALL) -d $(STAGINGDIR)/etc/rc1.d
	ln -sf ../init.d/firewall $(STAGINGDIR)/etc/rc1.d/S$(CONFIG_SAH_SERVICES_FW_ORDER)firewall
	ln -sf $(STAGINGDIR)/etc/init.d/firewall $(STAGINGDIR)/usr/lib/hgwcfg/backup/B10firewall
	ln -sf $(STAGINGDIR)/etc/init.d/firewall $(STAGINGDIR)/usr/lib/hgwcfg/restore/R10firewall
	ln -sf $(STAGINGDIR)/etc/init.d/firewall $(STAGINGDIR)/usr/lib/debuginfo/D10firewall
ifeq ($(CONFIG_SAH_SERVICES_FW_PCP),y)
	$(INSTALL) -m 0644 pcp/pcp.odl $(STAGINGDIR)/usr/lib/firewall
endif

release:
	@echo -en "\033[1m  ## Release firewall files ##\n\033[0m"
	$(INSTALL) -d $(ROOT_FS_DIR)/usr/lib/firewall
	$(INSTALL) -d $(ROOT_FS_DIR)/etc_ro_fs/init.d
	$(INSTALL) -d $(ROOT_FS_DIR)/etc_ro_fs/rc1.d
	$(INSTALL) -d $(ROOT_FS_DIR)/usr/lib/hgwcfg/backup
	$(INSTALL) -d $(ROOT_FS_DIR)/usr/lib/hgwcfg/restore
	$(INSTALL) -d $(ROOT_FS_DIR)/usr/lib/debuginfo
	$(INSTALL) -d $(ROOT_FS_DIR)/usr/lib/igd_mapping/firewall
	ln -sf ../init.d/firewall $(ROOT_FS_DIR)/etc_ro_fs/rc1.d/S$(CONFIG_SAH_SERVICES_FW_ORDER)firewall
	ln -sf /etc/init.d/firewall $(ROOT_FS_DIR)/usr/lib/hgwcfg/backup/B10firewall
	ln -sf /etc/init.d/firewall $(ROOT_FS_DIR)/usr/lib/hgwcfg/restore/R10firewall
	ln -sf /etc/init.d/firewall $(ROOT_FS_DIR)/usr/lib/debuginfo/D10firewall
ifeq ($(CONFIG_SAH_SERVICES_FW_PCP),y)
	$(INSTALL) -m 0644 $(STAGINGDIR)/usr/lib/firewall/pcp.odl $(ROOT_FS_DIR)/usr/lib/firewall/pcp.odl
endif

build_package:
	make -f Component.mk release ROOT_FS_DIR=$(COMP_PACKAGE_DIR)

release_package:
	cp -af $(COMP_PACKAGE_DIR)/* $(ROOT_FS_DIR)

doc:
	@echo -en "\033[1m  ## Generate fw.wiki file ##\n\033[0m"
	-pcb_docgen fw-definitions.odl > fw.xml
	-pcb_docgen --version "V5.0" --wiki fw.xml > fw.wiki


.PHONY: all clean install release build_package release_package doc
