#include <stdlib.h>
#include <debug/sahtrace.h>
#include <pcb/utils.h>
#include <pcb/core.h>
#include <pcb/utils/variant.h>

#include "fw_common.h"


//Generated function from ./fw.odl@162 for object Firewall
static request_t *build_save_request(const char *objectPath) {
    // create request
    request_t *req = request_create_executeFunction(objectPath,"save",0);
    if (!req) {
        SAH_TRACE_ERROR("Failed to create request");
        goto error;
    }

    // initialize variant
    variant_t val;
    if (!variant_initialize(&val, variant_type_unknown)) {
        SAH_TRACE_ERROR("Failed to initialize variant");
        goto error_cleanup_request;
    }

    // add value(s) to request

    variant_cleanup(&val);
    return req;

error_cleanup_request:
    request_destroy(req);
error:
    return NULL;
} // create request for function save end

//Generated function from ./fw.odl@162 for object Firewall
static bool parse_save_reply(request_t *req, bool *retval) {
    // parse reply for function call save
    // get the reply
    reply_t *reply = request_reply(req);
    if (!reply) {
        SAH_TRACE_ERROR("No reply available");
        goto exit_error;
    }

    // iterate the reply items
    reply_item_t *item = reply_firstItem(reply);
    while(item) {
        switch(reply_item_type(item)) {
            case reply_type_function_return: {
                const variant_t *val = reply_item_returnValue(item);
                *retval = variant_bool(val);
            }
            break;
            case reply_type_error: {
                SAH_TRACE_WARNING("Function call error (%d) - %s - %s",reply_item_error(item),reply_item_errorDescription(item),reply_item_errorInfo(item));
            }
            break;
            default:
                SAH_TRACE_WARNING("Unexpected reply item");
            break;
        }
        item = reply_nextItem(item);
    }

    return true;

exit_error:
    return false;
}  // parse reply for function save end

//Generated function from ..odl/fw@162 for object Firewall
bool  save(peer_info_t *dest, const char *objectPath) {

    bool  retval = true;
    if (!dest || !objectPath) {
        SAH_TRACE_ERROR("Invalid parameters specified");
        goto error;
    }
    // get pcb context
    pcb_t *pcb = pcb_fromConnection(peer_connection(dest));
    if (!pcb) {
        SAH_TRACE_ERROR("Invalid destination specified");
        goto error;
    }
    // build request
    request_t *req = build_save_request(objectPath);
    if (!req) {
        SAH_TRACE_ERROR("Failed to build request");
        goto error;
    }

    // send the request
    if (!pcb_sendRequest(pcb,dest,req)) {
        SAH_TRACE_ERROR("Failed to send request");
        goto error_cleanup_request;
    }

    // wait for reply
    SAH_TRACE_INFO("Function called, wait for reply");
    if (!pcb_waitForReply(pcb,req,NULL)) {
        SAH_TRACE_ERROR("Communication failed");
        goto error_cleanup_request;
    }

    if (!parse_save_reply(req, &retval)) {
        SAH_TRACE_ERROR("Failed to parse reply");
        goto error_cleanup_request;
    }

    request_destroy(req);
    return retval;

error_cleanup_request:
    request_destroy(req);
error:
    return retval;
}

//Generated function from ./fw.odl@163 for object Firewall
static request_t *build_reset_request(const char *objectPath) {
    // create request
    request_t *req = request_create_executeFunction(objectPath,"reset",0);
    if (!req) {
        SAH_TRACE_ERROR("Failed to create request");
        goto error;
    }

    // initialize variant
    variant_t val;
    if (!variant_initialize(&val, variant_type_unknown)) {
        SAH_TRACE_ERROR("Failed to initialize variant");
        goto error_cleanup_request;
    }

    // add value(s) to request

    variant_cleanup(&val);
    return req;

error_cleanup_request:
    request_destroy(req);
error:
    return NULL;
} // create request for function reset end

//Generated function from ./fw.odl@163 for object Firewall
static bool parse_reset_reply(request_t *req, bool *retval) {
    // parse reply for function call reset
    // get the reply
    reply_t *reply = request_reply(req);
    if (!reply) {
        SAH_TRACE_ERROR("No reply available");
        goto exit_error;
    }

    // iterate the reply items
    reply_item_t *item = reply_firstItem(reply);
    while(item) {
        switch(reply_item_type(item)) {
            case reply_type_function_return: {
                const variant_t *val = reply_item_returnValue(item);
                *retval = variant_bool(val);
            }
            break;
            case reply_type_error: {
                SAH_TRACE_WARNING("Function call error (%d) - %s - %s",reply_item_error(item),reply_item_errorDescription(item),reply_item_errorInfo(item));
            }
            break;
            default:
                SAH_TRACE_WARNING("Unexpected reply item");
            break;
        }
        item = reply_nextItem(item);
    }

    return true;

exit_error:
    return false;
}  // parse reply for function reset end

//Generated function from ..odl/fw@163 for object Firewall
bool  reset(peer_info_t *dest, const char *objectPath) {

    bool  retval = true;
    if (!dest || !objectPath) {
        SAH_TRACE_ERROR("Invalid parameters specified");
        goto error;
    }
    // get pcb context
    pcb_t *pcb = pcb_fromConnection(peer_connection(dest));
    if (!pcb) {
        SAH_TRACE_ERROR("Invalid destination specified");
        goto error;
    }
    // build request
    request_t *req = build_reset_request(objectPath);
    if (!req) {
        SAH_TRACE_ERROR("Failed to build request");
        goto error;
    }

    // send the request
    if (!pcb_sendRequest(pcb,dest,req)) {
        SAH_TRACE_ERROR("Failed to send request");
        goto error_cleanup_request;
    }

    // wait for reply
    SAH_TRACE_INFO("Function called, wait for reply");
    if (!pcb_waitForReply(pcb,req,NULL)) {
        SAH_TRACE_ERROR("Communication failed");
        goto error_cleanup_request;
    }

    if (!parse_reset_reply(req, &retval)) {
        SAH_TRACE_ERROR("Failed to parse reply");
        goto error_cleanup_request;
    }

    request_destroy(req);
    return retval;

error_cleanup_request:
    request_destroy(req);
error:
    return retval;
}

//Generated function from ./fw.odl@164 for object Firewall
static request_t *build_commit_request(const char *objectPath) {
    // create request
    request_t *req = request_create_executeFunction(objectPath,"commit",0);
    if (!req) {
        SAH_TRACE_ERROR("Failed to create request");
        goto error;
    }

    // initialize variant
    variant_t val;
    if (!variant_initialize(&val, variant_type_unknown)) {
        SAH_TRACE_ERROR("Failed to initialize variant");
        goto error_cleanup_request;
    }

    // add value(s) to request

    variant_cleanup(&val);
    return req;

error_cleanup_request:
    request_destroy(req);
error:
    return NULL;
} // create request for function commit end

//Generated function from ./fw.odl@164 for object Firewall
static bool parse_commit_reply(request_t *req, bool *retval) {
    // parse reply for function call commit
    // get the reply
    reply_t *reply = request_reply(req);
    if (!reply) {
        SAH_TRACE_ERROR("No reply available");
        goto exit_error;
    }

    // iterate the reply items
    reply_item_t *item = reply_firstItem(reply);
    while(item) {
        switch(reply_item_type(item)) {
            case reply_type_function_return: {
                const variant_t *val = reply_item_returnValue(item);
                *retval = variant_bool(val);
            }
            break;
            case reply_type_error: {
                SAH_TRACE_WARNING("Function call error (%d) - %s - %s",reply_item_error(item),reply_item_errorDescription(item),reply_item_errorInfo(item));
            }
            break;
            default:
                SAH_TRACE_WARNING("Unexpected reply item");
            break;
        }
        item = reply_nextItem(item);
    }

    return true;

exit_error:
    return false;
}  // parse reply for function commit end

//Generated function from ..odl/fw@164 for object Firewall
bool  commit(peer_info_t *dest, const char *objectPath) {

    bool  retval = true;
    if (!dest || !objectPath) {
        SAH_TRACE_ERROR("Invalid parameters specified");
        goto error;
    }
    // get pcb context
    pcb_t *pcb = pcb_fromConnection(peer_connection(dest));
    if (!pcb) {
        SAH_TRACE_ERROR("Invalid destination specified");
        goto error;
    }
    // build request
    request_t *req = build_commit_request(objectPath);
    if (!req) {
        SAH_TRACE_ERROR("Failed to build request");
        goto error;
    }

    // send the request
    if (!pcb_sendRequest(pcb,dest,req)) {
        SAH_TRACE_ERROR("Failed to send request");
        goto error_cleanup_request;
    }

    // wait for reply
    SAH_TRACE_INFO("Function called, wait for reply");
    if (!pcb_waitForReply(pcb,req,NULL)) {
        SAH_TRACE_ERROR("Communication failed");
        goto error_cleanup_request;
    }

    if (!parse_commit_reply(req, &retval)) {
        SAH_TRACE_ERROR("Failed to parse reply");
        goto error_cleanup_request;
    }

    request_destroy(req);
    return retval;

error_cleanup_request:
    request_destroy(req);
error:
    return retval;
}

//Generated function from ./fw.odl@167 for object Firewall
static request_t *build_setService_request(const char *objectPath, const char *id, const char *sourceInterface, const char *destinationPort, const char *protocol, int32_t ipversion, bool enable) {
    // create request
    request_t *req = request_create_executeFunction(objectPath,"setService",0);
    if (!req) {
        SAH_TRACE_ERROR("Failed to create request");
        goto error;
    }

    // initialize variant
    variant_t val;
    if (!variant_initialize(&val, variant_type_unknown)) {
        SAH_TRACE_ERROR("Failed to initialize variant");
        goto error_cleanup_request;
    }

    // add value(s) to request
    // add id
    //----------------------------------------------------
    if (!variant_setChar(&val,id)) {
        SAH_TRACE_ERROR("Failed to set value");
        goto error_cleanup_variant;
    }
    if (!request_addParameter(req,"id",&val)) {
        SAH_TRACE_ERROR("Failed to add parameter");
        goto error_cleanup_variant;
    }
    //----------------------------------------------------
    // add sourceInterface
    //----------------------------------------------------
    if (!variant_setChar(&val,sourceInterface)) {
        SAH_TRACE_ERROR("Failed to set value");
        goto error_cleanup_variant;
    }
    if (!request_addParameter(req,"sourceInterface",&val)) {
        SAH_TRACE_ERROR("Failed to add parameter");
        goto error_cleanup_variant;
    }
    //----------------------------------------------------
    // add destinationPort
    //----------------------------------------------------
    if (!variant_setChar(&val,destinationPort)) {
        SAH_TRACE_ERROR("Failed to set value");
        goto error_cleanup_variant;
    }
    if (!request_addParameter(req,"destinationPort",&val)) {
        SAH_TRACE_ERROR("Failed to add parameter");
        goto error_cleanup_variant;
    }
    //----------------------------------------------------
    // add protocol
    //----------------------------------------------------
    if (!variant_setChar(&val,protocol)) {
        SAH_TRACE_ERROR("Failed to set value");
        goto error_cleanup_variant;
    }
    if (!request_addParameter(req,"protocol",&val)) {
        SAH_TRACE_ERROR("Failed to add parameter");
        goto error_cleanup_variant;
    }
    //----------------------------------------------------
    // add ipversion
    //----------------------------------------------------
    if (!variant_setInt32(&val,ipversion)) {
        SAH_TRACE_ERROR("Failed to set value");
        goto error_cleanup_variant;
    }
    if (!request_addParameter(req,"ipversion",&val)) {
        SAH_TRACE_ERROR("Failed to add parameter");
        goto error_cleanup_variant;
    }
    //----------------------------------------------------
    // add enable
    //----------------------------------------------------
    if (!variant_setBool(&val,enable)) {
        SAH_TRACE_ERROR("Failed to set value");
        goto error_cleanup_variant;
    }
    if (!request_addParameter(req,"enable",&val)) {
        SAH_TRACE_ERROR("Failed to add parameter");
        goto error_cleanup_variant;
    }
    //----------------------------------------------------

    variant_cleanup(&val);
    return req;

error_cleanup_variant:
    variant_cleanup(&val);
error_cleanup_request:
    request_destroy(req);
error:
    return NULL;
} // create request for function setService end

//Generated function from ./fw.odl@167 for object Firewall
static bool parse_setService_reply(request_t *req, service_result_t **retval) {
    // parse reply for function call setService
    // get the reply
    reply_t *reply = request_reply(req);
    if (!reply) {
        SAH_TRACE_ERROR("No reply available");
        goto exit_error;
    }

    // iterate the reply items
    reply_item_t *item = reply_firstItem(reply);
    while(item) {
        switch(reply_item_type(item)) {
            case reply_type_function_return: {
                const variant_t *val = reply_item_returnValue(item);
                *retval = variant2service_result_t(val);
            }
            break;
            case reply_type_error: {
                SAH_TRACE_WARNING("Function call error (%d) - %s - %s",reply_item_error(item),reply_item_errorDescription(item),reply_item_errorInfo(item));
            }
            break;
            default:
                SAH_TRACE_WARNING("Unexpected reply item");
            break;
        }
        item = reply_nextItem(item);
    }

    return true;

exit_error:
    return false;
}  // parse reply for function setService end

//Generated function from ..odl/fw@167 for object Firewall
service_result_t* setService(peer_info_t *dest, const char *objectPath, const char *id, const char *sourceInterface, const char *destinationPort, const char *protocol, int32_t ipversion, bool enable) {

    service_result_t *retval = NULL;
    if (!dest || !objectPath) {
        SAH_TRACE_ERROR("Invalid parameters specified");
        goto error;
    }
    // get pcb context
    pcb_t *pcb = pcb_fromConnection(peer_connection(dest));
    if (!pcb) {
        SAH_TRACE_ERROR("Invalid destination specified");
        goto error;
    }
    // build request
    request_t *req = build_setService_request(objectPath, id, sourceInterface, destinationPort, protocol, ipversion, enable);
    if (!req) {
        SAH_TRACE_ERROR("Failed to build request");
        goto error;
    }

    // send the request
    if (!pcb_sendRequest(pcb,dest,req)) {
        SAH_TRACE_ERROR("Failed to send request");
        goto error_cleanup_request;
    }

    // wait for reply
    SAH_TRACE_INFO("Function called, wait for reply");
    if (!pcb_waitForReply(pcb,req,NULL)) {
        SAH_TRACE_ERROR("Communication failed");
        goto error_cleanup_request;
    }

    if (!parse_setService_reply(req, &retval)) {
        SAH_TRACE_ERROR("Failed to parse reply");
        goto error_cleanup_request;
    }

    request_destroy(req);
    return retval;

error_cleanup_request:
    request_destroy(req);
error:
    return retval;
}

//Generated function from ./fw.odl@168 for object Firewall
static request_t *build_deleteService_request(const char *objectPath, const char *id) {
    // create request
    request_t *req = request_create_executeFunction(objectPath,"deleteService",0);
    if (!req) {
        SAH_TRACE_ERROR("Failed to create request");
        goto error;
    }

    // initialize variant
    variant_t val;
    if (!variant_initialize(&val, variant_type_unknown)) {
        SAH_TRACE_ERROR("Failed to initialize variant");
        goto error_cleanup_request;
    }

    // add value(s) to request
    // add id
    //----------------------------------------------------
    if (!variant_setChar(&val,id)) {
        SAH_TRACE_ERROR("Failed to set value");
        goto error_cleanup_variant;
    }
    if (!request_addParameter(req,"id",&val)) {
        SAH_TRACE_ERROR("Failed to add parameter");
        goto error_cleanup_variant;
    }
    //----------------------------------------------------

    variant_cleanup(&val);
    return req;

error_cleanup_variant:
    variant_cleanup(&val);
error_cleanup_request:
    request_destroy(req);
error:
    return NULL;
} // create request for function deleteService end

//Generated function from ./fw.odl@168 for object Firewall
static bool parse_deleteService_reply(request_t *req, bool *retval) {
    // parse reply for function call deleteService
    // get the reply
    reply_t *reply = request_reply(req);
    if (!reply) {
        SAH_TRACE_ERROR("No reply available");
        goto exit_error;
    }

    // iterate the reply items
    reply_item_t *item = reply_firstItem(reply);
    while(item) {
        switch(reply_item_type(item)) {
            case reply_type_function_return: {
                const variant_t *val = reply_item_returnValue(item);
                *retval = variant_bool(val);
            }
            break;
            case reply_type_error: {
                SAH_TRACE_WARNING("Function call error (%d) - %s - %s",reply_item_error(item),reply_item_errorDescription(item),reply_item_errorInfo(item));
            }
            break;
            default:
                SAH_TRACE_WARNING("Unexpected reply item");
            break;
        }
        item = reply_nextItem(item);
    }

    return true;

exit_error:
    return false;
}  // parse reply for function deleteService end

//Generated function from ..odl/fw@168 for object Firewall
bool  deleteService(peer_info_t *dest, const char *objectPath, const char *id) {

    bool  retval = true;
    if (!dest || !objectPath) {
        SAH_TRACE_ERROR("Invalid parameters specified");
        goto error;
    }
    // get pcb context
    pcb_t *pcb = pcb_fromConnection(peer_connection(dest));
    if (!pcb) {
        SAH_TRACE_ERROR("Invalid destination specified");
        goto error;
    }
    // build request
    request_t *req = build_deleteService_request(objectPath, id);
    if (!req) {
        SAH_TRACE_ERROR("Failed to build request");
        goto error;
    }

    // send the request
    if (!pcb_sendRequest(pcb,dest,req)) {
        SAH_TRACE_ERROR("Failed to send request");
        goto error_cleanup_request;
    }

    // wait for reply
    SAH_TRACE_INFO("Function called, wait for reply");
    if (!pcb_waitForReply(pcb,req,NULL)) {
        SAH_TRACE_ERROR("Communication failed");
        goto error_cleanup_request;
    }

    if (!parse_deleteService_reply(req, &retval)) {
        SAH_TRACE_ERROR("Failed to parse reply");
        goto error_cleanup_request;
    }

    request_destroy(req);
    return retval;

error_cleanup_request:
    request_destroy(req);
error:
    return retval;
}

//Generated function from ./fw.odl@169 for object Firewall
static request_t *build_getService_request(const char *objectPath, const char *id) {
    // create request
    request_t *req = request_create_executeFunction(objectPath,"getService",0);
    if (!req) {
        SAH_TRACE_ERROR("Failed to create request");
        goto error;
    }

    // initialize variant
    variant_t val;
    if (!variant_initialize(&val, variant_type_unknown)) {
        SAH_TRACE_ERROR("Failed to initialize variant");
        goto error_cleanup_request;
    }

    // add value(s) to request
    // add id
    //----------------------------------------------------
    if (!variant_setChar(&val,id)) {
        SAH_TRACE_ERROR("Failed to set value");
        goto error_cleanup_variant;
    }
    if (!request_addParameter(req,"id",&val)) {
        SAH_TRACE_ERROR("Failed to add parameter");
        goto error_cleanup_variant;
    }
    //----------------------------------------------------

    variant_cleanup(&val);
    return req;

error_cleanup_variant:
    variant_cleanup(&val);
error_cleanup_request:
    request_destroy(req);
error:
    return NULL;
} // create request for function getService end

//Generated function from ./fw.odl@169 for object Firewall
static bool parse_getService_reply(request_t *req, variant_list_t **retval) {
    // parse reply for function call getService
    // get the reply
    reply_t *reply = request_reply(req);
    if (!reply) {
        SAH_TRACE_ERROR("No reply available");
        goto exit_error;
    }

    // iterate the reply items
    reply_item_t *item = reply_firstItem(reply);
    while(item) {
        switch(reply_item_type(item)) {
            case reply_type_function_return: {
                const variant_t *val = reply_item_returnValue(item);
                *retval = variant_array(val);
            }
            break;
            case reply_type_error: {
                SAH_TRACE_WARNING("Function call error (%d) - %s - %s",reply_item_error(item),reply_item_errorDescription(item),reply_item_errorInfo(item));
            }
            break;
            default:
                SAH_TRACE_WARNING("Unexpected reply item");
            break;
        }
        item = reply_nextItem(item);
    }

    return true;

exit_error:
    return false;
}  // parse reply for function getService end

//Generated function from ..odl/fw@169 for object Firewall
variant_list_t * getService(peer_info_t *dest, const char *objectPath, const char *id) {

    variant_list_t * retval = NULL;
    if (!dest || !objectPath) {
        SAH_TRACE_ERROR("Invalid parameters specified");
        goto error;
    }
    // get pcb context
    pcb_t *pcb = pcb_fromConnection(peer_connection(dest));
    if (!pcb) {
        SAH_TRACE_ERROR("Invalid destination specified");
        goto error;
    }
    // build request
    request_t *req = build_getService_request(objectPath, id);
    if (!req) {
        SAH_TRACE_ERROR("Failed to build request");
        goto error;
    }

    // send the request
    if (!pcb_sendRequest(pcb,dest,req)) {
        SAH_TRACE_ERROR("Failed to send request");
        goto error_cleanup_request;
    }

    // wait for reply
    SAH_TRACE_INFO("Function called, wait for reply");
    if (!pcb_waitForReply(pcb,req,NULL)) {
        SAH_TRACE_ERROR("Communication failed");
        goto error_cleanup_request;
    }

    if (!parse_getService_reply(req, &retval)) {
        SAH_TRACE_ERROR("Failed to parse reply");
        goto error_cleanup_request;
    }

    request_destroy(req);
    return retval;

error_cleanup_request:
    request_destroy(req);
error:
    return retval;
}

//Generated function from ./fw.odl@171 for object Firewall
static request_t *build_setRespondToPing_request(const char *objectPath, const char *sourceInterface, const service_enable_t *service_enable) {
    // create request
    request_t *req = request_create_executeFunction(objectPath,"setRespondToPing",0);
    if (!req) {
        SAH_TRACE_ERROR("Failed to create request");
        goto error;
    }

    // initialize variant
    variant_t val;
    if (!variant_initialize(&val, variant_type_unknown)) {
        SAH_TRACE_ERROR("Failed to initialize variant");
        goto error_cleanup_request;
    }

    // add value(s) to request
    // add sourceInterface
    //----------------------------------------------------
    if (!variant_setChar(&val,sourceInterface)) {
        SAH_TRACE_ERROR("Failed to set value");
        goto error_cleanup_variant;
    }
    if (!request_addParameter(req,"sourceInterface",&val)) {
        SAH_TRACE_ERROR("Failed to add parameter");
        goto error_cleanup_variant;
    }
    //----------------------------------------------------
    // add service_enable
    //----------------------------------------------------
    variant_t *var_service_enable = service_enable_t2variant(service_enable);
    if (!var_service_enable) {
        SAH_TRACE_ERROR("Failed to create variant");
        goto error_cleanup_variant;
    }
    if (!request_addParameter(req,"service_enable",var_service_enable)) {
        SAH_TRACE_ERROR("Failed to add parameter");
        variant_cleanup(var_service_enable);
        free(var_service_enable);
        goto error_cleanup_variant;
    }
    variant_cleanup(var_service_enable);
    free(var_service_enable);
    //----------------------------------------------------

    variant_cleanup(&val);
    return req;

error_cleanup_variant:
    variant_cleanup(&val);
error_cleanup_request:
    request_destroy(req);
error:
    return NULL;
} // create request for function setRespondToPing end

//Generated function from ./fw.odl@171 for object Firewall
static bool parse_setRespondToPing_reply(request_t *req, bool *retval) {
    // parse reply for function call setRespondToPing
    // get the reply
    reply_t *reply = request_reply(req);
    if (!reply) {
        SAH_TRACE_ERROR("No reply available");
        goto exit_error;
    }

    // iterate the reply items
    reply_item_t *item = reply_firstItem(reply);
    while(item) {
        switch(reply_item_type(item)) {
            case reply_type_function_return: {
                const variant_t *val = reply_item_returnValue(item);
                *retval = variant_bool(val);
            }
            break;
            case reply_type_error: {
                SAH_TRACE_WARNING("Function call error (%d) - %s - %s",reply_item_error(item),reply_item_errorDescription(item),reply_item_errorInfo(item));
            }
            break;
            default:
                SAH_TRACE_WARNING("Unexpected reply item");
            break;
        }
        item = reply_nextItem(item);
    }

    return true;

exit_error:
    return false;
}  // parse reply for function setRespondToPing end

//Generated function from ..odl/fw@171 for object Firewall
bool setRespondToPing(peer_info_t *dest, const char *objectPath, const char *sourceInterface, const service_enable_t *service_enable) {

    bool retval = NULL;
    if (!dest || !objectPath) {
        SAH_TRACE_ERROR("Invalid parameters specified");
        goto error;
    }
    // get pcb context
    pcb_t *pcb = pcb_fromConnection(peer_connection(dest));
    if (!pcb) {
        SAH_TRACE_ERROR("Invalid destination specified");
        goto error;
    }
    // build request
    request_t *req = build_setRespondToPing_request(objectPath, sourceInterface, service_enable);
    if (!req) {
        SAH_TRACE_ERROR("Failed to build request");
        goto error;
    }

    // send the request
    if (!pcb_sendRequest(pcb,dest,req)) {
        SAH_TRACE_ERROR("Failed to send request");
        goto error_cleanup_request;
    }

    // wait for reply
    SAH_TRACE_INFO("Function called, wait for reply");
    if (!pcb_waitForReply(pcb,req,NULL)) {
        SAH_TRACE_ERROR("Communication failed");
        goto error_cleanup_request;
    }

    if (!parse_setRespondToPing_reply(req, &retval)) {
        SAH_TRACE_ERROR("Failed to parse reply");
        goto error_cleanup_request;
    }

    request_destroy(req);
    return retval;

error_cleanup_request:
    request_destroy(req);
error:
    return retval;
}

//Generated function from ./fw_definitions.odl@246 for object Firewall
static request_t *build_getRespondToPing_request(const char *objectPath, const char *sourceInterface) {
    // create request
    request_t *req = request_create_executeFunction(objectPath,"getRespondToPing",0);
    if (!req) {
        SAH_TRACE_ERROR("Failed to create request");
        goto error;
    }

    // initialize variant
    variant_t val;
    if (!variant_initialize(&val, variant_type_unknown)) {
        SAH_TRACE_ERROR("Failed to initialize variant");
        goto error_cleanup_request;
    }

    // add value(s) to request
    // add sourceInterface
    //----------------------------------------------------
    if (!variant_setChar(&val,sourceInterface)) {
        SAH_TRACE_ERROR("Failed to set value");
        goto error_cleanup_variant;
    }
    if (!request_addParameter(req,"sourceInterface",&val)) {
        SAH_TRACE_ERROR("Failed to add parameter");
        goto error_cleanup_variant;
    }
    //----------------------------------------------------

    variant_cleanup(&val);
    return req;

error_cleanup_variant:
    variant_cleanup(&val);
error_cleanup_request:
    request_destroy(req);
error:
    return NULL;
} // create request for function getRespondToPing end

//Generated function from ./fw_definitions.odl@246 for object Firewall
static bool parse_getRespondToPing_reply(request_t *req, service_enable_t **retval) {
    // parse reply for function call getRespondToPing
    // get the reply
    reply_t *reply = request_reply(req);
    if (!reply) {
        SAH_TRACE_ERROR("No reply available");
        goto exit_error;
    }

    // iterate the reply items
    reply_item_t *item = reply_firstItem(reply);
    while(item) {
        switch(reply_item_type(item)) {
            case reply_type_function_return: {
                const variant_t *val = reply_item_returnValue(item);
                *retval = variant2service_enable_t(val);
            }
            break;
            case reply_type_error: {
                SAH_TRACE_WARNING("Function call error (%d) - %s - %s",reply_item_error(item),reply_item_errorDescription(item),reply_item_errorInfo(item));
            }
            break;
            default:
                SAH_TRACE_WARNING("Unexpected reply item");
            break;
        }
        item = reply_nextItem(item);
    }

    return true;

exit_error:
    return false;
}  // parse reply for function getRespondToPing end

//Generated function from ..odl/fw_definitions@246 for object Firewall
service_enable_t *getRespondToPing(peer_info_t *dest, const char *objectPath, const char *sourceInterface) {

    service_enable_t *retval = NULL;
    if (!dest || !objectPath) {
        SAH_TRACE_ERROR("Invalid parameters specified");
        goto error;
    }
    // get pcb context
    pcb_t *pcb = pcb_fromConnection(peer_connection(dest));
    if (!pcb) {
        SAH_TRACE_ERROR("Invalid destination specified");
        goto error;
    }
    // build request
    request_t *req = build_getRespondToPing_request(objectPath, sourceInterface);
    if (!req) {
        SAH_TRACE_ERROR("Failed to build request");
        goto error;
    }

    // send the request
    if (!pcb_sendRequest(pcb,dest,req)) {
        SAH_TRACE_ERROR("Failed to send request");
        goto error_cleanup_request;
    }

    // wait for reply
    SAH_TRACE_INFO("Function called, wait for reply");
    if (!pcb_waitForReply(pcb,req,NULL)) {
        SAH_TRACE_ERROR("Communication failed");
        goto error_cleanup_request;
    }

    if (!parse_getRespondToPing_reply(req, &retval)) {
        SAH_TRACE_ERROR("Failed to parse reply");
        goto error_cleanup_request;
    }

    request_destroy(req);
    return retval;

error_cleanup_request:
    request_destroy(req);
error:
    return retval;
}

//Generated function from ./fw.odl@174 for object Firewall
static request_t *build_setPortForwarding_request(const char *objectPath, const char *id, const char *origin, const char *sourceInterface, const char *externalPort, const char *internalPort, const char *destinationIPAddress, const char *sourcePrefix, const char *protocol, bool enable, bool persistent, const char *description) {
    // create request
    request_t *req = request_create_executeFunction(objectPath,"setPortForwarding",0);
    if (!req) {
        SAH_TRACE_ERROR("Failed to create request");
        goto error;
    }

    // initialize variant
    variant_t val;
    if (!variant_initialize(&val, variant_type_unknown)) {
        SAH_TRACE_ERROR("Failed to initialize variant");
        goto error_cleanup_request;
    }

    // add value(s) to request
    // add id
    //----------------------------------------------------
    if (!variant_setChar(&val,id)) {
        SAH_TRACE_ERROR("Failed to set value");
        goto error_cleanup_variant;
    }
    if (!request_addParameter(req,"id",&val)) {
        SAH_TRACE_ERROR("Failed to add parameter");
        goto error_cleanup_variant;
    }
    //----------------------------------------------------
    // add origin
    //----------------------------------------------------
    if (!variant_setChar(&val,origin)) {
        SAH_TRACE_ERROR("Failed to set value");
        goto error_cleanup_variant;
    }
    if (!request_addParameter(req,"origin",&val)) {
        SAH_TRACE_ERROR("Failed to add parameter");
        goto error_cleanup_variant;
    }
    //----------------------------------------------------
    // add sourceInterface
    //----------------------------------------------------
    if (!variant_setChar(&val,sourceInterface)) {
        SAH_TRACE_ERROR("Failed to set value");
        goto error_cleanup_variant;
    }
    if (!request_addParameter(req,"sourceInterface",&val)) {
        SAH_TRACE_ERROR("Failed to add parameter");
        goto error_cleanup_variant;
    }
    //----------------------------------------------------
    // add externalPort
    //----------------------------------------------------
    if (!variant_setChar(&val,externalPort)) {
        SAH_TRACE_ERROR("Failed to set value");
        goto error_cleanup_variant;
    }
    if (!request_addParameter(req,"externalPort",&val)) {
        SAH_TRACE_ERROR("Failed to add parameter");
        goto error_cleanup_variant;
    }
    //----------------------------------------------------
    // add internalPort
    //----------------------------------------------------
    if (!variant_setChar(&val,internalPort)) {
        SAH_TRACE_ERROR("Failed to set value");
        goto error_cleanup_variant;
    }
    if (!request_addParameter(req,"internalPort",&val)) {
        SAH_TRACE_ERROR("Failed to add parameter");
        goto error_cleanup_variant;
    }
    //----------------------------------------------------
    // add destinationIPAddress
    //----------------------------------------------------
    if (!variant_setChar(&val,destinationIPAddress)) {
        SAH_TRACE_ERROR("Failed to set value");
        goto error_cleanup_variant;
    }
    if (!request_addParameter(req,"destinationIPAddress",&val)) {
        SAH_TRACE_ERROR("Failed to add parameter");
        goto error_cleanup_variant;
    }
    //----------------------------------------------------
    // add sourcePrefix
    //----------------------------------------------------
    if (!variant_setChar(&val,sourcePrefix)) {
        SAH_TRACE_ERROR("Failed to set value");
        goto error_cleanup_variant;
    }
    if (!request_addParameter(req,"sourcePrefix",&val)) {
        SAH_TRACE_ERROR("Failed to add parameter");
        goto error_cleanup_variant;
    }
    //----------------------------------------------------
    // add protocol
    //----------------------------------------------------
    if (!variant_setChar(&val,protocol)) {
        SAH_TRACE_ERROR("Failed to set value");
        goto error_cleanup_variant;
    }
    if (!request_addParameter(req,"protocol",&val)) {
        SAH_TRACE_ERROR("Failed to add parameter");
        goto error_cleanup_variant;
    }
    //----------------------------------------------------
    // add enable
    //----------------------------------------------------
    if (!variant_setBool(&val,enable)) {
        SAH_TRACE_ERROR("Failed to set value");
        goto error_cleanup_variant;
    }
    if (!request_addParameter(req,"enable",&val)) {
        SAH_TRACE_ERROR("Failed to add parameter");
        goto error_cleanup_variant;
    }
    //----------------------------------------------------
    // add persistent
    //----------------------------------------------------
    if (!variant_setBool(&val,persistent)) {
        SAH_TRACE_ERROR("Failed to set value");
        goto error_cleanup_variant;
    }
    if (!request_addParameter(req,"persistent",&val)) {
        SAH_TRACE_ERROR("Failed to add parameter");
        goto error_cleanup_variant;
    }
    //----------------------------------------------------
    // add description
    //----------------------------------------------------
    if (!variant_setChar(&val,description)) {
        SAH_TRACE_ERROR("Failed to set value");
        goto error_cleanup_variant;
    }
    if (!request_addParameter(req,"description",&val)) {
        SAH_TRACE_ERROR("Failed to add parameter");
        goto error_cleanup_variant;
    }
    //----------------------------------------------------

    variant_cleanup(&val);
    return req;

error_cleanup_variant:
    variant_cleanup(&val);
error_cleanup_request:
    request_destroy(req);
error:
    return NULL;
} // create request for function setPortForwarding end

//Generated function from ./fw.odl@174 for object Firewall
static bool parse_setPortForwarding_reply(request_t *req, char **retval) {
    // parse reply for function call setPortForwarding
    // get the reply
    reply_t *reply = request_reply(req);
    if (!reply) {
        SAH_TRACE_ERROR("No reply available");
        goto exit_error;
    }

    // iterate the reply items
    reply_item_t *item = reply_firstItem(reply);
    while(item) {
        switch(reply_item_type(item)) {
            case reply_type_function_return: {
                const variant_t *val = reply_item_returnValue(item);
                *retval = variant_char(val);
            }
            break;
            case reply_type_error: {
                SAH_TRACE_WARNING("Function call error (%d) - %s - %s",reply_item_error(item),reply_item_errorDescription(item),reply_item_errorInfo(item));
            }
            break;
            default:
                SAH_TRACE_WARNING("Unexpected reply item");
            break;
        }
        item = reply_nextItem(item);
    }

    return true;

exit_error:
    return false;
}  // parse reply for function setPortForwarding end

//Generated function from ..odl/fw@174 for object Firewall
char * setPortForwarding(peer_info_t *dest, const char *objectPath, const char *id, const char *origin, const char *sourceInterface, const char *externalPort, const char *internalPort, const char *destinationIPAddress, const char *sourcePrefix, const char *protocol, bool enable, bool persistent, const char *description) {

    char * retval = NULL;
    if (!dest || !objectPath) {
        SAH_TRACE_ERROR("Invalid parameters specified");
        goto error;
    }
    // get pcb context
    pcb_t *pcb = pcb_fromConnection(peer_connection(dest));
    if (!pcb) {
        SAH_TRACE_ERROR("Invalid destination specified");
        goto error;
    }
    // build request
    request_t *req = build_setPortForwarding_request(objectPath, id, origin, sourceInterface, externalPort, internalPort, destinationIPAddress, sourcePrefix, protocol, enable, persistent, description);
    if (!req) {
        SAH_TRACE_ERROR("Failed to build request");
        goto error;
    }

    // send the request
    if (!pcb_sendRequest(pcb,dest,req)) {
        SAH_TRACE_ERROR("Failed to send request");
        goto error_cleanup_request;
    }

    // wait for reply
    SAH_TRACE_INFO("Function called, wait for reply");
    if (!pcb_waitForReply(pcb,req,NULL)) {
        SAH_TRACE_ERROR("Communication failed");
        goto error_cleanup_request;
    }

    if (!parse_setPortForwarding_reply(req, &retval)) {
        SAH_TRACE_ERROR("Failed to parse reply");
        goto error_cleanup_request;
    }

    request_destroy(req);
    return retval;

error_cleanup_request:
    request_destroy(req);
error:
    return retval;
}

//Generated function from ./fw.odl@175 for object Firewall
static request_t *build_deletePortForwarding_request(const char *objectPath, const char *id, const char *origin, const char* destinationIPAddress) {
    // create request
    request_t *req = request_create_executeFunction(objectPath,"deletePortForwarding",0);
    if (!req) {
        SAH_TRACE_ERROR("Failed to create request");
        goto error;
    }

    // initialize variant
    variant_t val;
    if (!variant_initialize(&val, variant_type_unknown)) {
        SAH_TRACE_ERROR("Failed to initialize variant");
        goto error_cleanup_request;
    }

    // add value(s) to request
    // add id
    //----------------------------------------------------
    if (!variant_setChar(&val,id)) {
        SAH_TRACE_ERROR("Failed to set value");
        goto error_cleanup_variant;
    }
    if (!request_addParameter(req,"id",&val)) {
        SAH_TRACE_ERROR("Failed to add parameter");
        goto error_cleanup_variant;
    }
    //----------------------------------------------------
    // add origin
    //----------------------------------------------------
    if (!variant_setChar(&val,origin)) {
        SAH_TRACE_ERROR("Failed to set value");
        goto error_cleanup_variant;
    }
    if (!request_addParameter(req,"origin",&val)) {
        SAH_TRACE_ERROR("Failed to add parameter");
        goto error_cleanup_variant;
    }
    //----------------------------------------------------
    // add destinationIPAddress
    //----------------------------------------------------
    if (!variant_setChar(&val,destinationIPAddress)) {
        SAH_TRACE_ERROR("Failed to set value");
        goto error_cleanup_variant;
    }
    if (!request_addParameter(req,"destinationIPAddress",&val)) {
        SAH_TRACE_ERROR("Failed to add parameter");
        goto error_cleanup_variant;
    }
    //----------------------------------------------------

    variant_cleanup(&val);
    return req;

error_cleanup_variant:
    variant_cleanup(&val);
error_cleanup_request:
    request_destroy(req);
error:
    return NULL;
} // create request for function deletePortForwarding end

//Generated function from ./fw.odl@175 for object Firewall
static bool parse_deletePortForwarding_reply(request_t *req, bool *retval) {
    // parse reply for function call deletePortForwarding
    // get the reply
    reply_t *reply = request_reply(req);
    if (!reply) {
        SAH_TRACE_ERROR("No reply available");
        goto exit_error;
    }

    // iterate the reply items
    reply_item_t *item = reply_firstItem(reply);
    while(item) {
        switch(reply_item_type(item)) {
            case reply_type_function_return: {
                const variant_t *val = reply_item_returnValue(item);
                *retval = variant_bool(val);
            }
            break;
            case reply_type_error: {
                SAH_TRACE_WARNING("Function call error (%d) - %s - %s",reply_item_error(item),reply_item_errorDescription(item),reply_item_errorInfo(item));
            }
            break;
            default:
                SAH_TRACE_WARNING("Unexpected reply item");
            break;
        }
        item = reply_nextItem(item);
    }

    return true;

exit_error:
    return false;
}  // parse reply for function deletePortForwarding end

//Generated function from ..odl/fw@175 for object Firewall
bool  deletePortForwarding(peer_info_t *dest, const char *objectPath, const char *id, const char* destinationIPAddress, const char *origin) {

    bool  retval = true;
    if (!dest || !objectPath) {
        SAH_TRACE_ERROR("Invalid parameters specified");
        goto error;
    }
    // get pcb context
    pcb_t *pcb = pcb_fromConnection(peer_connection(dest));
    if (!pcb) {
        SAH_TRACE_ERROR("Invalid destination specified");
        goto error;
    }
    // build request
    request_t *req = build_deletePortForwarding_request(objectPath, id, destinationIPAddress, origin);
    if (!req) {
        SAH_TRACE_ERROR("Failed to build request");
        goto error;
    }

    // send the request
    if (!pcb_sendRequest(pcb,dest,req)) {
        SAH_TRACE_ERROR("Failed to send request");
        goto error_cleanup_request;
    }

    // wait for reply
    SAH_TRACE_INFO("Function called, wait for reply");
    if (!pcb_waitForReply(pcb,req,NULL)) {
        SAH_TRACE_ERROR("Communication failed");
        goto error_cleanup_request;
    }

    if (!parse_deletePortForwarding_reply(req, &retval)) {
        SAH_TRACE_ERROR("Failed to parse reply");
        goto error_cleanup_request;
    }

    request_destroy(req);
    return retval;

error_cleanup_request:
    request_destroy(req);
error:
    return retval;
}

//Generated function from ./fw.odl@176 for object Firewall
static request_t *build_getPortForwarding_request(const char *objectPath, const char *id, const char *origin) {
    // create request
    request_t *req = request_create_executeFunction(objectPath,"getPortForwarding",0);
    if (!req) {
        SAH_TRACE_ERROR("Failed to create request");
        goto error;
    }

    // initialize variant
    variant_t val;
    if (!variant_initialize(&val, variant_type_unknown)) {
        SAH_TRACE_ERROR("Failed to initialize variant");
        goto error_cleanup_request;
    }

    // add value(s) to request
    // add id
    //----------------------------------------------------
    if (!variant_setChar(&val,id)) {
        SAH_TRACE_ERROR("Failed to set value");
        goto error_cleanup_variant;
    }
    if (!request_addParameter(req,"id",&val)) {
        SAH_TRACE_ERROR("Failed to add parameter");
        goto error_cleanup_variant;
    }
    //----------------------------------------------------
    // add origin
    //----------------------------------------------------
    if (!variant_setChar(&val,origin)) {
        SAH_TRACE_ERROR("Failed to set value");
        goto error_cleanup_variant;
    }
    if (!request_addParameter(req,"origin",&val)) {
        SAH_TRACE_ERROR("Failed to add parameter");
        goto error_cleanup_variant;
    }
    //----------------------------------------------------

    variant_cleanup(&val);
    return req;

error_cleanup_variant:
    variant_cleanup(&val);
error_cleanup_request:
    request_destroy(req);
error:
    return NULL;
} // create request for function getPortForwarding end

//Generated function from ./fw.odl@176 for object Firewall
static bool parse_getPortForwarding_reply(request_t *req, variant_list_t **retval) {
    // parse reply for function call getPortForwarding
    // get the reply
    reply_t *reply = request_reply(req);
    if (!reply) {
        SAH_TRACE_ERROR("No reply available");
        goto exit_error;
    }

    // iterate the reply items
    reply_item_t *item = reply_firstItem(reply);
    while(item) {
        switch(reply_item_type(item)) {
            case reply_type_function_return: {
                const variant_t *val = reply_item_returnValue(item);
                *retval = variant_array(val);
            }
            break;
            case reply_type_error: {
                SAH_TRACE_WARNING("Function call error (%d) - %s - %s",reply_item_error(item),reply_item_errorDescription(item),reply_item_errorInfo(item));
            }
            break;
            default:
                SAH_TRACE_WARNING("Unexpected reply item");
            break;
        }
        item = reply_nextItem(item);
    }

    return true;

exit_error:
    return false;
}  // parse reply for function getPortForwarding end

//Generated function from ..odl/fw@176 for object Firewall
variant_list_t * getPortForwarding(peer_info_t *dest, const char *objectPath, const char *id, const char *origin) {

    variant_list_t * retval = NULL;
    if (!dest || !objectPath) {
        SAH_TRACE_ERROR("Invalid parameters specified");
        goto error;
    }
    // get pcb context
    pcb_t *pcb = pcb_fromConnection(peer_connection(dest));
    if (!pcb) {
        SAH_TRACE_ERROR("Invalid destination specified");
        goto error;
    }
    // build request
    request_t *req = build_getPortForwarding_request(objectPath, id, origin);
    if (!req) {
        SAH_TRACE_ERROR("Failed to build request");
        goto error;
    }

    // send the request
    if (!pcb_sendRequest(pcb,dest,req)) {
        SAH_TRACE_ERROR("Failed to send request");
        goto error_cleanup_request;
    }

    // wait for reply
    SAH_TRACE_INFO("Function called, wait for reply");
    if (!pcb_waitForReply(pcb,req,NULL)) {
        SAH_TRACE_ERROR("Communication failed");
        goto error_cleanup_request;
    }

    if (!parse_getPortForwarding_reply(req, &retval)) {
        SAH_TRACE_ERROR("Failed to parse reply");
        goto error_cleanup_request;
    }

    request_destroy(req);
    return retval;

error_cleanup_request:
    request_destroy(req);
error:
    return retval;
}

//Generated function from ./fw.odl@179 for object Firewall
static request_t *build_setPinhole_request(const char *objectPath, const char *id, const char *origin, const char *sourceInterface, const char *sourcePort, const char *destinationPort, const char *destinationIPAddress, const char *sourcePrefix, const char *protocol, int32_t ipversion, bool enable, bool persistent, const char *description) {
    // create request
    request_t *req = request_create_executeFunction(objectPath,"setPinhole",0);
    if (!req) {
        SAH_TRACE_ERROR("Failed to create request");
        goto error;
    }

    // initialize variant
    variant_t val;
    if (!variant_initialize(&val, variant_type_unknown)) {
        SAH_TRACE_ERROR("Failed to initialize variant");
        goto error_cleanup_request;
    }

    // add value(s) to request
    // add id
    //----------------------------------------------------
    if (!variant_setChar(&val,id)) {
        SAH_TRACE_ERROR("Failed to set value");
        goto error_cleanup_variant;
    }
    if (!request_addParameter(req,"id",&val)) {
        SAH_TRACE_ERROR("Failed to add parameter");
        goto error_cleanup_variant;
    }
    //----------------------------------------------------
    // add origin
    //----------------------------------------------------
    if (!variant_setChar(&val,origin)) {
        SAH_TRACE_ERROR("Failed to set value");
        goto error_cleanup_variant;
    }
    if (!request_addParameter(req,"origin",&val)) {
        SAH_TRACE_ERROR("Failed to add parameter");
        goto error_cleanup_variant;
    }
    //----------------------------------------------------
    // add sourceInterface
    //----------------------------------------------------
    if (!variant_setChar(&val,sourceInterface)) {
        SAH_TRACE_ERROR("Failed to set value");
        goto error_cleanup_variant;
    }
    if (!request_addParameter(req,"sourceInterface",&val)) {
        SAH_TRACE_ERROR("Failed to add parameter");
        goto error_cleanup_variant;
    }
    //----------------------------------------------------
    // add sourcePort
    //----------------------------------------------------
    if (!variant_setChar(&val,sourcePort)) {
        SAH_TRACE_ERROR("Failed to set value");
        goto error_cleanup_variant;
    }
    if (!request_addParameter(req,"sourcePort",&val)) {
        SAH_TRACE_ERROR("Failed to add parameter");
        goto error_cleanup_variant;
    }
    //----------------------------------------------------
    // add destinationPort
    //----------------------------------------------------
    if (!variant_setChar(&val,destinationPort)) {
        SAH_TRACE_ERROR("Failed to set value");
        goto error_cleanup_variant;
    }
    if (!request_addParameter(req,"destinationPort",&val)) {
        SAH_TRACE_ERROR("Failed to add parameter");
        goto error_cleanup_variant;
    }
    //----------------------------------------------------
    // add destinationIPAddress
    //----------------------------------------------------
    if (!variant_setChar(&val,destinationIPAddress)) {
        SAH_TRACE_ERROR("Failed to set value");
        goto error_cleanup_variant;
    }
    if (!request_addParameter(req,"destinationIPAddress",&val)) {
        SAH_TRACE_ERROR("Failed to add parameter");
        goto error_cleanup_variant;
    }
    //----------------------------------------------------
    // add sourcePrefix
    //----------------------------------------------------
    if (!variant_setChar(&val,sourcePrefix)) {
        SAH_TRACE_ERROR("Failed to set value");
        goto error_cleanup_variant;
    }
    if (!request_addParameter(req,"sourcePrefix",&val)) {
        SAH_TRACE_ERROR("Failed to add parameter");
        goto error_cleanup_variant;
    }
    //----------------------------------------------------
    // add protocol
    //----------------------------------------------------
    if (!variant_setChar(&val,protocol)) {
        SAH_TRACE_ERROR("Failed to set value");
        goto error_cleanup_variant;
    }
    if (!request_addParameter(req,"protocol",&val)) {
        SAH_TRACE_ERROR("Failed to add parameter");
        goto error_cleanup_variant;
    }
    //----------------------------------------------------
    // add ipversion
    //----------------------------------------------------
    if (!variant_setInt32(&val,ipversion)) {
        SAH_TRACE_ERROR("Failed to set value");
        goto error_cleanup_variant;
    }
    if (!request_addParameter(req,"ipversion",&val)) {
        SAH_TRACE_ERROR("Failed to add parameter");
        goto error_cleanup_variant;
    }
    //----------------------------------------------------
    // add enable
    //----------------------------------------------------
    if (!variant_setBool(&val,enable)) {
        SAH_TRACE_ERROR("Failed to set value");
        goto error_cleanup_variant;
    }
    if (!request_addParameter(req,"enable",&val)) {
        SAH_TRACE_ERROR("Failed to add parameter");
        goto error_cleanup_variant;
    }
    //----------------------------------------------------
    // add persistent
    //----------------------------------------------------
    if (!variant_setBool(&val,persistent)) {
        SAH_TRACE_ERROR("Failed to set value");
        goto error_cleanup_variant;
    }
    if (!request_addParameter(req,"persistent",&val)) {
        SAH_TRACE_ERROR("Failed to add parameter");
        goto error_cleanup_variant;
    }
    //----------------------------------------------------
    // add description
    //----------------------------------------------------
    if (!variant_setChar(&val,description)) {
        SAH_TRACE_ERROR("Failed to set value");
        goto error_cleanup_variant;
    }
    if (!request_addParameter(req,"description",&val)) {
        SAH_TRACE_ERROR("Failed to add parameter");
        goto error_cleanup_variant;
    }
    //----------------------------------------------------

    variant_cleanup(&val);
    return req;

error_cleanup_variant:
    variant_cleanup(&val);
error_cleanup_request:
    request_destroy(req);
error:
    return NULL;
} // create request for function setPinhole end

//Generated function from ./fw.odl@179 for object Firewall
static bool parse_setPinhole_reply(request_t *req, char **retval) {
    // parse reply for function call setPinhole
    // get the reply
    reply_t *reply = request_reply(req);
    if (!reply) {
        SAH_TRACE_ERROR("No reply available");
        goto exit_error;
    }

    // iterate the reply items
    reply_item_t *item = reply_firstItem(reply);
    while(item) {
        switch(reply_item_type(item)) {
            case reply_type_function_return: {
                const variant_t *val = reply_item_returnValue(item);
                *retval = variant_char(val);
            }
            break;
            case reply_type_error: {
                SAH_TRACE_WARNING("Function call error (%d) - %s - %s",reply_item_error(item),reply_item_errorDescription(item),reply_item_errorInfo(item));
            }
            break;
            default:
                SAH_TRACE_WARNING("Unexpected reply item");
            break;
        }
        item = reply_nextItem(item);
    }

    return true;

exit_error:
    return false;
}  // parse reply for function setPinhole end

//Generated function from ..odl/fw@179 for object Firewall
char * setPinhole(peer_info_t *dest, const char *objectPath, const char *id, const char *origin, const char *sourceInterface, const char *sourcePort, const char *destinationPort, const char *destinationIPAddress, const char *sourcePrefix, const char *protocol, int32_t ipversion, bool enable, bool persistent, const char *description) {

    char * retval = NULL;
    if (!dest || !objectPath) {
        SAH_TRACE_ERROR("Invalid parameters specified");
        goto error;
    }
    // get pcb context
    pcb_t *pcb = pcb_fromConnection(peer_connection(dest));
    if (!pcb) {
        SAH_TRACE_ERROR("Invalid destination specified");
        goto error;
    }
    // build request
    request_t *req = build_setPinhole_request(objectPath, id, origin, sourceInterface, sourcePort, destinationPort, destinationIPAddress, sourcePrefix, protocol, ipversion, enable, persistent, description);
    if (!req) {
        SAH_TRACE_ERROR("Failed to build request");
        goto error;
    }

    // send the request
    if (!pcb_sendRequest(pcb,dest,req)) {
        SAH_TRACE_ERROR("Failed to send request");
        goto error_cleanup_request;
    }

    // wait for reply
    SAH_TRACE_INFO("Function called, wait for reply");
    if (!pcb_waitForReply(pcb,req,NULL)) {
        SAH_TRACE_ERROR("Communication failed");
        goto error_cleanup_request;
    }

    if (!parse_setPinhole_reply(req, &retval)) {
        SAH_TRACE_ERROR("Failed to parse reply");
        goto error_cleanup_request;
    }

    request_destroy(req);
    return retval;

error_cleanup_request:
    request_destroy(req);
error:
    return retval;
}

//Generated function from ./fw.odl@180 for object Firewall
static request_t *build_deletePinhole_request(const char *objectPath, const char *id, const char *origin) {
    // create request
    request_t *req = request_create_executeFunction(objectPath,"deletePinhole",0);
    if (!req) {
        SAH_TRACE_ERROR("Failed to create request");
        goto error;
    }

    // initialize variant
    variant_t val;
    if (!variant_initialize(&val, variant_type_unknown)) {
        SAH_TRACE_ERROR("Failed to initialize variant");
        goto error_cleanup_request;
    }

    // add value(s) to request
    // add id
    //----------------------------------------------------
    if (!variant_setChar(&val,id)) {
        SAH_TRACE_ERROR("Failed to set value");
        goto error_cleanup_variant;
    }
    if (!request_addParameter(req,"id",&val)) {
        SAH_TRACE_ERROR("Failed to add parameter");
        goto error_cleanup_variant;
    }
    //----------------------------------------------------
    // add origin
    //----------------------------------------------------
    if (!variant_setChar(&val,origin)) {
        SAH_TRACE_ERROR("Failed to set value");
        goto error_cleanup_variant;
    }
    if (!request_addParameter(req,"origin",&val)) {
        SAH_TRACE_ERROR("Failed to add parameter");
        goto error_cleanup_variant;
    }
    //----------------------------------------------------

    variant_cleanup(&val);
    return req;

error_cleanup_variant:
    variant_cleanup(&val);
error_cleanup_request:
    request_destroy(req);
error:
    return NULL;
} // create request for function deletePinhole end

//Generated function from ./fw.odl@180 for object Firewall
static bool parse_deletePinhole_reply(request_t *req, bool *retval) {
    // parse reply for function call deletePinhole
    // get the reply
    reply_t *reply = request_reply(req);
    if (!reply) {
        SAH_TRACE_ERROR("No reply available");
        goto exit_error;
    }

    // iterate the reply items
    reply_item_t *item = reply_firstItem(reply);
    while(item) {
        switch(reply_item_type(item)) {
            case reply_type_function_return: {
                const variant_t *val = reply_item_returnValue(item);
                *retval = variant_bool(val);
            }
            break;
            case reply_type_error: {
                SAH_TRACE_WARNING("Function call error (%d) - %s - %s",reply_item_error(item),reply_item_errorDescription(item),reply_item_errorInfo(item));
            }
            break;
            default:
                SAH_TRACE_WARNING("Unexpected reply item");
            break;
        }
        item = reply_nextItem(item);
    }

    return true;

exit_error:
    return false;
}  // parse reply for function deletePinhole end

//Generated function from ..odl/fw@180 for object Firewall
bool  deletePinhole(peer_info_t *dest, const char *objectPath, const char *id, const char *origin) {

    bool  retval = true;
    if (!dest || !objectPath) {
        SAH_TRACE_ERROR("Invalid parameters specified");
        goto error;
    }
    // get pcb context
    pcb_t *pcb = pcb_fromConnection(peer_connection(dest));
    if (!pcb) {
        SAH_TRACE_ERROR("Invalid destination specified");
        goto error;
    }
    // build request
    request_t *req = build_deletePinhole_request(objectPath, id, origin);
    if (!req) {
        SAH_TRACE_ERROR("Failed to build request");
        goto error;
    }

    // send the request
    if (!pcb_sendRequest(pcb,dest,req)) {
        SAH_TRACE_ERROR("Failed to send request");
        goto error_cleanup_request;
    }

    // wait for reply
    SAH_TRACE_INFO("Function called, wait for reply");
    if (!pcb_waitForReply(pcb,req,NULL)) {
        SAH_TRACE_ERROR("Communication failed");
        goto error_cleanup_request;
    }

    if (!parse_deletePinhole_reply(req, &retval)) {
        SAH_TRACE_ERROR("Failed to parse reply");
        goto error_cleanup_request;
    }

    request_destroy(req);
    return retval;

error_cleanup_request:
    request_destroy(req);
error:
    return retval;
}

//Generated function from ./fw.odl@181 for object Firewall
static request_t *build_getPinhole_request(const char *objectPath, const char *id, const char *origin) {
    // create request
    request_t *req = request_create_executeFunction(objectPath,"getPinhole",0);
    if (!req) {
        SAH_TRACE_ERROR("Failed to create request");
        goto error;
    }

    // initialize variant
    variant_t val;
    if (!variant_initialize(&val, variant_type_unknown)) {
        SAH_TRACE_ERROR("Failed to initialize variant");
        goto error_cleanup_request;
    }

    // add value(s) to request
    // add id
    //----------------------------------------------------
    if (!variant_setChar(&val,id)) {
        SAH_TRACE_ERROR("Failed to set value");
        goto error_cleanup_variant;
    }
    if (!request_addParameter(req,"id",&val)) {
        SAH_TRACE_ERROR("Failed to add parameter");
        goto error_cleanup_variant;
    }
    //----------------------------------------------------
    // add origin
    //----------------------------------------------------
    if (!variant_setChar(&val,origin)) {
        SAH_TRACE_ERROR("Failed to set value");
        goto error_cleanup_variant;
    }
    if (!request_addParameter(req,"origin",&val)) {
        SAH_TRACE_ERROR("Failed to add parameter");
        goto error_cleanup_variant;
    }
    //----------------------------------------------------

    variant_cleanup(&val);
    return req;

error_cleanup_variant:
    variant_cleanup(&val);
error_cleanup_request:
    request_destroy(req);
error:
    return NULL;
} // create request for function getPinhole end

//Generated function from ./fw.odl@181 for object Firewall
static bool parse_getPinhole_reply(request_t *req, variant_list_t **retval) {
    // parse reply for function call getPinhole
    // get the reply
    reply_t *reply = request_reply(req);
    if (!reply) {
        SAH_TRACE_ERROR("No reply available");
        goto exit_error;
    }

    // iterate the reply items
    reply_item_t *item = reply_firstItem(reply);
    while(item) {
        switch(reply_item_type(item)) {
            case reply_type_function_return: {
                const variant_t *val = reply_item_returnValue(item);
                *retval = variant_array(val);
            }
            break;
            case reply_type_error: {
                SAH_TRACE_WARNING("Function call error (%d) - %s - %s",reply_item_error(item),reply_item_errorDescription(item),reply_item_errorInfo(item));
            }
            break;
            default:
                SAH_TRACE_WARNING("Unexpected reply item");
            break;
        }
        item = reply_nextItem(item);
    }

    return true;

exit_error:
    return false;
}  // parse reply for function getPinhole end

//Generated function from ..odl/fw@181 for object Firewall
variant_list_t * getPinhole(peer_info_t *dest, const char *objectPath, const char *id, const char *origin) {

    variant_list_t * retval = NULL;
    if (!dest || !objectPath) {
        SAH_TRACE_ERROR("Invalid parameters specified");
        goto error;
    }
    // get pcb context
    pcb_t *pcb = pcb_fromConnection(peer_connection(dest));
    if (!pcb) {
        SAH_TRACE_ERROR("Invalid destination specified");
        goto error;
    }
    // build request
    request_t *req = build_getPinhole_request(objectPath, id, origin);
    if (!req) {
        SAH_TRACE_ERROR("Failed to build request");
        goto error;
    }

    // send the request
    if (!pcb_sendRequest(pcb,dest,req)) {
        SAH_TRACE_ERROR("Failed to send request");
        goto error_cleanup_request;
    }

    // wait for reply
    SAH_TRACE_INFO("Function called, wait for reply");
    if (!pcb_waitForReply(pcb,req,NULL)) {
        SAH_TRACE_ERROR("Communication failed");
        goto error_cleanup_request;
    }

    if (!parse_getPinhole_reply(req, &retval)) {
        SAH_TRACE_ERROR("Failed to parse reply");
        goto error_cleanup_request;
    }

    request_destroy(req);
    return retval;

error_cleanup_request:
    request_destroy(req);
error:
    return retval;
}

//Generated function from ./fw.odl@184 for object Firewall
static request_t *build_setDMZ_request(const char *objectPath, const char *id, const char *sourceInterface, const char *destinationIPAddress, const char *sourcePrefix, bool enable) {
    // create request
    request_t *req = request_create_executeFunction(objectPath,"setDMZ",0);
    if (!req) {
        SAH_TRACE_ERROR("Failed to create request");
        goto error;
    }

    // initialize variant
    variant_t val;
    if (!variant_initialize(&val, variant_type_unknown)) {
        SAH_TRACE_ERROR("Failed to initialize variant");
        goto error_cleanup_request;
    }

    // add value(s) to request
    // add id
    //----------------------------------------------------
    if (!variant_setChar(&val,id)) {
        SAH_TRACE_ERROR("Failed to set value");
        goto error_cleanup_variant;
    }
    if (!request_addParameter(req,"id",&val)) {
        SAH_TRACE_ERROR("Failed to add parameter");
        goto error_cleanup_variant;
    }
    //----------------------------------------------------
    // add sourceInterface
    //----------------------------------------------------
    if (!variant_setChar(&val,sourceInterface)) {
        SAH_TRACE_ERROR("Failed to set value");
        goto error_cleanup_variant;
    }
    if (!request_addParameter(req,"sourceInterface",&val)) {
        SAH_TRACE_ERROR("Failed to add parameter");
        goto error_cleanup_variant;
    }
    //----------------------------------------------------
    // add destinationIPAddress
    //----------------------------------------------------
    if (!variant_setChar(&val,destinationIPAddress)) {
        SAH_TRACE_ERROR("Failed to set value");
        goto error_cleanup_variant;
    }
    if (!request_addParameter(req,"destinationIPAddress",&val)) {
        SAH_TRACE_ERROR("Failed to add parameter");
        goto error_cleanup_variant;
    }
    //----------------------------------------------------
    // add sourcePrefix
    //----------------------------------------------------
    if (!variant_setChar(&val,sourcePrefix)) {
        SAH_TRACE_ERROR("Failed to set value");
        goto error_cleanup_variant;
    }
    if (!request_addParameter(req,"sourcePrefix",&val)) {
        SAH_TRACE_ERROR("Failed to add parameter");
        goto error_cleanup_variant;
    }
    //----------------------------------------------------
    // add enable
    //----------------------------------------------------
    if (!variant_setBool(&val,enable)) {
        SAH_TRACE_ERROR("Failed to set value");
        goto error_cleanup_variant;
    }
    if (!request_addParameter(req,"enable",&val)) {
        SAH_TRACE_ERROR("Failed to add parameter");
        goto error_cleanup_variant;
    }
    //----------------------------------------------------

    variant_cleanup(&val);
    return req;

error_cleanup_variant:
    variant_cleanup(&val);
error_cleanup_request:
    request_destroy(req);
error:
    return NULL;
} // create request for function setDMZ end

//Generated function from ./fw.odl@184 for object Firewall
static bool parse_setDMZ_reply(request_t *req, char **retval) {
    // parse reply for function call setDMZ
    // get the reply
    reply_t *reply = request_reply(req);
    if (!reply) {
        SAH_TRACE_ERROR("No reply available");
        goto exit_error;
    }

    // iterate the reply items
    reply_item_t *item = reply_firstItem(reply);
    while(item) {
        switch(reply_item_type(item)) {
            case reply_type_function_return: {
                const variant_t *val = reply_item_returnValue(item);
                *retval = variant_char(val);
            }
            break;
            case reply_type_error: {
                SAH_TRACE_WARNING("Function call error (%d) - %s - %s",reply_item_error(item),reply_item_errorDescription(item),reply_item_errorInfo(item));
            }
            break;
            default:
                SAH_TRACE_WARNING("Unexpected reply item");
            break;
        }
        item = reply_nextItem(item);
    }

    return true;

exit_error:
    return false;
}  // parse reply for function setDMZ end

//Generated function from ..odl/fw@184 for object Firewall
char * setDMZ(peer_info_t *dest, const char *objectPath, const char *id, const char *sourceInterface, const char *destinationIPAddress, const char *sourcePrefix, bool enable) {

    char * retval = NULL;
    if (!dest || !objectPath) {
        SAH_TRACE_ERROR("Invalid parameters specified");
        goto error;
    }
    // get pcb context
    pcb_t *pcb = pcb_fromConnection(peer_connection(dest));
    if (!pcb) {
        SAH_TRACE_ERROR("Invalid destination specified");
        goto error;
    }
    // build request
    request_t *req = build_setDMZ_request(objectPath, id, sourceInterface, destinationIPAddress, sourcePrefix, enable);
    if (!req) {
        SAH_TRACE_ERROR("Failed to build request");
        goto error;
    }

    // send the request
    if (!pcb_sendRequest(pcb,dest,req)) {
        SAH_TRACE_ERROR("Failed to send request");
        goto error_cleanup_request;
    }

    // wait for reply
    SAH_TRACE_INFO("Function called, wait for reply");
    if (!pcb_waitForReply(pcb,req,NULL)) {
        SAH_TRACE_ERROR("Communication failed");
        goto error_cleanup_request;
    }

    if (!parse_setDMZ_reply(req, &retval)) {
        SAH_TRACE_ERROR("Failed to parse reply");
        goto error_cleanup_request;
    }

    request_destroy(req);
    return retval;

error_cleanup_request:
    request_destroy(req);
error:
    return retval;
}

//Generated function from ./fw.odl@185 for object Firewall
static request_t *build_deleteDMZ_request(const char *objectPath, const char *id) {
    // create request
    request_t *req = request_create_executeFunction(objectPath,"deleteDMZ",0);
    if (!req) {
        SAH_TRACE_ERROR("Failed to create request");
        goto error;
    }

    // initialize variant
    variant_t val;
    if (!variant_initialize(&val, variant_type_unknown)) {
        SAH_TRACE_ERROR("Failed to initialize variant");
        goto error_cleanup_request;
    }

    // add value(s) to request
    // add id
    //----------------------------------------------------
    if (!variant_setChar(&val,id)) {
        SAH_TRACE_ERROR("Failed to set value");
        goto error_cleanup_variant;
    }
    if (!request_addParameter(req,"id",&val)) {
        SAH_TRACE_ERROR("Failed to add parameter");
        goto error_cleanup_variant;
    }
    //----------------------------------------------------

    variant_cleanup(&val);
    return req;

error_cleanup_variant:
    variant_cleanup(&val);
error_cleanup_request:
    request_destroy(req);
error:
    return NULL;
} // create request for function deleteDMZ end

//Generated function from ./fw.odl@185 for object Firewall
static bool parse_deleteDMZ_reply(request_t *req, bool *retval) {
    // parse reply for function call deleteDMZ
    // get the reply
    reply_t *reply = request_reply(req);
    if (!reply) {
        SAH_TRACE_ERROR("No reply available");
        goto exit_error;
    }

    // iterate the reply items
    reply_item_t *item = reply_firstItem(reply);
    while(item) {
        switch(reply_item_type(item)) {
            case reply_type_function_return: {
                const variant_t *val = reply_item_returnValue(item);
                *retval = variant_bool(val);
            }
            break;
            case reply_type_error: {
                SAH_TRACE_WARNING("Function call error (%d) - %s - %s",reply_item_error(item),reply_item_errorDescription(item),reply_item_errorInfo(item));
            }
            break;
            default:
                SAH_TRACE_WARNING("Unexpected reply item");
            break;
        }
        item = reply_nextItem(item);
    }

    return true;

exit_error:
    return false;
}  // parse reply for function deleteDMZ end

//Generated function from ..odl/fw@185 for object Firewall
bool  deleteDMZ(peer_info_t *dest, const char *objectPath, const char *id) {

    bool  retval = true;
    if (!dest || !objectPath) {
        SAH_TRACE_ERROR("Invalid parameters specified");
        goto error;
    }
    // get pcb context
    pcb_t *pcb = pcb_fromConnection(peer_connection(dest));
    if (!pcb) {
        SAH_TRACE_ERROR("Invalid destination specified");
        goto error;
    }
    // build request
    request_t *req = build_deleteDMZ_request(objectPath, id);
    if (!req) {
        SAH_TRACE_ERROR("Failed to build request");
        goto error;
    }

    // send the request
    if (!pcb_sendRequest(pcb,dest,req)) {
        SAH_TRACE_ERROR("Failed to send request");
        goto error_cleanup_request;
    }

    // wait for reply
    SAH_TRACE_INFO("Function called, wait for reply");
    if (!pcb_waitForReply(pcb,req,NULL)) {
        SAH_TRACE_ERROR("Communication failed");
        goto error_cleanup_request;
    }

    if (!parse_deleteDMZ_reply(req, &retval)) {
        SAH_TRACE_ERROR("Failed to parse reply");
        goto error_cleanup_request;
    }

    request_destroy(req);
    return retval;

error_cleanup_request:
    request_destroy(req);
error:
    return retval;
}

//Generated function from ./fw.odl@186 for object Firewall
static request_t *build_getDMZ_request(const char *objectPath, const char *id) {
    // create request
    request_t *req = request_create_executeFunction(objectPath,"getDMZ",0);
    if (!req) {
        SAH_TRACE_ERROR("Failed to create request");
        goto error;
    }

    // initialize variant
    variant_t val;
    if (!variant_initialize(&val, variant_type_unknown)) {
        SAH_TRACE_ERROR("Failed to initialize variant");
        goto error_cleanup_request;
    }

    // add value(s) to request
    // add id
    //----------------------------------------------------
    if (!variant_setChar(&val,id)) {
        SAH_TRACE_ERROR("Failed to set value");
        goto error_cleanup_variant;
    }
    if (!request_addParameter(req,"id",&val)) {
        SAH_TRACE_ERROR("Failed to add parameter");
        goto error_cleanup_variant;
    }
    //----------------------------------------------------

    variant_cleanup(&val);
    return req;

error_cleanup_variant:
    variant_cleanup(&val);
error_cleanup_request:
    request_destroy(req);
error:
    return NULL;
} // create request for function getDMZ end

//Generated function from ./fw.odl@186 for object Firewall
static bool parse_getDMZ_reply(request_t *req, variant_list_t **retval) {
    // parse reply for function call getDMZ
    // get the reply
    reply_t *reply = request_reply(req);
    if (!reply) {
        SAH_TRACE_ERROR("No reply available");
        goto exit_error;
    }

    // iterate the reply items
    reply_item_t *item = reply_firstItem(reply);
    while(item) {
        switch(reply_item_type(item)) {
            case reply_type_function_return: {
                const variant_t *val = reply_item_returnValue(item);
                *retval = variant_array(val);
            }
            break;
            case reply_type_error: {
                SAH_TRACE_WARNING("Function call error (%d) - %s - %s",reply_item_error(item),reply_item_errorDescription(item),reply_item_errorInfo(item));
            }
            break;
            default:
                SAH_TRACE_WARNING("Unexpected reply item");
            break;
        }
        item = reply_nextItem(item);
    }

    return true;

exit_error:
    return false;
}  // parse reply for function getDMZ end

//Generated function from ..odl/fw@186 for object Firewall
variant_list_t * getDMZ(peer_info_t *dest, const char *objectPath, const char *id) {

    variant_list_t * retval = NULL;
    if (!dest || !objectPath) {
        SAH_TRACE_ERROR("Invalid parameters specified");
        goto error;
    }
    // get pcb context
    pcb_t *pcb = pcb_fromConnection(peer_connection(dest));
    if (!pcb) {
        SAH_TRACE_ERROR("Invalid destination specified");
        goto error;
    }
    // build request
    request_t *req = build_getDMZ_request(objectPath, id);
    if (!req) {
        SAH_TRACE_ERROR("Failed to build request");
        goto error;
    }

    // send the request
    if (!pcb_sendRequest(pcb,dest,req)) {
        SAH_TRACE_ERROR("Failed to send request");
        goto error_cleanup_request;
    }

    // wait for reply
    SAH_TRACE_INFO("Function called, wait for reply");
    if (!pcb_waitForReply(pcb,req,NULL)) {
        SAH_TRACE_ERROR("Communication failed");
        goto error_cleanup_request;
    }

    if (!parse_getDMZ_reply(req, &retval)) {
        SAH_TRACE_ERROR("Failed to parse reply");
        goto error_cleanup_request;
    }

    request_destroy(req);
    return retval;

error_cleanup_request:
    request_destroy(req);
error:
    return retval;
}

//Generated function from ./fw.odl@189 for object Firewall
static request_t *build_setCustomRule_request(const char *objectPath, const char *id, const char *chain, const char *action, const char *destinationPort, const char *sourcePort, const char *destinationPrefix, const char *sourcePrefix, const char *protocol, int32_t ipversion, bool enable, const char *description) {
    // create request
    request_t *req = request_create_executeFunction(objectPath,"setCustomRule",0);
    if (!req) {
        SAH_TRACE_ERROR("Failed to create request");
        goto error;
    }

    // initialize variant
    variant_t val;
    if (!variant_initialize(&val, variant_type_unknown)) {
        SAH_TRACE_ERROR("Failed to initialize variant");
        goto error_cleanup_request;
    }

    // add value(s) to request
    // add id
    //----------------------------------------------------
    if (!variant_setChar(&val,id)) {
        SAH_TRACE_ERROR("Failed to set value");
        goto error_cleanup_variant;
    }
    if (!request_addParameter(req,"id",&val)) {
        SAH_TRACE_ERROR("Failed to add parameter");
        goto error_cleanup_variant;
    }
    //----------------------------------------------------
    // add chain
    //----------------------------------------------------
    if (!variant_setChar(&val,chain)) {
        SAH_TRACE_ERROR("Failed to set value");
        goto error_cleanup_variant;
    }
    if (!request_addParameter(req,"chain",&val)) {
        SAH_TRACE_ERROR("Failed to add parameter");
        goto error_cleanup_variant;
    }
    //----------------------------------------------------
    // add action
    //----------------------------------------------------
    if (!variant_setChar(&val,action)) {
        SAH_TRACE_ERROR("Failed to set value");
        goto error_cleanup_variant;
    }
    if (!request_addParameter(req,"action",&val)) {
        SAH_TRACE_ERROR("Failed to add parameter");
        goto error_cleanup_variant;
    }
    //----------------------------------------------------
    // add destinationPort
    //----------------------------------------------------
    if (!variant_setChar(&val,destinationPort)) {
        SAH_TRACE_ERROR("Failed to set value");
        goto error_cleanup_variant;
    }
    if (!request_addParameter(req,"destinationPort",&val)) {
        SAH_TRACE_ERROR("Failed to add parameter");
        goto error_cleanup_variant;
    }
    //----------------------------------------------------
    // add sourcePort
    //----------------------------------------------------
    if (!variant_setChar(&val,sourcePort)) {
        SAH_TRACE_ERROR("Failed to set value");
        goto error_cleanup_variant;
    }
    if (!request_addParameter(req,"sourcePort",&val)) {
        SAH_TRACE_ERROR("Failed to add parameter");
        goto error_cleanup_variant;
    }
    //----------------------------------------------------
    // add destinationPrefix
    //----------------------------------------------------
    if (!variant_setChar(&val,destinationPrefix)) {
        SAH_TRACE_ERROR("Failed to set value");
        goto error_cleanup_variant;
    }
    if (!request_addParameter(req,"destinationPrefix",&val)) {
        SAH_TRACE_ERROR("Failed to add parameter");
        goto error_cleanup_variant;
    }
    //----------------------------------------------------
    // add sourcePrefix
    //----------------------------------------------------
    if (!variant_setChar(&val,sourcePrefix)) {
        SAH_TRACE_ERROR("Failed to set value");
        goto error_cleanup_variant;
    }
    if (!request_addParameter(req,"sourcePrefix",&val)) {
        SAH_TRACE_ERROR("Failed to add parameter");
        goto error_cleanup_variant;
    }
    //----------------------------------------------------
    // add protocol
    //----------------------------------------------------
    if (!variant_setChar(&val,protocol)) {
        SAH_TRACE_ERROR("Failed to set value");
        goto error_cleanup_variant;
    }
    if (!request_addParameter(req,"protocol",&val)) {
        SAH_TRACE_ERROR("Failed to add parameter");
        goto error_cleanup_variant;
    }
    //----------------------------------------------------
    // add ipversion
    //----------------------------------------------------
    if (!variant_setInt32(&val,ipversion)) {
        SAH_TRACE_ERROR("Failed to set value");
        goto error_cleanup_variant;
    }
    if (!request_addParameter(req,"ipversion",&val)) {
        SAH_TRACE_ERROR("Failed to add parameter");
        goto error_cleanup_variant;
    }
    //----------------------------------------------------
    // add enable
    //----------------------------------------------------
    if (!variant_setBool(&val,enable)) {
        SAH_TRACE_ERROR("Failed to set value");
        goto error_cleanup_variant;
    }
    if (!request_addParameter(req,"enable",&val)) {
        SAH_TRACE_ERROR("Failed to add parameter");
        goto error_cleanup_variant;
    }
    //----------------------------------------------------
    // add description
    //----------------------------------------------------
    if (!variant_setChar(&val,description)) {
        SAH_TRACE_ERROR("Failed to set value");
        goto error_cleanup_variant;
    }
    if (!request_addParameter(req,"description",&val)) {
        SAH_TRACE_ERROR("Failed to add parameter");
        goto error_cleanup_variant;
    }
    //----------------------------------------------------

    variant_cleanup(&val);
    return req;

error_cleanup_variant:
    variant_cleanup(&val);
error_cleanup_request:
    request_destroy(req);
error:
    return NULL;
} // create request for function setCustomRule end

//Generated function from ./fw.odl@189 for object Firewall
static bool parse_setCustomRule_reply(request_t *req, char **retval) {
    // parse reply for function call setCustomRule
    // get the reply
    reply_t *reply = request_reply(req);
    if (!reply) {
        SAH_TRACE_ERROR("No reply available");
        goto exit_error;
    }

    // iterate the reply items
    reply_item_t *item = reply_firstItem(reply);
    while(item) {
        switch(reply_item_type(item)) {
            case reply_type_function_return: {
                const variant_t *val = reply_item_returnValue(item);
                *retval = variant_char(val);
            }
            break;
            case reply_type_error: {
                SAH_TRACE_WARNING("Function call error (%d) - %s - %s",reply_item_error(item),reply_item_errorDescription(item),reply_item_errorInfo(item));
            }
            break;
            default:
                SAH_TRACE_WARNING("Unexpected reply item");
            break;
        }
        item = reply_nextItem(item);
    }

    return true;

exit_error:
    return false;
}  // parse reply for function setCustomRule end

//Generated function from ..odl/fw@189 for object Firewall
char * setCustomRule(peer_info_t *dest, const char *objectPath, const char *id, const char *chain, const char *action, const char *destinationPort, const char *sourcePort, const char *destinationPrefix, const char *sourcePrefix, const char *protocol, int32_t ipversion, bool enable, const char* description) {

    char * retval = NULL;
    if (!dest || !objectPath) {
        SAH_TRACE_ERROR("Invalid parameters specified");
        goto error;
    }
    // get pcb context
    pcb_t *pcb = pcb_fromConnection(peer_connection(dest));
    if (!pcb) {
        SAH_TRACE_ERROR("Invalid destination specified");
        goto error;
    }
    // build request
    request_t *req = build_setCustomRule_request(objectPath, id, chain, action, destinationPort, sourcePort, destinationPrefix, sourcePrefix, protocol, ipversion, enable, description);
    if (!req) {
        SAH_TRACE_ERROR("Failed to build request");
        goto error;
    }

    // send the request
    if (!pcb_sendRequest(pcb,dest,req)) {
        SAH_TRACE_ERROR("Failed to send request");
        goto error_cleanup_request;
    }

    // wait for reply
    SAH_TRACE_INFO("Function called, wait for reply");
    if (!pcb_waitForReply(pcb,req,NULL)) {
        SAH_TRACE_ERROR("Communication failed");
        goto error_cleanup_request;
    }

    if (!parse_setCustomRule_reply(req, &retval)) {
        SAH_TRACE_ERROR("Failed to parse reply");
        goto error_cleanup_request;
    }

    request_destroy(req);
    return retval;

error_cleanup_request:
    request_destroy(req);
error:
    return retval;
}

//Generated function from ./fw.odl@190 for object Firewall
static request_t *build_deleteCustomRule_request(const char *objectPath, const char *id, const char *chain) {
    // create request
    request_t *req = request_create_executeFunction(objectPath,"deleteCustomRule",0);
    if (!req) {
        SAH_TRACE_ERROR("Failed to create request");
        goto error;
    }

    // initialize variant
    variant_t val;
    if (!variant_initialize(&val, variant_type_unknown)) {
        SAH_TRACE_ERROR("Failed to initialize variant");
        goto error_cleanup_request;
    }

    // add value(s) to request
    // add id
    //----------------------------------------------------
    if (!variant_setChar(&val,id)) {
        SAH_TRACE_ERROR("Failed to set value");
        goto error_cleanup_variant;
    }
    if (!request_addParameter(req,"id",&val)) {
        SAH_TRACE_ERROR("Failed to add parameter");
        goto error_cleanup_variant;
    }
    //----------------------------------------------------
    // add chain
    //----------------------------------------------------
    if (!variant_setChar(&val,chain)) {
        SAH_TRACE_ERROR("Failed to set value");
        goto error_cleanup_variant;
    }
    if (!request_addParameter(req,"chain",&val)) {
        SAH_TRACE_ERROR("Failed to add parameter");
        goto error_cleanup_variant;
    }
    //----------------------------------------------------

    variant_cleanup(&val);
    return req;

error_cleanup_variant:
    variant_cleanup(&val);
error_cleanup_request:
    request_destroy(req);
error:
    return NULL;
} // create request for function deleteCustomRule end

//Generated function from ./fw.odl@190 for object Firewall
static bool parse_deleteCustomRule_reply(request_t *req, bool *retval) {
    // parse reply for function call deleteCustomRule
    // get the reply
    reply_t *reply = request_reply(req);
    if (!reply) {
        SAH_TRACE_ERROR("No reply available");
        goto exit_error;
    }

    // iterate the reply items
    reply_item_t *item = reply_firstItem(reply);
    while(item) {
        switch(reply_item_type(item)) {
            case reply_type_function_return: {
                const variant_t *val = reply_item_returnValue(item);
                *retval = variant_bool(val);
            }
            break;
            case reply_type_error: {
                SAH_TRACE_WARNING("Function call error (%d) - %s - %s",reply_item_error(item),reply_item_errorDescription(item),reply_item_errorInfo(item));
            }
            break;
            default:
                SAH_TRACE_WARNING("Unexpected reply item");
            break;
        }
        item = reply_nextItem(item);
    }

    return true;

exit_error:
    return false;
}  // parse reply for function deleteCustomRule end

//Generated function from ..odl/fw@190 for object Firewall
bool  deleteCustomRule(peer_info_t *dest, const char *objectPath, const char *id, const char *chain) {

    bool  retval = true;
    if (!dest || !objectPath) {
        SAH_TRACE_ERROR("Invalid parameters specified");
        goto error;
    }
    // get pcb context
    pcb_t *pcb = pcb_fromConnection(peer_connection(dest));
    if (!pcb) {
        SAH_TRACE_ERROR("Invalid destination specified");
        goto error;
    }
    // build request
    request_t *req = build_deleteCustomRule_request(objectPath, id, chain);
    if (!req) {
        SAH_TRACE_ERROR("Failed to build request");
        goto error;
    }

    // send the request
    if (!pcb_sendRequest(pcb,dest,req)) {
        SAH_TRACE_ERROR("Failed to send request");
        goto error_cleanup_request;
    }

    // wait for reply
    SAH_TRACE_INFO("Function called, wait for reply");
    if (!pcb_waitForReply(pcb,req,NULL)) {
        SAH_TRACE_ERROR("Communication failed");
        goto error_cleanup_request;
    }

    if (!parse_deleteCustomRule_reply(req, &retval)) {
        SAH_TRACE_ERROR("Failed to parse reply");
        goto error_cleanup_request;
    }

    request_destroy(req);
    return retval;

error_cleanup_request:
    request_destroy(req);
error:
    return retval;
}

//Generated function from ./fw.odl@191 for object Firewall
static request_t *build_getCustomRule_request(const char *objectPath, const char *id, const char *chain) {
    // create request
    request_t *req = request_create_executeFunction(objectPath,"getCustomRule",0);
    if (!req) {
        SAH_TRACE_ERROR("Failed to create request");
        goto error;
    }

    // initialize variant
    variant_t val;
    if (!variant_initialize(&val, variant_type_unknown)) {
        SAH_TRACE_ERROR("Failed to initialize variant");
        goto error_cleanup_request;
    }

    // add value(s) to request
    // add id
    //----------------------------------------------------
    if (!variant_setChar(&val,id)) {
        SAH_TRACE_ERROR("Failed to set value");
        goto error_cleanup_variant;
    }
    if (!request_addParameter(req,"id",&val)) {
        SAH_TRACE_ERROR("Failed to add parameter");
        goto error_cleanup_variant;
    }
    //----------------------------------------------------
    // add chain
    //----------------------------------------------------
    if (!variant_setChar(&val,chain)) {
        SAH_TRACE_ERROR("Failed to set value");
        goto error_cleanup_variant;
    }
    if (!request_addParameter(req,"chain",&val)) {
        SAH_TRACE_ERROR("Failed to add parameter");
        goto error_cleanup_variant;
    }
    //----------------------------------------------------

    variant_cleanup(&val);
    return req;

error_cleanup_variant:
    variant_cleanup(&val);
error_cleanup_request:
    request_destroy(req);
error:
    return NULL;
} // create request for function getCustomRule end

//Generated function from ./fw.odl@191 for object Firewall
static bool parse_getCustomRule_reply(request_t *req, variant_list_t **retval) {
    // parse reply for function call getCustomRule
    // get the reply
    reply_t *reply = request_reply(req);
    if (!reply) {
        SAH_TRACE_ERROR("No reply available");
        goto exit_error;
    }

    // iterate the reply items
    reply_item_t *item = reply_firstItem(reply);
    while(item) {
        switch(reply_item_type(item)) {
            case reply_type_function_return: {
                const variant_t *val = reply_item_returnValue(item);
                *retval = variant_array(val);
            }
            break;
            case reply_type_error: {
                SAH_TRACE_WARNING("Function call error (%d) - %s - %s",reply_item_error(item),reply_item_errorDescription(item),reply_item_errorInfo(item));
            }
            break;
            default:
                SAH_TRACE_WARNING("Unexpected reply item");
            break;
        }
        item = reply_nextItem(item);
    }

    return true;

exit_error:
    return false;
}  // parse reply for function getCustomRule end

//Generated function from ..odl/fw@191 for object Firewall
variant_list_t * getCustomRule(peer_info_t *dest, const char *objectPath, const char *id, const char *chain) {

    variant_list_t * retval = NULL;
    if (!dest || !objectPath) {
        SAH_TRACE_ERROR("Invalid parameters specified");
        goto error;
    }
    // get pcb context
    pcb_t *pcb = pcb_fromConnection(peer_connection(dest));
    if (!pcb) {
        SAH_TRACE_ERROR("Invalid destination specified");
        goto error;
    }
    // build request
    request_t *req = build_getCustomRule_request(objectPath, id, chain);
    if (!req) {
        SAH_TRACE_ERROR("Failed to build request");
        goto error;
    }

    // send the request
    if (!pcb_sendRequest(pcb,dest,req)) {
        SAH_TRACE_ERROR("Failed to send request");
        goto error_cleanup_request;
    }

    // wait for reply
    SAH_TRACE_INFO("Function called, wait for reply");
    if (!pcb_waitForReply(pcb,req,NULL)) {
        SAH_TRACE_ERROR("Communication failed");
        goto error_cleanup_request;
    }

    if (!parse_getCustomRule_reply(req, &retval)) {
        SAH_TRACE_ERROR("Failed to parse reply");
        goto error_cleanup_request;
    }

    request_destroy(req);
    return retval;

error_cleanup_request:
    request_destroy(req);
error:
    return retval;
}

//Generated function from ./fw.odl@194 for object Firewall
static request_t *build_setFirewallLevel_request(const char *objectPath, const char *level) {
    // create request
    request_t *req = request_create_executeFunction(objectPath,"setFirewallLevel",0);
    if (!req) {
        SAH_TRACE_ERROR("Failed to create request");
        goto error;
    }

    // initialize variant
    variant_t val;
    if (!variant_initialize(&val, variant_type_unknown)) {
        SAH_TRACE_ERROR("Failed to initialize variant");
        goto error_cleanup_request;
    }

    // add value(s) to request
    // add level
    //----------------------------------------------------
    if (!variant_setChar(&val,level)) {
        SAH_TRACE_ERROR("Failed to set value");
        goto error_cleanup_variant;
    }
    if (!request_addParameter(req,"level",&val)) {
        SAH_TRACE_ERROR("Failed to add parameter");
        goto error_cleanup_variant;
    }
    //----------------------------------------------------

    variant_cleanup(&val);
    return req;

error_cleanup_variant:
    variant_cleanup(&val);
error_cleanup_request:
    request_destroy(req);
error:
    return NULL;
} // create request for function setFirewallLevel end

//Generated function from ./fw.odl@194 for object Firewall
static bool parse_setFirewallLevel_reply(request_t *req, bool *retval) {
    // parse reply for function call setFirewallLevel
    // get the reply
    reply_t *reply = request_reply(req);
    if (!reply) {
        SAH_TRACE_ERROR("No reply available");
        goto exit_error;
    }

    // iterate the reply items
    reply_item_t *item = reply_firstItem(reply);
    while(item) {
        switch(reply_item_type(item)) {
            case reply_type_function_return: {
                const variant_t *val = reply_item_returnValue(item);
                *retval = variant_bool(val);
            }
            break;
            case reply_type_error: {
                SAH_TRACE_WARNING("Function call error (%d) - %s - %s",reply_item_error(item),reply_item_errorDescription(item),reply_item_errorInfo(item));
            }
            break;
            default:
                SAH_TRACE_WARNING("Unexpected reply item");
            break;
        }
        item = reply_nextItem(item);
    }

    return true;

exit_error:
    return false;
}  // parse reply for function setFirewallLevel end

//Generated function from ..odl/fw@194 for object Firewall
bool  setFirewallLevel(peer_info_t *dest, const char *objectPath, const char *level) {

    bool  retval = true;
    if (!dest || !objectPath) {
        SAH_TRACE_ERROR("Invalid parameters specified");
        goto error;
    }
    // get pcb context
    pcb_t *pcb = pcb_fromConnection(peer_connection(dest));
    if (!pcb) {
        SAH_TRACE_ERROR("Invalid destination specified");
        goto error;
    }
    // build request
    request_t *req = build_setFirewallLevel_request(objectPath, level);
    if (!req) {
        SAH_TRACE_ERROR("Failed to build request");
        goto error;
    }

    // send the request
    if (!pcb_sendRequest(pcb,dest,req)) {
        SAH_TRACE_ERROR("Failed to send request");
        goto error_cleanup_request;
    }

    // wait for reply
    SAH_TRACE_INFO("Function called, wait for reply");
    if (!pcb_waitForReply(pcb,req,NULL)) {
        SAH_TRACE_ERROR("Communication failed");
        goto error_cleanup_request;
    }

    if (!parse_setFirewallLevel_reply(req, &retval)) {
        SAH_TRACE_ERROR("Failed to parse reply");
        goto error_cleanup_request;
    }

    request_destroy(req);
    return retval;

error_cleanup_request:
    request_destroy(req);
error:
    return retval;
}
