/*
 * Copyright (c) 2009 SoftAtHome
 *
 * The information and source code contained herein is the exclusive
 * property of SoftAtHome and may not be disclosed, examined, or
 * reproduced in whole or in part without explicit written authorization
 * of the copyright owner.
 *
 * fw_common.h - SoftAtHome firewall plug-in: common functions;
 *
 */

#ifndef __fw_common_h__
#define __fw_common_h__

#ifdef __cplusplus
    extern "C"
{
#endif

#include <pcb/core.h>
#include <sys/time.h>

#define FW "firewall"
#define SAVE_FILE "/etc/firewall-config.odl"
#define SAVE_FILE_TMP "/etc/firewall-config.odl.tmp"
#define DEFAULTS_FILE "/usr/lib/firewall/fw-defaults.odl"

#define INTERFACE_LEN 16
#define IP_LEN 150
#define MAC_LEN 18
#define DSCP_VALUE_LEN 10
#define NAME_LEN 64

#define NR_OF_PROTOCOLS 10

#define FWERR 0x00050000

typedef struct {
	bool enableIPv4;
	bool enableIPv6;
} service_enable_t;

typedef struct {
	char *name;
	char *port;
} service_result_t;


/** 
 * helper function to retrieve int argument of a pcb function 
 */
bool arg_getServiceEnableValue(uint32_t attr, argument_value_list_t *args, service_enable_t **value, const char *name);

/**
 * helper functions to convert between a variant and service_enable_t
 */
service_enable_t *variant2service_enable_t(const variant_t *var);
variant_t *service_enable_t2variant(const service_enable_t *data); 
bool variant_setServiceEnable(variant_t *variant, service_enable_t service_enable);

/**
 * helper functions to convert between a variant and service_result_t
 */
service_result_t *variant2service_result_t(const variant_t *var);

bool checkIpAddress(char **address);

void start_timer(struct timeval *start);
void stop_timer(struct timeval *start, struct timeval *res);

#ifdef __cplusplus
}
#endif

#endif
