/*
 * Copyright (c) 2009 SoftAtHome
 *
 * The information and source code contained herein is the exclusive
 * property of SoftAtHome and may not be disclosed, examined, or
 * reproduced in whole or in part without explicit written authorization
 * of the copyright owner.
 *
 */

#ifndef __IPTABLES_FACADE_H__
#define __IPTABLES_FACADE_H__

#include "fw_common.h"

/*
 * iptf stands for IPTables Facade
 */
#define IPTF_NEW      0x1
#define IPTF_DELETED  0x2
#define IPTF_MODIFIED 0x4

#define IPTF_INTERFACE_LEN INTERFACE_LEN
#define IPTF_IP_LEN IP_LEN
#define IPTF_MAC_LEN MAC_LEN
#define IPTF_TABLE_LEN 16
#define IPTF_CHAIN_LEN 32

enum iptfStatus {
	IPTF_STATUS_DISABLED=0,
	IPTF_STATUS_ERROR,
	IPTF_STATUS_ENABLED
};

enum iptfPolicy {
	IPTF_POLICY_DROP=0,
	IPTF_POLICY_ACCEPT,
	IPTF_POLICY_MASQUERADE,
	IPTF_POLICY_REJECT
};

enum iptfTarget {
	IPTF_TARGET_RETURN,
	IPTF_TARGET_MARK,
	IPTF_TARGET_DSCP,
	IPTF_TARGET_QUEUE,
	IPTF_TARGET_PBIT,
	IPTF_TARGET_POLICY,
	IPTF_TARGET_CHAIN,
	IPTF_TARGET_DNAT
};

// keep in sync with kernel, not found in kernel header files...
#define ADDR_TYPE_UNSPEC 0
#define ADDR_TYPE_UNICAST 1
#define ADDR_TYPE_LOCAL 2
#define ADDR_TYPE_BROADCAST 3
#define ADDR_TYPE_ANYCAST 4
#define ADDR_TYPE_MULTICAST 5

struct dnat {
	char ip[16];
	unsigned short minport;
	unsigned short maxport;
};

struct iptfRule {
	struct iptfRule * prev; // RO
	struct iptfRule * next; // RO
	int flags; // RO
	
	enum iptfStatus    status; // RO
	int                enable;
	char               table[IPTF_TABLE_LEN];
	char               chain[IPTF_CHAIN_LEN];
	int                ipv4;
	struct {
		char            inInterface[IPTF_INTERFACE_LEN];
		char            outInterface[IPTF_INTERFACE_LEN];
		char            destination[IPTF_IP_LEN];
		char            destinationMask[IPTF_IP_LEN];
		int             destinationExclude;
		char            source[IPTF_IP_LEN];
		char            sourceMask[IPTF_IP_LEN];
		int             sourceExclude;
		int             protocol;
		int             protocolExclude;
		int             destinationPort;
		int             destinationPortRangeMax;
		int             destinationPortExclude;
		int             sourcePort;
		int             sourcePortRangeMax;
		int             sourcePortExclude;
		unsigned int    ipLengthMin;
		unsigned int    ipLengthMax;
		int 		ipLengthExclude;
		char		dscpCheck;
		int             dscpExclude;
		char		sourceMacAddress[IPTF_MAC_LEN];
		int		sourceMacExclude;
		char		destMacAddress[IPTF_MAC_LEN];
		int		destMacExclude;
		unsigned char	icmpType;
		unsigned short  sourceAddressType;
		unsigned short  destAddressType;
	} criteria;
	enum iptfTarget    target;
	union {
		unsigned int     dscp;
		unsigned int     mark;
		unsigned int     queue;
		unsigned int     pbit;
		enum iptfPolicy  policy;
		char             chain[64];
		struct dnat      dnat;
	} targets; 
};

struct iptfRule *  iptfCreateRule(struct iptfRule * predecessor);
void               iptfModifyRule(struct iptfRule * rule);
void               iptfDeleteRule(struct iptfRule * rule);

struct iptfRule *  iptfCreateRuleSet(void);
void               iptfDeleteRuleSet(struct iptfRule * ruleset);
void               iptfImplementChanges(struct iptfRule * ruleset, int do_commit);

void setPortRange(unsigned short min, unsigned int max);

void commit();

#endif

