/*
 * This file is partly the contents of the Linux header file linux/ip6_tunnel.h
 * If the Linux header file from the toolchain doesn't include the definition
 * for 'struct ip6_tnl_parm2', GRE over IPv6 support cannot be built into iproute2.
 * Therefore, until the toolchain is updated, this file is available.
 *
 * Also a one-liner from linux/if_arp.h is added
 *
 * And some stuff from if_tunnel.h
 */
#ifndef __IP6TUNNEL_H__
#define __IP6TUNNEL_H__

#include <linux/types.h>

#define ARPHRD_IP6GRE 823

enum
{
	X_IFLA_GRE_UNSPEC,
	X_IFLA_GRE_LINK,
	X_IFLA_GRE_IFLAGS,
	X_IFLA_GRE_OFLAGS,
	X_IFLA_GRE_IKEY,
	X_IFLA_GRE_OKEY,
	X_IFLA_GRE_LOCAL,
	X_IFLA_GRE_REMOTE,
	X_IFLA_GRE_TTL,
	X_IFLA_GRE_TOS,
	X_IFLA_GRE_PMTUDISC,
	X_IFLA_GRE_ENCAP_LIMIT,
	X_IFLA_GRE_FLOWINFO,
	X_IFLA_GRE_FLAGS,
	__X_IFLA_GRE_MAX,
};

#define X_IFLA_GRE_MAX    (__X_IFLA_GRE_MAX - 1)

struct ip6_tnl_parm2 {
	char name[IFNAMSIZ];    /* name of tunnel device */
	int link;       /* ifindex of underlying L2 interface */
	__u8 proto;     /* tunnel protocol */
	__u8 encap_limit;   /* encapsulation limit for tunnel */
	__u8 hop_limit;     /* hop limit for tunnel */
	__be32 flowinfo;    /* traffic class and flowlabel for tunnel */
	__u32 flags;        /* tunnel flags */
	struct in6_addr laddr;  /* local tunnel end-point address */
	struct in6_addr raddr;  /* remote tunnel end-point address */
	
	__be16          i_flags;
	__be16          o_flags;
	__be32          i_key;
	__be32          o_key;
};

#endif
