/* Copyright (C) 2006 Ikanos Communications
 * See IKANOS_PROP_LICENSE.txt for license information. */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <syslog.h>
#include <fcntl.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <string.h>

#include "utils.h"
#include "tc_util.h"

#include <wfqlib.h>
/*
#define SETWT_USAGE "Usage: fusiv_wfq type <dwpfq/pq> bw <value in Kbps/Mbps/Gbps > borrow <mask in hex> tcp-ack-priority <mask in hex> contribute <mask in hex> qwts <Q0_wt> <Q1_wt> <Q2_wt> <Q3_wt> <Q4_wt> <Q5_wt> <Q6_wt> <Q7_wt> \n\n\ */

#define SETWT_USAGE "Usage: fusiv_wfq type <dwpfq/pq> bw <value in Kbps/Mbps/Gbps > borrow <mask in hex> tcp-ack-priority <mask in hex> contribute <mask in hex> qinfo <Q0_pri>:<Q0_minbw>/<Q0_maxbw> ...<Q7_pri>:<Q7_minbw>/<Q7_maxbw>\n\n\
Mandatory Parameters \n\
  type             - Scheduler Type \n\
                     dwpfq  -  Deficit Weighted Priority Based Fair Queuing (DWPFQ) \n\
                     pq     -  Priority Queuing (PQ) \n\
  bw               - Scheduler Bandwidth Expressed in Kbps/Mbps/Gbps\n\n\
Conditional Mandatory\n\
  qinfo             - Scheduler Q Min and Max bandwith in Kbps/Mbps/Gbps\n\
                     ** Mandatory for Deficit Weighted Priority based Fair Queuing (DWPFQ) ** \n\n\
Optional Parameters \n \
  borrow           -  Borrow Mask\n \
                      Each bit number mapped to the scheduler Q priority. [OPTIONAL, default 0xFF] \n \
                      when set, the Q's un-used  bandwidth can be given to other Qs othewise reserved. \n \
  tcp-ack-priority -  TCP-ACK Priority Mask\n \
                      Each bit number mapped to the scheduler Q priority. [OPTIONAL, default 0xFF] \n \
                      when set, TCP ACKs are always transmitted before any other packet \n \
                      in that particular Q. \n \
  contribute       -  Contribute Mask\n \
                      Each bit number mapped to the scheduler Q priority. [OPTIONAL, default 0xFF] \n \
                      when set, the un-used bandwidth from the Qs can be given to this Q.\n"


#define usage() return(-1)

static void explain(void)
{
	fprintf(stderr, SETWT_USAGE);
}

static int VerifyDigit(char *argv)
{
	char *ptr = argv;
	while(*ptr != '\0')
	{
		if(isdigit(*ptr) == 0)
			return -1;
		ptr++;
	}
	return atoi(argv);
}



static int wfq_parse_opt(struct qdisc_util *qu, int argc, char **argv, struct nlmsghdr *n)
{
  wfq_config_t opt;
	int total = 0;
	int i,j;
	int value;
        int queueno,k;
       char buf[50],buf2[50],*tmp1,*tmp2;


        memset(&opt, 0, sizeof(opt));
	opt.borrow = 0xFF;
	opt.contribute = 0xff;
	opt.ackPriority = 0x0;

	while(argc > 0) 
	{
		if(!strcasecmp(*argv,"type"))
		{
			/* Scheduler Configuration */
			NEXT_ARG();
			/* Configure Type of the scheduler, deficit-weighted or simply priority queue */
			if(!strcasecmp(*argv,"dwpfq"))
				opt.schedType = DEFICIT_WT_PRIORITY;
			else if(!strcasecmp(*argv,"pq"))
				opt.schedType = SIMPLE_PRIORITY;
			else
			{
				fprintf(stderr,"Invalid \"type\" %s \n",*argv);
		    goto parse_failed;
			}
		}
		else if(!strcasecmp(*argv,"bw"))
		{
			/* Configure Link Speed */
			NEXT_ARG();

                       strcpy(buf,*argv);
                       tmp1 = buf;

                       /* Get the number before Kbps/Mbps/Gbps specifier.
                        * Internally, the scheduler bw is handled in terms
                        * of Kbps,so convert Mbps/Gbps to Kbps
                        */

                       if((tmp2 = strstr(buf,"Kbps")))
                               buf[tmp2-tmp1] = '\0';
                       else if ((tmp2 = strstr(buf,"Mbps")))
                       {
                               buf[tmp2-tmp1] = '\0';
                               strcat(buf,"000");
                       }
                       else if ((tmp2 = strstr(buf,"Gbps")))
                       {
                               buf[tmp2-tmp1] = '\0';
                               strcat(buf,"000000");
                       }
                       else
                       {
                               fprintf(stderr,"Invalid \"bw\" %s \n",*argv);
                               goto parse_failed;
                       }
			if( (opt.bandwidth = VerifyDigit(buf)) == -1)
			{
				fprintf(stderr,"Invalid \"bw\" %s \n",*argv);
		    goto parse_failed;
			}
    }
		else if(!strcasecmp(*argv,"borrow"))
		{
			/* Borrow Mask */
      NEXT_ARG();
			if(sscanf(*argv,"%x",&value) == 0)
			{
				fprintf(stderr,"Invalid \"borrow\" %s \n",*argv);
		    goto parse_failed;
			}
			opt.borrow = value;
		}
		else if(!strcasecmp(*argv,"tcp-ack-priority"))
		{
			/* AckPriority Mask */
      NEXT_ARG();
			if(sscanf(*argv,"%x",&value) == 0)
			{
				fprintf(stderr,"Invalid \"tcp-ack-priority\" %s \n",*argv);
		    goto parse_failed;
			}
			opt.ackPriority = value;
		}
		else if(!strcasecmp(*argv,"contribute"))
		{
			/* Contribute Mask */
      NEXT_ARG();
			if(sscanf(*argv,"%x",&value) == 0)
			{
				fprintf(stderr,"Invalid \"contribute\" %s \n",*argv);
		    goto parse_failed;
			}
			opt.contribute = value;
		}
#if 0
		else if(!strcasecmp(*argv,"qwts"))
		{
			/* Scheduler Q weights */
			NEXT_ARG();
			for(i=0; i<DEFAULT_QSIZE; i++)
			{
			  if( (opt.PerQ[i].weight = VerifyDigit(*argv)) == -1)
			  {
			  	fprintf(stderr,"Invalid \"qwts\" %s \n",*argv);
		      goto parse_failed;
			  }
				if( i < (DEFAULT_QSIZE -1))
				{
					NEXT_ARG();
				}
			  total += opt.PerQ[i].weight;
      }
			if(total != 100)
			{
				fprintf(stderr,"Invalid \"qwts\" total should be 100%\n");
		    goto parse_failed;
			}
		}
#else
                else if(!strcasecmp(*argv,"qinfo"))
                {
                        /* Scheduler Q min and max bandwidths */
                        if(!opt.bandwidth)
                        {
                          fprintf(stderr,"No bandwidth to assign \"qinfo\" %s \n",*argv);
                          goto parse_failed;
                        }
                            
                        NEXT_ARG();
                        queueno = -1;
                        for(i=0; i<DEFAULT_QSIZE; i++)
                        {
                          strcpy(buf,*argv);
                          tmp1 = buf;
                          if((tmp2 = strstr(buf,":")))
                          {
                            buf[tmp2-tmp1] = '\0';
                            if( (queueno = VerifyDigit(buf)) == -1)
                            {
                                fprintf(stderr,"Invalid \"qinfo\" %s \n",*argv);
                                goto parse_failed;
                            }
                            if((queueno <0)||(queueno>7))
                            {
                                fprintf(stderr,"Invalid Priority queue \"qinfo\" %s \n",*argv);
                      		goto parse_failed;
                            }
                            tmp1 = ++tmp2;
                            for(j=0; j<2; j++) /* Check for Min and Max BWs */
                            {
                              if((tmp2 = strstr(tmp1,"Kbps")))
                                tmp1[tmp2-tmp1] = '\0';
                              else if ((tmp2 = strstr(tmp1,"Mbps")))
                              {
                                tmp1[tmp2-tmp1] = '\0';
                                strcat(tmp1,"000");
                              }
                              else if ((tmp2 = strstr(tmp1,"Gbps")))
                              {
                                strcpy(buf2,tmp2);
                                tmp1[tmp2-tmp1] = '\0';
                                tmp2 = buf2;
                                strcat(tmp1,"000000");
                              }
                              else
                              {
                                fprintf(stderr,"Invalid \"bw\" %s \n",*argv);
                                goto parse_failed;
                              }
                              if( (k = VerifyDigit(tmp1)) == -1)
                              {
                                  fprintf(stderr,"Invalid \"bw\" %s \n",*argv);
                                goto parse_failed;
                              }
                              if((j == 0)&&(!opt.PerQ[queueno].minbw)) 
                                opt.PerQ[queueno].minbw = k;
                              else if((j == 1)&&(!opt.PerQ[queueno].maxbw))
                                opt.PerQ[queueno].maxbw = k;
                              else
                              {
                                fprintf(stderr,"Queue duplication %s \n",*argv);
                                goto parse_failed;
                              }
                              tmp1 = tmp2 + 5; /* to parse "/Mbps or /Kbps or /Gbps"*/
                            }
                            if((opt.PerQ[queueno].minbw > opt.PerQ[queueno].maxbw)||
                               (opt.PerQ[queueno].maxbw > opt.bandwidth )||
                                (opt.PerQ[0].minbw == 0))
                            {
                                fprintf(stderr,"Improper min and max BWs or No BW for Default queue \"qinfo\" %s \n",*argv);
                                goto parse_failed;
                            }
                          }
                          else
                          {
                                fprintf(stderr,"Improper command \"qinfo\" %s \n",*argv);
                                goto parse_failed;
                          }
                          if( i < (DEFAULT_QSIZE -1))
                          {
                            if(--argc <= 0)  /* This is to accept the command even if it has only one queue's "qinfo" */
                              break;
		            argv++;
                          }
                        }
                        for(i=0; i<DEFAULT_QSIZE; i++)
                          total += opt.PerQ[i].minbw;

                        if(total > opt.bandwidth)
                        {
                          fprintf(stderr,"Invalid \"qinfo\" total bw should be < link bandwidth\n");
                          goto parse_failed;
                        }
                }
#endif
		else 
		{
      fprintf(stderr, "What is \"%s\"?\n", *argv);
		  goto parse_failed;
    }
		argc--;
		argv++;
  }
	if(n->nlmsg_type != RTM_DELQDISC)
	{
		if(opt.schedType == 0)
		{
			fprintf(stderr,"Parameter  \"type\"is mandatory\n");
			goto parse_failed;
		}
		if(opt.bandwidth == 0)
		{
			fprintf(stderr,"Parameter  \"bw\"is mandatory\n");
			goto parse_failed;
		}
		if(opt.schedType == DEFICIT_WT_PRIORITY && total == 0)
		{
			fprintf(stderr,"Parameter  \"qinfo\"is mandatory for Deficit Weighted Priority scheduler\n");
			goto parse_failed;
		}
	}
	addattr_l(n, 1024, TCA_OPTIONS, &opt, sizeof(opt));
	return 0;
parse_failed:
	explain();
	return -1;
}


static int wfq_print_opt(struct qdisc_util *qu, FILE *f, struct rtattr *opt)
{
	wfq_config_t *data;
	int i;
  if (opt == NULL)
    return 0;
  if(RTA_PAYLOAD(opt) < sizeof (wfq_config_t))
	  return -1;

   data = RTA_DATA (opt);
   fprintf(f,"Scheduler Configurations : \n");
	 if(data->schedType == DEFICIT_WT_PRIORITY)
		 fprintf(f,"  type             - Deficit Weighted Priority based Fair Queuing (DWPFQ)\n");
	 else if(data->schedType == SIMPLE_PRIORITY)
		 fprintf(f,"  type             - Priority Queuing (PQ)\n");
	 else
     fprintf(f,"  type             - UNKNOWN \n");

	 fprintf(f,"  bw               - %d Kbps \n",data->bandwidth);			
	 fprintf(f,"  borrow           - 0x%x\n",data->borrow);			
	 fprintf(f,"  tcp-ack-priority - 0x%x\n",data->ackPriority);			
	 fprintf(f,"  contribute       - 0x%x\n",data->contribute);			
	 if(data->schedType == DEFICIT_WT_PRIORITY)
	 {
	   fprintf(f,"qinfo-QNo:MIN/MAX BW\n");
     for(i=0; i<DEFAULT_QSIZE; i++)
		   fprintf(f,"  %d:%d Kbps/%d Kbps\n",i,data->PerQ[i].minbw,data->PerQ[i].maxbw);
	   fprintf(f,"\n");
	 }
   return 0;
}

static int wfq_print_xstats(struct qdisc_util *qu, FILE *f, struct rtattr *xstats)
{
	wfq_config_t *data;
	int i;

  if (xstats == NULL)
    return 0;

  if (RTA_PAYLOAD(xstats) <sizeof (wfq_config_t))
	  return -1;

  data = RTA_DATA (xstats);
  for(i=0; i<DEFAULT_QSIZE; i++)
  {
    printf("Q.NO=%d :Sent_pkt=%u:Drop_pkt=%u:Sent_bytes=%lld:Drop_bytes=%lld\n",
      i,data->PerQ[i].sent_packets,data->PerQ[i].drop_packets,
      data->PerQ[i].sent_bytes,data->PerQ[i].drop_bytes);
  }
	return 0;
}

struct qdisc_util fusiv_wfq_qdisc_util = {
	NULL,
	"fusiv_wfq",
	wfq_parse_opt,
	wfq_print_opt,
	wfq_print_xstats,
};
