#ifndef _NEMO_WRAPPER_H_
#define _NEMO_WRAPPER_H_

#include "upnp/upnp.h"
#include <pcb/pcb_client.h>

typedef struct _nemo_intf {
  bool linkIsUp;                         // flag link is up/down -> physical interface is up
  bool isConnected;                      // flag connection is connected/disconnected -> both physical and logical interface is up
#ifdef UPNP_ENABLE_IPV6
  char ipv6Address[INET6_ADDRSTRLEN+1];
#endif // UPNP_ENABLE_IPV6
  char ipv4Address[INET6_ADDRSTRLEN+1];
  char luckyIntfName[IFNAMSIZ+1];        // name of the lucky interface (e.g. ppp_vdata)
  char nemoInterface[IFNAMSIZ+1];        // nemo id oof the wan interface (e.g. data)
} nemo_interface;

typedef enum {DSL=0, ETHERNET} wan_access_type_t;

typedef struct _nemo_link_layer_max_bit_rates {
  wan_access_type_t wanAccessType;
  int upstreamMaxBitRate;
  int downStreamMaxBitRate;
  int UpstreamCurrRate;
  int DownstreamCurrRate;
} nemo_link_layer_max_bit_rates;

// call this init function first
bool NeMoInitWrapper(const char *nemoWanInterfaceName);

bool NeMoUpdateWanStatus(pcb_t *client, peer_info_t *peer, bool physicalIsUp, bool logicalIsUp);

bool NeMoGetWanStatus(nemo_interface *nemo_intf);

// NeMoGetWanStatus() has to be called first to update the current status
// returns Up or Down
char* NeMoGetEthernetLinkStatus(nemo_interface *nemo_intf);

// NeMoGetWanStatus() has to be called first to update the current status
// returns Coonected or Disconnected
char* NeMoGetConnectionStatus(nemo_interface *nemo_intf);

// this function can only be called from inside the PCB thread, not from any of the lib_upnp threads !!!!
bool NeMoGetLinkLayerMaxBitRates(nemo_link_layer_max_bit_rates *bitRates);


#endif // _NEMO_WRAPPER_H_
