#!/usr/bin/env bash
#set -x

DIRECTORY=$(cd `dirname $0` && pwd)
cd $DIRECTORY

. ../shell_tools.sh
. ../shell_unit.sh
. ../testsources.sh

testplan local_standalone
# testplan remote
# testplan local_standalone
# testplan remote_standalone

IPV4_ADDRES_TO_USE=192.168.1.254 # IPv4 address to use for SSDP messages
LOG_FILE=test.log

testplan_setup()
{
	echo "=== Entering testplan_setup ===="
	echo "" > $LOG_FILE  # create log file
}

testplan_teardown()
{
    # TODO
	echo "Entering testplan_teardown"
}

testExternalPort_InternalPortRanges()
{
	echo "++++ Verify the External/Internal port ranges (1024 .. 65535) ++++" | tee -a $LOG_FILE
	OUT=`./upnpc-static -m $IPV4_ADDRES_TO_USE -a $IPV4_ADDRES_TO_USE -1 6000 TCP 3600`
	echo -e $OUT >> $LOG_FILE
	echo $OUT | grep "failed with code 402" | grep "Invalid Args" > /dev/null
	assert "$? -eq 0" "allowed value for NewInternalPort cannot be <0 when executing AddPortMapping"

	OUT=`./upnpc-static -m $IPV4_ADDRES_TO_USE -a $IPV4_ADDRES_TO_USE 65536 6000 TCP 3600`
	echo -e $OUT >> $LOG_FILE
	echo $OUT | grep "failed with code 601" | grep "Argument Value Out of Range" > /dev/null
	assert "$? -eq 0" "allowed value for NewInternalPort cannot be > 65535 when executing AddPortMapping"

	OUT=`./upnpc-static -m $IPV4_ADDRES_TO_USE -a $IPV4_ADDRES_TO_USE 6000 -1 TCP 3600`
	echo -e $OUT >> $LOG_FILE
	echo $OUT | grep "failed with code 402" | grep "Invalid Args" > /dev/null
	assert "$? -eq 0" "allowed value for NewExternalPort cannot be <0 when executing AddPortMapping"

	OUT=`./upnpc-static -m $IPV4_ADDRES_TO_USE -a $IPV4_ADDRES_TO_USE 6000 65536 TCP 3600`
	echo -e $OUT >> $LOG_FILE
	echo $OUT | grep "failed with code 601" | grep "Argument Value Out of Range" > /dev/null
	assert "$? -eq 0" "allowed value for NewExternalPort cannot be > 65535 when executing AddPortMapping"

	# NOW THE ADDANYPortMapping
		OUT=`./upnpc-static -m $IPV4_ADDRES_TO_USE -k $IPV4_ADDRES_TO_USE -1 6000 TCP 3600`
	echo -e $OUT >> $LOG_FILE
	echo $OUT | grep "failed with code 402" | grep "Invalid Args" > /dev/null
	assert "$? -eq 0" "allowed value for NewInternalPort cannot be <0 when executing AddANYPortMapping"

	OUT=`./upnpc-static -m $IPV4_ADDRES_TO_USE -k $IPV4_ADDRES_TO_USE 65536 6000 TCP 3600`
	echo -e $OUT >> $LOG_FILE
	echo $OUT | grep "failed with code 601" | grep "Argument Value Out of Range" > /dev/null
	assert "$? -eq 0" "allowed value for NewInternalPort cannot be > 65535 when executing AddANYPortMapping"

	OUT=`./upnpc-static -m $IPV4_ADDRES_TO_USE -k $IPV4_ADDRES_TO_USE 6000 -1 TCP 3600`
	echo -e $OUT >> $LOG_FILE
	echo $OUT | grep "failed with code 402" | grep "Invalid Args" > /dev/null
	assert "$? -eq 0" "allowed value for NewExternalPort cannot be <0 when executing AddANYPortMapping"

	OUT=`./upnpc-static -m $IPV4_ADDRES_TO_USE -k $IPV4_ADDRES_TO_USE 6000 65536 TCP 3600`
	echo -e $OUT >> $LOG_FILE
	echo $OUT | grep "failed with code 601" | grep "Argument Value Out of Range" > /dev/null
	assert "$? -eq 0" "allowed value for NewExternalPort cannot be > 65535 when executing AddANYPortMapping"
}

UPNPIGD_0041()
{
	echo "++++ Allowed values for NewInternalPort parameter of AddPortMapping and AddAnyPortMapping actions ++++" | tee -a $LOG_FILE
	# allowed value for NewInternalPort cannot be equal to 0, otherwise return error 732 "WildCardNotPermittedinIntPort"
	OUT=`./upnpc-static -m $IPV4_ADDRES_TO_USE -Y 1 0`
	echo -e $OUT >> $LOG_FILE
	echo $OUT | grep "failed with code 732" | grep "WildCardNotPermittedinIntPort" > /dev/null
	assert "$? -eq 0" "allowed value for NewInternalPort cannot be equal to 0 when executing AddPortMapping"

	OUT=`./upnpc-static -m $IPV4_ADDRES_TO_USE -X 1 0`
	echo -e $OUT >> $LOG_FILE
	echo $OUT | grep "failed with code 732" | grep "WildCardNotPermittedinIntPort" > /dev/null
	assert "$? -eq 0" "allowed value for NewInternalPort cannot be equal to 0 when executing AddANYPortMapping"

	# allowed value for NewInternalPort cannot be less then 1024, otherwise return error 606 "Action not authorized"
	OUT=`./upnpc-static -m $IPV4_ADDRES_TO_USE -Y 1 1023`
	echo -e $OUT >> $LOG_FILE
	echo $OUT | grep "failed with code 606" | grep "Action not authorized" > /dev/null
	assert "$? -eq 0" "allowed value for NewInternalPort cannot be less then 1024 when executing AddPortMapping"

	OUT=`./upnpc-static -m $IPV4_ADDRES_TO_USE -X 1 1023`
	echo -e $OUT >> $LOG_FILE
	echo $OUT | grep "failed with code 606" | grep "Action not authorized" > /dev/null
	assert "$? -eq 0" "allowed value for NewInternalPort cannot be less then 1024 when executing AddANYPortMapping"
}

UPNPIGD_0050()
{
	echo "++++ Static NAT rules persistency ++++" | tee -a $LOG_FILE

	# To solve interoperability issues with legacy UPnP IGD Control Points, the Orange residential gateway MUST allow a control point
  # to create persistent NAT rules through the UPnP action AddPortMapping (with the parameter NewLeaseDuration set to 0).
	OUT=`./upnpc-static -m $IPV4_ADDRES_TO_USE -Y 1 60000 0`
	STATUS=$?
	echo -e $OUT >> $LOG_FILE
	assert "$STATUS -eq 0" "AddPortMapping HAS to allow parameter NewLeaseDuration set to 0"

	# However; as specified in [ A5 A7A7], the Orange residential gateway MUST NOT allow a control point to create persistent NAT rules
  # through the UPnP action AddAnyPortMapping (with the parameter NewLeaseDuration set to 0).
	OUT=`./upnpc-static -m $IPV4_ADDRES_TO_USE -X 1 60001 0`
	echo $OUT | grep "failed with code 606" | grep "Argument Value Out of Range" > /dev/null
	assert "$? -eq 1" "AddANYPortMapping MUST NOT allow NewLeaseDuration set to 0"
}

locateUPnP-IGDControlPoint()
{
	echo "++++ try to find a  UPnP-IGD device ++++" | tee -a $LOG_FILE

	./upnpc-static -m $IPV4_ADDRES_TO_USE -l >> $LOG_FILE
	STATUS=$?
	assert "$STATUS -eq 0" "Try to find an UPnP-IGD device"
	if [ $STATUS -ne 0 ]
	then
		echo "NO UPnP-IGD device found, nothing to test, exiting" | tee -a $LOG_FILE
		exit 1
	fi
}

test_ft_testplan()
{
	echo "==== Entering ft_testplan ====" | tee -a $LOG_FILE
	run_test locateUPnP-IGDControlPoint  "Locate UPnP-IGD control point"
	run_test testExternalPort_InternalPortRanges  "Verify the External/Internal port ranges (1024 .. 65535)"
	run_test UPNPIGD_0041  "Allowed values for NewInternalPort parameter of AddPortMapping and AddAnyPortMapping actions"
	run_test UPNPIGD_0050  "Static NAT rules persistency"
	run_test_done
}

run_testplan test_ft_testplan



