#include <stdarg.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <sys/socket.h>
#include <netinet/in.h>

#include <sys/select.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#include <arpa/inet.h>

#include <net/if.h>
#include <sys/ioctl.h>

#ifndef IN6_IS_ADDR_GLOBAL
#define IN6_IS_ADDR_GLOBAL(a) \
    (((((__const uint8_t *) (a))[0] & htonl(0xff000000)) <= htonl(0x3f000000)   \
        && (((__const uint8_t *) (a))[0] & htonl(0xff000000)) >= htonl(0x20000000)))
#endif /* IS ADDR GLOBAL */

#ifndef IN6_IS_ADDR_ULA
#define IN6_IS_ADDR_ULA(a) \
    ((((__const uint32_t *) (a))[0] & htonl(0xfe000000)) \
    == htonl (0xfc000000))
#endif /* IS ADDR ULA */


#define MAX(a,b) ((a)>(b) ? (a) : (b))

#define SSDP_IP   "239.255.255.250"
#define SSDP_IPV6_LINKLOCAL "FF02::C"
#define SSDP_IPV6_SITELOCAL "FF05::C"
#define MAX_INTERFACES 256

static int g_ipv4Socket = 0;
static int g_ipv6Socket = 0;
static int g_serverPort = 1900;
static int g_bindToDevice = 0;
static int g_stopServer = 0;
static unsigned int gIF_INDEX = 0;

static char* g_logFileName = NULL;
static FILE *g_logFileFP = NULL;

static char* g_ifName = NULL;                       // interface name
static int g_ifNameStrLength = 0;                   // g_ifName strlength
static char g_IPv4Address[22] = {'\0'};             // IPv4 address of the interface
static char g_IPv6Address[65] = {'\0'};             // IPv6 address of the interface
static char g_IPv6_ULA_GUA_Address[65] = {'\0'};    // IPv6 address of the interface

void Log(const char *format,...)
{
  va_list args;
  static char str[1024];

  va_start(args, format);
  vsprintf(str, format, args);
  va_end(args);

  fprintf(g_logFileFP ? g_logFileFP : stdout, "%s\n", str);
  if (g_logFileFP) fflush (g_logFileFP);   // we do not close the fp, thus flush here
}

int GetIPAddresses()
{
  char szBuffer[MAX_INTERFACES * sizeof(struct ifreq)];
  struct ifconf ifConf;
  struct ifreq ifReq;
  FILE *inet6_procfd;
  size_t i;
  int LocalSock;
  struct in6_addr v6_addr;
  unsigned if_idx;
  char addr6[8][5];
  char buf[65];		/* INET6_ADDRSTRLEN */  
  int valid_addr_found = 0;

  // Create an unbound datagram socket to do the SIOCGIFADDR ioctl on.
  if ((LocalSock = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP)) < 0) {
    Log("Can't create addrlist socket\n");
    return 0;
  }
  // Get the interface configuration information...
  ifConf.ifc_len = sizeof(szBuffer);
  ifConf.ifc_ifcu.ifcu_buf = (caddr_t) szBuffer;

  if (ioctl(LocalSock, SIOCGIFCONF, &ifConf) < 0) {
    Log("DiscoverInterfaces: SIOCGIFCONF returned error\n");
    return 0;
  }
  // Cycle through the list of interfaces looking for IP addresses.
  for (i = 0; i < (size_t)ifConf.ifc_len;) {
    struct ifreq *pifReq = (struct ifreq *)((caddr_t) ifConf.ifc_req + i);
    i += sizeof(*pifReq);
    // See if this is the sort of interface we want to deal with.
    strcpy(ifReq.ifr_name, pifReq->ifr_name);
    if (ioctl(LocalSock, SIOCGIFFLAGS, &ifReq) < 0) {
      Log("Can't get interface flags for %s:\n", ifReq.ifr_name);
    }
    // Skip LOOPBACK interfaces, DOWN interfaces and interfaces that
    // don't support MULTICAST.
    if ((ifReq.ifr_flags & IFF_LOOPBACK)
        || (!(ifReq.ifr_flags & IFF_UP))
        || (!(ifReq.ifr_flags & IFF_MULTICAST))) {
      continue;
    }

    if (strncmp(g_ifName, pifReq->ifr_name, g_ifNameStrLength) != 0) {
        // This is not the interface we're looking for.
        continue;
    }
    /* Check address family. */
    if (pifReq->ifr_addr.sa_family == AF_INET) {
      // Copy interface name, IPv4 address and interface index.

      strncpy(g_IPv4Address, inet_ntoa(((struct sockaddr_in *)&(pifReq->ifr_addr))->sin_addr), sizeof(g_IPv4Address));
      Log("IPv4 Address of interface \"%s\" is %s", g_ifName, g_IPv4Address);
      //gIF_INDEX = if_nametoindex(g_IPv4Address);
      gIF_INDEX = if_nametoindex(g_ifName);
      valid_addr_found = 1;
      break;
    }
  }
  close(LocalSock);
  // Failed to find a valid address.
  if (valid_addr_found == 0) {
    Log("Failed to find an adapter with valid IP addresses for use.\n");
    return 0;
  }
  // Try to get the IPv6 address for the same interface
  // from "/proc/net/if_inet6", if possible.
  inet6_procfd = fopen("/proc/net/if_inet6", "r");
  if (inet6_procfd) {
    while (fscanf(inet6_procfd,
            "%4s%4s%4s%4s%4s%4s%4s%4s %02x %*02x %*02x %*02x %*20s\n",
            addr6[0], addr6[1], addr6[2], addr6[3],
            addr6[4], addr6[5], addr6[6], addr6[7],
            &if_idx) != EOF) {
      /* Get same interface as IPv4 address retrieved. */
      if (gIF_INDEX == if_idx) {
        snprintf(buf, sizeof(buf),
           "%s:%s:%s:%s:%s:%s:%s:%s", addr6[0],
           addr6[1], addr6[2], addr6[3], addr6[4],
           addr6[5], addr6[6], addr6[7]);
        /* Validate formed address and check for link-local. */
        if (inet_pton(AF_INET6, buf, &v6_addr) > 0) {
          if (IN6_IS_ADDR_ULA(&v6_addr)) {
            /* Got valid IPv6 ula. */
            strncpy(g_IPv6_ULA_GUA_Address, buf, sizeof(g_IPv6_ULA_GUA_Address));
          } else if (IN6_IS_ADDR_GLOBAL(&v6_addr)
               && strlen(g_IPv6_ULA_GUA_Address)
               == 0) {
            // got a GUA, should store it while no ULA is found
            strncpy(g_IPv6_ULA_GUA_Address, buf, sizeof(g_IPv6_ULA_GUA_Address));
          } else
              if (IN6_IS_ADDR_LINKLOCAL(&v6_addr)
            && strlen(g_IPv6Address) == 0) {
            // got a Link local IPv6 address.
            strncpy(g_IPv6Address, buf, sizeof(g_IPv6Address));
          }
        }
      }
    }
    fclose(inet6_procfd);
  }

  Log("Interface name=%s, index=%d, v4=%s, v6=%s, ULA or GUA v6=%s\n",
       g_ifName, gIF_INDEX, g_IPv4Address, g_IPv6Address, g_IPv6_ULA_GUA_Address);

  return 1;
}

int CreateSSDPv4Socket()
{
  int one = 1;
  struct sockaddr_storage __ss;
  struct sockaddr_in *ssdpAddr4 = (struct sockaddr_in *)&__ss;
  struct ip_mreq ssdpMcastAddr;
  struct in_addr addr;

  if ( (g_ipv4Socket = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
    Log("Error: Unable to create IPv4 socket (%s)", strerror(errno));
    return 0;
  }
  else {
    Log("* IPv4 socket \"%i\" created", g_ipv4Socket);
  }

  if (0 != setsockopt(g_ipv4Socket, SOL_SOCKET, SO_REUSEADDR, (void *)&one, sizeof(one))) {
    Log("Error: Unable to set SO_REUSEADDR for IPv4 socket (%s)", strerror(errno));
    return 0;
  }
  else {
    Log("* IPv4 socket SO_REUSEADDR OK");
  }

  if (g_bindToDevice && g_ifNameStrLength) {
      if (0 != setsockopt(g_ipv4Socket, SOL_SOCKET, SO_BINDTODEVICE, g_ifName, g_ifNameStrLength)) {
        Log("Error: Unable to set SO_BINDTODEVICE for IPv4 socket (%s)", strerror(errno));
        return 0;
      }
      else {
        Log("* IPv4 socket SO_BINDTODEVICE OK");
      }
  }

  memset(&__ss, 0, sizeof(__ss));
  ssdpAddr4->sin_family = AF_INET;
  ssdpAddr4->sin_addr.s_addr = htonl(INADDR_ANY);
  ssdpAddr4->sin_port = htons(g_serverPort);

  if (0 != bind(g_ipv4Socket, (struct sockaddr *)ssdpAddr4, sizeof(*ssdpAddr4))) {
    Log("Error in bind() for IPv4 socket, addr=0x%08X, port=%d: %s\n", INADDR_ANY, g_serverPort, strerror(errno));
    return 0;
  }
  else {
    Log("* IPv4 socket bind OK");
  }

  // =======================================

  // join multicast SSDP group
  memset((void *)&ssdpMcastAddr, 0, sizeof(struct ip_mreq));      
  ssdpMcastAddr.imr_interface.s_addr = inet_addr(g_IPv4Address);
  ssdpMcastAddr.imr_multiaddr.s_addr = inet_addr(SSDP_IP);  
  if (-1 == setsockopt(g_ipv4Socket, IPPROTO_IP, IP_ADD_MEMBERSHIP, (char *)&ssdpMcastAddr, sizeof(struct ip_mreq))) {
    Log("Error in setsockopt() IP_ADD_MEMBERSHIP (join multicast group) for IPv4 socket: %s\n", strerror(errno));
    return 0;
  }
  else {
    Log("* IPv4 setsockopt() IP_ADD_MEMBERSHIP (join multicast group OK");
  }

  // Set multicast interface: from now on all multicast traffic generated in this socket will be output from the chosen interface
  memset((void *)&addr, 0, sizeof(struct in_addr));
  addr.s_addr = inet_addr(g_IPv4Address);
  if (-1 == setsockopt(g_ipv4Socket, IPPROTO_IP, IP_MULTICAST_IF, (char *)&addr, sizeof addr)) {
    Log("Error in setsockopt() IP_MULTICAST_IF (set multicast interface): %s\n", strerror(errno));
    // This is probably not a critical error, so let's continue.
  }

  // set the TTL of multicast diagrams to 1, to prevent them to be forwarded beyond the local network
  if (-1 == setsockopt(g_ipv4Socket, IPPROTO_IP, IP_MULTICAST_TTL, &one, sizeof(one))) {
    Log("Error in setsockopt() IP_MULTICAST_TTL (set TTL): %s\n", strerror(errno));
  }
  // SO_BROADCAST : Allows UDP datagram (SOCK_DGRAM) sockets to send and receive packets sent to and from the broadcast address.
  if (-1 == setsockopt(g_ipv4Socket, SOL_SOCKET, SO_BROADCAST, &one, sizeof(one))) {
    Log("Error in setsockopt() SO_BROADCAST: %s\n", strerror(errno));
  }

  return 1;
}

int CreateSSDPv6Socket()
{
  int one = 1;
  struct ipv6_mreq ssdpMcastAddr;
  struct sockaddr_storage __ss;
  struct sockaddr_in6 *ssdpAddr6 = (struct sockaddr_in6 *)&__ss;

  if ( (g_ipv6Socket = socket(AF_INET6, SOCK_DGRAM, 0)) < 0) {
    Log("Error: Unable to create IPv6 socket (%s)", strerror(errno));
    return 0;
  }
  else {
    Log("** IPv6 socket \"%i\" created", g_ipv6Socket);
  }

  if (0 != setsockopt(g_ipv6Socket, SOL_SOCKET, SO_REUSEADDR, (void *)&one, sizeof(one))) {
    Log("Error: Unable to set SO_REUSEADDR for IPv6 socket (%s)", strerror(errno));
    return 0;
  }
  else {
    Log("** IPv6 socket SO_REUSEADDR OK");
  }

  if (g_bindToDevice && g_ifNameStrLength) {
    if (0 != setsockopt(g_ipv6Socket, SOL_SOCKET, SO_BINDTODEVICE, g_ifName, g_ifNameStrLength)) {
      Log("Error: Unable to set SO_BINDTODEVICE for IPv6 socket (%s)", strerror(errno));
      return 0;
    }
    else {
      Log("** IPv6 socket SO_BINDTODEVICE OK");
    }
  }

  memset(&__ss, 0, sizeof(__ss));
  ssdpAddr6->sin6_family = AF_INET6;
  ssdpAddr6->sin6_addr = in6addr_any;
  ssdpAddr6->sin6_scope_id = gIF_INDEX;
  ssdpAddr6->sin6_port = htons(g_serverPort);  
  if ( -1 == bind(g_ipv6Socket, (struct sockaddr *)ssdpAddr6, sizeof(*ssdpAddr6))) {
    Log("Error in bind(), addr=0x%032lX, port=%d: %s\n",  0lu, g_serverPort, strerror(errno));
    return 0;
  }  
  else {
    Log("** IPv6 socket bind OK");
  }

  memset((void *)&ssdpMcastAddr, 0, sizeof(ssdpMcastAddr));
  ssdpMcastAddr.ipv6mr_interface = gIF_INDEX;
  inet_pton(AF_INET6, SSDP_IPV6_LINKLOCAL, &ssdpMcastAddr.ipv6mr_multiaddr);
  //inet_pton(AF_INET6, SSDP_IPV6_SITELOCAL, &ssdpMcastAddr.ipv6mr_multiaddr);

  if ( -1 == setsockopt(g_ipv6Socket, IPPROTO_IPV6, IPV6_JOIN_GROUP, (char *)&ssdpMcastAddr, sizeof(ssdpMcastAddr))) {
    Log("Error in setsockopt() IPV6_JOIN_GROUP (join multicast group): %s\n",strerror(errno));
    return 0;
  }  
  else {
    Log("** IPv6 setsockopt IPV6_JOIN_GROUP OK");
  }

  if (-1 == setsockopt(g_ipv6Socket, SOL_SOCKET, SO_BROADCAST, (char *)&one, sizeof(one))) {
    Log("Error in setsockopt() SO_BROADCAST (set broadcast): %s\n", strerror(errno));
    return 0;
  }  

  return 1;
}

void ReadFromSocket(int socket)
{  
  char staticBuf[1024] = {'\0'};
  struct sockaddr_storage __ss;
  socklen_t socklen = sizeof(__ss);
  ssize_t byteReceived = 0;
  char ntop_buf[64];

  byteReceived = recvfrom(socket, staticBuf, sizeof(staticBuf)-1, 0, (struct sockaddr *)&__ss, &socklen);
  if (byteReceived > 0) {
    staticBuf[byteReceived] = '\0';
    if (__ss.ss_family == AF_INET)
      inet_ntop(AF_INET, &((struct sockaddr_in *)&__ss)->sin_addr, ntop_buf, sizeof(ntop_buf));
    else if (__ss.ss_family == AF_INET6) {
      inet_ntop(AF_INET6, &((struct sockaddr_in6 *)&__ss)->sin6_addr, ntop_buf, sizeof(ntop_buf));

      if (NULL != strstr(ntop_buf, "::ffff:")) {
        /* some kernel versions claim that the destination IP is IPv6, although it is an IPv4 address */
        Log("------------------ Ipv4 address in Ipv6 format: %s\n", ntop_buf);
        __ss.ss_family = AF_INET;
      }
    }
    else
      strncpy(ntop_buf, "<Invalid address family>", sizeof(ntop_buf));

    Log( "Start of received response ----------------------------------------------------\n"
         "%s\n"
         "End of received response ------------------------------------------------------\n"
         "From host %s", staticBuf, ntop_buf);

  }

}

int StartServer()
{
  int rc = 0;
  int ret = 0;
  int maxSocket = 0;
  fd_set expSet;
  fd_set rdSet;

  maxSocket = MAX(g_ipv4Socket, g_ipv6Socket);
  maxSocket++;

  g_stopServer = 0;
  Log("* Start SSDP server, max socket: %i", maxSocket);

  while (!g_stopServer) {
    FD_ZERO(&rdSet);
    FD_ZERO(&expSet);

    FD_SET(g_ipv4Socket, &rdSet);
    FD_SET(g_ipv6Socket, &rdSet);

    /* select() */
    ret = select(maxSocket, &rdSet, NULL, &expSet, NULL);
    if (-1 == ret && errno == EINTR) {
      continue;
    }

    if (FD_ISSET(g_ipv4Socket, &rdSet)) {
      ReadFromSocket(g_ipv4Socket);
    }

    if (FD_ISSET(g_ipv6Socket, &rdSet)) {
      ReadFromSocket(g_ipv6Socket);
    }

  }

  return rc;
}

void PrintUsageAndExit()
{
  printf("\n usage: ssdp_tester [-b] [-h] [-l <logfile>] <interface_name>\n\n"
         "            -b               : bind to the device (default is no)\n"
         "            -l               : log to a file iso to stdout\n"
         "            <interface_name> : name of the interface to start listening on\n\n");

  exit(1);
}

int ParseCommandLineParameters(int argc, char *argv[])
{  
  int i = 0;

  for (i = 1; i< argc; i++) {
    char *param = argv[i];
    if (argv[i][0] == '-') {  // is it an option?
      if (!strcmp(param, "-b")){
        g_bindToDevice = 1;
      }
      else if (!strcmp(param, "-l")){
        ++i;
        if (argc > i) {
          g_logFileName = strdup(argv[i]);
        }
        else {
          printf("Error: <logfile> option is missing \n\n");
          return 0;
        }
      }
      else {
        printf("Error: Unknown option \"%s\"\n\n", param);
        return 0;
      }
    }
    else {
      if (!g_ifName) {
        g_ifName = strdup(param);
      }
      else {
        printf("Error: <interface_name> \"%s\" is already defined, what about \"%s\"\n\n", g_ifName, param);
        return 0;
      }
    }
  }

  if (g_logFileName && NULL == (g_logFileFP = fopen ( g_logFileName , "w" ))) {
    printf("Error: Unable to open logfile \"%s\"\n\n", g_logFileName);
    return 0;
  }

  if (!g_ifName) {
    printf("Error: missing <interface_name>\n\n");
    return 0;
  }
  g_ifNameStrLength = strlen(g_ifName);

  return 1;
}

int main(int argc, char *argv[]) {

  // parse the command line parameters
  if (!ParseCommandLineParameters(argc, argv))
    PrintUsageAndExit();  

  if (GetIPAddresses() &&
      CreateSSDPv4Socket() &&
      CreateSSDPv6Socket())
    StartServer();

  // TODO close sockets and logfile

	return 0;
}
