################################################################
# project.lst build flags
#
# SAH_DEBUG..........: No optimalization, trace level set to 500
# SAH_DEBUG_NO_STRIP.: No stripping during release step
#
################################################################

ifndef STAGINGDIR
  $(error STAGINGDIR not defined, stopping build)
endif

include $(STAGINGDIR)/components.config

INSTALL ?= install
STRIP ?= strip
PKG_CONFIG_LIBDIR ?= $(STAGINGDIR)/build/pkg-config

SYSBUSLVL=01
MBUSMAPLVL=71
EMANAGERLVL=73

export CFLAGS += -fPIC -Werror

# Currently only build grom (mbus proxy using pcb)
# All other tools are not used

all:
ifeq ($(CONFIG_PCB_DDW_SERIALIZER),y)
	$(MAKE) -C pcb_serializers/pcb_serialize_ddw all
endif
ifeq ($(CONFIG_PCB_TPL_SERIALIZER),y)
	$(MAKE) -C pcb_serializers/pcb_serialize_tpl all
endif
ifeq ($(CONFIG_PCB_HTTP_SERIALIZER),y)
	$(MAKE) -C pcb_serializers/pcb_serialize_http all
endif
ifeq ($(CONFIG_PCB_MBUS_SUPPORT),y)
	$(MAKE) -C pcb_mbus/src all
	$(MAKE) -C grom/src all
endif
ifeq ($(CONFIG_PCB_ODL_SERIALIZER),y)
	$(MAKE) -C pcb_serializers/pcb_serialize_odl all
endif
ifeq ($(CONFIG_PCB_PLUGIN),y)
	$(MAKE) -C pcb_plugin/src all
endif
ifeq ($(CONFIG_PCB_CLIENT),y)
	$(MAKE) -C pcb_client/src all
endif
ifeq ($(CONFIG_PCB_SYSBUS),y)
	$(MAKE) -C pcb_sysbus/src all
	m4 -DSTAGINGDIR=$(STAGINGDIR) pcb_sysbus/sysbus.sh.m4 > pcb_sysbus/sysbus.sh
endif
ifeq ($(CONFIG_PCB_EVENTMANAGER),y)
	$(MAKE) -C eventmanager/src all
endif
ifeq ($(CONFIG_PCB_CLI),y)
	$(MAKE) -C pcb_cli/src all
endif

clean:
ifeq ($(CONFIG_PCB_DDW_SERIALIZER),y)
	$(MAKE) -C pcb_serializers/pcb_serialize_ddw clean
endif
ifeq ($(CONFIG_PCB_TPL_SERIALIZER),y)
	$(MAKE) -C pcb_serializers/pcb_serialize_tpl clean
endif
ifeq ($(CONFIG_PCB_HTTP_SERIALIZER),y)
	$(MAKE) -C pcb_serializers/pcb_serialize_http clean
endif
ifeq ($(CONFIG_PCB_MBUS_SUPPORT),y)
	$(MAKE) -C pcb_mbus/src clean
	$(MAKE) -C grom/src clean
endif
ifeq ($(CONFIG_PCB_ODL_SERIALIZER),y)
	$(MAKE) -C pcb_serializers/pcb_serialize_odl clean
endif
ifeq ($(CONFIG_PCB_PLUGIN),y)
	$(MAKE) -C pcb_plugin/src clean
endif
ifeq ($(CONFIG_PCB_CLIENT),y)
	$(MAKE) -C pcb_client/src clean
endif
ifeq ($(CONFIG_PCB_SYSBUS),y)
	$(MAKE) -C pcb_sysbus/src clean
	rm -f pcb_sysbus/sysbus.sh
endif
ifeq ($(CONFIG_PCB_EVENTMANAGER),y)
	$(MAKE) -C eventmanager/src clean
endif
ifeq ($(CONFIG_PCB_CLI),y)
	$(MAKE) -C pcb_cli/src clean
endif
	$(MAKE) -C test clean

install:
ifeq ($(CONFIG_PCB_DDW_SERIALIZER),y)
	$(INSTALL) -D -m 0755 pcb_serializers/pcb_serialize_ddw/libpcb_serialize_ddw.so $(STAGINGDIR)/usr/lib/pcb/libpcb_serialize_ddw.so
endif
ifeq ($(CONFIG_PCB_TPL_SERIALIZER),y)
	$(INSTALL) -D -m 0755 pcb_serializers/pcb_serialize_tpl/libpcb_serialize_tpl.so $(STAGINGDIR)/usr/lib/pcb/libpcb_serialize_tpl.so
endif
ifeq ($(CONFIG_PCB_HTTP_SERIALIZER),y)
	$(INSTALL) -D -m 0755 pcb_serializers/pcb_serialize_http/libpcb_serialize_http.so $(STAGINGDIR)/usr/lib/pcb/libpcb_serialize_http.so
endif
ifeq ($(CONFIG_PCB_MBUS_SUPPORT),y)
	$(INSTALL) -D -m 0755 grom/src/grom $(STAGINGDIR)/bin/grom
endif
ifeq ($(CONFIG_PCB_ODL_SERIALIZER),y)
	$(INSTALL) -D -m 0755 pcb_serializers/pcb_serialize_odl/libpcb_serialize_odl.so $(STAGINGDIR)/usr/lib/pcb/libpcb_serialize_odl.so
endif
ifeq ($(CONFIG_PCB_PLUGIN),y)
	$(INSTALL) -D -m 0755 pcb_plugin/src/pcb_plugin $(STAGINGDIR)/bin/pcb_plugin
endif
ifeq ($(CONFIG_PCB_CLIENT),y)
	$(INSTALL) -D -m 0755 pcb_client/src/pcb_client $(STAGINGDIR)/bin/pcb_client
endif
ifeq ($(CONFIG_PCB_SYSBUS),y)
	$(INSTALL) -D -m 0755 pcb_sysbus/src/pcb_sysbus $(STAGINGDIR)/bin/pcb_sysbus
	$(INSTALL) -d $(STAGINGDIR)/etc/rc1.d
	$(INSTALL) -m 755 -D pcb_sysbus/sysbus.sh $(STAGINGDIR)/etc/init.d/sysbus
	cd $(STAGINGDIR)/etc/rc1.d && ln -fs ../init.d/sysbus S$(SYSBUSLVL)sysbus
ifeq ($(CONFIG_PCB_MBUS_SUPPORT),y)
ifneq ($(CONFIG_PCB_INSTALL_TARGET_STB),y)
	$(INSTALL) -m 755 -D pcb_sysbus/mbusmap.sh $(STAGINGDIR)/etc/init.d/mbusmap
	cd $(STAGINGDIR)/etc/rc1.d && ln -fs ../init.d/igdbus S$(MBUSMAPLVL)mbusmap
	$(INSTALL) -D -m 0644 $(STAGINGDIR)/config/mbus.map $(STAGINGDIR)/usr/lib/grom/mbus.map
endif
endif
endif
ifeq ($(CONFIG_PCB_EVENTMANAGER),y)
	$(INSTALL) -D -m 0644 eventmanager/eventmanager.odl $(STAGINGDIR)/usr/lib/eventmanager/eventmanager.odl
	$(INSTALL) -D -m 0755 eventmanager/src/eventmanager.so $(STAGINGDIR)/usr/lib/eventmanager/eventmanager.so
	$(INSTALL) -d $(STAGINGDIR)/etc/rc1.d
	$(INSTALL) -m 755 -D eventmanager/eventmanager.sh $(STAGINGDIR)/etc/init.d/eventmanager
	cd $(STAGINGDIR)/etc/rc1.d && ln -fs ../init.d/eventmanager S$(EMANAGERLVL)eventmanager
endif
ifeq ($(CONFIG_PCB_CLI),y)
	$(INSTALL) -D -m 0755 pcb_cli/src/pcb_cli $(STAGINGDIR)/bin/pcb_cli
endif

release:
ifeq ($(CONFIG_PCB_DDW_SERIALIZER),y)
	$(INSTALL) -D -m 755 $(STAGINGDIR)/usr/lib/pcb/libpcb_serialize_ddw.so $(ROOT_FS_DIR)/usr/lib/pcb/libpcb_serialize_ddw.so
ifndef SAH_DEBUG_NO_STRIP
	$(STRIP) $(ROOT_FS_DIR)/usr/lib/pcb/libpcb_serialize_ddw.so
endif
endif
ifeq ($(CONFIG_PCB_TPL_SERIALIZER),y)
	$(INSTALL) -D -m 755 $(STAGINGDIR)/usr/lib/pcb/libpcb_serialize_tpl.so $(ROOT_FS_DIR)/usr/lib/pcb/libpcb_serialize_tpl.so
ifndef SAH_DEBUG_NO_STRIP
	$(STRIP) $(ROOT_FS_DIR)/usr/lib/pcb/libpcb_serialize_tpl.so
endif
endif
ifeq ($(CONFIG_PCB_HTTP_SERIALIZER),y)
	$(INSTALL) -D -m 755 $(STAGINGDIR)/usr/lib/pcb/libpcb_serialize_http.so $(ROOT_FS_DIR)/usr/lib/pcb/libpcb_serialize_http.so
ifndef SAH_DEBUG_NO_STRIP
	$(STRIP) $(ROOT_FS_DIR)/usr/lib/pcb/libpcb_serialize_http.so
endif
endif
ifeq ($(CONFIG_PCB_MBUS_SUPPORT),y)
	$(INSTALL) -D -m 0755 $(STAGINGDIR)/bin/grom $(ROOT_FS_DIR)/bin/grom
ifndef SAH_DEBUG_NO_STRIP
	$(STRIP) $(ROOT_FS_DIR)/bin/grom
endif
endif
ifeq ($(CONFIG_PCB_ODL_SERIALIZER),y)
	$(INSTALL) -D -m 755 $(STAGINGDIR)/usr/lib/pcb/libpcb_serialize_odl.so $(ROOT_FS_DIR)/usr/lib/pcb/libpcb_serialize_odl.so
ifndef SAH_DEBUG_NO_STRIP
	$(STRIP) $(ROOT_FS_DIR)/usr/lib/pcb/libpcb_serialize_odl.so
endif
endif
ifeq ($(CONFIG_PCB_PLUGIN),y)
	$(INSTALL) -D -m 0755 $(STAGINGDIR)/bin/pcb_plugin $(ROOT_FS_DIR)/bin/pcb_plugin
ifndef SAH_DEBUG_NO_STRIP
	$(STRIP) $(ROOT_FS_DIR)/bin/pcb_plugin
endif
endif
ifeq ($(CONFIG_PCB_CLIENT),y)
	$(INSTALL) -D -m 0755 $(STAGINGDIR)/bin/pcb_client $(ROOT_FS_DIR)/bin/pcb_client
ifndef SAH_DEBUG_NO_STRIP
	$(STRIP) $(ROOT_FS_DIR)/bin/pcb_client
endif
endif
ifeq ($(CONFIG_PCB_SYSBUS),y)
	$(INSTALL) -D -m 0755 $(STAGINGDIR)/bin/pcb_sysbus $(ROOT_FS_DIR)/bin/pcb_sysbus
ifndef SAH_DEBUG_NO_STRIP
	$(STRIP) $(ROOT_FS_DIR)/bin/pcb_sysbus
endif
ifeq ($(CONFIG_PCB_INSTALL_TARGET_STB),y)
	$(INSTALL) -d $(ROOT_FS_DIR)/etc/rc.d/rcS.d
	$(INSTALL) -D -m 755 -D $(STAGINGDIR)/etc/init.d/sysbus $(ROOT_FS_DIR)/etc/init.d/sysbus
	cd $(ROOT_FS_DIR)/etc/rc.d/rcS.d && ln -fs ../../init.d/sysbus S58sysbus.sh
else
	$(INSTALL) -d $(ROOT_FS_DIR)/etc_ro_fs/rc1.d
	$(INSTALL) -D -m 755 -D $(STAGINGDIR)/etc/init.d/sysbus $(ROOT_FS_DIR)/etc_ro_fs/init.d/sysbus
	cd $(ROOT_FS_DIR)/etc_ro_fs/rc1.d && ln -fs ../init.d/sysbus S$(SYSBUSLVL)sysbus
ifeq ($(CONFIG_PCB_MBUS_SUPPORT),y)
	$(INSTALL) -D -m 755 -D $(STAGINGDIR)/etc/init.d/mbusmap $(ROOT_FS_DIR)/etc_ro_fs/init.d/mbusmap
	cd $(ROOT_FS_DIR)/etc_ro_fs/rc1.d && ln -fs ../init.d/mbusmap S$(MBUSMAPLVL)mbusmap
	$(INSTALL) -D -m 0644 $(STAGINGDIR)/usr/lib/grom/mbus.map $(ROOT_FS_DIR)//usr/lib/grom/mbus.map
endif
endif
endif
ifeq ($(CONFIG_PCB_EVENTMANAGER),y)
ifeq ($(CONFIG_PCB_INSTALL_TARGET_STB),y)
else
	$(INSTALL) -D -m 0755 $(STAGINGDIR)/usr/lib/eventmanager/eventmanager.so $(ROOT_FS_DIR)/usr/lib/eventmanager/eventmanager.so
ifndef SAH_DEBUG_NO_STRIP
	$(STRIP) $(ROOT_FS_DIR)/usr/lib/eventmanager/eventmanager.so
endif
	$(INSTALL) -D -m 0644 $(STAGINGDIR)/usr/lib/eventmanager/eventmanager.odl $(ROOT_FS_DIR)/usr/lib/eventmanager/eventmanager.odl
	$(INSTALL) -d $(ROOT_FS_DIR)/etc_ro_fs/rc1.d
	$(INSTALL) -D -m 755 -D $(STAGINGDIR)/etc/init.d/eventmanager $(ROOT_FS_DIR)/etc_ro_fs/init.d
	cd $(ROOT_FS_DIR)/etc_ro_fs/rc1.d && ln -fs ../init.d/eventmanager S$(EMANAGERLVL)eventmanager
endif
endif
ifeq ($(CONFIG_PCB_CLI),y)
	$(INSTALL) -D -m 0755 pcb_cli/src/pcb_cli $(ROOT_FS_DIR)/bin/pcb_cli
ifndef SAH_DEBUG_NO_STRIP
	$(STRIP) $(ROOT_FS_DIR)/bin/pcb_cli
endif
endif

test:
	@echo "Creating pcb library unit tests"
	@echo " Variable  COMPONENTNAME=$(COMPONENTNAME)"
	@echo " Variable  LOCALTESTDIR=$(LOCALTESTDIR)"
	$(MAKE) -C test
	echo export COMPONENT_LOCATION=`pwd` > $(LOCALTESTDIR)/component.location
	$(INSTALL) -m 755 test/run $(LOCALTESTDIR)/
	$(INSTALL) -m 755 test/test.sh $(LOCALTESTDIR)/
	$(INSTALL) -d $(LOCALTESTDIR)/unittests
	$(INSTALL) -m 644 test/pcb_unit_test_plugin/test_plugin.odl $(LOCALTESTDIR)
	$(INSTALL) -m 644 test/pcb_unit_test_plugin/test_plugin-definition.odl $(LOCALTESTDIR)
	$(INSTALL) -m 644 test/pcb_unit_test_plugin/test_plugin-data.odl $(LOCALTESTDIR)
	$(INSTALL) -m 755 test/pcb_unit_test_plugin/plugin/test_plugin.so $(LOCALTESTDIR)
	$(INSTALL) -m 755 test/test_libpcb_client/*.so $(LOCALTESTDIR)/unittests
	$(INSTALL) -m 644 test/test_libpcb_client/*.so.list.xml $(LOCALTESTDIR)/unittests
	$(INSTALL) -m 755 test/pcb_plugin_testplan.sh $(LOCALTESTDIR)
	$(INSTALL) -m 755 test/pcb_plugin_r_testplan.sh $(LOCALTESTDIR)
	$(INSTALL) -m 755 test/check_datamodel_testplan.sh $(LOCALTESTDIR)
	$(INSTALL) -m 755 test/check_datamodel_r_testplan.sh $(LOCALTESTDIR)
	@echo "Done!"

.PHONY: all clean install release test

