%{
#include <stdlib.h>

#include <debug/sahtrace.h>

#include <pcb/core/object.h>
#include <pcb/core/parameter.h>
#include <pcb/core/validator.h>
#include <pcb/core/datamodel.h>
#include <pcb/core/acl.h>
#include <pcb/utils/string_list.h>

#include "objectloader.h"
#include "objectloader.tab.h"

extern string_t description;
extern string_t description_line;

void yy_state_set(int new_state) {
    BEGIN(new_state);
}

static void start_description();

static inline void trim_description() {
#ifdef PCB_HELP_SUPPORT
    string_trim(&description);
#endif
}

static inline void add_description(const char *text) {
#ifdef PCB_HELP_SUPPORT
    string_fromChar(&description_line, text);
    string_trim(&description_line);
    if (!string_isEmpty(&description_line)) {
      string_append(&description, &description_line);
    }
#else
  (void)text;
#endif
}

static void end_description() {
#ifdef PCB_HELP_SUPPORT
    string_appendChar(&description,"\\n");
#endif
}

extern bool functionArgs;

%}

%option stack

%x COMMENT DESCRIPTION LINECOMMENT LONGTEXT

NL  \r?\n

%%

"\"\""              {
                      strncpy(odl_yylval.text,"",MAX_TEXT_SIZE);
                      return TEXT;
                    }


"/*"                yy_push_state( COMMENT );

"/**"               { start_description(); }

"//"                yy_push_state( LINECOMMENT );

"\""                { memset(odl_yylval.text,0,MAX_TEXT_SIZE + 1); SAH_TRACEZ_INFO("odl","LONGTEXT START"); yy_push_state( LONGTEXT ); }

[\n]                { yylineno++;}

[ \t]

<COMMENT>{
      "*/"          yy_pop_state();
      "*"
      [^*\n]+
      [^*\n]*{NL}   { yylineno++;}
}

<DESCRIPTION>{
      "*/"          {
                      trim_description();
                      yy_pop_state();
                    }
      "*"
      [^*\n]+       {
                      add_description(yytext);
                    }
      {NL}          { yylineno++;
                      end_description();
                    }
}

<LINECOMMENT>{
      {NL}          { yylineno++; yy_pop_state(); }
      "*"
      [^*\n]+
}

<LONGTEXT>{
      {NL}           yy_pop_state();
      [^\\"]+        { strncat(odl_yylval.text,yytext,MAX_TEXT_SIZE - strlen(odl_yylval.text)); SAH_TRACEZ_INFO("odl", " ==> Normal text [%s]",yytext);}
      [\\].          { strncat(odl_yylval.text,yytext, MAX_TEXT_SIZE - strlen(odl_yylval.text)); SAH_TRACEZ_INFO("odl", " ==> Escaped char [%s]",yytext); }
      "\""           { yy_pop_state(); SAH_TRACEZ_INFO("odl", "LONGTEXT END [%s]", odl_yylval.text); return TEXT; }
}

<<EOF>> {
          FILE *prev = yyin;
          odl_yypop_buffer_state();
          if (!YY_CURRENT_BUFFER) {
              yyterminate();
          } else {
              fclose(prev);
          }
          return FILE_DONE;
        }

include                 { if (functionArgs) {
                            strncpy(odl_yylval.text,yytext,MAX_TEXT_SIZE);
                            return WORD;
                          } else {
                            odl_yylval.keyword = keyword_include;
                            return INCLUDE;
                          }
                        }

define                  { if (functionArgs) {
                            strncpy(odl_yylval.text,yytext,MAX_TEXT_SIZE);
                            return WORD;
                          } else {
                            odl_yylval.keyword = keyword_define;
                            return DEFINE;
                          }
                        }

object                  {  if (functionArgs) {
                            strncpy(odl_yylval.text,yytext,MAX_TEXT_SIZE);
                            return WORD;
                          } else {
                            odl_yylval.keyword = keyword_object;
                            return OBJECT;
                          }
                        }

extends                 { if (functionArgs) {
                            strncpy(odl_yylval.text,yytext,MAX_TEXT_SIZE);
                            return WORD;
                          } else {
                            odl_yylval.keyword = keyword_object;
                            return EXTENDS;
                          }
                        }

acl                     {  if (functionArgs) {
                            strncpy(odl_yylval.text,yytext,MAX_TEXT_SIZE);
                            return WORD;
                          } else {
                            odl_yylval.keyword = keyword_acl;
                            return ACL;
                          }
                        }

user                    { if (functionArgs) {
                            strncpy(odl_yylval.text,yytext,MAX_TEXT_SIZE);
                            return WORD;
                          } else {
                            odl_yylval.keyword = keyword_user;
                            return USER;
                          }
                        }

group                   { if (functionArgs) {
                            strncpy(odl_yylval.text,yytext,MAX_TEXT_SIZE);
                            return WORD;
                          } else {
                            odl_yylval.keyword = keyword_group;
                            return GROUP;
                          }
                        }

world                   { if (functionArgs) {
                            strncpy(odl_yylval.text,yytext,MAX_TEXT_SIZE);
                            return WORD;
                          } else {
                            odl_yylval.keyword = keyword_world;
                            return WORLD;
                          }
                        }

add                     { if (functionArgs) {
                            strncpy(odl_yylval.text,yytext,MAX_TEXT_SIZE);
                            return WORD;
                          } else {
                            odl_yylval.keyword = keyword_add;
                            return ADD;
                          }
                        }

delete                  { if (functionArgs) {
                            strncpy(odl_yylval.text,yytext,MAX_TEXT_SIZE);
                            return WORD;
                          } else {
                            odl_yylval.keyword = keyword_delete;
                            return DELETE;
                          }
                        }

read                    { if (functionArgs) {
                            strncpy(odl_yylval.text,yytext,MAX_TEXT_SIZE);
                            return WORD;
                          } else {
                            odl_yylval.keyword = keyword_read;
                            odl_yylval.attributes = acl_read;
                            return READ;
                          }
                        }

write                   { if (functionArgs) {
                            strncpy(odl_yylval.text,yytext,MAX_TEXT_SIZE);
                            return WORD;
                          } else {
                            odl_yylval.keyword = keyword_write;
                            odl_yylval.attributes = acl_write;
                            return WRITE;
                          }
                        }

translate               { if (functionArgs) {
                            strncpy(odl_yylval.text,yytext,MAX_TEXT_SIZE);
                            return WORD;
                          } else {
                            odl_yylval.keyword = keyword_translate;
                            return TRANSLATE;
                          }
                        }

execute                 { if (functionArgs) {
                            strncpy(odl_yylval.text,yytext,MAX_TEXT_SIZE);
                            return WORD;
                          } else {
                            odl_yylval.attributes = acl_execute;
                            return EXECUTE;
                          }
                        }

read_value              { if (functionArgs) {
                            strncpy(odl_yylval.text,yytext,MAX_TEXT_SIZE);
                            return WORD;
                          } else {
                            odl_yylval.attributes = acl_read_value;
                            return EXECUTE;
                          }
                        }

none                    { if (functionArgs) {
                            strncpy(odl_yylval.text,yytext,MAX_TEXT_SIZE);
                            return WORD;
                          } else {
                            odl_yylval.attributes = 0;
                            return NONE;
                          }
                        }

with                    { if (functionArgs) {
                            strncpy(odl_yylval.text,yytext,MAX_TEXT_SIZE);
                            return WORD;
                          } else {
                            odl_yylval.keyword = keyword_with;
                            return WITH;
                          }
                        }

using                   { if (functionArgs) {
                            strncpy(odl_yylval.text,yytext,MAX_TEXT_SIZE);
                            return WORD;
                          } else {
                            odl_yylval.keyword = keyword_using;
                            return USING;
                          }
                        }

default                 { if (functionArgs) {
                            strncpy(odl_yylval.text,yytext,MAX_TEXT_SIZE);
                            return WORD;
                          } else {
                            odl_yylval.keyword = keyword_default;
                            return DEFAULT;
                          }
                        }

constraint              { if (functionArgs) {
                            strncpy(odl_yylval.text,yytext,MAX_TEXT_SIZE);
                            return WORD;
                          } else {
                            odl_yylval.keyword = keyword_constraint;
                            return CONSTRAINT;
                          }
                        }

read-only               { if (functionArgs) {
                            strncpy(odl_yylval.text,yytext,MAX_TEXT_SIZE);
                            return WORD;
                          } else {
                            odl_yylval.attributes = attr_readonly;
                            return ATTRIBUTE;
                          }
                        }

persistent              { if (functionArgs) {
                            strncpy(odl_yylval.text,yytext,MAX_TEXT_SIZE);
                            return WORD;
                          } else {
                            odl_yylval.attributes = attr_persistent;
                            return ATTRIBUTE;
                          }
                        }

transient               { if (functionArgs) {
                            strncpy(odl_yylval.text,yytext,MAX_TEXT_SIZE);
                            return WORD;
                          } else {
                            odl_yylval.attributes = attr_not_persistent;
                            return ATTRIBUTE;
                          }
                        }

volatile                { if (functionArgs) {
                            strncpy(odl_yylval.text,yytext,MAX_TEXT_SIZE);
                            return WORD;
                          } else {
                            odl_yylval.attributes = attr_volatile;
                            return ATTRIBUTE;
                          }
                        }

template-only           { if (functionArgs) {
                            strncpy(odl_yylval.text,yytext,MAX_TEXT_SIZE);
                            return WORD;
                          } else {
                            odl_yylval.attributes = attr_templateonly;
                            return ATTRIBUTE;
                          }
                        }

accept-parameters       { if (functionArgs) {
                            strncpy(odl_yylval.text,yytext,MAX_TEXT_SIZE);
                            return WORD;
                          } else {
                            odl_yylval.attributes = attr_acceptparameters;
                            return ATTRIBUTE;
                          }
                        }

message                 { if (functionArgs) {
                            strncpy(odl_yylval.text,yytext,MAX_TEXT_SIZE);
                            return WORD;
                          } else {
                            odl_yylval.attributes = attr_message;
                            return ATTRIBUTE;
                          }
                        }

in                      { odl_yylval.attributes = attr_in;
                          return ATTRIBUTE;
                        }

out                     { odl_yylval.attributes = attr_out;
                          return ATTRIBUTE;
                        }

mandatory               { odl_yylval.attributes = attr_mandatory;
                          return ATTRIBUTE;
                        }

void                    { odl_yylval.dataType=type_void;
                          return  DATATYPE;
                        }

string                  { odl_yylval.dataType=type_string;
                          return  DATATYPE;
                        }

int8                    { odl_yylval.dataType=type_int8;
                          return  DATATYPE;
                        }

int16                   { odl_yylval.dataType=type_int16;
                          return  DATATYPE;
                        }

int32                   { odl_yylval.dataType=type_int32;
                          return  DATATYPE;
                        }

int64                   { odl_yylval.dataType=type_int64;
                          return  DATATYPE;
                        }

uint8                   { odl_yylval.dataType=type_uint8;
                          return  DATATYPE;
                        }

uint16                  { odl_yylval.dataType=type_uint16;
                          return  DATATYPE;
                        }

uint32                  { odl_yylval.dataType=type_uint32;
                          return  DATATYPE;
                        }

uint64                  { odl_yylval.dataType=type_uint64;
                          return  DATATYPE;
                        }

bool                    { odl_yylval.dataType=type_bool;
                          return  DATATYPE;
                        }

datetime                { odl_yylval.dataType=type_date_time;
                          return  DATATYPE;
                        }

reference               { odl_yylval.dataType=type_reference;
                          return  DATATYPE;
                        }

list                    {  odl_yylval.dataType=type_list;
                           return  DATATYPE;
                        }

fd                      {  odl_yylval.dataType=type_file_descriptor;
                           return  DATATYPE;
                        }

byte_array              { odl_yylval.dataType=type_byte_array;
                          return  DATATYPE;
                        }

variant                 { odl_yylval.dataType=type_variant;
                          return  DATATYPE;
                        }

minvalue                { if (functionArgs) {
                            strncpy(odl_yylval.text,yytext,MAX_TEXT_SIZE);
                            return WORD;
                          } else {
                            odl_yylval.keyword = keyword_minvalue;
                            return MINVALUE;
                          }
                        }

maxvalue                { if (functionArgs) {
                            strncpy(odl_yylval.text,yytext,MAX_TEXT_SIZE);
                            return WORD;
                          } else {
                            odl_yylval.keyword = keyword_maxvalue;
                            return MAXVALUE;
                          }
                        }

range                   { if (functionArgs) {
                            strncpy(odl_yylval.text,yytext,MAX_TEXT_SIZE);
                            return WORD;
                          } else {
                            odl_yylval.keyword = keyword_range;
                            return RANGE;
                          }
                        }

enum                    { if (functionArgs) {
                            strncpy(odl_yylval.text,yytext,MAX_TEXT_SIZE);
                            return WORD;
                          } else {
                            odl_yylval.keyword = keyword_enum;
                            return ENUM;
                          }
                        }

custom                  { if (functionArgs) {
                            strncpy(odl_yylval.text,yytext,MAX_TEXT_SIZE);
                            return WORD;
                          } else {
                            odl_yylval.keyword = keyword_custom;
                            return CUSTOM;
                          }
                        }

datamodel               { if (functionArgs) {
                            strncpy(odl_yylval.text,yytext,MAX_TEXT_SIZE);
                            return WORD;
                          } else {
                            odl_yylval.keyword = keyword_datamodel;
                            return DATAMODEL;
                          }
                        }

instance                { if (functionArgs) {
                            strncpy(odl_yylval.text,yytext,MAX_TEXT_SIZE);
                            return WORD;
                          } else {
                            odl_yylval.keyword = keyword_instance;
                            return INSTANCE;
                          }
                        }

of                      { if (functionArgs) {
                            strncpy(odl_yylval.text,yytext,MAX_TEXT_SIZE);
                            return WORD;
                          } else {
                            odl_yylval.keyword = keyword_of;
                            return OF;
                          }
                        }

destination             { if (functionArgs) {
                            strncpy(odl_yylval.text,yytext,MAX_TEXT_SIZE);
                            return WORD;
                          } else {
                            odl_yylval.keyword = keyword_destination;
                            return DESTINATION;
                          }
                        }

from                    { if (functionArgs) {
                            strncpy(odl_yylval.text,yytext,MAX_TEXT_SIZE);
                            return WORD;
                          } else {
                            odl_yylval.keyword = keyword_from;
                            return FROM;
                          }
                        }

parameter               { if (functionArgs) {
                            strncpy(odl_yylval.text,yytext,MAX_TEXT_SIZE);
                            return WORD;
                          } else {
                            odl_yylval.keyword = keyword_parameter;
                            return PARAMETER;
                          }
                        }

function                { if (functionArgs) {
                            strncpy(odl_yylval.text,yytext,MAX_TEXT_SIZE);
                            return WORD;
                          } else {
                            odl_yylval.keyword = keyword_function;
                            return FUNCTION;
                          }
                        }

recursive               { if (functionArgs) {
                            strncpy(odl_yylval.text,yytext,MAX_TEXT_SIZE);
                            return WORD;
                          } else {
                            odl_yylval.keyword = keyword_recursive;
                            return RECURSIVE;
                          }
                        }

accept                  { if (functionArgs) {
                            strncpy(odl_yylval.text,yytext,MAX_TEXT_SIZE);
                            return WORD;
                          } else {
                            odl_yylval.keyword = keyword_accept;
                            return ACCEPT;
                          }
                        }

drop                    { if (functionArgs) {
                            strncpy(odl_yylval.text,yytext,MAX_TEXT_SIZE);
                            return WORD;
                          } else {
                            odl_yylval.keyword = keyword_drop;
                            return DROP;
                          }
                        }

rename                  { if (functionArgs) {
                            strncpy(odl_yylval.text,yytext,MAX_TEXT_SIZE);
                            return WORD;
                          } else {
                            odl_yylval.keyword = keyword_rename;
                            return RENAME;
                          }
                        }

request                 { if (functionArgs) {
                            strncpy(odl_yylval.text,yytext,MAX_TEXT_SIZE);
                            return WORD;
                          } else {
                            odl_yylval.keyword = keyword_rename;
                            return REQUEST;
                          }
                        }

cast                    { if (functionArgs) {
                            strncpy(odl_yylval.text,yytext,MAX_TEXT_SIZE);
                            return WORD;
                          } else {
                            odl_yylval.keyword = keyword_cast;
                            return CAST;
                          }
                        }

"..."                   { strncpy(odl_yylval.text,yytext,MAX_TEXT_SIZE);
                          return DOTS;
                        }

"true"                  { strcpy(odl_yylval.text,"true");
                          return TEXT;
                        }

"false"                 { strcpy(odl_yylval.text,"false");
                          return TEXT;
                        }

-[[:digit:]]+           { strncpy(odl_yylval.text,yytext,MAX_TEXT_SIZE);
                          return NUMBER;
                        }

[[:digit:]]+            { strncpy(odl_yylval.text,yytext,MAX_TEXT_SIZE);
                          return NUMBER;
                        }

[a-zA-Z0-9\-_]+         { strncpy(odl_yylval.text,yytext,MAX_TEXT_SIZE);
                          return WORD;
                        }

[a-zA-Z0-9\-_/]+         { strncpy(odl_yylval.text,yytext,MAX_TEXT_SIZE);
                          return PATH;
                        }

\n|.                    { return yytext[0];
                        }

%%

static void start_description() {
#ifdef PCB_HELP_SUPPORT
  if (strcmp(getenv("PCB_DESCRIPTION")?getenv("PCB_DESCRIPTION"):"enabled","enabled") == 0) {
      yy_push_state( DESCRIPTION );
      string_clear(&description);
      string_clear(&description_line);
  } else {
      yy_push_state( COMMENT );
  }
#else
  yy_push_state( COMMENT );
#endif
}
