
# Pretty

ifneq ($(filter $(strip $(V)),1 2),)
VERBOSE:=
QUIET:=@echo 1>/dev/null 
else
VERBOSE:=@
QUIET:=@echo 
endif

# Debug
ifneq ($(DEBUG),1)
CPPFLAGS+=-Os -DNDEBUG
ifeq ($(PROFILING),1)
CPPFLAGS+=-g
else
LDFLAGS+=-Wl,--strip-debug
endif
else
CPPFLAGS+=-g3 -O0
endif

# Werror
WERROR?=
ifeq ($(WERROR),1)
CPPFLAGS+=-Werror
endif

EXTRA_CPPFLAGS?=
EXTRA_CFLAGS?=
EXTRA_CXXFLAGS?=
EXTRA_LDFLAGS?=
CPPFLAGS+=$(EXTRA_CPPFLAGS)
CFLAGS+=$(EXTRA_CFLAGS)
CXXFLAGS+=$(EXTRA_CXXFLAGS)
LDFLAGS+=$(EXTRA_LDFLAGS)

# Include dep files
-include $(OBJS:.o=.d)

# Rule for C files
%.o: %.c
	$(QUIET)"  CC   " $@
	@[ -d "$(dir $@)" ] || mkdir -p "$(dir $@)"
	$(VERBOSE)$(CC) $(CPPFLAGS) $(CFLAGS) -fPIC -MD -MP $< -c -o $@

%.o: %.cpp
	$(QUIET)"  CXX  " $@
	@[ -d "$(dir $@)" ] || mkdir -p "$(dir $@)"
	$(VERBOSE)$(CXX) $(CPPFLAGS) $(CXXFLAGS) -fPIC -MD -MP $< -c -o $@

ifeq ($(type),lib)

$(out): $(OBJS)
ifeq ($(SRCS_CXX),)
	$(QUIET)"  LIB  " $@
	$(VERBOSE)$(CC) -fPIC -shared -o $@ $(filter %.c %.cpp %.o,$^) $(LDFLAGS)
else
	$(QUIET)" LIBXX " $@
	$(VERBOSE)$(CXX) -fPIC -shared -o $@ $(filter %.c %.cpp %.o,$^) $(LDFLAGS)
endif

else ifeq ($(type),dllib)

# Same than lib
$(out): $(OBJS)
ifeq ($(SRCS_CXX),)
	$(QUIET)" DLLIB " $@
	$(VERBOSE)$(CC) -fPIC -shared -o $@ $(filter %.c %.cpp %.o,$^) $(LDFLAGS)
else
	$(QUIET)"DLLIBXX" $@
	$(VERBOSE)$(CXX) -fPIC -shared -o $@ $(filter %.c %.cpp %.o,$^) $(LDFLAGS)
endif

else ifeq ($(type),staticlib)
$(out): $(OBJS)
	$(QUIET)"  AR   " $@
	$(VERBOSE)$(AR) rc $@ $(filter %.c %.cpp %.o,$^)
else
$(out): $(OBJS)
ifeq ($(SRCS_CXX),)
	$(QUIET)"  BIN  " $@
	$(VERBOSE)$(CC) -o $@ $(filter %.c %.cpp %.o,$^) $(LDFLAGS)
else
	$(QUIET)" BINXX " $@
	$(VERBOSE)$(CXX) -o $@ $(filter %.c %.cpp %.o,$^) $(LDFLAGS)
endif
endif

# Generic rules
# Note that this doesn't include header dependencies
#
# We don't add $(ADDONS_RULES) dependency because this would make all
# additional binaries (e.g. tests) depend on the addons rules, so the
# compilation would be paused until addons are compiled.
#
#  * If external binaries depend on the main output file (a library for
#    example) the $(out) dependency have to be added.
#  * If the binary does not depend on the main output file but a file in a
#    addon, $(ADDONS_RULES) can be used, or the specific addon rule
#    (for example if we need add-on/feature1/impl2, the rule is
#    feature1/impl2.addon)

lib%.so:
	$(QUIET)" SOLIB " $@
	$(VERBOSE)$(CC) $(filter %.c %.cpp %.o,$^) -o $@ $(LDFLAGS) -fPIC -shared

%: %.o
ifeq ($(SRCS_CXX),)
	$(QUIET)"  BIN  " $@
	$(VERBOSE)$(CC) $(filter %.c %.cpp %.o,$^) -o $@ $(LDFLAGS)
else
	$(QUIET)" BINXX " $@
	$(VERBOSE)$(CXX) $(filter %.c %.cpp %.o,$^) -o $@ $(LDFLAGS)
endif

%: %.c
	$(QUIET)"  BIN  " $@
	$(VERBOSE)$(CC) $(filter %.c %.cpp %.o,$^) -o $@ $(CPPFLAGS) $(CFLAGS) $(LDFLAGS)

%: %.cpp
	$(QUIET)" BINXX " $@
	$(VERBOSE)$(CXX) $(filter %.c %.cpp %.o,$^) -o $@ $(CPPFLAGS) $(CXXFLAGS) $(LDFLAGS)

