
# If BASEDIR is not set by multiarch.mk (stand-alone?)
BASEDIR?=.

# Define CPPFLAGS and TARGET_ARCH for generic gcc rules
CPPFLAGS?=
TARGET_ARCH?=

#----- FUNCTIONS -----
,=,# To allow commas as $(,)
echolist=$(shell echo -e 1>&2 \
           '$(1) $(if $(2),$(addprefix \n    [,$(addsuffix ],$(2))),None)')
echoone=$(shell echo -e 1>&2 '$(1)')
getconfvar=$(shell [ -f "$(1)" ] && sed -rn -e 's/^$(2)=(.*)$$/\1/ p' $(1))
#----- ----- ----- -----

name?=$(shell echo $(BASEDIR)/ | sed -r -e 's|^.*/([^/]+)/*$$|\1|')
type?=lib

-include $(BASEDIR)/mk.spec

ifndef out
ifeq ($(type),bin)
out:=$(name).bin
else ifeq ($(type),lib)
out:=lib$(name).so
else ifeq ($(type),dllib)
out:=lib$(name).so
else ifeq ($(type),staticlib)
out:=lib$(name).a
endif
endif

DEBUG?=0

ifneq ($(DEBUG),1)
RESOURCEKIND=release
else
RESOURCEKIND=debug
endif


CORE_SRC_DIRS:=$(wildcard $(BASEDIR)/src/     $(BASEDIR)/core/src/)
CORE_SRC_DIRS:=$(shell find $(CORE_SRC_DIRS) -type d)
CORE_INC_DIRS:=$(wildcard $(BASEDIR)/include/ $(BASEDIR)/core/include/)

RESOURCEDIR:=$(BASEDIR)/resource/$(CCMACHINE)/$(CCVERSION)
RES_LIB_DIRS:=$(wildcard $(RESOURCEDIR)/$(RESOURCEKIND)/lib/)
RES_INC_DIRS:=$(wildcard $(RESOURCEDIR)/$(RESOURCEKIND)/include/)

AVAILABLE_IMPLS:=$(shell ls -d $(BASEDIR)/add-on/*/src $(BASEDIR)/add-on/*/*/src 2>/dev/null | sed -e 's|^$(BASEDIR)/add-on/||' | while read dir ; do addon="$$(dirname $$dir)" ; parent="$$(dirname $$addon)" ; [ "$$parent" = "." ] && echo "./$$addon" || echo "$$addon" ; done)
AVAILABLE_FEATURES:=$(shell for dir in $(AVAILABLE_IMPLS) ; do echo $$(dirname $$dir) ; done | sort | uniq)

# Add staging dir includes if it is defined
STAGINGDIR?=
ifdef STAGINGDIR
STAGE_LIB_DIRS:=$(wildcard $(STAGINGDIR)/lib $(STAGINGDIR)/usr/lib $(STAGINGDIR)/usr/local/lib)
STAGE_INC_DIRS:=$(wildcard $(STAGINGDIR)/include $(STAGINGDIR)/usr/include $(STAGINGDIR)/usr/local/include)
else
STAGE_LIB_DIRS?=
STAGE_INC_DIRS?=
endif

V?=0
ifneq ($(filter $(strip $(V)),2),)
$(call echoone,Summary:)
$(call echoone,* Name: $(name)$(,) type: $(type)$(,) out: $(out))
$(call echolist,* Core src dirs:,$(CORE_SRC_DIRS))
$(call echolist,* Core inc dirs:,$(CORE_INC_DIRS))
$(call echolist,* Resource lib dirs:,$(RES_LIB_DIRS))
$(call echolist,* Resource inc dirs:,$(RES_INC_DIRS))
ifdef STAGINGDIR
$(call echolist,* Staging inc dirs:,$(STAGE_INC_DIRS))
$(call echolist,* Staging lib dirs:,$(STAGE_LIB_DIRS))
endif
$(call echoone,* Build dir: $(CURDIR))
$(call echoone,* Add-ons:,)
$(foreach feature,$(AVAILABLE_FEATURES),$(call echoone,  - "$(feature)": $(foreach i,$(filter $(feature)/%,$(AVAILABLE_IMPLS)),$(if $(filter $(addprefix $(feature)/,$(value ADDON_$(feature))),$(i)),[$(notdir $(i))],$(notdir $(i))))))
$(call echoone,)
endif

# Check Add-on selection
$(foreach feature,$(AVAILABLE_FEATURES),$(if $(filter-out $(AVAILABLE_IMPLS),$(addprefix $(feature)/,$(value ADDON_$(feature)))),$(error Invalid implementation choice for feature "$(feature)": "$(value ADDON_$(feature))"),))

# Get Add-on list
ADDONS:=$(foreach feature,$(AVAILABLE_FEATURES),$(addprefix $(feature)/,$(value ADDON_$(feature))))

# Check Add-on vars
$(foreach addon,$(ADDONS),$(if $(call getconfvar,$(BASEDIR)/add-on/$(addon)/mk.spec,type),,$(error "type" is not defined in $(BASEDIR)/add-on/$(addon)/mk.spec)))
$(foreach addon,$(ADDONS),$(if $(call getconfvar,$(BASEDIR)/add-on/$(addon)/mk.spec,name),,$(error "name" is not defined in $(BASEDIR)/add-on/$(addon)/mk.spec)))

# The build directory for add-ons is the same than our
ADDONS_BUILDDIR:=$(REL_BUILDDIR)

# Add-on flags
ADDONS_CPPFLAGS+=$(addprefix -I$(BASEDIR)/add-on/,$(addsuffix /include,$(ADDONS)))
ADDONS_LDFLAGS+=$(addprefix -L$(BASEDIR)/add-on/,$(addsuffix /$(ADDONS_BUILDDIR),$(ADDONS)))
# Dynamic libs
ADDONS_LIBS+=$(foreach addon,$(ADDONS),$(if $(filter $(call getconfvar,$(BASEDIR)/add-on/$(addon)/mk.spec,type),lib),-l$(call getconfvar,$(BASEDIR)/add-on/$(addon)/mk.spec,name)))
# Static libs
ADDONS_LIBS+=$(foreach addon,$(ADDONS),$(if $(filter $(call getconfvar,$(BASEDIR)/add-on/$(addon)/mk.spec,type),staticlib),-Wl$(,)--whole-archive$(,)-l$(call getconfvar,$(BASEDIR)/add-on/$(addon)/mk.spec,name)$(,)--no-whole-archive))

# Add core source dirs
# > Use this to put all .o in a single folder.
# > This fails if two files have the same name
# > vpath %.c $(CORE_SRC_DIRS)
VPATH+=$(BASEDIR)/

# Add core include dir and resource include dir in CFLAGS
CPPFLAGS+=$(addprefix -I,$(CORE_INC_DIRS) $(RES_INC_DIRS))
CPPFLAGS+=$(addprefix -isystem ,$(STAGE_INC_DIRS))

# Add resource lib dir
LDFLAGS+=$(addprefix -L,$(RES_LIB_DIRS) $(if $(STAGINGDIR),$(STAGE_LIB_DIRS))) \
         $(addprefix -Wl$(,)-rpath-link$(,),$(RES_LIB_DIRS) $(if $(STAGINGDIR),$(STAGE_LIB_DIRS)))

# Add add-on flags
SUBADDON_CPPFLAGS?=
SUBADDON_CFLAGS?=
SUBADDON_CXXFLAGS?=
SUBADDON_LDFLAGS?=
ADDONS_CPPFLAGS?=
ADDONS_CFLAGS?=
ADDONS_CXXFLAGS?=
ADDONS_LDFLAGS?=
CPPFLAGS+=$(ADDONS_CPPFLAGS) $(SUBADDON_CPPFLAGS)
CFLAGS+=$(ADDONS_CFLAGS) $(SUBADDON_CFLAGS)
CXXFLAGS+=$(ADDONS_CXXFLAGS) $(SUBADDON_CXXFLAGS)
LDFLAGS+=$(ADDONS_LDFLAGS) $(ADDONS_LIBS) $(SUBADDON_LDFLAGS)

CPPFLAGS+=-Wall -Wextra -Wno-unused
LDFLAGS+=-Wl,--no-undefined

ifneq ($(filter $(strip $(V)),2),)
$(info Debug:)
$(info - CPPFLAGS=$(CPPFLAGS))
$(info - LDFLAGS=$(LDFLAGS))
$(info )
endif
# Define objects
# > Use this to put all .o in a single folder
# > This fails if two files have the same name
# > OBJS:=$(patsubst %.c,%.o,$(notdir $(wildcard $(addsuffix /*.c,$(CORE_SRC_DIRS)))))
SRCS_C:=$(subst $(BASEDIR)/,,$(wildcard $(addsuffix /*.c,$(CORE_SRC_DIRS))))
SRCS_CXX:=$(subst $(BASEDIR)/,,$(wildcard $(addsuffix /*.cpp,$(CORE_SRC_DIRS))))
SRCS:=$(SRCS_C) $(SRCS_CXX)
OBJS:=$(shell echo $(patsubst %.c,%.o,$(SRCS_C)) $(patsubst %.cpp,%.o,$(SRCS_CXX)) \
              | sed -e 's|/\+|/|g')

ADDONS_RULES:=$(addsuffix .addon,$(ADDONS))
ADDONS_INSTALL:=$(addsuffix .addon-install,$(ADDONS))
ADDONS_RELEASE:=$(addsuffix .addon-release,$(ADDONS))

ALL=$(ADDONS_RULES) $(out)
$(out): $(ADDONS_RULES)

-include $(BASEDIR)/mk.inc
include $(MKDIR)/build.mk

# Define the "all" default rule
.DEFAULT_GOAL := all
.PHONY: all install_headers
all: $(ALL)

PREFIX?=usr/local

INSTALL_TYPE_bin=bin
INSTALL_TYPE_lib=lib
INSTALL_TYPE_staticlib=lib
INSTALL_TYPE_dllib=lib
INSTALL_TYPE=$(value INSTALL_TYPE_$(type))

ifndef install_dir
ifeq ($(type),dllib)
ifdef SUBADDON_PARENT
install_dir:=$(PREFIX)/$(INSTALL_TYPE)/$(SUBADDON_PARENT)
else
install_dir:=$(PREFIX)/$(INSTALL_TYPE)/$(name)
endif
else
install_dir:=$(PREFIX)/$(INSTALL_TYPE)
endif
endif

install: install_headers $(ADDONS_INSTALL)
release:                 $(ADDONS_RELEASE)

# Don't install/release static libs
ifneq ($(type),staticlib)
install: $(DESTDIR)/$(install_dir)/$(out)
release: $(DESTDIR)/$(install_dir)/$(out)
endif

ifneq ($(filter $(strip $(V)),1 2),)
MAKEV:=
INSTALLV:=
INSTALL_V:=-v
else
MAKEV:=@
INSTALLV:=@
INSTALL_V:=
endif

%.addon:
	$(MAKEV)$(MAKE) -f $(MKDIR)/main.mk -C $(BASEDIR)/add-on/$(@:.addon=) \
	    SUBADDON_PARENT="$(name)" \
	    SUBADDON_CPPFLAGS="$(addprefix -I,$(CORE_INC_DIRS) $(RES_INC_DIRS)) $(EXTRA_CPPFLAGS)"\
	    SUBADDON_LDFLAGS="$(addprefix -L,$(RES_LIB_DIRS)) $(addprefix -Wl$(,)-rpath-link$(,),$(RES_LIB_DIRS))"

%.addon-install:
	$(MAKEV)$(MAKE) -f $(MKDIR)/main.mk -C $(BASEDIR)/add-on/$(@:.addon-install=) \
	    SUBADDON_PARENT="$(name)" \
	    SUBADDON_CPPFLAGS="$(addprefix -I,$(CORE_INC_DIRS) $(RES_INC_DIRS)) $(EXTRA_CPPFLAGS)" \
	    SUBADDON_LDFLAGS="$(addprefix -L,$(RES_LIB_DIRS)) $(addprefix -Wl$(,)-rpath-link$(,),$(RES_LIB_DIRS))" \
	    install

%.addon-release:
	$(MAKEV)$(MAKE) -f $(MKDIR)/main.mk -C $(BASEDIR)/add-on/$(@:.addon-release=) \
	    SUBADDON_PARENT="$(name)" \
	    SUBADDON_CPPFLAGS="$(addprefix -I,$(CORE_INC_DIRS) $(RES_INC_DIRS)) $(EXTRA_CPPFLAGS)" \
	    SUBADDON_LDFLAGS="$(addprefix -L,$(RES_LIB_DIRS)) $(addprefix -Wl$(,)-rpath-link$(,),$(RES_LIB_DIRS))" \
	    release

$(DESTDIR)/$(install_dir)/$(out): $(out)
	$(INSTALLV)$(INSTALL) $(INSTALL_V) -D $(out) $(DESTDIR)/$(install_dir)/$(out)

install_headers:
	$(INSTALLV)$(INSTALL) $(INSTALL_V) -d $(DESTDIR)/$(PREFIX)/include/
	$(INSTALLV)if [ -d "$(BASEDIR)/include" ] ; \
	then \
	find $(BASEDIR)/include/ -type f -name "*.h" \
	| while read f ; \
	do \
		REL="$$(echo "$$f" | sed -e 's|$(BASEDIR)/*||')" ; \
		echo "$$REL" ; \
		$(INSTALL) $(INSTALL_V) -m 644 -D "$$f" "$(DESTDIR)/$(PREFIX)/$$REL" ; \
	done ; \
	fi

