# Include this makefile to compile simple components
#
# Simple components are defined this way:
#  - include/ contains public headers
#  - src/ contains source files (including subfolders)
#  - mk.spec contains basic configuration:
#    * name=<basename>
#    * type=<lib|bin>
#    (if type is lib, the final binary will be lib<basename>.so)
#    (if type is bin, the final binary will be <basename>.bin)
#  - mk.inc contains Makefile code to modify vars, for example:
#    EXTRA_CFLAGS
#    EXTRA_LDFLAGS
#
# Possible configuration:
# SIMPLE_PREFIX: the prefix to use (default /usr)

SIMPLE_PREFIX?=/usr

export PREFIX=$(SIMPLE_PREFIX)
export EXTRA_CPPFLAGS=$(shell echo $(CONFIG_CROSS_COMPILE_CPPFLAGS))
export EXTRA_CFLAGS=$(shell echo $(CONFIG_CROSS_COMPILE_CFLAGS))
export EXTRA_CXXFLAGS=$(shell echo $(CONFIG_CROSS_COMPILE_CXXFLAGS))
export EXTRA_LDFLAGS=$(shell echo $(CONFIG_CROSS_COMPILE_LDFLAGS))
export DEBUG=$(if $(filter y,$(CONFIG_DEBUG)),1,0)
export WERROR=1

include $(dir $(lastword $(MAKEFILE_LIST)))/tmpdir.mk

simple_all:
	$(MAKE) -f $(STAGINGDIR)/buildutils/mk/main.mk

#IDESTPREFIX:=$(shell echo $$PWD)/dest
IDESTPREFIX:=$(realpath .)/dest
TMPDIR_SRC:=$(IDESTPREFIX)/
TMPDIR_TEMPFILE:=$(IDESTPREFIX).list
TMPDIR_DST:=$(STAGINGDIR)

simple_subinstall:
	rm -rf "$(TMPDIR_SRC)"
	mkdir -p "$(TMPDIR_SRC)"
	# Install in dest
	$(MAKE) -f $(STAGINGDIR)/buildutils/mk/main.mk install \
	        DESTDIR=$(TMPDIR_SRC)
tmpdir_install: simple_subinstall
simple_install: tmpdir_install

simple_release:
	$(MAKE) -f $(STAGINGDIR)/buildutils/mk/main.mk release \
	        DESTDIR=$(ROOT_FS_DIR)

simple_clean:
	$(MAKE) -f $(STAGINGDIR)/buildutils/mk/main.mk clean
	rm -rf $(TMPDIR_SRC)
	rm -f $(TMPDIR_TEMPFILE)

